package ns.gui;

import javax.swing.*;
import javax.swing.text.DefaultCaret;
import java.awt.*;
import java.awt.event.*;
import java.util.logging.Logger;


/**
 * a frame-modal dialog that covers the whole frame and optionally allows the underlying frame to show through.
 *
 * To use:
 *  1. create instance
 * 2. call setVisble(true/false)
 * 3. addKeyListener() to get key events
 *
 */
public class GlassDialog extends JDialog{
    private static Logger log = Logger.getLogger(GlassDialog.class.getName());

    private JFrame parent;
    private JPanel rootPan,viewPan = new JPanel();
    private Color overlayColor;

    /**
     * this color will be painted over the background component, if it has an alpha-value background will show through
     * @param c if null use Color(0xff,0xff,0xff,0xff)  (full transparency)
     */
    public void setOverlayColor(Color c){
        this.overlayColor=c;
    }

    private final FocusAdapter focus = new FocusAdapter() {
        public void focusGained(FocusEvent e) {
            if(isVisible()){
                log.fine("foucAdapter focusGained");
                requestFocus();
               // getFocusTraversalPolicy().getDefaultComponent(GlassDialog.this).requestFocus();
            }
        }
        public void focusLost(FocusEvent e) {
        }
    };
    private ComponentAdapter  thisSizeMove = new ComponentAdapter() {
        public void componentMoved(ComponentEvent e) {
            if(parent.getX()!=getX() || parent.getY()!=getY()){
                parent.setLocation(getX(),getY());
            }
        }
        public void componentResized(ComponentEvent e) {
            if(parent.getWidth()!=getWidth() || parent.getHeight()!=getHeight()){
                parent.setSize(getWidth(),getHeight());
            }
        }
    };
    private ComponentListener parentSizeMove = new ComponentAdapter() {
        public void componentMoved(ComponentEvent e) {
            if(parent.getX()!=getX() || parent.getY()!=getY()){
                setLocation(parent.getX(),parent.getY());
            }
        }
        public void componentResized(ComponentEvent e) {
            if(parent.getWidth()!=getWidth() || parent.getHeight()!=getHeight()){
                setSize(parent.getWidth(),parent.getHeight());
            }
        }
    };


    /**
     *
     * @param parent
     * @param overlayColor  color which will be overlaid over parent component's background (can be null which means transparent)
     */
    public GlassDialog(final JFrame parent, Color overlayColor, JPanel mainPanel) {
        super(parent,parent.getTitle(),false);
        this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        this.parent=parent;
        this.overlayColor = overlayColor;
        setUndecorated(false);
        rootPan = new JPanel();
        rootPan.setLayout(new OverlayLayout(rootPan));
        getContentPane().add(rootPan);
        rootPan.add(viewPan);
        rootPan.add(backPanel);
        this.setFocusable(true);
        viewPan.setOpaque(false);
        viewPan.setLayout(new BorderLayout());
        if(mainPanel!=null) setMainPanel(mainPanel);
    }




    private void bindToParent(boolean b, final JFrame parent) {
        if (b) {
            parent.addFocusListener(focus);
            addComponentListener(thisSizeMove);
            parent.addComponentListener(parentSizeMove);
        } else {
            parent.removeFocusListener(focus);
            removeComponentListener(thisSizeMove);
            parent.removeComponentListener(parentSizeMove);
        }
    }
    public void setVisible(boolean b) {
        if(b){
            if(parent.getX()!=getX() || parent.getY()!=getY()){
                setLocation(parent.getX(),parent.getY());
            }
            if(parent.getWidth()!=getWidth() || parent.getHeight()!=getHeight()){
                setSize(parent.getWidth(),parent.getHeight());
            }
        }
        bindToParent(b,parent);

        super.setVisible(b);
    }



    public void paintAll(Graphics g) {
        super.paintAll(g);    //To change body of overridden methods use File | Settings | File Templates.
    }

    private JPanel backPanel = new JPanel(){
        public void paint(Graphics g) {
            Insets i = parent.getInsets();
            Graphics g1 = g.create();
            g1.translate(-i.left,-i.top);

            parent.paint(g1);
            g1.translate(i.left,i.top);
            //now paint a transparent area over the whole lot
            if (overlayColor!=null) {
                g1.setColor(overlayColor);
                g1.fillRect(getX(),getY(),getWidth(),getHeight());
                log.finer("filled rect with color="+overlayColor+" x="+getX()+" y="+getY()+" h="+getHeight()+" w="+getWidth());
            }
        }
    };

    /**
     * if non-null this panel will be stretched to cover the area of teh parent component, it can be transparent in part.
     * @param mainPanel if null no parent panel is added
     */
    public void setMainPanel(JPanel mainPanel) {
        viewPan.add(mainPanel);
    }


    /**
     * for testing, creates some windows - veryify the work as expected
     * @param args
     */
    public static void main(String[] args) {
        for(int i=0;i<2;i++) createDialog();
    }

    private static void createDialog() {
        final JLabel jLabel = new JLabel("status");
        JButton jb = new JButton("press me");
        JFrame jf = new JFrame("test");
        JPanel jp = new JPanel(new BorderLayout());
        jf.getContentPane().add(jp);
        JPanel mainP = new JPanel(new FlowLayout());
        JPanel bP = new JPanel(new FlowLayout());
        jp.add(mainP);
        bP.add(jb);
        jp.add(bP,BorderLayout.SOUTH);
        mainP.add(jLabel);
        JPanel jta = createMessageArea();
        jta.setPreferredSize(new Dimension(100,100));


        jb.addActionListener(new ActionListener() {
            int i=0;
            public void actionPerformed(ActionEvent e) {
                jLabel.setText("pressed "+(++i)+" times");
            }
        });
        JPanel diaPanel = new JPanel(new FlowLayout());
        final GlassDialog gd = new GlassDialog(jf,null,diaPanel);
        JButton jb1 = new JButton("activate dialog");
        bP.add(jb1);
        jb1.addActionListener(new ActionListener() {
            private boolean activ=false;
            public void actionPerformed(ActionEvent e) {
                gd.setVisible(true);

            }
        }
        );


        //gd.setOverlayColor(new Color(0xff,0x00,0x00,0xff/2));

        JButton dispose = new JButton("hide");
        diaPanel.add(dispose);
        diaPanel.add(new JTextField("hello"));
        diaPanel.setOpaque(false);
        dispose.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                gd.setVisible(false);
            }
        });
        diaPanel.setBackground(new Color(0x00,0x00,0xff,0xff/2));
      //  gd.setMainPanel(diaPanel);
        gd.addKeyListener(new KeyAdapter() {
            int i=0;
            public void keyPressed(KeyEvent e) {
                System.out.println("keypresed="+(++i));
            }

            public void keyReleased(KeyEvent e) {
                System.out.println("keyReleased="+(++i));
            }

            public void keyTyped(KeyEvent e) {
                System.out.println("keyTyped="+(++i));
            }
        });
        diaPanel.addMouseListener(new MouseAdapter() {
            int i=0;
            public void mouseClicked(MouseEvent e) {
                System.out.println("mouseclicked="+(++i));
            }
        });
        diaPanel.add(new JScrollPane(jta));

        jf.setSize(300,200);
        jf.setVisible(true);
    }

    private static JPanel createMessageArea() {
        JPanel jp = new JPanel();
        OverlayLayout ol = new OverlayLayout(jp);
        jp.setLayout(ol);
        JPanel glassPanel = new JPanel();
        glassPanel.setOpaque(false);
        glassPanel.addKeyListener(new KeyAdapter() {});
        glassPanel.addMouseListener(new MouseAdapter(){});
        glassPanel.setFocusable(false);
        jp.add(glassPanel);
        JTextArea messageArea = new JTextArea();
        jp.add(messageArea);
        messageArea.setEditable(false);
        messageArea.setOpaque(true);
        messageArea.setForeground(Color.LIGHT_GRAY);
        messageArea.setBackground(new Color(0x0,0x0,0x0,0xff/2)); //50% black
        messageArea.setFont(new Font("monospaced",Font.BOLD,15));
        DefaultCaret dc = new DefaultCaret(){
            public Dimension getSize() {
                return new Dimension((int)getWidth(),(int)getHeight());
            }
            public double getWidth() {
                return 0;
            }
            public double getHeight() {
                return 0;
            }
        };
        messageArea.setCaret(dc);
        messageArea.setWrapStyleWord(true);
        messageArea.setLineWrap(true);
        messageArea.setHighlighter(null);
        messageArea.setFocusable(false);
        messageArea.setEditable(false);
        messageArea.setText("try clicking on me, then typing some keystrokes (are they absorbed by the jtextarea?)") ;


        return jp;
    }


}
