/**
 * Title: ${NAME}
 * Copyright:    Copyright (c) 17-Jan-2006
 * @author spurlinn
 */
package uk.co.hcsoftware.commander;

import ns.MacApp;

import java.awt.*;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Logger;

/**
 * global properties that don't fit naturally anywhere else, and are shared by various things
 */
public class AppProperties {
    private static final Logger log = Logger.getLogger(AppProperties.class.getName());
    private static AppProperties ourInstance = new AppProperties();
    
    public final static Font plainFont = new Font("lucida sans",Font.PLAIN,12);
    public final static Font italicPlainFont = new Font("lucida sans",Font.ITALIC /*| Font.BOLD*/,12);
    public final static Font boldFont = new Font("lucida sans",Font.BOLD,12);
     public final static Font italicBoldFont=new Font("lucida sans",Font.ITALIC | Font.BOLD,12);

    public final static int DEFAULT_FORE=Color.black.getRGB(),DEFAULT_BACK=new Color(230,230,230).getRGB(),DEFAULT_SEL=new Color(121,118,227).getRGB();
//    public final static int DEFAULT_FORE=Color.green.getRGB(),DEFAULT_BACK=Color.black.getRGB(),DEFAULT_SEL=Color.green.getRGB();


    public static AppProperties getInstance() {
        return ourInstance;
    }

    private AppProperties() {
    }
    File prefsDir;
    public synchronized File getPrefsDirectory(){
        if (prefsDir==null) {
            //by default bookmarks go in ${HOME}.commanderbond/bookmarks.xml
            File homeDir = new File(System.getProperty("user.home"));
            prefsDir= new File(homeDir,".commanderbond");
        }
        return prefsDir;
    }

    private boolean gotCtrlMask;
    private int ctrl_mask;
    public synchronized int getCtrlMask(){
        if (!gotCtrlMask) {
            if (MacApp.isMac()) {
                log.fine("mac!");
                ctrl_mask=Event.META_MASK;
            }
            else ctrl_mask=Event.CTRL_MASK;
            log.fine("ctrl_mask="+ctrl_mask);
            gotCtrlMask=true;
        }
        return ctrl_mask;
    }

    /**
     * attempt to convert arg into a File object (object may be invalid, as uses new File(text) if all else fails)
     * @param text
     * @return
     */
    public static File getFile(String text) {
          if(text.startsWith("file:")){
            try {
                URL u=new URL(text);
                return new File(u.getFile());
            } catch (MalformedURLException e) {
                Main.log.warning("couldn't convert converted urltext="+text+" to file");
            }
        }
        return new File(text);
    }
}
