package uk.co.hcsoftware.commander;

import ns.MacApp;
import uk.co.hcsoftware.commander.gui.CommanderFrame;
import uk.co.hcsoftware.commander.gui.StatusManager;
import uk.co.hcsoftware.commander.io.TempFileManager;
import uk.co.hcsoftware.commander.io.UpdateChecker;
import uk.co.hcsoftware.commander.prefs.UpdatesPanel;

import javax.swing.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.util.prefs.Preferences;

/**

 */ 
public class Main {
    public static final Logger log=Logger.getLogger(Main.class.getName());
    public static String getCtrlName(){ return MacApp.isMac()?"\u2318":"CTRL";}
    public static void main(final String[] args) {
        final StatusManager statusManager = new StatusManager();
        final CommanderFrame cf = new CommanderFrame(statusManager);
//        Spin.setDefaultProxyFactory(new CGLibProxyFactory());
//        java.lang.IllegalArgumentException: Superclass has no null constructors but no arguments were given
        Runnable runnable = new Runnable() {
            public void run() {
                //process the args
                List<File> filesToOpen=new ArrayList<File>();
                for(String arg:args){
                    //from jnlp spec (jsr056) 20060214
                    //association element: The optional association element is a hint to the JNLP client that it wishes to be registered with the
//operating system as the primary handler of certain extensions and a certain mime-type. The association element must have the
//extensions and mime-type attributes[...] An application making such a request should be prepared to have its main method invoked with the arguments -open filename
//and -print filename instead of any arguments listed with the application-desc element.
                    if("-open".equals(arg)){ //ignore the open arg, since default behaviour is to look for files in args
                        continue;
                    }
                    else if("-print".equals(arg)){  //don't support printing
                        filesToOpen.clear();
                        break;
                    }
                    else{
                        File f = AppProperties.getFile(arg);
                        if(f.exists()) filesToOpen.add(f);
                    }
                }
                cf.setVisible(true);
                cf.initDividerLocations();
                if(!filesToOpen.isEmpty()){
                    cf.openFiles(filesToOpen.toArray(new File[filesToOpen.size()]));
                }
                else cf.loadDefaultView(true);
                cf.setDefaultFocus();
            }
        };
        SwingUtilities.invokeLater(runnable);
        TempFileManager.cleanup(true);
        if (Preferences.userNodeForPackage(UpdatesPanel.class).getBoolean("checkForUpates",true)) {
            log.fine("checkForUpates");
            final UpdateChecker updateChecker = new UpdateChecker();
            Action a = updateChecker.getUpdatesAction(cf); //WARNING: this frame might get closed..
            if (a!=null) {
                log.fine("adding checkForUpates action");
                statusManager.addAllFramesAction(a);
            }
            else log.fine("checkForUpates action was null: assuming no updates");
        }
    }

}
