package uk.co.hcsoftware.commander;

import java.util.logging.Logger;


/**
 * Title: WildcardMatcher
 * Copyright:    Copyright (c) 17-Feb-2006
 *
 * @author N Spurling
 */
public class WildcardMatcher {
    public static final Logger log = Logger.getLogger(WildcardMatcher.class.getName());

    public boolean isMatch(String pattern, char wildcard, String test, boolean caseSensitive) {
        if(!caseSensitive){
            pattern=pattern.toLowerCase();
            test=test.toLowerCase();
        }
        return isMatch(pattern.toCharArray(), wildcard, test.toCharArray());
    }

    public boolean isMatch(String pattern, char wildcard, String test) {
        return isMatch(pattern, wildcard, test, true);
    }

    public boolean isMatch(char[] ch, char wildcard, char[] tst) {
        int pos = 0;
        int bpos = 0;
        int bstart = 0;
        for (int i = 0; i < ch.length; i++) {
            if (ch[i] == wildcard) {
                pos = indexOf(ch, bstart, bpos, tst, pos);
                if (pos == -1) {
                    return false;
                }
                pos += bpos;
                bstart += (bpos + 1);
                bpos = 0;
            }
            else {
                bpos++;
            }
        }
        if (bpos > 0) {
            pos = indexOf(ch, bstart, bpos, tst, pos);
            if (pos == -1) {
                return false;
            }
        }
        if (ch.length > 0 && !(ch[0] == wildcard)) {
            //doesn't start with wc
            pos = indexOf(wildcard, ch);
            if (pos == -1) {
                pos = tst.length;
            }
            if (!isSubstring(tst, 0, pos, ch, 0)) {
                return false;
            }
        }
        if (!(ch[ch.length - 1] == wildcard)) {
            //doesn't end with wc
            pos = lastIndexOf(wildcard, ch);
            if (pos == -1) {
                pos = 0;
            }
            else {
                pos++;
            }
            bpos = ch.length - pos;
            if (!isSubstring(tst, tst.length - bpos, bpos, ch, pos)) {
                return false;
            }
        }
        return true;
    }

    private boolean isSubstring(char[] find, int find_start, int find_len, char[] in, int start) {
        int i = 0;
        while (i < find_len && start < in.length) {
            if (find[find_start + i] != in[start++]) {
                return false;
            }
            i++;
        }
        return i == find_len && start <= in.length;
    }

    private int indexOf(char c, char[] in) {
        for (int i = 0; i < in.length; i++) {
            if (c == in[i]) {
                return i;
            }
        }
        return -1;
    }


    private int lastIndexOf(char c, char[] in) {
        for (int i = in.length - 1; i >= 0; i--) {
            if (c == in[i]) {
                return i;
            }
        }
        return -1;
    }

    //index of cs1 in cs2
    private int indexOf(char[] cs1, int cs1_start, int cs1_len, char[] cs2, int cs2_start) {
        for (; cs2_start < cs2.length; cs2_start++) {
            if (isSubstring(cs1, cs1_start, cs1_len, cs2, cs2_start)) {
                return cs2_start;
            }
        }
        return -1;
    }



}
