package uk.co.hcsoftware.commander.bookmarks;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Logger;


/**
 * Dialog for editing a bookmark
 *
 * Title: BookmarkEditor
 * Copyright:    Copyright (c) 25-Oct-2005
 *
 * @author spurlinn
 */
public class BookmarkEditor extends JDialog{
    public static final Logger log = Logger.getLogger(BookmarkEditor.class.getName());

    private JTextField name=new JTextField(),url1=new JTextField(),url2=new JTextField();
    private Bookmark b;
    private Bookmark editMark;
    private static String toString(URL url){
        if(url==null) return "";
        return url.toExternalForm();
    }
    public BookmarkEditor(Dialog owner,Bookmark b) throws HeadlessException {
        super(owner,"edit bookmark: "+b.getName(),true);
        init(b);
    }
    public BookmarkEditor(Frame owner,Bookmark b) throws HeadlessException {
        super(owner,"edit bookmark: "+b.getName(),true);
        init(b);
    }

    private void init(Bookmark b) {
        super.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        this.b=b;
        JPanel mainPan = new JPanel(new BorderLayout(0,5));
        getContentPane().add(mainPan);

        JPanel propsPan = new JPanel();
        //name, url1,url2 (path) in the future, sorted and filter as well
        name.setText(b.getName());
//        url1.setText(toString(b.getF1()));
//        url2.setText(toString(b.getF2()));
        final GridBagLayout layout = new GridBagLayout();
        propsPan.setLayout(layout);
        propsPan.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        GridBagConstraints gbc=new GridBagConstraints();
        JTextField[] jtf={name,url1,url2};
        String[] labels = {"Name:","Location (left):","Location (right):"};
        gbc.insets=new Insets(2,2,2,2);
        int ro=0;
        for(JTextField j:jtf){
            gbc.gridy=ro;
            gbc.gridx=0;
            gbc.weightx=0;
            gbc.anchor=GridBagConstraints.WEST;
            gbc.fill=GridBagConstraints.NONE ;

            final JLabel jl = new JLabel(labels[ro]);
            layout.setConstraints(jl,gbc);
            propsPan.add(jl);
            gbc.gridx=1;
            gbc.weightx=1;
            gbc.weighty=1;
            gbc.fill=GridBagConstraints.BOTH ;
            JScrollPane jsp = new JScrollPane(j);
            jsp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
            jsp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
            layout.setConstraints(jsp,gbc);
            propsPan.add(jsp);
            ro++;
        }
        mainPan.add(propsPan);

        JPanel buttonPan=new JPanel(new FlowLayout(FlowLayout.RIGHT));
        mainPan.add(buttonPan,BorderLayout.SOUTH);
        Action canAction=new AbstractAction("Cancel") {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        };
        Action okAction = new AbstractAction("Save") {
            public void actionPerformed(ActionEvent e) {
                try {
                    save();
                    dispose();
                } catch (InvalidLocationException e1) {
                    JOptionPane.showMessageDialog(BookmarkEditor.this
                            ,"Value for "+e1.getName()+" is invalid"
                            ,"Invalid location",JOptionPane.ERROR_MESSAGE);
                }
            }
        };
        final JButton ok = new JButton(okAction),can=new JButton(canAction);
        buttonPan.add(can);
        buttonPan.add(ok);


        getRootPane().setDefaultButton(ok);
        final InputMap inputMap = mainPan.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        final ActionMap actionMap = mainPan.getActionMap();
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),"esc");
        actionMap.put("esc",canAction);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),"ok");
        actionMap.put("ok",okAction);

        //pack();
        setSize(400,200);
    }

    static class InvalidLocationException extends Exception{
        private String name,value;

        public InvalidLocationException(String name, String value) {
            this.name = name;
            this.value = value;
        }

        public String getName() {
            return name;
        }

        public String getValue() {
            return value;
        }

    }
    private void save() throws InvalidLocationException {
//        try {
//            Bookmark b2=(Bookmark) b.clone();
//            // update the bm and repaint if nec  throw ex if invalid
//            b2.setName(name.getText());
//            try {
//                b2.setF1(toURL(url1.getText()));
//            } catch (MalformedURLException e) {
//                throw new InvalidLocationException("left location",url1.getText());
//            }
//            try {
//                b2.setF2(toURL(url2.getText()));
//            } catch (MalformedURLException e) {
//                throw new InvalidLocationException("right location",url1.getText() );
//            }
//            editMark=b2;
//        } catch (CloneNotSupportedException e) {
//            log.log(Level.WARNING, "CloneNotSupportedException!", e);
//        }
    }

    private URL toURL(final String text) throws MalformedURLException {
        if(text==null || text.length()==0) return null;
        return new URL(text);
    }

    /**
     *
     * @param owner
     * @param b
     * @return new, edited bookmark or null if action was cancelled
     */
    public static Bookmark showEditDialog(Component owner,Bookmark b){
        Window window =SwingUtilities.getWindowAncestor(owner);
        final BookmarkEditor be;
        if (window instanceof Frame) {
            be=new BookmarkEditor((Frame)window,b);
        } else if(window instanceof Dialog){
            be=new BookmarkEditor((Dialog)window,b);
        }else {
            be=new BookmarkEditor((Frame)null,b);
        }
        be.setLocationRelativeTo(owner);
        be.setVisible(true);
        return be.editMark;
    }

    public static void main(String[] args) {
            final Bookmark b = new Bookmark();
            b.setName("test bookmark1");
          //  b.setF1(new URL("file://C:\\Documents and Settings\\spurlinn\\.xql\\Documents and Settings\\spurlinn\\.xql"));
            Bookmark ret=BookmarkEditor.showEditDialog(null,b);
//        } catch (MalformedURLException e) {
//            log.log(Level.WARNING, "exception", e);
//        }
    }
}
