package uk.co.hcsoftware.commander.bookmarks;

import org.jdom.Element;
import uk.co.hcsoftware.commander.AppProperties;
import uk.co.hcsoftware.commander.gui.*;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.gui.file.Vfile;
import uk.co.hcsoftware.commander.io.ImageProvider;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 */
public class BookmarksBar extends JPanel implements Themeable{
    private static final Logger log = Logger.getLogger(BookmarksBar.class.getName());

    private Opener opener;
    private static List<BookmarksBar> bars = new ArrayList<BookmarksBar>();

    private static DefaultTreeModel bookmarks;
    private static BookmarkReader br = BookmarkReader.getInstance();
    private JTree jt;
    private static File tmpUrl,homeUrl;
    private Action mkdirAction;
    private static final Icon bookmarkIcon=ImageProvider.getInstance().getIcon("icons/16/bookmark.png");
    private final AbstractAction duplicateAction = new AbstractAction("Duplicate.."
                    ,ImageProvider.getInstance().getIcon("icons/16/edit_add.png")) {
                public void actionPerformed(ActionEvent e) {
                    duplicate();
                }
            };
    private final AbstractAction renameAction = new AbstractAction("Rename..",ImageProvider.getInstance().getIcon("icons/16/filesaveas.png")) {
                public void actionPerformed(ActionEvent e) {
                    rename();
                }
            };
    private final AbstractAction deleteAction = new AbstractAction("Delete",ImageProvider.getInstance().getIcon("icons/16/editdelete.png")) {
                public void actionPerformed(ActionEvent e) {
                    delete();
                }
            };

    static{
        bookmarks=br.readBookmarks(true);
        homeUrl=new File(System.getProperty("user.home"));
        tmpUrl=new File(System.getProperty("java.io.tmpdir"));
        log.fine("bookmarks="+getBookmarksPrintout(bookmarks));
    }

    private static String getBookmarksPrintout(TreeModel bm){
        StringBuffer sb = new StringBuffer();
        int level=0;
        appendNode(sb,(DefaultMutableTreeNode) bm.getRoot(),level);
        return sb.toString();
    }

    private static void appendNode(StringBuffer sb, DefaultMutableTreeNode root, int level) {
        for(int i=0;i<level;i++) sb.append(' ');
        sb.append((root).getUserObject());
        sb.append('\n');
        for(Enumeration e=root.children();e.hasMoreElements();){
            appendNode(sb,(DefaultMutableTreeNode) e.nextElement(),level+1);
        }
    }


    public BookmarksBar(Opener opener, CommanderFocus cf) {
        super(new BorderLayout(3,3));
        bars.add(this);
        this.opener = opener;

        JPanel homePanel = new JPanel(new GridLayout(1,3,2,2));

        homePanel.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
        add(homePanel,BorderLayout.NORTH);
        homePanel.add(getHomeButton());
        homePanel.add(getTmpButton());
        homePanel.add(getMkDirButton());

        JPanel treePan = new JPanel(new BorderLayout(2,2));
        add(treePan);

        jt = new JTree(bookmarks);
        setupTree(jt);
        treePan.add(new JScrollPane(jt));
        setupDnD();
        new ThemeListener(this);

        if (tss.saveFileExists()) {
            log.fine("restoring bookmark treestate");
            try {
                tss.restoreTreeState(jt);
            } catch (TreeExpansionStateSaver.CannotRestoreStateException e) {
                log.log(Level.WARNING, "couldn't restore tree state", e);
            }
        } else {  //by default expand all the level 1 dirs
            log.fine("tss savefile !exists: expanding level1");
            final TreeModel model = jt.getModel();
            Object root = model.getRoot();
            if(root!=null){
                final int childCount = model.getChildCount(root);
                Object[] path=new Object[2];
                path[0]=root;
                for(int i=0;i<childCount;i++){
                    path[1]=model.getChild(root,i);
                    jt.expandPath(new TreePath(path));
                }
            }
        }

        jt.addTreeExpansionListener(new TreeExpansionListener() {
            public void treeExpanded(TreeExpansionEvent event) {
                saveTreeState();
            }
            public void treeCollapsed(TreeExpansionEvent event) {
                saveTreeState();
            }
        });
    }
    /**
     * use me when the JTree state has changed (cannot listen to the model as this is shared by multiple instances of this class)
     *
     */
    private void saveTreeState() {
        tss.saveTreeState(jt);
    }

    TreeExpansionStateSaver tss = new TreeExpansionStateSaver(new File(AppProperties.getInstance().getPrefsDirectory(),"bookmarktreestate.xml"), true);

    final static Font plainFont = new Font("lucida sans",Font.PLAIN,12);
    final static Font italicPlainFont = new Font("lucida sans",Font.ITALIC /*| Font.BOLD*/,12);
    final static Font boldFont = new Font("lucida sans",Font.BOLD,12);

    private void setupTree(JTree jt) {
        jt.setRootVisible(false);
        jt.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        jt.setAutoscrolls(true);
        jt.setCellRenderer(new DefaultTreeCellRenderer(){
            final Border b = BorderFactory.createEmptyBorder(0,3,0,0);
            public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
                DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode) value;
                final JLabel jl = (JLabel) super.getTreeCellRendererComponent(tree, null, sel, expanded, leaf, row, hasFocus);
                jl.setBorder(b);
                jl.setOpaque(true);
                if (dmtn!=null) {
                    Bookmark b = getBookmark(dmtn);
                    if (b!=null) {
                        jl.setText(b.getName());
                        if(b.isDirectory()){
                            jl.setFont(boldFont);
                            jl.setText(jl.getText());//+"..");
                            jl.setIcon(null);
                        }
                        else if(b.isViewBookmark() && !b.isSinglePanelBookmark()){
                            jl.setIcon(bookmarkIcon);
                            jl.setFont(plainFont);//jl.setFont(italicPlainFont);
                            jl.setVerticalTextPosition(SwingConstants.CENTER);
                            jl.setHorizontalTextPosition(SwingConstants.RIGHT);
                        }
                        else{
                            jl.setIcon(null);
                            jl.setFont(plainFont);
                        }
                    }
                }
                setColors(jl,sel);
                return jl;
            }
            private void setColors(JLabel jl, boolean isSelected){
                //jl.setIcon(null);
                if(backColor!=null&&foreColor!=null&&selColor!=null){
                    if(isSelected){
                        jl.setBackground(selColor);
                        jl.setForeground(backColor);
                    }
                    else{
                        jl.setBackground(backColor);
                        jl.setForeground(foreColor);
                    }
                }
            }
        });
        jt.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if(e.getClickCount()==2){
                    openSelectedBookmark(false);
                }
            }
        });

        //delete deletes selected bookmakr
        final AbstractAction delAction = new AbstractAction("delete") {
            public void actionPerformed(ActionEvent e) {
                delete();
            }
        };
        final AbstractAction openAction = new AbstractAction("open") {
            public void actionPerformed(ActionEvent e) {
                 openSelectedBookmark(false);
            }
        };
        final AbstractAction openInNewFrameAction = new AbstractAction("open_new_frame") {
            public void actionPerformed(ActionEvent e) {
                 openSelectedBookmark(true);
            }
        };

        InputMap im = jt.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        ActionMap am = jt.getActionMap();
        am.put("deleteaction",delAction);
        am.put("openaction",openAction);
        am.put("open_new_frame",openInNewFrameAction);

        im.put(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE,0),"deleteaction");
        im.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0),"deleteaction");
        im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),"openaction");
        im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,AppProperties.getInstance().getCtrlMask()),"open_new_frame");

        setupPopup(jt);
    }

    private void setupPopup(JTree jt) {
        //right-click menu
        jt.addMouseListener(new MouseAdapter() {
            private void moused(MouseEvent e){
                if(e.isPopupTrigger()){
                    final JPopupMenu menu = buildMenu(e);
                    menu.show(BookmarksBar.this.jt,e.getX(),e.getY());
                }
            }
            public void mousePressed(MouseEvent e) {moused(e);}
            public void mouseReleased(MouseEvent e) {moused(e);}
        });
    }

    /**
     *
     * @param me can be null
     * @return
     */
    private JPopupMenu buildMenu(MouseEvent me) {
        final JPopupMenu menu=new JPopupMenu();
        menu.add(mkdirAction);
        menu.addSeparator();
        menu.add(duplicateAction);
        menu.add(renameAction);
        menu.add(deleteAction);

        if(me!=null){
            // if bookmark under mouse is a view-bookmark then offer to open in new frame
            final Bookmark bm = getBookmarkAtMouse(me);
            if(bm!=null && bm.isViewBookmark()){
                menu.addSeparator();
                menu.add(new AbstractAction("Open "+bm.getName()+" in new window") {
                    public void actionPerformed(ActionEvent e) {
                         opener.restoreFrameViews(bm.getName(), bm.getEl(),true);
                    }
                });
            }
        }
        return menu;
    }

    /**
     *
     * @param openViewBookmarkInNewFrame open the new bookmark in a new frame - only has an effect if the bookmark is a view bookmark, otherwise opens in current frame
     */
    private void openSelectedBookmark(boolean openViewBookmarkInNewFrame) {
        Bookmark b = getSelectedBookmark();
        if(b!=null && !b.isDirectory()){
            try {
                if (b.isViewBookmark()) {
                    if(b.isSinglePanelBookmark()){
                        opener.restoreSingleView(b.getEl());
                    }
                    else opener.restoreFrameViews(b.getName(), b.getEl(), openViewBookmarkInNewFrame);
                } else {
                    opener.open(b.getFiles());
                }
            } catch (CannotOpenBookmarkException e) {
                JOptionPane.showMessageDialog(this,"Couldn't open bookmark:\n " +
                        "  "+e.getMessage(),"Open bookmark",JOptionPane.ERROR_MESSAGE);
            }
        }
    }


    private Bookmark getBookmark(TreeNode treeNode) {
        log.finest("getBookmark treeNode hasparent="+(treeNode.getParent()!=null));
        Bookmark b = (Bookmark) ((DefaultMutableTreeNode)treeNode).getUserObject();
        return b;
    }

    private JButton getHomeButton() {
        final Action homeAction = new AbstractAction("home") {
            public void actionPerformed(ActionEvent e) {
                try {
                    opener.open(Collections.singletonList(homeUrl));
                } catch (CannotOpenBookmarkException e1) {
                    JOptionPane.showMessageDialog(BookmarksBar.this,"Couldn't open home bookmark:\n " +
                        "  "+e1.getMessage()+"\n(url="+homeUrl+")","Go Home",JOptionPane.ERROR_MESSAGE);
                }
            }
        };
        homeAction.putValue(Action.SHORT_DESCRIPTION,"Open your home directory");
        homeAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_H);
        //NS I like the home icon, but on Mac OS X the buttons without icons have rounded edges and the ones with
        //have squared edges, so either tmp and home buttons need icons or neither (and I can't find an icon for tmp)
        homeAction.putValue(Action.SMALL_ICON, ImageProvider.getInstance().getIcon("icons/16/gohome.png"));
        final JButton jb = new JButton(homeAction);
        jb.setFocusable(false);
        jb.setText("");
        return jb;
    }
    private JButton getTmpButton() {

        final Action homeAction = ActionBuilder.getInstance().init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                try {
                    opener.open(Collections.singletonList(tmpUrl));
                } catch (CannotOpenBookmarkException e1) {
                    JOptionPane.showMessageDialog(BookmarksBar.this,"Couldn't open home bookmark:\n " +
                        "  "+e1.getMessage()+"\n(url="+homeUrl+")","Go Home",JOptionPane.ERROR_MESSAGE);
                }
            }
        })
                .tooltip("Open Temporary Directory")
                .icon(ImageProvider.getInstance().getIcon("icons/16/file_temporary.png"))
                .name("tmp")
                .mnemonicKey(KeyEvent.VK_T)
                .get();
        final JButton jb = new JButton(homeAction);
        jb.setFocusable(false);
        jb.setText("");
        return jb;
    }
    private JButton getMkDirButton() {
        mkdirAction = ActionBuilder.getInstance().init(new AbstractAction() {
                    public void actionPerformed(ActionEvent e) {
                        mkdir();
                    }
                })
                .tooltip("New Bookmarks Folder")
                .icon(ImageProvider.getInstance().getIcon("icons/16/folder_new.png"))
                .name("Create Folder")
                .get();
        final JButton jb = new JButton(mkdirAction);
        jb.setText("");
        jb.setFocusable(false);
        return jb;
    }


    //Themeable interface
    private Color backColor,foreColor,selColor;
    public void setColors(Color backColor, Color foreColor, Color selColor) {
        jt.setBackground(backColor);
        jt.setForeground(foreColor);
        this.backColor=backColor;
        this.foreColor=foreColor;
        this.selColor=selColor;
        repaint();
    }



    public interface Opener{
        public void open(List<File> files) throws CannotOpenBookmarkException;
        public void restoreFrameViews(String title, Element viewElement, boolean newFrame);
        public void restoreSingleView(Element viewElement);
    }

    private void setupDnD() {
        final DropTargetListener dtl = new DropTargetListener(){
            public void dragEnter(DropTargetDragEvent evt) {
                 if(evt.isDataFlavorSupported(ViewStateTransferData.dataFlavor)){
                      evt.acceptDrag(DnDConstants.ACTION_MOVE);
                 }
                else if(evt.isDataFlavorSupported(DataFlavor.javaFileListFlavor)){
                   evt.acceptDrag(DnDConstants.ACTION_COPY);
                }
                else if(evt.isDataFlavorSupported(bookmarkFlavor)){
                    evt.acceptDrag(DnDConstants.ACTION_MOVE);
                }
                else{
                    log.finer("reject drag supported flavors="+evt.getCurrentDataFlavorsAsList());
                    evt.rejectDrag();
                }
            }
            public void dragOver(DropTargetDragEvent evt) {; }
            public void dragExit(DropTargetEvent evt) {;}
            public void dropActionChanged(DropTargetDragEvent evt) {;}
            public void drop(DropTargetDropEvent evt) {
                log.fine("drop isEdt="+SwingUtilities.isEventDispatchThread());
                onDrop(evt);
            }
        };
        jt.setDropTarget(new DropTarget(this,dtl));

        final DragSource ds = new DragSource();
       // ds.createDefaultDragGestureRecognizer(table,DnDConstants.ACTION_COPY,dragGestureListener);
        ds.createDefaultDragGestureRecognizer(jt,DnDConstants.ACTION_MOVE,new DragGestureListener() {
            public void dragGestureRecognized(DragGestureEvent dge) {
                onDrag(dge, ds);
            }
        });
    }

    private void onDrag(DragGestureEvent dge, final DragSource ds) {
        Transferable t=getTransferable();
        if (t!=null) {
            log.finer("starting drag now");
            //NS 20051024: have noticed on Mac OS X 10.2.4/java1.5.0 sometimes drop doesn't succeed e.g. drop onto external window
            //and cursor changes back to normal. After this trags don't get initiated, e.g. call startDrag(..) and nothing happens
            //restarting app cures it..
            ds.startDrag (dge, DragSource.DefaultCopyDrop, t, new DragSourceListener() {
                public void dragEnter(DragSourceDragEvent dsde) {log.finest("dragenter");}
                public void dragOver(DragSourceDragEvent dsde) {log.finest("dragOver");}
                public void dropActionChanged(DragSourceDragEvent dsde) {log.finest("dropActionChanged"); }
                public void dragExit(DragSourceEvent dse) {log.finest("dragExit");}
                public void dragDropEnd(DragSourceDropEvent dsde) {log.finest("dragDropEnd");}
            });
        } else {
            log.fine("nothing to transfer");
        }
    }
    private static final class BookmarkTransferData{}
    private static final DataFlavor bookmarkFlavor = new DataFlavor(BookmarkTransferData.class,"CommanderBond bookmark");

    private Transferable getTransferable(){
        final DefaultMutableTreeNode b = getSelectedNode();
        if(b==null || b.getUserObject()==null || !(b.getUserObject() instanceof Bookmark)){
            log.warning("ignore drag as bad/no sel");
            return null;
        }

        log.finer("start drag sels="+b);
        Transferable t = new Transferable() {
            public DataFlavor[] getTransferDataFlavors() {
                return new DataFlavor[]{
                    bookmarkFlavor
                };
            }
            public boolean isDataFlavorSupported(DataFlavor flavor) {
                return bookmarkFlavor.equals(flavor);
            }
            public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException {
                final boolean dataFlavorSupported = isDataFlavorSupported(flavor);
                log.finer("getTransferData dataFlavorSupported="+dataFlavorSupported);
                if(!dataFlavorSupported) throw new UnsupportedFlavorException(flavor);
                return b;
            }
        };
        return t;
    }


    private void onDrop(final DropTargetDropEvent evt) {
        try {
            log.finer("onDrop evt="+evt);
            if(evt.isDataFlavorSupported(ViewStateTransferData.dataFlavor)){
                evt.acceptDrop(evt.getDropAction());
                log.fine("accept drop ViewStateTransferData");
                // add a new bookmark with this element (cf. bookmark view)
                Object data = evt.getTransferable().getTransferData(ViewStateTransferData.dataFlavor);
                ViewStateTransferData transferData=(ViewStateTransferData) data;
                final Element viewElement = transferData.getViewElement();

                String name = (String) JOptionPane.showInputDialog(BookmarksBar.this
                        ,"enter a name for the bookmark\n"
                        ,"Bookmark View"
                        ,JOptionPane.QUESTION_MESSAGE,null,null
                        ,null
                ) ;
                if (name!=null && name.length()>0) {
                    addBookmark(viewElement, name, null, -1);
                }
            }
            else if(evt.isDataFlavorSupported(DataFlavor.javaFileListFlavor)){
                evt.acceptDrop(evt.getDropAction());
                log.fine("accept drop javafilelist");

                Object data=evt.getTransferable().getTransferData(DataFlavor.javaFileListFlavor);
                List<File> files = (List<File>)data;
                //just drop the 1st Vfile
                if(files.size()>0){
                    final File f = files.get(0);
                    log.finer("drop file="+f);
                    final Vfile vfile = new Vfile(f);
                    //think there might be a swing bug circa 1.4 (seen on Mac OS 10.3)
                    //createBookmark springs a JDialog, which hangs the prog..
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            final Point location = evt.getLocation();
                            createBookmark(getNearestParent(location),vfile, getInsertIndex(location));
                        }
                    }) ;
                }

            }
            else if(evt.isDataFlavorSupported(bookmarkFlavor)){
                evt.acceptDrop(evt.getDropAction());
                log.fine("accept drop bookmark");
                dropBookmark(evt);
            }
            else{
                log.finer("no flavors supported");
            }
        } catch (UnsupportedFlavorException e) {
            log.log(Level.WARNING,"UnsupportedFlavorException",e);  //todo dialog?
        } catch (IOException e) {
            log.log(Level.WARNING,"IOException",e);
        }
    }

    private void dropBookmark(final DropTargetDropEvent evt) {
        try {
            DefaultMutableTreeNode bookmarkNode=(DefaultMutableTreeNode) evt.getTransferable().getTransferData(bookmarkFlavor);
            Point dropLocation = evt.getLocation();
            final DefaultMutableTreeNode newParent = getNearestParent(dropLocation);
            final DefaultMutableTreeNode oldParent = (DefaultMutableTreeNode) bookmarkNode.getParent();
            log.fine("oldParent="+oldParent.getUserObjectPath()+" newP="+newParent.getUserObjectPath());
            int newIndex = getInsertIndex(dropLocation);
            final int oldIndex=oldParent.getIndex(bookmarkNode);

            log.fine("move node="+bookmarkNode+" from index="+oldIndex+" to index="+newIndex);
            if(oldParent==newParent && oldIndex==newIndex){
                log.fine("ignoring drop as is no-op");
                return;
            }
            if(newParent.isNodeAncestor(bookmarkNode)){
                log.fine("ignoring drop as dropNode is ancestor of target node");
                return;
            }
            //delete the node from its prev parent
            final boolean removeNodes = oldIndex>=0;
            if(removeNodes) oldParent.remove(oldIndex);
            else log.warning("couldn't remove node="+bookmarkNode+" from parent="+oldParent+" as index was="+oldIndex);
            //add the node to its new parent, at the specified index
            if(newIndex<0){
                newParent.insert(bookmarkNode,0);
                newIndex=0;
            }
            else{
                if(removeNodes && newParent==oldParent && oldIndex<newIndex){
                    newIndex--;   //we have removed 1 node from above our insertion point, so the insertion point has changed
                }
                newParent.insert(bookmarkNode,newIndex);
            }

            //now fire events
            if(removeNodes) bookmarks.nodesWereRemoved(oldParent,new int[]{oldIndex},new Object[]{bookmarkNode});
            bookmarks.nodesWereInserted(newParent,new int[]{newIndex});
            saveTreeState();
            //now select the new bookmarkNode
            selectNode(bookmarkNode);
        } catch (UnsupportedFlavorException e) {
            log.log(Level.WARNING,"UnsupportedFlavorException",e);
        } catch (IOException e) {
            log.log(Level.WARNING,"IOException",e);
        }
    }



    private void createBookmark(DefaultMutableTreeNode parent, Vfile vf, int index) {
        if (vf!=null) {
            if (!vf.exists() || vf.isDirectory()) {
                log.finer("prompt user for create bookmark vfile="+vf);
                String name = (String) JOptionPane.showInputDialog(this
                        ,"enter a name for the bookmark\n"
                        +vf.getName()
                        ,"Create Bookmark"
                        ,JOptionPane.OK_CANCEL_OPTION,null,null
                        ,vf.getName()
                ) ;
                if(name!=null){
                    addBookmark(name, parent, index,vf.getFile());
                }
            }
            else{
                JOptionPane.showMessageDialog(this,"can't bookmark files, only directories","Create Bookmark",JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    /**
     *
     * @param name
     * @param parentNode if null add it under root
     * @param index  if -1 then add to end
     */
    public void addBookmark(String name, DefaultMutableTreeNode parentNode, int index,File... files) {
        log.fine("addBookmark name="+name);
        final Bookmark b = Bookmark.createBookmark(name,files);
        log.fine("b="+b);
        addBookmark(b, parentNode, index);
    }

    private void addBookmark(final Bookmark b, DefaultMutableTreeNode parentNode, int index) {
        final DefaultMutableTreeNode parent = (DefaultMutableTreeNode) (parentNode==null?jt.getModel().getRoot():parentNode);
        DefaultMutableTreeNode child = new DefaultMutableTreeNode(b);
        child.setAllowsChildren(false);
        DefaultTreeModel dtm = bookmarks;
        final int i;
        if(index<0){
            parent.add(child);
            i=parent.getChildCount()-1;
        }
        else{
            parent.insert(child,index);
            i=index;
        }
        log.fine("inserted node="+child+" @"+index+" i="+i);
        dtm.nodesWereInserted(parent,new int[]{i});
        //now select the new b
        selectNode(child);
    }

    /**
     *
     * @param el jdom element holding all the info necessary to restore this state from the Opener.restoreState(Element) method
     * @param name name to associate this bookmark with in the GUI (doesn't have to be unique)
     * @param parent
     * @param index
     */
    public void addBookmark(Element el, String name, DefaultMutableTreeNode parent, int index) {
         final Bookmark b = Bookmark.createBookmark(name,el);
        addBookmark(b, parent, index);
    }

    /**
     *
     * @return 'current working directory' If selectd node is a leaf then it's parent, if not then itself, if no selection then the root
     */
    private DefaultMutableTreeNode getCwd() {
        TreePath tp = jt.getSelectionPath();
        TreeNode result;
        if(tp==null) result = (DefaultMutableTreeNode) jt.getModel().getRoot();
        else{
            TreeNode sel = (TreeNode) tp.getLastPathComponent();
            result = getNearestParent(sel);
        }
        return (DefaultMutableTreeNode) result;
    }

    private DefaultMutableTreeNode getNearestParent(Point location){
        TreePath tp = jt.getClosestPathForLocation(location.x,location.y);
        if(tp==null){
            log.fine("not closestpath adding to root");
            return (DefaultMutableTreeNode) jt.getModel().getRoot();
        }
        else{
            final TreeNode nearestNode = (TreeNode) tp.getLastPathComponent();
            return (DefaultMutableTreeNode) getNearestParent(nearestNode);
        }
    }
    private int getInsertIndex(Point insertLocation){
        TreePath tp = jt.getClosestPathForLocation(insertLocation.x,insertLocation.y);
        if(tp==null){
            return -1;
        }
        else{
            final TreeNode nearestNode = (TreeNode) tp.getLastPathComponent();
            final DefaultMutableTreeNode parent = getNearestParent(insertLocation);
            //if nearest node is a parent, then make it the lastchild, else after the nearestnode
            return parent==nearestNode?parent.getChildCount():parent.getIndex(nearestNode)+1;
        }
    }

    private TreeNode getNearestParent(TreeNode sel) {
        if(!sel.getAllowsChildren()){
            return  sel.getParent();
        }
        else return sel;
    }


    // commandable interface impl
    public void delete() {
        DefaultMutableTreeNode dmtn = getSelectedNode();
        if(dmtn!=null){
            Bookmark b = getBookmark(dmtn);
            String message = b==null?"Delete bookmark directory?":"Delete bookmark "+b.getName()+" ?";
            if(JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(this, message
                    ,"Delete",JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE)){
                DefaultMutableTreeNode parent=(DefaultMutableTreeNode) dmtn.getParent();
                if(parent!=null){
                    Object[] nodes = {dmtn};
                    int[] indices={bookmarks.getIndexOfChild(parent,dmtn)};
                    parent.remove(dmtn);
                    bookmarks.nodesWereRemoved(parent,indices,nodes);
                    saveTreeState();
                }
                else log.warning("couddn't remove bookmark as its parent was null! bm="+dmtn);
            }
        }
    }
    public void mkdir() {
        log.fine("mkdir pressed");
        String dest=(String)JOptionPane.showInputDialog(this,
                "Create Bookmarks Directory:"
                ,"Create Bookmarks Directory",JOptionPane.QUESTION_MESSAGE,null
                ,null,"");
        if(dest!=null && dest.trim().length()>0){
            DefaultMutableTreeNode parent = getCwd();
            Bookmark b = new Bookmark(dest);
            DefaultMutableTreeNode child = new DefaultMutableTreeNode(b);
            parent.add(child);
            bookmarks.nodesWereInserted(parent,new int[]{parent.getChildCount()-1});
            saveTreeState();
            //now select the new dir
            selectNode(child);
        }
    }
    private void selectNode(DefaultMutableTreeNode child){
        if(child!=null){
            TreePath tp = new TreePath(child.getPath());
            jt.setSelectionPath(tp);
            //scroll to selected
            jt.scrollPathToVisible(tp);
        }
    }
    private Bookmark getBookmarkAtMouse(MouseEvent me){
        final TreePath tp = jt.getPathForLocation(me.getX(),me.getY());//jt.getSelectionPath();
        if(tp!=null){
            DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode) tp.getLastPathComponent();//.getSelectedNode();
            if(dmtn!=null){
                return getBookmark(dmtn);
            }
        }
        return null;
    }

    private Bookmark getSelectedBookmark(){
        DefaultMutableTreeNode dmtn = getSelectedNode();
        if(dmtn!=null){
            return getBookmark(dmtn);
        }
        return null;
    }

    private DefaultMutableTreeNode getSelectedNode() {
        final TreePath tp = jt.getSelectionPath();
        if(tp==null) return null;
        DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode) tp.getLastPathComponent();//.getSelectedNode();
        return dmtn;
    }

    public void rename() {
        DefaultMutableTreeNode dmtn = getSelectedNode();
        Bookmark b;
        if (dmtn!=null && (b=(Bookmark) dmtn.getUserObject())!=null) {
            String dest=(String)JOptionPane.showInputDialog(this,
                    "rename "+b.getName()+" to:"
                    ,"Rename",JOptionPane.QUESTION_MESSAGE,null
                    ,null,b.getName());
            if(dest!=null){
                b.setName(dest);
                bookmarks.nodeChanged(dmtn);
            }
        }
    }
    public void duplicate() {
        DefaultMutableTreeNode dmtn = getSelectedNode();
        Bookmark b;
        if (dmtn!=null && (b=(Bookmark) dmtn.getUserObject())!=null && !b.isDirectory()) {
            String dest=(String)JOptionPane.showInputDialog(this,
                    "duplicate "+b.getName()+" new name:"
                    ,"Duplicate",JOptionPane.QUESTION_MESSAGE,null
                    ,null,b.getName());
            if(dest!=null){
                final TreeNode parent = dmtn.getParent();
                try {
                    Bookmark b2=(Bookmark) b.clone();
                    b2.setName(dest);
                    addBookmark(b2, (DefaultMutableTreeNode) parent, parent.getIndex(dmtn));
                } catch (CloneNotSupportedException e) {
                    log.log(Level.WARNING, "exception", e);
                }
            }
        }
    }

}
