package uk.co.hcsoftware.commander.gui;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: AutocompleteJumper
 * Copyright:    Copyright (c) 13-Jun-2006
 *
 * @author N Spurling
 */
public abstract class AutocompleteJumper extends KeyAdapter {
    private static final Logger log = Logger.getLogger(AutocompleteJumper.class.getName());
    protected final RowIdentifier ri;
    protected static int id=0;
    protected final int myId;
    private long lastTimeCharTyped=-1;
    private final long maxInterval_ms=2000;
    private String isearch="";

    public AutocompleteJumper(RowIdentifier ri) {
        this.ri = ri;
        myId=id++;
    }


    public void keyTyped(KeyEvent e) {   //jump to letter if typing
        char c = e.getKeyChar();
        //if it's more than 2s since last key was typed then reset isearch
        long time=System.currentTimeMillis();
        if(lastTimeCharTyped>0 && time-lastTimeCharTyped > maxInterval_ms){
            reset();
        }
        lastTimeCharTyped=time;

        log.log(Level.FINER,"chartyped="+c+" jumper="+toString()+" ke="+e/*,new Throwable()*/);
        if(c==KeyEvent.VK_BACK_SPACE){
            if(isearch.length()==1)
                reset();
            else if(isearch.length()>1) isearch= isearch.substring(0,isearch.length()-1);
        }
        else if(c==KeyEvent.VK_ESCAPE || c==KeyEvent.VK_DOWN || c==KeyEvent.VK_ENTER || c==KeyEvent.VK_UP){
            reset();
            return;
        }
        else if(e.getModifiers()==0 && e.getModifiersEx()==0 /*&& (Character.isLetter(c) || Character.isDigit(c))*/){
            isearch+=Character.toLowerCase(c);  //don't ignore non-alphanumeric as there might be punctuation in name
        }
        else{
            log.fine("ignorning char="+c);
            return;
        }


        // if(e.getModifiers()==0 && e.getModifiersEx()==0 && (Character.isLetter(c) || Character.isDigit(c))){
        //  char upper = Character.toUpperCase(c);
        //select the first vfile beginning with this letter
        int row=getSelectedRow()+1;// start from selrow+1 and visit every row in sequence (i.e. wrap at end)
        if(isearch.length()>1 && row>0){ //if there is an isearch ongoing then start searching from the current row
            //not the nextrow (in practise this seems more intuitive)
            row--;
        }
        // (if selrow==-1 does the right thing)
        final int rowCount = getRowCount();
        for(int i=0;i<rowCount;i++,row++){
            if(row>=rowCount) row=0;
            String name = ri.getRowId(row);
            if(name==null) continue;
            name=name.toLowerCase();
            if (name.startsWith(isearch)) {
                log.finer("found matching row for char="+c+" index="+row+" fname="+name);
                setSelectionInterval(row,row);
                scrollToVisible(row);
                return;
            }
        }
        log.finer("no file having firstletter matching c="+c);
    }

    protected abstract void setSelectionInterval(int start, int end);

    protected abstract int getRowCount();

    protected abstract int getSelectedRow();

    protected abstract void scrollToVisible(int row);

    private void reset() {
        isearch="";
    }

    public String toString() {
        return "autocomplete jumper id="+myId;
    }

    public static interface RowIdentifier{
        public String getRowId(int viewRow);
    }
}
