package uk.co.hcsoftware.commander.gui;

import ns.MacApp;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import uk.co.hcsoftware.commander.AppProperties;
import uk.co.hcsoftware.commander.CannotSaveStateException;
import uk.co.hcsoftware.commander.Main;
import uk.co.hcsoftware.commander.bookmarks.BookmarksBar;
import uk.co.hcsoftware.commander.gui.file.DirectoryPanel;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.gui.util.ActionDispatcher;
import uk.co.hcsoftware.commander.gui.util.GuiUtilities;
import uk.co.hcsoftware.commander.gui.util.WeakListener;
import uk.co.hcsoftware.commander.gui.usercommands.CommandManager;
import uk.co.hcsoftware.commander.io.ImageProvider;
import uk.co.hcsoftware.commander.io.SaveTarget;
import uk.co.hcsoftware.commander.prefs.PrefsDialog;
import uk.co.hcsoftware.commander.prefs.GeneralConfigPanel;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.BackingStoreException;
import java.util.prefs.PreferenceChangeEvent;
import java.util.prefs.PreferenceChangeListener;
import java.util.prefs.Preferences;
/**  todo: F3 views file (open in viewer) F4 edits file (open in notepad?) ENTER 'launches' file
 *   todo: remember sorting?
 * todo: filter? (not sure about this as breaks statelessness - maybe in new window?)
 * todo: integrated texteditor? give it some fancy features, maybe used nio so it can browse large files.. choices such as:
 * charset, bin/text mode, syntax highlighting?
 *
 * todo: change tar to virtual browser (like zip)
 *
 * todo: use netbenas filesystem api/spi ?
 *
 *
 * todo: recycle bin impl (write custom)
 * todo: manual/quick guide
 *
 *
 */
public class CommanderFrame extends  JFrame implements PreferenceChangeListener{
    private static final Logger log=Logger.getLogger(CommanderFrame.class.getName());
    private static final Preferences prefs = Preferences.userNodeForPackage(CommanderFrame.class);

    private static final int GAP = 6;

    private JToolBar buttonPanel= new JToolBar();
    private static AboutDialog aboutDialog;
    private final JMenu fileMenu = new JMenu("File");
    private static boolean registeredMac;
    private final BookmarksBar bookmarsBar;
    private final CommanderFocus cf ;

    private final static int ctrlMask=AppProperties.getInstance().getCtrlMask();
    private final JSplitPane bookmarksViewSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);


    private static final List<CommanderFrame> commanderFrames=new LinkedList<CommanderFrame>();
    private final StatusManager statusManager;
    private ActionBuilder actionBuilder = ActionBuilder.getInstance();
    private final CommanderFrame.StatusPanel statusPanel;

    public CommanderFrame(StatusManager statusManager) throws HeadlessException {
        super("CommanderBond");
        this.statusManager = statusManager;
        super.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        super.setIconImage(ImageProvider.getInstance().getImage("icons/16/file-manager.png"));
        setGlassPane(glassPanel);
        this.cf=new CommanderFocus();
        setupSplitPane(bookmarksViewSplitPane, 0f); //give all the space on resize to the main view panel
        final Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());
        viewsPanel=new ViewPanel(this,new DirectoryPanel(this,cf));
        viewsPanel.split(new DirectoryPanel(this,cf),JSplitPane.HORIZONTAL_SPLIT);
        setupActions();
        final BookmarksBar.Opener opener = new BookmarksBar.Opener() {
            private void setCommandule(Commandule oldCommandule, Commandule newCommandule){
                ViewPanel vp=viewsPanel.findOwner(oldCommandule);
                if(vp!=null){
                    vp.setCommandule(newCommandule);
                }
            }
            private void setDirPanel(Commandule c, final File f) {
                if (c!=null) {
                    DirectoryPanel newCommandule = new DirectoryPanel(CommanderFrame.this,cf);
                    setCommandule(c,newCommandule);
                    newCommandule.openFile(f);
                }
            }

            public void open(List<File> files)  {
                if (!files.isEmpty()) {
                    Commandule c = getCurrentCommandule();
                    if(c!=null) setDirPanel(c, files.get(0));
                    if(files.size()>1){
                        c=getNextCommandule();
                        if(c!=null) setDirPanel(c,files.get(1));
                    }

                }
            }


            public void restoreFrameViews(String title, Element viewElement, boolean newFrame) {
                if (!newFrame) {
                    viewsPanel.restoreState(viewElement);
//                    setWindowTitle(title);
                } else {
                    CommanderFrame cf = createNewFrame();
                    cf.viewsPanel.restoreState(viewElement);
                    cf.getFirstCommandule().getFocus();
                    cf.setWindowTitle(title);
                }
            }

            public void restoreSingleView(Element viewStateElement) {
                Commandule c = getCurrentCommandule();
                final Commandule newCommandule;
                if (c!=null) {
                    final Element viewElement = viewStateElement.getChild("view");
                    newCommandule = ViewPanel.getCommandule(CommanderFrame.this,viewElement);
                    setCommandule(c,newCommandule);
                }
            }
        };
        bookmarsBar = new BookmarksBar(opener, cf);

        createMenuBar();

        final JMenu helpMenu = new JMenu("Help");
        getJMenuBar().add(helpMenu);
        helpMenu.add(new AbstractAction("Keyboard shortcuts"){
            ShortcutsDialog sd;
            public void actionPerformed(ActionEvent e) {
                if (sd==null) {
                    List<ShortcutsDialog.ShortcutDescriptor> l=new ArrayList<ShortcutsDialog.ShortcutDescriptor>();
//                    for(Action a:actions){
//                        l.add(new ShortcutsDialog.ShortcutDescriptor(a));
//                    }
                    sd = new ShortcutsDialog(CommanderFrame.this,l);
                    sd.pack();
                }
                sd.setLocationRelativeTo(CommanderFrame.this);
                sd.setVisible(true);
                sd.requestFocus();
            }
        });
        setupMac(helpMenu);


        buttonPanel.setBorder(BorderFactory.createEmptyBorder(GAP,GAP,GAP,GAP));
        buttonPanel.setFloatable(false);
        if (GeneralConfigPanel.showCommanderButtonsPref.getValue()) {
            contentPane.add(buttonPanel,BorderLayout.NORTH);
        }

        bookmarksViewSplitPane.setOneTouchExpandable(true);

        contentPane.add(bookmarksViewSplitPane);
        bookmarksViewSplitPane.add(bookmarsBar);
        JPanel commandulesPanel = new JPanel(new BorderLayout(2,2));
        commandulesPanel.add(viewsPanel);

        statusPanel = new StatusPanel();
        commandulesPanel.add(statusPanel,BorderLayout.SOUTH);

        bookmarksViewSplitPane.add(commandulesPanel);
        //contentPane.add(bookmarsBar,BorderLayout.WEST);
        bookmarsBar.setBorder(BorderFactory.createEmptyBorder(GAP,GAP,GAP,GAP));


        //getFirstCommandule().getFocus();
        pack();
        log.fine("getfocus="+getFirstCommandule().getFocus());

        //restore size
        int h=prefs.getInt("frameheight",600);
        int w=prefs.getInt("framewidth",800);
        log.finer("restore framesize w="+w+" h="+h);
        if(w<10 || h<10){
            log.warning("unsensible framesizes, using defaults w="+w+" h="+h);
            w=800; h=600;
        }
        setSize(w,h);
        addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
                final int h = getHeight();
                final int w = getWidth();
                log.finer("saving framesize w="+w+" h="+h);
                prefs.putInt("frameheight",h);
                prefs.putInt("framewidth",w);
            }
        });
        commanderFrames.add(this);
        addWindowListener(new WindowAdapter() {
            public void windowClosed(WindowEvent e) {// save viewstate on close
                log.fine("windowClosed, saving state to prefs");
                saveCurrentState();
                commanderFrames.remove(CommanderFrame.this);
                if(commanderFrames.isEmpty()){
                    log.info("0 commanderframes left, forcing system exit");
                    System.exit(0);
                }
            }
        });


        //add preflistener for commanderbuttons event
        final Preferences viewPrefs=GeneralConfigPanel.showCommanderButtonsPref.getPrefs();
        WeakListener wl = new WeakListener(this,PreferenceChangeListener.class){
            public Object onNullObserver() {
                viewPrefs.removePreferenceChangeListener((PreferenceChangeListener) getProxy());
                return null;
            }
        };
        viewPrefs.addPreferenceChangeListener((PreferenceChangeListener) wl.getProxy());
    }

    private class StatusPanel extends JPanel{
        private final JLabel frameStatusLabel,mainStatusLabel;
        private JPanel rightPanel,leftPanel;
        public StatusPanel() {
            super(new BorderLayout());
            rightPanel=new JPanel(new FlowLayout(FlowLayout.RIGHT));
            leftPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
            add(rightPanel,BorderLayout.EAST);
            add(leftPanel);//centre
            frameStatusLabel = new JLabel(" ");
            frameStatusLabel.setHorizontalAlignment(JLabel.RIGHT);
            mainStatusLabel = new JLabel(" ");
            mainStatusLabel.setHorizontalAlignment(JLabel.LEFT);
            leftPanel.add(mainStatusLabel);
            frameStatusLabel.setBorder(BorderFactory.createEmptyBorder(2,2,2,6));
            setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
            statusManager.addListener(new StatusManager.Listener() {
                public void actionsChanged(List<Action> a) {
                    buildRightPanel();
                }
                public void mainStatusTextChanged(String newMainStatusText){
                    setMainStatusText(newMainStatusText);
                }
            });
            buildRightPanel();
        }
        public void buildRightPanel(Action... frameActions){
            rightPanel.removeAll();
            rightPanel.add(frameStatusLabel);
            List<Action> actions = new ArrayList<Action>(Arrays.asList(frameActions));
            actions.addAll(statusManager.getActions());
            log.fine("adding "+actions.size()+" statusactions");
            for(Action a:actions){
                final JButton jb = new JButton(a);
                rightPanel.add(jb);
                final Object actionName = a.getValue(Action.NAME);
                if (actionName==null || ((String)actionName).length()==0) {
                    jb.setPreferredSize(new Dimension(20,20)); //icons are 16px^2 so make button about that size
                }
                jb.setFocusable(false);
            }
            validate();
        }

        public void setMainStatusText(String statusText) {
            if (statusText!=null && statusText.length()>0) {
                mainStatusLabel.setText(statusText);
            } else {
                mainStatusLabel.setText(" ");
            }
        }
        public void setFrameStatusText(String statusText, Action... frameAction) {
            if (statusText!=null && statusText.length()>0) {
                frameStatusLabel.setText(statusText);
            } else {
                frameStatusLabel.setText(" ");
            }
            buildRightPanel(frameAction);
        }
    }


    public void preferenceChange(PreferenceChangeEvent evt) {
        if(GeneralConfigPanel.showCommanderButtonsPref.isThisPref(evt)){//viewcontrols value has changed
            log.fine("received prefchagned evt="+evt);
            final boolean show=GeneralConfigPanel.showCommanderButtonsPref.getValue();
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    if(show) getContentPane().add(buttonPanel,BorderLayout.NORTH);
                    else getContentPane().remove(buttonPanel);
                    validate();
                }
            });
        }
    }


    private void setupMac(JMenu helpMenu) {
        if(MacApp.isMac()){
            try {
                if (!registeredMac) {
                    log.info("is mac! registering application");
                    new MacApp(new MacApp.MacImplementation() {
                        public boolean hasPrefsDialog() {
                            return true;
                        }
                        public void about() { showHelpDialog(CommanderFrame.this); }
                        public void preferences() { showPrefs(CommanderFrame.this);}
                        public void openFile(File f) {
                            log.fine("open file="+f);
                            try {
                                CommanderFrame.this.openFile(f,getFirstCommandule());
                            } catch (Exception e) {
                                log.log(Level.WARNING, "exception", e);
                            }
                        }
                        public boolean quit() {
                            saveCurrentState(); //don't seem to get windowclose (or don't have time to save) on mac, so make sure we save here
                            return true;
                        }
                        public void printFile(File f) {
                        }
                        public void reOpenApplication() {
                        }
                        public void openApplication() {
                        }
                    });
                    registeredMac=true;
                }
            } catch (MacApp.CouldntSetupMacException e) {
                log.log(Level.WARNING,"error",e);
            }
        }
        else{
            //not a mac so add a help menu item
            helpMenu.addSeparator();
            helpMenu.add(new AbstractAction("about..."){
                public void actionPerformed(ActionEvent e) {
                    showHelpDialog(CommanderFrame.this);
                }
            });
            fileMenu.addSeparator();
            final AbstractAction prefsAction = new AbstractAction("Preferences.."){
                public void actionPerformed(ActionEvent e) {
                    showPrefs(CommanderFrame.this);
                }
            };
            actionBuilder.init(prefsAction)
                    .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_COMMA,ctrlMask))
                    .name("preferences")
                    .icon(getIcon("configure"))
                    .get();
            fileMenu.add(prefsAction);//add it before teh Exit item
            fileMenu.addSeparator();
            fileMenu.add(new AbstractAction("Exit") {
                public void actionPerformed(ActionEvent e) {
                    saveCurrentState();
                    System.exit(0);
                }
            }) ;
        }
    }

    private void saveCurrentState() {
        try {
            Element el = viewsPanel.saveState();
            Document d = new Document(el);
            XMLOutputter xmlo = new XMLOutputter(Format.getCompactFormat());
            final String xmlString = xmlo.outputString(d);
            log.fine("state="+xmlString);
            prefs.put("viewstate",xmlString);
            try {
                prefs.flush();
            } catch (BackingStoreException e) {
                log.log(Level.WARNING, "exception", e);
            }
            log.fine("saved state.");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public CommanderFocus getCommanderFocus() {
        return cf;
    }

    private static void showPrefs(CommanderFrame cf){
        PrefsDialog pd = new PrefsDialog(cf);
        pd.setLocationRelativeTo(cf);
        pd.setVisible(true);
    }

    /**
     * attempts to open the file, does nothign if that is impossible (e.g. because the c is null)
     * @param f
     * @param c  can be null
     * @return  true if successfully caused c to open f
     */
    private boolean openFile(File f, Commandule c){
        if(c instanceof DirectoryPanel){
            DirectoryPanel dp = (DirectoryPanel) c;
            dp.openFile(f);
            return true;
        }
        return false;
    }
    /**
     * open these directories one per view, for all the views that exist; ignore any files for which there are not views
     * any files that are nonextant or not dirs are skipped
     * @param dirs
     */
    public void openFiles(File... dirs){
        int commandule=0;
        for(File f:dirs){
            if(!f.exists() || !f.isDirectory()){
                log.warning("ignoring openfile="+f+" because doesn't exist or not a dir");
                continue;
            }
            Commandule c = getCommandule(commandule);
            if(c!=null){
                openFile(f,c);
            }
            else{
                log.info("no more commandules, ignoring further files to open..");
                break;
            }
        }
    }

    public void loadDefaultView(boolean restorePrefsView) {
        final File homeDir = new File(System.getProperty("user.home"));
        String prefsState;
        if(restorePrefsView && (prefsState=prefs.get("viewstate","")).length()>0){
            try {
                SAXBuilder db = new SAXBuilder();
                Document d=db.build(new StringReader(prefsState));
                viewsPanel.restoreState(d.getRootElement());
                return;
            } catch (JDOMException e) {
                log.log(Level.WARNING, "exception", e);
            } catch (IOException e) {
                log.log(Level.WARNING, "exception", e);
            }
            log.info("failed to restore state, defaulting to homedirs");
        }
        //load home dir in both panes
        openFile(homeDir,getFirstCommandule());
        openFile(homeDir,getCommandule(1));
    }

    private Commandule getView(){
        Commandule c = getCurrentCommandule();
        if(c==null){
            final List<Commandule> allCommandules = getAllCommandules();
            c = (Commandule) cf.getMostRecentFocusHolder(allCommandules);
            if(c==null){
                log.fine("neither pane has the focus, defaulting to pane1");
                c= getFirstCommandule();
            }
        }
        return c;
    }

    private Commandule getFirstCommandule() {
        return getCommandule(0);//NB: make efficienter
    }
    private Commandule getCommandule(int i) {
        final List<Commandule> allCommandules = getAllCommandules();
        if(i<0 || i>=allCommandules.size()) return null;
        return allCommandules.get(i);//NB: make efficienter
    }
    private void setupActions() {
        final JComponent contentPane = ((JComponent)getContentPane());
        final InputMap contentPaneInputMap = contentPane.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        final ActionMap contentPaneActionMap = contentPane.getActionMap();

        contentPaneInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB,0),"tab");
        contentPaneActionMap.put("tab",new AbstractAction() {  //tab forward
            public void actionPerformed(ActionEvent e) {
                moveFocus(true);
            }
        });
        contentPaneInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB,KeyEvent.SHIFT_MASK),"shifttab");
        contentPaneActionMap.put("shifttab",new AbstractAction() { //tab backward
            public void actionPerformed(ActionEvent e) {
                moveFocus(false);
            }
        });

        final ActionMap viewsPanelActionMap = viewsPanel.getActionMap();
//        final InputMap viewsPanelInputMap = viewsPanel.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

        contentPaneInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_B,ctrlMask),"bookmarks");
        viewsPanelActionMap.put("bookmarks",new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                bookmarsBar.requestFocus();
            }
        });

//        GuiUtilities.removeBindings(contentPane,KeyStroke.getKeyStroke(KeyEvent.VK_F2,0));
         //NS: don't need to register this as JMenu automatically fires them using their keystrokes
//        Action[] actions = {
//            newFrameAction,
//            syncAction,//+Main.getCtrlName()+"+E"),
//            swapAction,//+Main.getCtrlName()+"+U"),
//            unsplitCurr,
//            unsplitAll,
//            splitH,
//            splitV,
//            closeCurr
//        };


//        for(Action a:actions){
//            viewsPanelInputMap.put((KeyStroke) a.getValue(Action.ACCELERATOR_KEY),a.getValue(Action.ACTION_COMMAND_KEY));
//            viewsPanelActionMap.put(a.getValue(Action.ACTION_COMMAND_KEY),a);
//        }
//        for(Action a:commanderActions){
//            log.fine("reg action name="+a.getValue(Action.NAME)+" a="+a);
//            viewsPanelInputMap.put((KeyStroke) a.getValue(Action.ACCELERATOR_KEY),a.getValue(Action.NAME));
//            viewsPanelActionMap.put(a.getValue(Action.NAME),a);
//        }

        setupButtonPanel();
    }

    private void moveFocus(boolean directionForward) {
        if(log.isLoggable(Level.FINER)) log.finer("moveFocus fwd="+directionForward);
        Commandule current = getCurrentCommandule();
        while(current!=null){
            Commandule next = getNextCommandule(current,directionForward);
            if(log.isLoggable(Level.FINER)) log.finer("next="+next+" curr="+current);
            if(next==current){//there is only 1 frame
                return;
            }
            if(next!=null){
                if(next.getFocus()){
                    return;
                }
                else{//try the next one
                    current=next;
                }
            }
            else{
                log.fine("no focusable next commandule! ignoring tab");
                return;
            }
        }
    }

    final List<Action> commanderActions = ActionDispatcher.getInstance().createCommanderActions(new ActionDispatcher.CommanduleSource() {
        public Commandule getCommandule() {
            return getView();
        }
    });
    private void setupButtonPanel() {
        for(Action a:commanderActions) buttonPanel.add(createButton(a));
        buttonPanel.addSeparator();
    }
    private void createMenuBar(){
        JMenuBar jmb = new JMenuBar();
        fileMenu.setMnemonic('f');
        fileMenu.add(newFrameAction);
        fileMenu.add(editWindowTitleAction);
        fileMenu.addSeparator();
        for(Action a:commanderActions) fileMenu.add(a);

        jmb.add(fileMenu);

        JMenu usercommands = new JMenu("Commands");
//        usercommands.setMnemonic('');
        jmb.add(usercommands);

        usercommands.add(actionBuilder.init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                Commandule c = getCurrentCommandule();
                if(c!=null){
                    c.usercommand();
                }
            }
        })
                .name("Run..")
                .get()

        );
        usercommands.addSeparator();
        usercommands.add(actionBuilder.init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                CommandManager.getInstance().showEditor(CommanderFrame.this);
            }
        })
                .name("Configure..")
                .get());

        JMenu view = new JMenu("View");
        view.setMnemonic('v');

        view.add(recall);
        view.addSeparator();
        view.add(syncAction);
        view.add(swapAction);

        view.addSeparator();
        view.add(unsplitCurr);
        view.add(unsplitAll);
        view.add(splitH);
        view.add(splitV);
        view.add(closeCurr);

        view.add(new AbstractAction("Reset to default view",ImageProvider.getInstance().getIcon("icons/16/view_left_right.png")) {
            public void actionPerformed(ActionEvent e) {
              Object[] possibleValues = { "Reset", "Cancel" };
            int selectedValue = JOptionPane.showOptionDialog(CommanderFrame.this,
                    "Are you sure you want to remove all splits and go back to the default (twin) view?","Reset to default view",
                    JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                    possibleValues, possibleValues[0]);
            if(0==selectedValue){
                Commandule first = getFirstCommandule();
                Commandule second = getCommandule(1);
                viewsPanel.unsplit(first);
                if(second!=null){
                    viewsPanel.split(second,JSplitPane.HORIZONTAL_SPLIT);
                }
                first.getFocus();
            }
        }});

        view.addSeparator();
        final String title = "Bookmark Current View";
        view.add(new AbstractAction(title,ImageProvider.getInstance().getIcon("icons/16/bookmark.png")) {
            public void actionPerformed(ActionEvent e) {
                bookmarkCurrentView();
            }
        });

        jmb.add(view);

        if ("true".equals(System.getProperty("debug"))) {
            JMenu debug = new JMenu("debug");
            jmb.add(debug);
            debug.add(new AbstractAction("block input") {
                public void actionPerformed(ActionEvent e) {
                    blockInput("Input blocked\n\n(press ESC to unblock)"
                            ,new KeyAdapter() {
                                public void keyPressed(KeyEvent e) {
                                    log.fine("keyTyped="+e);
                                    if(e.getKeyCode()==KeyEvent.VK_ESCAPE){
                                        unblockInput();
                                    }
                                }
                            }
                    );
                }
            });
        }
        setJMenuBar(jmb);
    }

    private void bookmarkCurrentView() {
        String name = (String) JOptionPane.showInputDialog(CommanderFrame.this
                ,"enter a name for the bookmark\n"
                ,"Bookmark Current View"
                ,JOptionPane.QUESTION_MESSAGE,null,null
                ,null
        ) ;
        if(name!=null){
            Element el = viewsPanel.saveState();
            bookmarsBar.addBookmark(el, name, null, -1);
        }
    }
    private final GlassPanel glassPanel=new GlassPanel();
    public void blockInput(String text){
        blockInput(text,null);
    }
    public void blockInput(String text, KeyListener kl){
        if(kl!=null) glassPanel.setKeyListener(kl);
        updateBlockedMessage(text);
        glassPanel.setVisible(true);
        glassPanel.repaint();//sometimes shows up with previous message for a while.. hopefully this will cure it
        final boolean b = glassPanel.requestFocusInWindow();
        log.finest("requestFocusInWindow="+b);
        if(!b) glassPanel.requestFocus();
    }

    public void updateBlockedMessage(String text) {
        glassPanel.setMessage(text);
    }

    public void unblockInput(){
        glassPanel.setVisible(false);
    }




    private JButton createButton(Action a){
        JButton jb = new JButton(a);
        jb.setFocusable(false);
        KeyStroke accelerator = (KeyStroke) a.getValue(Action.ACCELERATOR_KEY);
        String acceleratorText;
        if (accelerator != null) {
            acceleratorText = getAcceleratorText(accelerator);
            if (acceleratorText.length()>0) {
                jb.setText("<html>"+jb.getText()+"<font color=\"gray\"><small> "+acceleratorText+"</small></font></html>");
            }
        }
        return jb;
    }

    public static String getAcceleratorText(KeyStroke keyStroke) {
        String acceleratorText="";
        int modifiers = keyStroke.getModifiers();
        if (modifiers > 0) {
            log.fine("creatbutton mods="+modifiers+" mac="+MacApp.isMac()+" crtlname="+Main.getCtrlName());
            if (MacApp.isMac() && (modifiers & ctrlMask)!=0) {
                acceleratorText=Main.getCtrlName();
            } else {
                acceleratorText = KeyEvent.getKeyModifiersText(modifiers);
            }
            acceleratorText += '+';
        }

        int keyCode = keyStroke.getKeyCode();
        if (keyCode != 0) {
            acceleratorText += KeyEvent.getKeyText(keyCode);
        } else {
            acceleratorText += keyStroke.getKeyChar();
        }
        return acceleratorText;
    }

    private ImageIcon getIcon(String iconname) {
        return ImageProvider.getInstance().getIcon("icons/16/"+iconname+".png");
    }

//    Action copyAction = new AbstractAction("Copy F5") {
//        public void actionPerformed(ActionEvent e) {
//            final Commandule view = getView();
//            if (view!=null) {
//                view.copy();
//            }
//        }
//    };
//    Action duplicateAction = new AbstractAction("Duplicate SHFT+F5") {
//        public void actionPerformed(ActionEvent e) {
//            final Commandule view = getView();
//            if (view!=null) {
//                view.duplicate();
//            }
//        }
//    };
//    Action renameAction = new AbstractAction("Rename SHFT+F6") {
//        public void actionPerformed(ActionEvent e) {
//            final Commandule view = getView();
//            if (view!=null) {
//                view.rename();
//            }
//        }
//    };
//    Action moveAction = new AbstractAction("Move F6") {
//        public void actionPerformed(ActionEvent e) {
//            final Commandule view = getView();
//            if (view!=null) {
//                view.move();
//            }
//        }
//    };
//    Action deleteAction = new AbstractAction("Delete F8") {
//        public void actionPerformed(ActionEvent e) {
//            final Commandule view = getView();
//            if (view!=null) {
//                view.delete();
//            }
//        }
//    };
//    Action mkdirAction = new AbstractAction("Make Dir F7") {
//        public void actionPerformed(ActionEvent e) {
//            final Commandule view = getView();
//            if (view!=null) {
//                view.mkdir();
//            }
//        }
//    };
    Action syncAction=actionBuilder.init(new AbstractAction("Sync "+Main.getCtrlName()+"+E") {
        public void actionPerformed(ActionEvent e) {
            try {

                final Commandule currentCommandule = getCurrentCommandule();
                final Commandule target = getNextCommandule();

                if (currentCommandule!=null && target!=null) {
                    String otherF = currentCommandule.saveState();
                    if (otherF!=null) {
                        target.restoreState(otherF);
                    }
                }
            } catch (CannotSaveStateException e1) {
                log.fine("cannot save state");
                JOptionPane.showMessageDialog(CommanderFrame.this,"At least one of the views cannot save its state","Swap views",JOptionPane.ERROR_MESSAGE);
            } catch (CannotRestoreSavedStateException e1) {
                log.fine("couldn't sync: "+e1.getMessage());
                JOptionPane.showMessageDialog(CommanderFrame.this,"The target view cannot display this view's data","Sync views",JOptionPane.ERROR_MESSAGE);
            }
        }
    }).tooltip("Set next view to same as current")
                                        .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_E,ctrlMask))
                                        .name("Sync current->next")
                                        .get();
    Action swapAction=actionBuilder.init(new AbstractAction("Swap") {
        public void actionPerformed(ActionEvent e) {
            final Commandule currentCommandule = getCurrentCommandule();
            final Commandule target = getNextCommandule();

            try {
                if (currentCommandule!=null && target!=null) {
                    String url1 = currentCommandule.saveState();
                    String url2 = target.saveState();
                    target.restoreState(url1);
                    currentCommandule.restoreState(url2);
                }
            } catch (CannotSaveStateException e1) {
                log.fine("cannot save state");
                JOptionPane.showMessageDialog(CommanderFrame.this,"At least one of the views cannot save its state","Swap views",JOptionPane.ERROR_MESSAGE);
            } catch (CannotRestoreSavedStateException e1) {
                log.fine("couldn't sync: "+e1.getMessage());
                JOptionPane.showMessageDialog(CommanderFrame.this,"The views cannot display each others' data","Swap views",JOptionPane.ERROR_MESSAGE);
            }
        }
    }) .tooltip("Swap current and next views' positions").keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_U,ctrlMask))
                                        .name("Swap current/next")
                                        .get();
    Action recall=actionBuilder.init(new AbstractAction("open previous dir") {
        public void actionPerformed(ActionEvent e) {
            final Commandule directoryPanel = getCurrentCommandule();
            if (directoryPanel!=null) {
                try {
                    directoryPanel.restorePreviousState();
                } catch (CannotRestoreSavedStateException e1) {
                    log.log(Level.WARNING, "exception", e1);
                }
            }
        }
    })      .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,KeyEvent.ALT_DOWN_MASK))
                        .name("Back")
                        .icon(getIcon("back"))
                        .tooltip("Restore previous view")
                        .get();
//    Action unrecall=new AbstractAction("open next dir") {
//        public void actionPerformed(ActionEvent e) {
//            final Commandule directoryPanel = getCurrentCommandule();
//            if (directoryPanel!=null) {
//                try {
//                    directoryPanel.restorePreviousState();
//                } catch (CannotRestoreSavedStateException e1) {
//                    log.log(Level.WARNING, "exception", e1);
//                }
//            }
//        }
//    };


    Action moveFocusAction=new AbstractAction("tab") {
        public void actionPerformed(ActionEvent e) {
            Commandule c = getNextCommandule();
            if(c!=null) c.getFocus();
        }
    };
//    *  ctrl+0 unsplit current
// * ctrl+1 unsplit all
// * ctrl+2  split current horizontally
// * ctrl+3 split current vertically
// * ctrl+4 resotre split ??  (restores previous split condition to current pane)
    Action unsplitCurr=actionBuilder.init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            Commandule c = getCurrentCommandule();
            unsplit(c);
            c.getFocus();
        }
    })
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_0,ctrlMask))
                .name("Unsplit current")
                .icon(getIcon("view_right"))
                .tooltip("Unsplit current view")
                .get();


    /**
     * unsplit the view, preserving this commandule
     * @param preserve
     */
    private void unsplit(Commandule preserve) {
        if(preserve!=null){
            ViewPanel th = this.viewsPanel.findOwner(preserve);
            if (th!=null) {
                final ViewPanel parentViewPanel = th.getParentViewPanel();
                if (parentViewPanel!=null) {
                    parentViewPanel.unsplit(preserve);
                }
            }
        }
    }

    /**
     * close the selected commandule (if possible)
     */
    Action closeCurr=actionBuilder.init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            Commandule c = getCurrentCommandule();
            //this is the one to close, so find it's counterpart (to preserve)
            if(c!=null){
                ViewPanel th = CommanderFrame.this.viewsPanel.findOwner(c);
                if (th!=null) {
                    th.close();
                }
            }
        }
    })
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_W,ctrlMask))
                .name("Close current")
                .icon(getIcon("editdelete"))
                .tooltip("Close the current view")
                .get();
    Action unsplitAll=actionBuilder.init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            final Commandule currentCommandule = getCurrentCommandule();
            viewsPanel.unsplit(currentCommandule);
            currentCommandule.getFocus();
        }
    })
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_1,ctrlMask))
                .name("Unsplit all")
                .icon(getIcon("view_remove"))
                .tooltip("Unsplit all views")
                .get();
    Action splitH=actionBuilder.init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            split(JSplitPane.VERTICAL_SPLIT);
        }
    })
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_2,ctrlMask))
                .name("Split current horizontally")
                .icon(getIcon("view_top_bottom"))
                .tooltip("Split current view horizontally")
                .get();
     Action splitV=actionBuilder.init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            split(JSplitPane.HORIZONTAL_SPLIT);
            //this is intentional: JSplitPane parlance is the opposite way round to jEdit parlance, and i assume users speak jEdit (which I assume is copied from emacs) not JSplitPane
        }
    })
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_3,ctrlMask))
                .name("Split current vertically")
                .icon(getIcon("view_left_right"))
                .tooltip("Split current view vertically")
                .get();

    private void split(int orientation) {
        Commandule c = getCurrentCommandule();
        if(c!=null){
            ViewPanel th = this.viewsPanel.findOwner(c);
            if (th!=null) {
                th.split(orientation);
            }
        }
    }


    private static void showHelpDialog(CommanderFrame cf) {
        if(aboutDialog==null){
            aboutDialog = new AboutDialog(cf,"about CommanderBond");
        }
        aboutDialog.setLocationRelativeTo(cf);
        aboutDialog.setVisible(true);
    }


    Action newFrameAction = actionBuilder.init(new AbstractAction("New Window") {
        public void actionPerformed(ActionEvent e) {
            CommanderFrame cf = createNewFrame();
            cf.loadDefaultView(false);
            cf.getFirstCommandule().getFocus();
        }
    })      .tooltip("New window")
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_N,ctrlMask))
            .icon(getIcon("window_new"))
    .get();
    Action editWindowTitleAction = actionBuilder.init(new AbstractAction("Edit Window Title") {
        public void actionPerformed(ActionEvent e) {
            String title = JOptionPane.showInputDialog(CommanderFrame.this,"New window title");
            if(title!=null){
                setWindowTitle(title);
            }
        }
    })      .tooltip("Change the title of this window")
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_T,ctrlMask | Event.ALT_MASK ))
    .get();

    private void setWindowTitle(String title) {
        if (title!=null) {
            setTitle(title+" - CommanderBond");
        } else {
            setTitle("CommanderBond");
        }
    }

    private CommanderFrame createNewFrame() {
        CommanderFrame cf = new CommanderFrame(statusManager);
        cf.setSize(this.getSize());
        cf.setLocationByPlatform(true);
        cf.setVisible(true);
        cf.initDividerLocations();
        return cf;
    }

    public void initDividerLocations() {
        final double divloc = prefs.getFloat("mainpan.dividerloc",0.2f);
        log.fine("initing bookmarksViewSplitPane divloc="+divloc);
        bookmarksViewSplitPane.setDividerLocation(divloc);
        //bookmarksViewSplitPane

        bookmarksViewSplitPane.validate(); //make sure bookmarksViewSplitPane causes fileViewPan to recalc its width before we set its dividerlocation
        //fileViewPan.setDividerLocation(divlocFv);
        bookmarksViewSplitPane.addPropertyChangeListener("dividerLocation",new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                log.finer("name="+evt.getPropertyName()+" newval="+evt.getNewValue());
               // int divLoc = ((Integer) evt.getNewValue()).intValue();
                float f = getDividerLocation(bookmarksViewSplitPane);
                log.finer("bookmarksViewSplitPane divLocPc="+f);
                prefs.putFloat("mainpan.dividerloc",f);
            }
        });
    }

    public float getDividerLocation(JSplitPane jsp) {
        int width = jsp.getWidth();
        width-=jsp.getDividerSize();
        BigDecimal divl = new BigDecimal(jsp.getDividerLocation());
        BigDecimal locpc=divl.divide(new BigDecimal(width),2,RoundingMode.HALF_DOWN);
        float f = locpc.floatValue();
        return f;
    }


    public SaveTarget getSaveTarget() {
        Commandule c = getNextCommandule();
        return c;
    }

    public void setDefaultFocus() {
        getFirstCommandule().getFocus();
    }


    private final ViewPanel viewsPanel;
    private List<Commandule> getAllCommandules(){
        return viewsPanel.getCommandules();
    }
    /**
     *
     */
    public void addTabule(Commandule addTo, Commandule newCommandule, int splitOrientation) {
        ViewPanel th = this.viewsPanel.findOwner(addTo);
        th.split(newCommandule,splitOrientation);
    }
    private Commandule getNextCommandule(Commandule current, boolean directionForward,final List<Commandule> allCommandules){
        ListIterator<Commandule> iterator= directionForward?allCommandules.listIterator() :allCommandules.listIterator(allCommandules.size());
        while (directionForward?iterator.hasNext():iterator.hasPrevious()) {
            Commandule c = directionForward?iterator.next():iterator.previous();
            if(current==c){
                //return the next commandule, wrap to first if necessary
                if(directionForward?iterator.hasNext():iterator.hasPrevious()){
                    return directionForward?iterator.next():iterator.previous();
                }
                return directionForward?allCommandules.get(0):allCommandules.get(allCommandules.size()-1);  //must be at least one
            }
        }
        return null;
    }
     public Commandule getNextCommandule(Commandule current, boolean directionForward){
          return getNextCommandule(current,directionForward,getAllCommandules());
     }
    public Commandule getNextCommandule(){
        List<Commandule> all = getAllCommandules();
        final Commandule currentCommandule = getCurrentCommandule(all);
        if(currentCommandule==null) return null;
        return getNextCommandule(currentCommandule,true,all);
    }
    public Commandule getCurrentCommandule(){
         return getCurrentCommandule(getAllCommandules());
    }
    private Commandule getCurrentCommandule(final List<Commandule> allCommandules){
        for(Commandule c:allCommandules){
            if(cf.hasFocus((Component) c)){
                return c;
            }
        }
        return null;
    }

    public void setupSplitPane(JSplitPane jsp, float resizeWeight){
        GuiUtilities.removeBindings(jsp,KeyStroke.getKeyStroke(KeyEvent.VK_F8,0));
        GuiUtilities.removeBindings(jsp,KeyStroke.getKeyStroke(KeyEvent.VK_F6,0));
        jsp.setOneTouchExpandable(true);
        jsp.setResizeWeight(resizeWeight);
    }

    /**
     * updates the statustext *for this frame* (other frames are not affected)
     * @param statusText
     */
    public void setFrameStatusText(String statusText) {
        statusPanel.setFrameStatusText(statusText);
    }

    /**
     *
     * @param statusText
     * @param frameAction actions that will be turned into buttons to the immediate right of the text
     */
    public void setFrameStatusText(String statusText, Action... frameAction) {
        statusPanel.setFrameStatusText(statusText,frameAction);
    }
}
