package uk.co.hcsoftware.commander.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.logging.Logger;


/**
 * Title: CreateFileDialogBuilder
 * Copyright:    Copyright (c) 24-Apr-2006
 *
 * @author N Spurling
 */
public class CreateFileDialogBuilder{
    public static final Logger log = Logger.getLogger(CreateFileDialogBuilder.class.getName());

    private String message,title;
    private String pathprefix,pathsuffix;
    private String enteredName;
    private boolean okPressed=false;
    private String okText="Create";

    private void setup(final JDialog jd) {
        final CreateFilePanel cfp = new CreateFilePanel();
        jd.setLayout(new BorderLayout());
        jd.getContentPane().add(cfp);
        jd.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        final JTextField filenameField = cfp.getFilenameField();
        final Action enterAction = new AbstractAction(okText) {
            public void actionPerformed(ActionEvent e) {
                enteredName=filenameField.getText();
                okPressed=true;
                 jd.dispose();
            }
        };
        Action canAction =new AbstractAction("Cancel") {
            public void actionPerformed(ActionEvent e) {
                 jd.dispose();
            }
        };

        jd.getRootPane().setDefaultButton(cfp.getCreateButton());
        final InputMap inputMap = cfp.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        final ActionMap actionMap = cfp.getActionMap();
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),"esc");
        actionMap.put("esc",canAction);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),"ok");
        actionMap.put("ok",enterAction);
        cfp.getCreateButton().setAction(enterAction);
        cfp.getCancelButton().setAction(canAction);
        if (message!=null) {
            cfp.getMessageLabel().setText(message);
        }
        if(title!=null){
            jd.setTitle(title);
        }
        else throw new IllegalStateException("must provide a title");
        String val="";
        int carrot_position=0 ;

        if(pathprefix!=null){
            val+=pathprefix;
            carrot_position=pathprefix.length();
        }
        if(pathsuffix!=null) val+=pathsuffix;
        filenameField.setText(val);
        filenameField.setCaretPosition(carrot_position);
//        try {
//            filenameField.scrollRectToVisible(filenameField.modelToView(carrot_position));
//        } catch (BadLocationException e) {
//            log.log(Level.WARNING, "exception", e);
//        }
    }

    public CreateFileDialogBuilder withMessage(String message){
        this.message=message;
        return this;
    }
    public CreateFileDialogBuilder withTitle(String title){
        this.title=title;
        return this;
    }
    public CreateFileDialogBuilder withPathprefix(String pathprefix, boolean addTrailingDirSepIfNotPresent){
        this.pathprefix=pathprefix;
        if(addTrailingDirSepIfNotPresent && !pathprefix.endsWith(File.separator)){
            this.pathprefix+=File.separator;
        }
        return this;
    }
    public CreateFileDialogBuilder withPathsuffix(String pathsuffix){
        this.pathsuffix=pathsuffix;
        return this;
    }
    public CreateFileDialogBuilder withOkText(String text){
        this.okText=text;
        return this;
    }
    public String showDialog(Component c){
        Window window = //JOptionPane.getWindowForComponent(parentComponent);
        SwingUtilities.getWindowAncestor(c);
        JDialog jd;
        if (window instanceof Frame) {
            jd = new JDialog((Frame)window,true);
        } else {
            jd = new JDialog((Dialog)window,true);
        }
        setup(jd);
        jd.pack();
        jd.setLocationRelativeTo(c);
        jd.setVisible(true);
        return okPressed?enteredName:null;
    }
    public File showDialogForFile(Component c, File parentDir){
        withPathprefix(parentDir.getAbsolutePath(),true);
        String s = showDialog(c);
        if(s==null) return null;
        log.fine("s="+s);
        if(s.contains(File.separator)){
            //assume it's an absolute path
            log.fine("create file path="+s);
            return new File(s);
        }
        else{
            log.fine("create file path="+s+" parent="+parentDir);
            return new File(parentDir,s);
        }
    }






}
