package uk.co.hcsoftware.commander.gui;

import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.*;
import java.util.logging.Logger;


/**
 *
 * Place for the common dnd code - implementation must be provided by DndHandler object
 * Title: DndHelper
 * Copyright:    Copyright (c) 10-Feb-2006
 *
 * @author spurlinn
 */
public class DndHelper {
    public static final Logger log = Logger.getLogger(DndHelper.class.getName());

    private final DataFlavor[] receiveFlavors;
    private final Component component;
    private final DndHandler handler;
    private Component[] otherDropTargets;

//    private DndHelper(DataFlavor[] receiveFlavors, Component component,DndHandler dndh){
//          this(receiveFlavors,component,dndh,new Component[0]);
//     }
    private DndHelper(DataFlavor[] receiveFlavors, Component ct,DndHandler dndh) {
        this.receiveFlavors = receiveFlavors;
        this.component = ct;
        this.handler = dndh;
    }
    private void setupDnD() {
        if (receiveFlavors.length>0) {
            final DropTargetListener dropTargetListener = new DropTargetListener(){
                public void dragEnter(DropTargetDragEvent evt) {
                    //ns 20060302 java 1.5 win2k, seem to get no-entry sign even though calling acceptDrop(), regular dragsymbol is preferable.. so commented this lot out
//                    for(DataFlavor df: receiveFlavors){
//                        if(evt.isDataFlavorSupported(df)){
//                            evt.acceptDrag(DnDConstants.ACTION_COPY);
//                            log.finer("dragEnter flavor supported: acceptdrop");
//                            return;
//                        }
//                    }
//                    log.finer("no flavor supported: reject drag");
//                    evt.rejectDrag();
                }
                public void dragOver(DropTargetDragEvent evt) {
//                    for(DataFlavor df: receiveFlavors){
//                        if(evt.isDataFlavorSupported(df)){
//                            evt.acceptDrag(DnDConstants.ACTION_COPY);
//                            log.finer("dragOver flavor supported: acceptdrop");
//                            return;
//                        }
//                    }
//                    log.finer("no flavor supported: reject drag");
//                    evt.rejectDrag();
                }
                public void dragExit(DropTargetEvent evt) {}
                public void dropActionChanged(DropTargetDragEvent evt) {}
                public void drop(DropTargetDropEvent evt) {
                    onDrop(evt);
                }
            };

            component.setDropTarget(new DropTarget(component,dropTargetListener));
            for(Component ct:otherDropTargets){
                ct.setDropTarget(new DropTarget(ct,dropTargetListener));
            }
        }
        final DragSource ds = new DragSource();
        ds.createDefaultDragGestureRecognizer(component,DnDConstants.ACTION_COPY,new DragGestureListener() {
            public void dragGestureRecognized(DragGestureEvent dge) {
               if(sd!=null && !sd.shouldStartDrag(dge)) return;
                onDrag(dge, ds);
            }
        });
    }
    private StartDrag sd=null;
    public interface StartDrag{
        public boolean shouldStartDrag(DragGestureEvent dge) ;
    }

    private void onDrop(DropTargetDropEvent evt) {
        log.finer("src="+evt.getSource());
        for(DataFlavor df: receiveFlavors){
            if(evt.isDataFlavorSupported(df)){
                log.finer("flavor supported, handling drop");
                evt.acceptDrop(evt.getDropAction());
                handler.drop(evt,evt.getTransferable());
                return;
            }
        }
        evt.rejectDrop();
    }
    private void onDrag(DragGestureEvent dge, final DragSource ds) {
        Transferable t=handler.getTransferable(dge);
        if (t!=null) {
            log.finer("starting drag now");
            //NS 20051024: have noticed on Mac OS X 10.2.4/java1.5.0 sometimes drop doesn't succeed e.g. drop onto external window
            //and cursor changes back to normal. After this trags don't get initiated, e.g. call startDrag(..) and nothing happens
            //restarting app cures it..
//            NS 20060406: here is a stacktrace that I think is the same issue:
//  uname -a:     Darwin mini.local 8.5.0 Darwin Kernel Version 8.5.0: Sun Jan 22 10:38:46 PST 2006; root:xnu-792.6.61.obj~1/RELEASE_PPC Power Macintosh powerpc
//
//Exception in thread "AWT-EventQueue-0" java.awt.dnd.InvalidDnDOperationException: Drag and drop in progress
//	at sun.awt.dnd.SunDragSourceContextPeer.setDragDropInProgress(SunDragSourceContextPeer.java:298)
//	at java.awt.dnd.DragSource.startDrag(DragSource.java:286)
//	at java.awt.dnd.DragSource.startDrag(DragSource.java:403)
//	at uk.co.hcsoftware.commander.gui.DndHelper.onDrag(DndHelper.java:105)
//	at uk.co.hcsoftware.commander.gui.DndHelper.access$200(DndHelper.java:18)
//	at uk.co.hcsoftware.commander.gui.DndHelper$2.dragGestureRecognized(DndHelper.java:77)
//	at java.awt.dnd.DragGestureRecognizer.fireDragGestureRecognized(DragGestureRecognizer.java:339)
//	at apple.awt.CMouseDragGestureRecognizer.mouseDragged(CMouseDragGestureRecognizer.java:144)
//	at java.awt.AWTEventMulticaster.mouseDragged(AWTEventMulticaster.java:262)
//	at java.awt.AWTEventMulticaster.mouseDragged(AWTEventMulticaster.java:261)
//	at java.awt.Component.processMouseMotionEvent(Component.java:5602)
//	at javax.swing.JComponent.processMouseMotionEvent(JComponent.java:3144)
//	at java.awt.Component.processEvent(Component.java:5323)
//	at java.awt.Container.processEvent(Container.java:2010)
            if (!dragInProgress) {
                ds.startDrag (dge, DragSource.DefaultCopyDrop, t, new DragSourceListener() {
                    public void dragEnter(DragSourceDragEvent dsde) {log.finest("dragenter");

                    }
                    public void dragOver(DragSourceDragEvent dsde) {log.finest("dragOver");}
                    public void dropActionChanged(DragSourceDragEvent dsde) {log.finest("dropActionChanged"); }
                    public void dragExit(DragSourceEvent dse) {log.finest("dragExit");}
                    public void dragDropEnd(DragSourceDropEvent dsde) {log.finest("dragDropEnd");
                        dragInProgress=false;
                    }
                });
                dragInProgress=true;
            }
            else{
                log.info("ignoring draggesture as dragInProgress=true");
            }
        } else {
            log.finer("nothing to transfer");
        }
    }
    private boolean dragInProgress=false;
    public static class DndHelperBuilder{
        private final DataFlavor[] receiveFlavors;
        private final Component ct;
        private final DndHandler dndh;
        private Component[] otherDropTargets=new Component[0];
        private StartDrag sd;

        public DndHelperBuilder(DataFlavor[] receiveFlavors, Component ct, DndHandler dndh) {
            this.receiveFlavors = receiveFlavors;
            this.ct = ct;
            this.dndh = dndh;
        }
        public DndHelperBuilder otherDropTargets(Component... otherDropTargets){
             this.otherDropTargets=otherDropTargets;
            return this;
        }
        public DndHelperBuilder shouldStartDragCheck(StartDrag sd){
            this.sd=sd;
            return this;
        }

        public DndHelper create(){
            final DndHelper dndHelper = new DndHelper(receiveFlavors, ct, dndh);
            dndHelper.otherDropTargets=this.otherDropTargets;
            dndHelper.sd = this.sd;
            dndHelper.setupDnD();
            return dndHelper;
        }


    }
//    public static DndHelper createDndHelper(DataFlavor[] receiveFlavors, Component component, DndHandler dndh, Component... otherDropTargets) {
//        return new DndHelper(receiveFlavors, component, dndh, otherDropTargets);
//    }
//
//    public static DndHelper createDndHelper(DataFlavor[] receiveFlavors, Component component, DndHandler dndh) {
//        return new DndHelper(receiveFlavors, component, dndh);
//    }

    public interface DndHandler{
         void drop(DropTargetDropEvent evt, Transferable transferable) ;
        /**
         *
         * @return may return null (in which case op is aborted)
         * @param dge
         */
         Transferable getTransferable(DragGestureEvent dge) ;
    }
}
