package uk.co.hcsoftware.commander.gui;

import com.Ostermiller.util.Browser;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: FileLauncher
 * Copyright:    Copyright (c) 29-Nov-2005
 *
 * @author spurlinn
 */
public class FileLauncher {
    public static final Logger log = Logger.getLogger(FileLauncher.class.getName());

    private static boolean browserInited=false;
    public void launch(File... f) throws IOException {
        launchFiles(Arrays.asList(f));
    }
    public void launchFiles(List<File> l) throws IOException {
        List<String> strings = new ArrayList<String>(l.size());
        for(File f:l) strings.add(f.toURL().toString());
        log.fine("display urls="+strings);
        launchUrls(strings);

    }
    public void launchUrls(String... urls) throws IOException {//launch the vfile
        launchUrls(Arrays.asList(urls));
    }
    public void launchUrls(List<String> strings) throws IOException {//launch the vfile
        synchronized (Browser.class) {
            if(!browserInited){
                Browser.init();
                browserInited=true;
            }
        }

        // Browser.displayURLs(strings.toArray(new String[strings.size()]));
        for(String ss:strings) Browser.displayURL(ss);
    }

    public void launchWithPrompt(Component host, List<File> l){
        if(!l.isEmpty()){
            String s = l.size()==1?l.get(0).getName():l.size()+" files";
            String[] options = new String[]{"launch","cancel"};
            if ( 0== JOptionPane.showOptionDialog(host
                    ,"launch "+s+"?","Launch"
                    ,JOptionPane.YES_NO_CANCEL_OPTION
                    ,JOptionPane.QUESTION_MESSAGE,null
                    ,options,options[0])) {
                try {
                    launchFiles(l);
                } catch (IOException e) {
                    log.log(Level.WARNING,"ioe",e);
                    JOptionPane.showMessageDialog(host,"Error executing external process\n" +
                            "["+e.getMessage()+"]","Launch",JOptionPane.ERROR_MESSAGE);
                }
            }
        }


    }

    /**
     *
     * @param f
     * @return true if success
     */
    public boolean tryLaunch(File... f) {
        try {
            launch(f);
            return true;
        } catch (IOException e) {
            log.log(Level.WARNING, "exception", e);
        }
        return false;
    }
    /**
     *
     * @param f
     * @return true if success
     */
    public boolean tryLaunch(List<File> f) {
        try {
            launchFiles(f);
            return true;
        } catch (IOException e) {
            log.log(Level.WARNING, "exception", e);
        }
        return false;
    }

   

}
