package uk.co.hcsoftware.commander.gui;

/**
 * means for controlling how the GUI appears while performing some long task.
 *
 * you MUST call unblockInput() if you call blockInput..() otherwise the GUI might be blocked forever.
 * It is best to put the unblockInput() call in a finally block or outside any exception handling, to make sure
 * it doesn't get skipped.
 *
 * call wasCancelled(), periodically, to determine if the user decided to cancel the action halfway through
 * and take steps to abort the operation.
 *  *
 *
 * Title: InputBlocker
 * Copyright:    Copyright (c) 14-Feb-2006
 *
 * @author N Spurling
 */
public interface InputBlocker {

    boolean wasCancelled();

    /**
     * call from EDT
     *
     * block gui input in cancellable way. If ESC is pressed wasCancelled() will return true (at which point you might want to abort what you're
     * doing and call unblockinput);
     * @param text  initial message adds "(ESC to cancel)" to this
     */
    void blockInputWithCancel(String text);
    /**
     * call from EDT
     *
     * block GUI input (e.g. keys, mouse) by putting a glasspane over the whole frame
     * blocking is not cancellable (there is no way for user to do anything until unblockinput is called)
     * @param text to display on the glasspane
     * @param pauseBeforeBlockMillis
     */
    void blockInput(String text, long pauseBeforeBlockMillis);

    void updateBlockMessage(String status);
    /**
     * call from EDT
     */
    void unblockInput();

}
