/*
* SearchForm.java
*
* Created on 17 February 2006, 11:04
*/

package uk.co.hcsoftware.commander.gui;

import javax.swing.*;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.*;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParseException;

/**
 * @author  spurlinn
 */
public abstract class SearchForm extends JDialog {
    private static final Logger log = Logger.getLogger(SearchForm.class.getName());

    private DateFormat dateFormat=new SimpleDateFormat(DATEFORMAT_STRING);
//    private List<File> searchRoots;
    private Action cancelAction=new AbstractAction("cancel") {
        public void actionPerformed(ActionEvent e) {
            dispose();
        }
    };
    private boolean beenSeenBefore=false;
    private static final String DATE_FMT="dd-MMM-yyyy",TIME_FMT="HH:mm",DATEFORMAT_STRING = DATE_FMT+" "+TIME_FMT;//
    private SearchForm.DateAction modBeforeAction;
    private SearchForm.DateAction modAfterAction;
    // "dd-MMM-yyyy HH:mm";

    public SearchForm(Frame owner, Component parent) {
        super(owner,"Find",false);
        setLocationRelativeTo(parent);
        initComponents();

        find.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    final SearchParameters sp = getParameters();
                    if (!sp.searchRoots.isEmpty()) {
                        dispose();
                        onFindPressed(sp);
                    }
                    else{
                        JOptionPane.showMessageDialog(SearchForm.this,"No valid startdirectories entered!\n Please" +
                                " type the names directory(s) in which to start the search.\n " +
                                "If more than one separate with "+File.separator
                        ,"Search",JOptionPane.ERROR_MESSAGE);
                    }
                } catch (ParseException e1) {
                    JOptionPane.showMessageDialog(SearchForm.this,"Invalid dates entered\n Please enter dates in dd-MMM-yyyy format, and times in HH:mm (24hr)\n" +
                            " e.g. 12-Aug-2006 19:30"
                        ,"Search",JOptionPane.ERROR_MESSAGE);
                }
            }
        });
        cancel.setAction(cancelAction);
        getRootPane().getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),"cancel");

        getRootPane().getActionMap().put("cancel",cancelAction);

        //populate date/time fields
        Date now = new Date();
        SimpleDateFormat dte = new SimpleDateFormat(DATE_FMT),tm=new SimpleDateFormat(TIME_FMT);

        modBeforeDateField.setText(dte.format(now));
        modBeforeTimeField.setText(tm.format(now));

        modAfterDateField.setText(dte.format(now));
        modAfterTimeField.setText("");

        //add actions to enable/disable textfields
        modBeforeAction = new DateAction(modBeforeCb, modBeforeDateField, modBeforeTimeField);
        modBeforeCb.addActionListener(modBeforeAction);
        modAfterAction = new DateAction(modAfterCb, modAfterDateField, modAfterTimeField);
        modAfterCb.addActionListener(modAfterAction);
    }
    private class DateAction extends AbstractAction{
        private List<JTextComponent> textFieldsToEnable;
        private final JCheckBox src;

        public DateAction(JCheckBox src, JTextComponent... textFieldsToEnable) {
            this.src = src;
            this.textFieldsToEnable = Arrays.asList(textFieldsToEnable);
        }

        public void actionPerformed(ActionEvent e) {
            for(JTextComponent jtc:textFieldsToEnable) jtc.setEditable(src.isSelected());
        }
    }



    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        patternField = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        caseSensitive = new javax.swing.JCheckBox();
        find = new javax.swing.JButton();
        cancel = new javax.swing.JButton();
        baseDir = new javax.swing.JLabel();
        wildcard = new javax.swing.JCheckBox();
        searchRootsField = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        containsField = new javax.swing.JTextField();
        containsCaseJcb = new javax.swing.JCheckBox();
        contentWildcard = new javax.swing.JCheckBox();
        modBeforeCb = new javax.swing.JCheckBox();
        jLabel3 = new javax.swing.JLabel();
        modAfterCb = new javax.swing.JCheckBox();
        modBeforeDateField = new javax.swing.JTextField();
        modAfterDateField = new javax.swing.JTextField();
        modBeforeTimeField = new javax.swing.JTextField();
        modAfterTimeField = new javax.swing.JTextField();

        setTitle("Find");
        patternField.setText("*");

        jLabel1.setText("Filename pattern:");

        caseSensitive.setText("case sensitive");
        caseSensitive.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        caseSensitive.setMargin(new java.awt.Insets(0, 0, 0, 0));

        find.setText("Find");

        cancel.setText("Cancel");

        baseDir.setText("Search from:");

        wildcard.setSelected(true);
        wildcard.setText("use * as wildcard");
        wildcard.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        wildcard.setMargin(new java.awt.Insets(0, 0, 0, 0));

        jLabel2.setText("File contains text:");

        containsCaseJcb.setText("case sensitive");
        containsCaseJcb.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        containsCaseJcb.setMargin(new java.awt.Insets(0, 0, 0, 0));

        contentWildcard.setText("use * as wildcard");
        contentWildcard.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        contentWildcard.setMargin(new java.awt.Insets(0, 0, 0, 0));

        modBeforeCb.setText("before:");
        modBeforeCb.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        modBeforeCb.setMargin(new java.awt.Insets(0, 0, 0, 0));

        jLabel3.setLabelFor(modBeforeCb);
        jLabel3.setText("File modified:");

        modAfterCb.setText("after:");
        modAfterCb.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        modAfterCb.setMargin(new java.awt.Insets(0, 0, 0, 0));

        modBeforeDateField.setEditable(false);
        modBeforeDateField.setText("12-Aug-2006");

        modAfterDateField.setEditable(false);
        modAfterDateField.setText("12-Aug-2006");

        modBeforeTimeField.setEditable(false);
        modBeforeTimeField.setText("12:25");
        modBeforeTimeField.setMinimumSize(new java.awt.Dimension(43, 20));

        modAfterTimeField.setEditable(false);
        modAfterTimeField.setText("12:25");
        modAfterTimeField.setMinimumSize(new java.awt.Dimension(43, 20));

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 157, Short.MAX_VALUE)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                                        .add(wildcard)
                                        .add(14, 14, 14)
                                        .add(caseSensitive))
                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                                        .add(cancel)
                                        .add(19, 19, 19)
                                        .add(find, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                            .add(layout.createSequentialGroup()
                                .add(121, 121, 121)
                                .add(contentWildcard)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 50, Short.MAX_VALUE)
                                .add(containsCaseJcb))
                            .add(jLabel2)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(baseDir)
                                    .add(jLabel1))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 5, Short.MAX_VALUE)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, containsField)
                                    .add(patternField)
                                    .add(searchRootsField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 265, Short.MAX_VALUE))))
                        .addContainerGap())
                    .add(layout.createSequentialGroup()
                        .add(jLabel3)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 27, Short.MAX_VALUE)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(modBeforeCb)
                            .add(modAfterCb))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(modBeforeDateField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 107, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(modAfterDateField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 107, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(modBeforeTimeField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(modAfterTimeField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 53, Short.MAX_VALUE))
                        .add(50, 50, 50))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(baseDir)
                    .add(searchRootsField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 26, Short.MAX_VALUE)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(patternField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(caseSensitive)
                    .add(wildcard))
                .add(15, 15, 15)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel3)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(modBeforeCb)
                            .add(modBeforeDateField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(modBeforeTimeField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(modAfterCb)
                            .add(modAfterDateField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(modAfterTimeField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                .add(14, 14, 14)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(containsField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(containsCaseJcb)
                    .add(contentWildcard))
                .add(24, 24, 24)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(find)
                    .add(cancel))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents



    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel baseDir;
    private javax.swing.JButton cancel;
    private javax.swing.JCheckBox caseSensitive;
    private javax.swing.JCheckBox containsCaseJcb;
    private javax.swing.JTextField containsField;
    private javax.swing.JCheckBox contentWildcard;
    private javax.swing.JButton find;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JCheckBox modAfterCb;
    private javax.swing.JTextField modAfterDateField;
    private javax.swing.JTextField modAfterTimeField;
    private javax.swing.JCheckBox modBeforeCb;
    private javax.swing.JTextField modBeforeDateField;
    private javax.swing.JTextField modBeforeTimeField;
    private javax.swing.JTextField patternField;
    private javax.swing.JTextField searchRootsField;
    private javax.swing.JCheckBox wildcard;
    // End of variables declaration//GEN-END:variables

    private SearchParameters getParameters() throws ParseException {
        final SearchParameters sp = new SearchParameters(readSearchRoots());
        sp.searchPattern= patternField.getText();
        sp.wildcardSelected= wildcard.isSelected();
        sp.caseSensitiveSelected=caseSensitive.isSelected();

        sp.containsCaseSensitive=containsCaseJcb.isSelected();
        sp.containsWildcardSelected=contentWildcard.isSelected();
        sp.containsPattern=containsField.getText();

        if(modBeforeCb.isSelected()){
            sp.before=readDate(true,modBeforeDateField.getText(), modBeforeTimeField.getText()).getTime();
        }
        if(modAfterCb.isSelected()){
            sp.after=readDate(false, modAfterDateField.getText(), modAfterTimeField.getText()).getTime();
        }
        log.fine("contains="+containsField.getText());
        return sp;
    }

    private Date readDate(boolean beforeDate, String dateValue, String timeValue) throws ParseException {
        if(nullOrEmpty(timeValue)){
            if(beforeDate){
                //time is 00:00, so is before-exclusive
                timeValue="00:00";
            }
            else{//time is 23:59 so is after-exclusive
                timeValue="23:59";
            }
        }
        return dateFormat.parse(dateValue+" "+timeValue);
    }

    private boolean nullOrEmpty(String timeValue) {
        return timeValue==null || timeValue.trim().length()==0;
    }




    public void setParameters(SearchParameters sp) {
        patternField.setText(sp.searchPattern);
        wildcard.setSelected(sp.wildcardSelected);
        caseSensitive.setSelected(sp.caseSensitiveSelected);
        searchRootsField.setText(getSearchText(sp.searchRoots));

        contentWildcard.setSelected(sp.containsWildcardSelected);
        containsCaseJcb.setSelected(sp.containsCaseSensitive);
        containsField.setText(sp.containsPattern);
        // set dateparams
        SimpleDateFormat dte = new SimpleDateFormat(DATE_FMT),tm=new SimpleDateFormat(TIME_FMT);
        if(sp.hasBefore()){
            modBeforeDateField.setText(dte.format(sp.before));
            modBeforeTimeField.setText(tm.format(sp.before));
            modBeforeCb.setSelected(true);
        }
        if(sp.hasAfter()){
            modAfterDateField.setText(dte.format(sp.after));
            modAfterTimeField.setText(tm.format(sp.after));
            modAfterCb.setSelected(true);
        }
        //enable/disable entryfields based on checkbox state
        modAfterAction.actionPerformed(null);
        modBeforeAction.actionPerformed(null);
    }



    /**
     * struct holding all the search params, saves lots of refactoring when add/remove parameters.
     * Typically if add a variable have to:
     * 1. add instance var (with default variable or value supplied in constructor)
     * 2. adjust setParameters to populate the control on this form
     * 3. edit getParameters to read the new control value
     * 4. visit the places where SearchParameters are created and supply a value if required
     * 5. visit places where find is implemented and take account of the value
     */
    public static class SearchParameters implements Cloneable{
        public static final String WILDCARD="*";
        public List<File> searchRoots;
        public String searchPattern=WILDCARD;
        public boolean wildcardSelected=true, caseSensitiveSelected=false;

        public String containsPattern="";
        public boolean containsCaseSensitive=false,containsWildcardSelected=false;
        public long before=-1,after=-1;//millis since epoch: if <0 means any

        public SearchParameters(List<File> searchRoots) {
            this.searchRoots = searchRoots;
        }

        public String toString() {
            return "searchPattern="+searchPattern+" wc="+wildcardSelected+" cs="+caseSensitiveSelected+
                    " containsPattern="+containsPattern+" containsCs="+containsCaseSensitive+" containsWild="+containsWildcardSelected;
        }

        public Object clone() {
            try {
                SearchParameters sp= (SearchParameters) super.clone();
                if(searchRoots!=null){
                    sp.searchRoots=new ArrayList<File>(searchRoots);
                }
                return sp;
            } catch (CloneNotSupportedException e) {
                log.log(Level.SEVERE, "exception", e);
                return null;
            }
        }

        public boolean matchesAllNames() {
            return wildcardSelected && searchPattern!=null && WILDCARD.equals(searchPattern.trim());
        }
        public boolean matchesAllContents() {
            return containsPattern==null || containsPattern.trim().length()==0
                    || (containsWildcardSelected && WILDCARD.equals(containsPattern.trim()));
        }
        private Pattern createPattern(String input, boolean caseSensitiveSelected, boolean wildcardSelected){
            final String regex;
            if (wildcardSelected) {
                StringBuilder sb = new StringBuilder();
                //split at the wildcards
                //replace each wildcard with its regex equivalent (.*) and escape the chars in between
                char[] c = input.toCharArray();
                char WC=WILDCARD.charAt(0);
                int from=0;
                for (int i = 0; i < c.length; i++) {
                    char c1 = c[i];
                    if(c1==WC){
                        final String s = input.substring(from,i);
                        if(s.length()>0) sb.append(Pattern.quote(s));
                        from=i+1;//+1 to skip the wildcard
                        sb.append(".*");//the wildcard
                    }
                }
                if(!input.endsWith(WILDCARD)){//add the suffix chars
                    final int wcIndex = input.lastIndexOf(WILDCARD);
                    final String s;
                    if(wcIndex==-1) s=input;
                    else  s = input.substring(wcIndex+1);
                    if(s.length()>0) sb.append(Pattern.quote(s));
                }
                regex=sb.toString();
            } else {
                regex=Pattern.quote(input);
            }
            int flags=0;
            if(!caseSensitiveSelected) flags |= Pattern.CASE_INSENSITIVE;
            log.fine("converted input="+input+" to regex="+regex);
            return Pattern.compile(regex,flags);
        }
        public Pattern createSearchPattern(){
            return createPattern(searchPattern,caseSensitiveSelected,wildcardSelected);
        }
         public Pattern createContainsPattern(){
            return createPattern(containsPattern,containsCaseSensitive,containsWildcardSelected);
        }

        public boolean hasAfter() {
            return after>=0;
        }

        public boolean hasBefore() {
            return before>=0;
        }
    }
    public abstract void onFindPressed(SearchParameters sp) ;
    private List<File> readSearchRoots() {
        String paths = searchRootsField.getText();
        String[] fpaths=paths.split(File.pathSeparator);
        List<File> l=new ArrayList<File>(fpaths.length);
        for(String fp:fpaths){
            File f = new File(fp.trim());
            if(f.exists()){
                l.add(f);
            }
        }
        return l;
    }
    private String getSearchText(List<File> searchRoots) {
        StringBuilder sb=new StringBuilder();
        for (Iterator<File> iterator = searchRoots.iterator(); iterator.hasNext();) {
            File file = iterator.next();
            sb.append(file.getAbsolutePath());
            if(iterator.hasNext()) sb.append(File.pathSeparator);
        }
        return sb.toString();
    }

    public void setVisible(boolean b) {
        super.setVisible(b);
        if(b){
            getRootPane().setDefaultButton(find);
            if(!beenSeenBefore){
                patternField.selectAll();
                patternField.requestFocus();
                beenSeenBefore=true;
            }
        }
    }
}
