package uk.co.hcsoftware.commander.gui;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.util.List;
import java.util.logging.Logger;

public class ShortcutsDialog extends JDialog{
    private static final Logger log = Logger.getLogger(ShortcutsDialog.class.getName());

    public ShortcutsDialog(Frame owner,List<ShortcutDescriptor> l) throws HeadlessException {
        super(owner,"Keyboard shortcuts",false);
        super.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        JPanel mainPan = new JPanel(new BorderLayout());
        getContentPane().add(mainPan);
        mainPan.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));

        JEditorPane jep = new JEditorPane("text/html",getText(l));
        jep.setEditable(false);

        mainPan.add(new JScrollPane(jep));
//        setSize(450,280);
//        setLocationRelativeTo(owner);
    }

    private String getText(List<ShortcutDescriptor> l) {
        StringBuffer sb = new StringBuffer();
        sb.append("<html>");

        sb.append("<table>");
//        sb.append("<h1>Shortcut Keys</h1>");
        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_ESCAPE))+getDescHtml("Clear selection and marked files")+"</tr>");
        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_TAB))+getDescHtml("Cycle forwards through views")+"</tr>");
        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyModifiersText(KeyEvent.SHIFT_MASK)+"+"+KeyEvent.getKeyText(KeyEvent.VK_TAB))+getDescHtml("Cycle backwards through views")+"</tr>");
        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_SPACE))+getDescHtml("Mark/Unmark file")+"</tr>");
        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_ENTER))+getDescHtml("Open selected directory")+"</tr>");
        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyModifiersText(KeyEvent.CTRL_MASK)+"+"+KeyEvent.getKeyText(KeyEvent.VK_ENTER))
                +getDescHtml("Open selected directory in a new (split) view")+"</tr>");

//        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_UP))+getDescHtml("Select file above")+"</tr>");
//        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_DOWN))+getDescHtml("Select file below")+"</tr>");
        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_HOME))+getDescHtml("Select first file in list")+"</tr>");
               sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_END))+getDescHtml("Select last file in list")+"</tr>");
               sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_LEFT))+getDescHtml("Open parent directory")+"</tr>");
        sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_RIGHT))+getDescHtml("Open selected directory")+"</tr>");
        sb.append("<tr>");
        for(ShortcutDescriptor sd:l){
            sb.append("<tr>"+getCmdHtml(sd.getAcceleratorText())+getDescHtml(sd.getDescription())+"</tr>");
        }
        //sb.append("<tr>"+getCmdHtml(KeyEvent.getKeyText(KeyEvent.VK_C))+getDescHtml("copy selected filename to clipboard")+"</tr>");

        sb.append("</table>");
        sb.append("</html>");
        return sb.toString();
    }
    private String getCmdHtml(String cmd){
        return "<td><b>"+cmd+"</b></td>";
    }
    private String getDescHtml(String desc){
        return "<td>"+desc+"</td>";
    }
    public static class ShortcutDescriptor{
        private KeyStroke accelerator;
        private String description;

        public ShortcutDescriptor(Action a) {
            this((KeyStroke) a.getValue(Action.ACCELERATOR_KEY),(String)a.getValue(Action.SHORT_DESCRIPTION));
        }

        public ShortcutDescriptor(KeyStroke accelerator, String description) {
            this.accelerator = accelerator;
            this.description = description;
        }
        public String getAcceleratorText(){
            return CommanderFrame.getAcceleratorText(accelerator);
        }

        public String getDescription() {
            return description;
        }
    }
}
