package uk.co.hcsoftware.commander.gui;

import uk.co.hcsoftware.commander.gui.ctable.CommanderSelectionUtilities;
import uk.co.hcsoftware.commander.gui.ctable.CommanderTable;
import uk.co.hcsoftware.commander.gui.ctable.MarkableTableModel;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import java.awt.*;
import java.util.logging.Logger;


/**
 * convenience class updates commanderFrame status
 *
 * // need to refreshStatus(); whenever:
        //dir changes
        //selection changes
        //markstatus chagnes
 *
 * Title: StatusUpdater
 * Copyright:    Copyright (c) 08-Feb-2006
 *
 * @author spurlinn
 */
public class StatusUpdater {
    private static final Logger log = Logger.getLogger(StatusUpdater.class.getName());

    private final CommanderTable table;

    private String statusText;
    private final CommanderFrame commanderFrame;
    private final CommanderFocus cf;
    private final Component focusComponent;


    public StatusUpdater(CommanderTable ct, CommanderFocus cf, final Component focusComponent, CommanderFrame commanderFrame) {
        this.table = ct;
        this.commanderFrame=commanderFrame;
        this.cf=cf;
        this.focusComponent=focusComponent;
        refreshStatus();


        table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                Runnable runnable = new Runnable() {
                    public void run() {
                        refreshStatus();
                    }
                };
                SwingUtilities.invokeLater(runnable);
            }
        });
        //listen for mark status changes (bit of a sledgehammer..)
        // problem was that listening to the model, sometimes we receive the changeevent before the table has got it.
        //we then ask the table for the rowcount in order to get the selection (don't ask the model as there may be filters between the model and the view)
        //however, the table is still out of date, so tells us the wrong rowcount, which causes an aioob when we try to fetch that row from the model..
        //SO: now we listen to the table (not the tablemodel) which only forwards events after it has handled them
        table.addTableModelListener(new TableModelListener() {
            public void tableChanged(TableModelEvent e) {
                 refreshStatus();
            }
        });

        //update status panel
        cf.addListener(new CommanderFocus.CommanderFocusListener() {
            public void focusChanged(Component lostFocus, Component gainedFocus) {
                if(gainedFocus==focusComponent && !disableFocusStatusUpdate){
                    log.finest("gained focus, setstatus");
                    StatusUpdater.this.commanderFrame.setFrameStatusText(statusText);
                }
            }
        });
    }
    boolean disableFocusStatusUpdate;

    public void setDisableFocusStatusUpdate(boolean disableFocusStatusUpdate) {
        this.disableFocusStatusUpdate = disableFocusStatusUpdate;
    }

    private void refreshStatus(){
        final int tableRowCount = table.getRowCount();
        refreshStatus(tableRowCount);
    }

    private void refreshStatus(final int tableRowCount) {
        log.finer("refreshing status");
        //clear disablefocus flag
        disableFocusStatusUpdate=false;
        StringBuilder sb = new StringBuilder();

        sb.append(tableRowCount==1?"1 file":tableRowCount+" files");

        final int len = getSelectedRowcount();
        if(len>0) sb.append(", "+len+" selected/marked");
        // add sort text?
        final String s = sb.toString();
        log.finer(" status="+s);
        if(!s.equals(statusText) && cf.hasFocus(focusComponent)){
            statusText= s;
            commanderFrame.setFrameStatusText(statusText);
        }
    }

    private int getSelectedRowcount() {
        TableModel tm = table.getModel();
        final int len;
        if(tm instanceof MarkableTableModel){
            MarkableTableModel mtm = (MarkableTableModel) tm;
            len = CommanderSelectionUtilities.getSelectedRowCount(table,mtm,true,true);//table.getSelectedRowCount(true,true);
        }
        else{
            len = table.getSelectedRowCount();
        }
        return len;
    }


}
