package uk.co.hcsoftware.commander.gui.ctable;

import uk.co.hcsoftware.commander.gui.util.TableLayout;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import java.util.List;


public class ColumnSortDialog extends JDialog{
    private boolean cancelled;

    public ColumnSortDialog(Frame f, java.util.List<String> columnNames) throws HeadlessException {
        super(f,"Configure Sorting",true);
        super.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
        JPanel mainPan = new JPanel(new BorderLayout());
        getContentPane().add(mainPan);
        JPanel buttonP=new JPanel(new FlowLayout(FlowLayout.CENTER));
        final JButton ok = new JButton("OK");
        final JButton cancel = new JButton("Cancel");
        final JButton clear = new JButton("Clear Orders");
        ActionListener al = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object src = e.getSource();
                if(src==ok) dispose();
                else if(src==cancel){
                    cancelled=true;
                    dispose();
                }
                else if(src==clear){
                    clearOrders();
                }
            }
        };
        ok.addActionListener(al);
        cancel.addActionListener(al);
        clear.addActionListener(al);
        buttonP.add(ok);
        buttonP.add(cancel);
        buttonP.add(clear);



        mainPan.add(buttonP,BorderLayout.SOUTH);

        JPanel controlP = new JPanel();
        JPanel interPan = new JPanel(new BorderLayout());
        interPan.add(controlP);
        mainPan.add(interPan);
        double[] rows = new double[columnNames.size()+1];//+1 for the title-row
        for (int i = 0; i < rows.length; i++) {
            rows[i]=TableLayout.PREFERRED;
        }
        TableLayout tl = new TableLayout(
                new double[]{TableLayout.PREFERRED,80d,TableLayout.PREFERRED,TableLayout.PREFERRED} //columnNames
               , rows //rows
        );
        tl.setHGap(5);
        tl.setVGap(8);
        controlP.setLayout(tl);
        controlP.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        //headings
        controlP.add(new JLabel("Column Name"),"0,0,l,c");
        controlP.add(new JLabel("Order"),"1,0,c,c");
        controlP.add(new JLabel("Direction"),"2,0,3,0,c,c");

        //rows
        int ro=1;
        for (String name:columnNames) {
            Control c=new Control(ro-1,name, true);
            allControls.add(c);
            //x,y == col,row
            controlP.add(new JLabel(c.getName()),"0,"+ro);
            controlP.add(c.getButton(),"1,"+ro);
            controlP.add(c.getAsc(),"2,"+ro);
            controlP.add(c.getDesc(),"3,"+ro);
            ro++   ;
        }

        pack();
    }
    public void setDefaultSortDirections(List<SortDirection> directions){
        int i=0;
        for(SortDirection sd:directions){
            if(i<allControls.size()){
                Control c = allControls.get(i);
                c.setAscending(sd==SortDirection.ascending);
            }
            i++;
        }
    }

    public void clearOrders() {
        selectedControls.clear();
        renumberControls();
    }

    public void setTerms(List<SortTerm> defaultSort) {
        clearOrders();
        for (SortTerm st:defaultSort) {
            for (Control control:allControls) {
                if(control.index==st.getColumnIndex()){
                    control.select(true);
                    control.setAscending(st.isAscending());
                }
            }
        }
    }

    public static class SortTerm{
        private int columnIndex;
        private SortDirection ascending;

        public SortTerm(SortDirection ascending, int columnIndex) {
            this.ascending = ascending;
            this.columnIndex = columnIndex;
        }

        public boolean isAscending() {
            return ascending==SortDirection.ascending;
        }

        public int getColumnIndex() {
            return columnIndex;
        }

    }

    /**
     *
     * @return List<SortTerm>
     */
    public List<SortTerm> showDialog(){
       // this.ss=ss;
        cancelled=false;
        setVisible(true);
        if(cancelled) return null;
        return getSortTerms();
    }
    private LinkedList<Control> selectedControls = new LinkedList<Control>();
    private List<Control> allControls = new ArrayList<Control>();
    public void renumberControls(){
        for (Iterator iterator = allControls.iterator(); iterator.hasNext();) {
            Control control = (Control) iterator.next();
            control.setText(" ");
        }
        int i=1;
        for (Iterator iterator = selectedControls.iterator(); iterator.hasNext();i++) {
            Control control = (Control) iterator.next();
            control.setText(""+i);
        }
    }
    private class Control{
        private JButton button=new JButton(" ");
        private JRadioButton asc=new JRadioButton("asc",true);
        private JRadioButton desc = new JRadioButton("desc",false);
        public void setText(String text){
            button.setText(text);
        }
        public Control(int index, String name, boolean ascSelected) {
            this.index = index;
            this.name = name;
            ButtonGroup bg = new ButtonGroup();
            bg.add(asc);
            bg.add(desc);
            asc.setSelected(ascSelected);
            button.setFocusable(false);
            button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    select(!isSelected());
                }
            });
        }
        public boolean isSelected(){
            return selectedControls.contains(Control.this);
        }
        public void select(boolean select){
            if(select ){
                if( !isSelected()){
                    selectedControls.add(Control.this);
                    renumberControls();
                }
            }
            else{
                if(isSelected()){
                    selectedControls.remove(Control.this);
                    renumberControls();
                }
            }
        }

        private int index;
        private String name;

        public String getName() {
            return name;
        }

        public SortTerm getSortSpec(){
            return new SortTerm(asc.isSelected()?SortDirection.ascending:SortDirection.descending,index);
        }
        public JRadioButton getAsc() {
            return asc;
        }

        public JButton getButton() {
            return button;
        }

        public JRadioButton getDesc() {
            return desc;
        }

        public void setAscending(boolean ascending) {
             asc.setSelected(ascending);
            desc.setSelected(!ascending);
        }
    }
    /**
     *
     * @return List <SortTerm>
     */
    private java.util.List<SortTerm> getSortTerms() {
        java.util.List<SortTerm> l = new ArrayList<SortTerm>();
        for (Iterator iterator = selectedControls.iterator(); iterator.hasNext();) {
            Control c = (Control) iterator.next();
            l.add(c.getSortSpec());
        }
        return l;
    }

    public static void main(String[] args) {
        List preds = Arrays.asList("col 1","col2","col3","col4");
        JFrame jf = new JFrame();
        jf.setVisible(true);
        final ColumnSortDialog cs = new ColumnSortDialog(jf
                 , preds
                );
        cs.setVisible(true);
    }

}
