package uk.co.hcsoftware.commander.gui.ctable;

import javax.swing.table.AbstractTableModel;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;


/**
 * extends tablemodel with support for holding a boolean 'marked' value for each row (wraps underlying elements in a Node that holds the marked state)
 *
 *
 *
 * Title: MarkableTableModel
 * Copyright:    Copyright (c) 20-Mar-2006
 *
 * @author N Spurling
 */
public abstract class MarkableTableModel<E> extends AbstractTableModel {
    public static final Logger log = Logger.getLogger(MarkableTableModel.class.getName());
    protected E dir;
    protected List<Node> children=Collections.emptyList();


    public void setDir(E dir, List<E> children){
        this.children=new ArrayList<Node>(children.size());
        for(E zn:children) this.children.add(new Node(zn));
        this.dir=dir;
        log.finer("set children len="+this.children.size());
        fireTableDataChanged();
        
    }

    public boolean isMarked(int modelRow) {
        return children.get(modelRow).isMarked();
    }

    public void toggleMarked(int... modelRows) {
        int lo=-1, hi=-1;
        for(int ro:modelRows){
            final Node node = children.get(ro);
            node.setMarked(!node.isMarked());
            if(lo==-1 || ro<lo) lo=ro;
            if(hi==-1 || ro>hi) hi=ro;
        }
        if (lo!=-1 && hi!=-1) {
            log.fine("fireTableRowsUpdated lo="+lo+" hi="+hi);
            fireTableRowsUpdated(lo,hi);
        }
    }

    public void clearMarked() {
        int lo=-1, hi=-1;
        int ro=0;
        for(Node node:children){
            if (node.isMarked()) {
                node.setMarked(false);
                if(lo==-1 || ro<lo) lo=ro;
                if(hi==-1 || ro>hi) hi=ro;
            }
            ro++;
        }
        if (lo!=-1 && hi!=-1) {
            log.fine("fireTableRowsUpdated lo="+lo+" hi="+hi);
            fireTableRowsUpdated(lo,hi);
        }
    }

    public int getRowCount() {
        return children.size();
    }

    public E getElement(int modelIndex) {
        return children.get(modelIndex).getElement();
    }

    public int indexOf(E f) {
        int i=0;
        for(Node n:children){
            if(f.equals(n.getElement())) return i;
            i++;
        }
        return -1;
    }

    public E getDirectory() {
        return dir;
    }

    protected final class Node{
        private E element;
        private boolean marked;

        public Node(E vf) {
            this.element = vf;
        }

        public E getElement() {
            return element;
        }

        public void setElement(E vf) {
            this.element = vf;
        }

        public boolean isMarked() {
            return marked;
        }

        public void setMarked(boolean marked) {
            this.marked = marked;
        }
    }
}
