package uk.co.hcsoftware.commander.gui.ctable;

import uk.co.hcsoftware.commander.io.FileRecognizer;

import javax.swing.*;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.util.logging.Logger;


/**
 * Title: RowHeaderTable
 * Copyright:    Copyright (c) 06-Apr-2006
 *
 * good advice at
 * //        When the JViewport's viewPosition attribute changes in the main viewport (the one with the main table), the y coordinate of the row header's viewport stays in sync. For some reason, this does not work the other way (the Swing folks claim that it's by design - huh?). Becuase of this, it is possible that you could set focus to your row header table and then change the view position using Page Up or Down. To fix this, add a ChangeListener to the row header viewport which updates the main viewport's y position (NOT x position).
//        http://www.chka.de/swing/table/row-headers/JTable.html
 *
 * @author N Spurling
 */
public class RowHeaderTable extends JTable{
    public static final Logger log = Logger.getLogger(RowHeaderTable.class.getName());

    private CommanderTable ct;
    private RowNamer rowNamer;

    public RowHeaderTable(CommanderTable ct, RowNamer rowNamer) {
        this.ct = ct;
        this.rowNamer = rowNamer;

        setRowSelectionAllowed(false);
        setFocusable(false);
        getTableHeader().setFocusable(false);
        setShowGrid(false);
        setRowHeight(ct.getRowHeight());
        setPreferredScrollableViewportSize(new Dimension(20,ct.getPreferredScrollableViewportSize().height));

        setModel(new Model());
    }
    private final class Model extends AbstractTableModel implements TableModelListener {
        FileRecognizer fr = FileRecognizer.getInstance();

        public Model() {
            ct.getModel().addTableModelListener(this);
        }

        public int getRowCount() { return ct.getRowCount(); }
        public int getColumnCount() {  return 1;  }
        public String getColumnName(int columnIndex) {  return ""; }
        public Class<?> getColumnClass(int columnIndex) {  return Icon.class; }
        public Object getValueAt(int viewRow, int viewCol) {
            final String rowName = rowNamer.getRowName(viewRow);
            if (rowName!=null) {
                return fr.getIcon(rowName, rowNamer.isDirectory(viewRow));
            }
            return null;
        }

        public void tableChanged(TableModelEvent e) {
              fireTableChanged(e);
        }
    }
}
