package uk.co.hcsoftware.commander.gui.file;

import uk.co.hcsoftware.commander.io.FileUtils;
import uk.co.hcsoftware.commander.gui.InputBlocker;

import java.io.*;
import java.util.List;
import java.util.Arrays;
import java.util.logging.Logger;

/**
 * $Log: ArchiveCreator.java,v $
 * Revision 1.1  2006/12/10 12:01:35  ns
 * update to 3-nov-2006 version
 *
 */
public class ArchiveCreator {
    private static final Logger log = Logger.getLogger(ArchiveCreator.class.getName());
    private File structureRoot;
    private final InputBlocker ib;
    private int addedFiles;
    private String zipFileName;

    public ArchiveCreator(InputBlocker ib, File structureRoot, String zipFileName) {
        this.ib = ib;
        this.structureRoot = structureRoot;
        this.zipFileName = zipFileName;
    }

    public static interface ArkOutputStream{

        void closeEntry() throws IOException;
        void writeDirectory(String dirPath) throws IOException;
        void writeFile(String dirPath, InputStream is) throws  IOException;
        void putNextEntry(String zipPathString) throws IOException;

        void write(byte[] buf, int i, int i1) throws IOException;
    }

    public void writeToZip(List<File> l, ArkOutputStream zos) throws IOException {

        // if preserve struc, path should be path relative to structureRoot (ending with /)
        log.finest("write tozip files="+l);
        String[] pathToRoot =null;
        FileUtils fu = FileUtils.getInstance();
        if(structureRoot!=null){
            pathToRoot =fu.splitPath(structureRoot.getCanonicalPath());
        }
        for(File vf:l){
            if(ib.wasCancelled()){
                log.finest("returning as ib cancelled");
                return;
            }
            String[] zipPath=new String[0];
            String canPath = vf.getParentFile().getCanonicalPath();
            if (pathToRoot !=null) {  //ensure this file's zip-path is relative to this ancestor file
                String[] pathToFile =fu.splitPath(canPath);
                zipPath=new String[pathToFile.length-pathToRoot.length];
                System.arraycopy(pathToFile,pathToRoot.length,zipPath,0,zipPath.length);
                log.fine("converted inputpath="+canPath+" to="+ Arrays.asList(zipPath)+" base="+pathToRoot);
            }
            log.finest("add file="+vf);
            addFile(vf,zipPath,zos);
        }
    }

    // Here is a sample printout of ZipEntry.getName() from a well known zipfile (top-level dir is commanderbond-0.1)
//    commanderbond-0.1/src/uk/co/hcsoftware/commander/treeplugin/file/
//    commanderbond-0.1/src/uk/co/hcsoftware/commander/treeplugin/zip/
//    commanderbond-0.1/src/uk/co/hcsoftware/commander/vfs/
//
    /**
     *
     * @param f
     * @param zipPath path to this element in the zip
     * @param zos
     * @throws IOException
     */
    private void addFile(File f,String[] zipPath, ArkOutputStream zos) throws IOException {
        if(ib.wasCancelled()) return;
        log.finest("add File="+f+" path="+Arrays.asList(zipPath));
        if(f.isDirectory()){
            zipPath=path(zipPath,f.getName()); //the file is the dir, so add it to the path
            String zipPathString = toZipDirPath(zipPath);
            log.finest("add entry="+zipPathString);
            zos.putNextEntry(zipPathString);
            zos.closeEntry();
            File[] children = f.listFiles();
            if(children.length>0){
                for(File c:children){
                    addFile(c,zipPath,zos);
                }
            }
        }
        else{
            String zipPathString = toZipPath(zipPath, f.getName());
            log.finest("add entry="+zipPathString);
            zos.putNextEntry(zipPathString);
            InputStream is = new BufferedInputStream(new FileInputStream(f));
            try {
                int i;
                byte[] buf=new byte[1024];
                while ((i = is.read(buf)) > 0) {
                    zos.write(buf, 0, i);
                }
                zos.closeEntry();
            } finally {
                is.close();
            }
        }
        addedFiles++;
        ib.updateBlockMessage(getMessage());
    }

    /**
     *
     * @param zipPath directory elements
     * @return zippath to directory
     */
    private String toZipDirPath(String[] zipPath) {
        StringBuilder sb = new StringBuilder();
        for(String s:zipPath){
            sb.append(s);
            sb.append("/");
        }
        return sb.length()==0?"/":sb.toString(); //keep trailing slash since this is a dir
    }
    private String toZipPath(String[] zipPath, String fileName) {
        return toZipDirPath(zipPath)+fileName;
    }

    private String[] path(String[] old, String plus){
        String[] ret = new String[old.length+1];
        System.arraycopy(old,0,ret,0,old.length);
        ret[ret.length-1]=plus;
        return ret;
    }

    private String getMessage() {
        final String s = "Creating file "+zipFileName;
        if (addedFiles>0) {
            return s+"\n (added "+addedFiles+" entries)";
        } else {
            return s;
        }
    }
}
