package uk.co.hcsoftware.commander.gui.file;

import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.io.ImageProvider;
import uk.co.hcsoftware.commander.AppProperties;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.*;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: CopyContentsAction
 * Copyright:    Copyright (c) 01-Mar-2006
 *
 * @author N Spurling
 */
public class CopyContentsAction extends AbstractAction{
    public static final Logger log = Logger.getLogger(CopyContentsAction.class.getName());
    private final Component c;
    private final FileSelector fs;
    private static final String TITLE = "Copy contents to clipboard";

    public CopyContentsAction(Component c,FileSelector fs) {
        super(TITLE);
        this.c = c;
        this.fs = fs;
        ActionBuilder.getInstance().init(this)
                .name("Copy file contents to clipboard")
                .tooltip("Copy the contents of these files to the clipboard")
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_C,AppProperties.getInstance().getCtrlMask()| KeyEvent.SHIFT_DOWN_MASK | KeyEvent.ALT_DOWN_MASK))
                .command("copycontents")
                .icon(ImageProvider.getInstance().getIcon("icons/16/editcopy.png"));
    }
    public void actionPerformed(ActionEvent e) {
        try {
            Transferable copyResult = getCopyResult();
            if (copyResult!=null) {
                try {
                    Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
                    cb.setContents(
                            copyResult
                            , new ClipboardOwner() { public void lostOwnership(Clipboard clipboard, Transferable contents) { }}
                    );
                } catch (Exception e1) {
                    log.log(Level.FINE, "couldn't copy "+copyResult+" to clipboard", e1);
                    JOptionPane.showMessageDialog(c,"Couldn't put contents of file on clipboard:\n  "+e1,TITLE,JOptionPane.ERROR_MESSAGE);
                }
            }
            else{
                JOptionPane.showMessageDialog(c,"Please select a file to copy",TITLE,JOptionPane.WARNING_MESSAGE);
            }
        } catch (IOException e1) {
            log.log(Level.FINE, "exception", e1);
            JOptionPane.showMessageDialog(c,"Couldn't read contents of file into text:\n  "+e1,TITLE,JOptionPane.ERROR_MESSAGE);
        }
    }


    protected Transferable getCopyResult() throws IOException {
        //copy the selected names to the clipboard
        java.util.List<File> l = fs.getSelectedFiles(true,true,false);
        if (!l.isEmpty()){//l.size()==1 && l.get(0).isFile()) {
            StringBuilder sb = new StringBuilder();
            for (File f:l) {
                final BufferedInputStream is = new BufferedInputStream(new FileInputStream(f));
                try {
                    if(sb.length()>0) sb.append("\n");
                    sb.append(new String(readData(is),"UTF8"));
                } finally {
                    is.close();
                }
            }
            return new StringSelection(sb.toString());
        }
        return null;
    }
    private byte[] readData(InputStream is) throws IOException {
        if(is==null) throw new IOException("No such resource");
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            byte[] buf=new byte[1024];
            int i;
            while((i=is.read(buf))!=-1){
                baos.write(buf,0,i);
            }
            return baos.toByteArray();
        } finally {
            is.close();
        }
    }

}
