package uk.co.hcsoftware.commander.gui.file;

import uk.co.hcsoftware.commander.AppProperties;
import uk.co.hcsoftware.commander.io.ImageProvider;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: CopyNamesAction
 * Copyright:    Copyright (c) 01-Mar-2006
 *
 * @author N Spurling
 */
public class CopyNamesAction extends AbstractAction{
    public static final Logger log = Logger.getLogger(CopyNamesAction.class.getName());
    private final Component c;
    private final FileSelector fs;
    private static final String TITLE = "Copy file name(s) to clipboard";

    public CopyNamesAction(Component c, FileSelector fs) {
        super(TITLE);
        this.c = c;
        this.fs = fs;
         ActionBuilder.getInstance().init(this)
                .tooltip(TITLE)
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_C,AppProperties.getInstance().getCtrlMask()))
                .command("copynames")
                .icon(ImageProvider.getInstance().getIcon("icons/16/editcopy.png"));
    }
    public void actionPerformed(ActionEvent e) {
        Transferable copyResult = getCopyResult(false);
        if (copyResult!=null) {
            try {
                Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
                cb.setContents(
                        copyResult
                        , new ClipboardOwner() { public void lostOwnership(Clipboard clipboard, Transferable contents) { }}
                );
            } catch (Exception e1) {
                log.log(Level.FINE, "couldn't copy "+copyResult+" to clipboard", e1);
                JOptionPane.showMessageDialog(c,"Couldn't put contents of file on clipboard:\n  "+e1,TITLE,JOptionPane.ERROR_MESSAGE);
            }
        }    else{
            JOptionPane.showMessageDialog(c,"Please select a file to copy",TITLE,JOptionPane.WARNING_MESSAGE);
        }
    }


    protected Transferable getCopyResult(boolean copyFullPath){
        //copy the selected names to the clipboard
        List<File> l = fs.getSelectedFiles(true,true,false);
        if (!l.isEmpty()) {
            StringBuffer sb = new StringBuffer();
            for(File vf:l){
                if(sb.length()>0) sb.append("\n");
                sb.append(copyFullPath?vf.getAbsolutePath():vf.getName());
            }
            return new StringSelection(sb.toString());
        }
        return null;
    }

}
