package uk.co.hcsoftware.commander.gui.file;

import uk.co.hcsoftware.commander.AppProperties;
import uk.co.hcsoftware.commander.io.ImageProvider;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: CopyNamesAction
 * Copyright:    Copyright (c) 01-Mar-2006
 *
 * @author N Spurling
 */
public class CopyPathsAction extends CopyNamesAction{
    public static final Logger log = Logger.getLogger(CopyPathsAction.class.getName());
    private final Component c;
    private static final String TITLE = "Copy file path(s) to clipboard";

    public CopyPathsAction(Component c, FileSelector fs) {
        super(c, fs);
        this.c = c;
        ActionBuilder.getInstance().init(this)
                .name(TITLE)
                .tooltip(TITLE)
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_C,AppProperties.getInstance().getCtrlMask()| KeyEvent.SHIFT_DOWN_MASK))
                .command("copypaths")
                .icon(ImageProvider.getInstance().getIcon("icons/16/editcopy.png"));
    }
    public void actionPerformed(ActionEvent e) {
        Transferable copyResult = getCopyResult(true);
        if (copyResult!=null) {
            try {
                Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
                cb.setContents(
                        copyResult
                        , new ClipboardOwner() { public void lostOwnership(Clipboard clipboard, Transferable contents) { }}
                );
            } catch (Exception e1) {
                log.log(Level.FINE, "couldn't copy "+copyResult+" to clipboard", e1);
                JOptionPane.showMessageDialog(c,"Couldn't put contents of file on clipboard:\n  "+e1,TITLE,JOptionPane.ERROR_MESSAGE);
            }
        }
        else{
            JOptionPane.showMessageDialog(c,"Please select a file to copy",TITLE,JOptionPane.WARNING_MESSAGE);
        }
    }
}
