package uk.co.hcsoftware.commander.gui.file;

import ns.TaskTimer;
import ns.gui.SwingWorker3;
import org.apache.commons.codec.binary.Hex;
import uk.co.hcsoftware.commander.gui.HtmlDialog;
import uk.co.hcsoftware.commander.io.FileUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: HarvestDataAction
 * Copyright:    Copyright (c) 13-Feb-2006
 *
 * @author spurlinn
 */
public class CreateDigestAction extends AbstractAction {
    private static final Logger log = Logger.getLogger(CreateDigestAction.class.getName());
    private DirectoryPanel directoryPanel;
    private final String title;
    private String digest;

    public CreateDigestAction(DirectoryPanel directoryPanel, String digestName) {
        super("Calculate "+digestName+" digest"/*, ImageProvider.getInstance().getIcon("icons/16/ark.png")*/);
        this.title=(String) super.getValue(Action.NAME);//getTitle();
        this.directoryPanel = directoryPanel;
        this.digest=digestName;
    }


    public void actionPerformed(ActionEvent e) {
        final List<Vfile> l = directoryPanel.getSelectedVfiles(true, true, false);
        final List<File> fs = Vfile.getFiles(l);
        FileUtils.getInstance().weed(fs,false,true);
        if (!l.isEmpty()) {
            try {
                final MessageDigest md = MessageDigest.getInstance(digest);
                showDigest(fs,md);
            } catch (NoSuchAlgorithmException e1) {
                log.log(Level.WARNING, "exception", e1);
                JOptionPane.showMessageDialog(directoryPanel,
                                "Couldn't find that digest!",title
                                ,JOptionPane.ERROR_MESSAGE);
                        return;
            }
        }
        else{
            JOptionPane.showMessageDialog(directoryPanel,
                                "Please some fs",title
                                ,JOptionPane.ERROR_MESSAGE);
                        return;
        }
    }
    private String getMessage(File files, int filesToGo){
        final String s;
        if(filesToGo==1) s= files.getName();
        else s=filesToGo+" files";
        return "Calculating "+digest+" digest for "+ s ;
    }
    public void showDigest(final List<File> files,final MessageDigest md) {
        final File file1 = files.get(0);
        directoryPanel.blockInputWithCancel(getMessage(file1,files.size()));
        new SwingWorker3(){
            public Object construct() {
                try {
                    final StringBuffer sb = new StringBuffer();
                    sb.append("<html");
                    sb.append("<pre>");
                    int filesToGo=files.size();
                    TaskTimer tt = new TaskTimer();
                    for(File f:files){
                        directoryPanel.updateBlockMessage(getMessage(file1,filesToGo--));
                        InputStream is = new BufferedInputStream(new FileInputStream(f));
                        try {
                            byte[] buf = new byte[1024*10];
                            int i;
                            while((i=is.read(buf))!=-1){
                                if(directoryPanel.wasCancelled()) return null;
                                md.update(buf,0,i);
                            }
                            final byte[] bytes = md.digest();
                            String sbb = new String(Hex.encodeHex(bytes));//toHexString(bytes);
                            sb.append("<b>"+f.getName()+":</b> "+sbb+"<br>");
                        } finally {
                            is.close();
                        }
                    }
                    sb.append("</pre></html>");
                    log.fine("took "+tt.stopString()+" to calc "+digest+" digest for "+files.size()+" files");
                    return sb.toString();
                } catch (Exception e1) {
                    log.log(Level.WARNING,"e1",e1);
                    return e1;
                }
            }

            public void finished() {
                directoryPanel.unblockInput();
                if(directoryPanel.wasCancelled()) return;
                if(get()instanceof Exception){
                    JOptionPane.showMessageDialog(directoryPanel,
                            "couldn't gather data:\n"+ ((Exception) get()).getMessage(),title
                            ,JOptionPane.ERROR_MESSAGE);
                    return;
                }
                Window f = SwingUtilities.getWindowAncestor(directoryPanel);
                HtmlDialog td = new HtmlDialog((Frame) f, title, false);
                td.loadHtml((String) get());
                td.setSize(500,300);
                td.setLocationRelativeTo(directoryPanel);
                td.setVisible(true);
            }
        }.start();
    }
}
