package uk.co.hcsoftware.commander.gui.file;

import com.ice.tar.TarEntry;
import com.ice.tar.TarHeader;
import com.ice.tar.TarOutputStream;
import ns.gui.SwingWorker3;
import uk.co.hcsoftware.commander.io.FileUtils;
import uk.co.hcsoftware.commander.io.ImageProvider;
import uk.co.hcsoftware.commander.io.FileEventManager;
import uk.co.hcsoftware.commander.gui.CreateFileDialogBuilder;
import uk.co.hcsoftware.commander.gui.CommanduleContext;
import uk.co.hcsoftware.commander.gui.InputBlocker;
import uk.co.hcsoftware.commander.gui.CreateArkPanel2;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.*;
import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: CreateTarAction
 * Copyright:    Copyright (c) 13-Feb-2006
 *
 * @author spurlinn
 */
public class CreateTarAction extends AbstractAction {
    private static final Logger log = Logger.getLogger(CreateTarAction.class.getName());
    private static final String title="Create tar file";
//    private DirectoryPanel directoryPanel;
    private int addedFiles=0;
    private File tarFile;
    private Compression c;
    private Component parent;
    private CommanduleContext cc;
    private final FileSelector fs;
    private final InputBlocker ib;

     private ZipPathHelper zph = new ZipPathHelper();
    private File structureRoot;

    public CreateTarAction(Component parent, CommanduleContext cc, FileSelector fs, InputBlocker pu) {
        super(title, ImageProvider.getInstance().getIcon("icons/16/tar.png"));
        this.parent = parent;
        this.cc = cc;
        this.fs = fs;
        this.ib = pu;
//        this.directoryPanel = directoryPanel;
    }
    enum Compression{
        gzip("Create GZipped tar",".tar.gz")
        ,bzip2("Create Bzip2ed tar",".tar.bz2")
        //    ,lzma("Create LZMA tar",".tar.lzma")
        ,none("Create regular tar",".tar");
        private final String createMessage;
        private final String fileSuffix;
        Compression(String createMessage, String fileSuffix){
            this.createMessage = createMessage;
            this.fileSuffix = fileSuffix;
        }

        public String getCreateMessage() {
            return createMessage;
        }

        public String getFileSuffix() {
            return fileSuffix;
        }

        public String toString() {
            return createMessage;
        }
    }
    public void actionPerformed(ActionEvent e) {
        File vf = cc.getDirectory();
        final File parentDir = vf==null?new File(System.getProperty("user.home")):vf;
        final List<File> l = cc.getSelectedFiles();//directoryPanel.getSelectedVfiles(true, true, false);

        if (l.size()>0) {
            //prompt for gzip
            List poss = new ArrayList(Arrays.asList(Compression.values()));
            poss.add("Cancel");
            Object[] possibleValues = poss.toArray();
            int selectedValue = JOptionPane.showOptionDialog(parent,
                    "Do you want to compress the tar?",title,
                    JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                    possibleValues, possibleValues[0]);
            if(selectedValue>=poss.size()){
                return;//cancel selected, so abort
            }
            c = (Compression) poss.get(selectedValue);
            final String suffix =c.getFileSuffix();
//            File dest = new CreateFileDialogBuilder()
//                    .withTitle(title)
//                    .withMessage("Create tar file:")
//                    .withPathsuffix(suffix)
//                    .showDialogForFile(parent,parentDir);
             CreateArkPanel2.CreateArkDialogBuilder builder = CreateArkPanel2.getBuilder();
        File dest = builder
                .withTitle(title)
                .withMessage("Create zip file:")
                .withPathsuffix(suffix)
                .withInputFiles(l)
                .showDialogForFile(parent,parentDir);
        if(builder.isPreserveStructure()){
            log.fine("preservestruc commonanc="+builder.getRootAncestorFile());
            structureRoot=builder.getRootAncestorFile();
        }

            if(dest!=null/* && dest.trim().length()>0*/){

                tarFile = dest;//new File(parentDir,dest);
                final boolean exists = tarFile.exists();
                if(exists){
                    if(!tarFile.isFile()){
                        JOptionPane.showMessageDialog(parent,"That name is already in use by a directory!",title,JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                    else if(JOptionPane.YES_OPTION!=JOptionPane.showConfirmDialog(parent,
                            "Replace existing file?\n"+tarFile.getName(),title,JOptionPane.YES_NO_CANCEL_OPTION
                            ,JOptionPane.QUESTION_MESSAGE)){
                        return;
                    }
                }
                ib.blockInputWithCancel(getMessage());
                new SwingWorker3(){
                    public Object construct() {
                        try {
                            final TarOutputStream tos= FileUtils.getInstance().getTarOutputStream(tarFile);// = new TarOutputStream(new FileOutputStream(tarFile));
                            try {
                                writeToTar(l,tos);
                                return null;
                            } finally {
                                tos.close();
                            }
                        } catch (Exception e1) {
                            log.log(Level.WARNING, "exception", e1);
                            return e1;
                        }
                    }
                    public void finished() {
                        ib.unblockInput();
                        if(get() instanceof Exception){
                            if(tarFile.exists()) fireEvents(exists);
                            Exception e1=(Exception) get();
                            JOptionPane.showMessageDialog(parent,"Couldn't create tar file\n" +
                                    "["+e1.getMessage()+"]",title,JOptionPane.ERROR_MESSAGE);
                            return;
                        }
                        fireEvents(exists);
                        fs.selectFiles(Collections.singletonList(tarFile));
                    }
                }.start();

            }
            else log.fine("ignoring "+title+" as null name entered");
        }
        else JOptionPane.showMessageDialog(parent,"Please select some files to go in the tar",title,JOptionPane.ERROR_MESSAGE);
    }

    private void fireEvents(final boolean exists) {
        if (!exists) {
            FileEventManager.getInstance().fireAdded(tarFile);
        } else {
            FileEventManager.getInstance().fireChanged(tarFile);
        }
    }

    private String getMessage() {
        final String s = "Creating " +
                ( c==Compression.gzip?"Gzipped ":c==Compression.bzip2?"Bzipped ":"" )+
                "tarfile \n "+tarFile;
        if (addedFiles>0) {
            return s+"\n (added "+addedFiles+" entries)";
        } else {
            return s;
        }
    }

    private String getCompressionMessage() {
        if(c==Compression.gzip) return "Gzipped";
        else if(c==Compression.bzip2) return "Bzipped";
        return "";
    }

    private void writeToTar(List<File> l, TarOutputStream zos) throws IOException {
        // if preserve struc, path should be path relative to structureRoot (ending with /)
        log.finest("write tozip files="+l);
        String[] pathToRoot =null;
        FileUtils fu = FileUtils.getInstance();
        if(structureRoot!=null){
            pathToRoot =fu.splitPath(structureRoot.getCanonicalPath());
        }
        for(File vf:l){
            if(ib.wasCancelled()){
                log.finest("returning as ib cancelled");
                return;
            }
            String[] zipPath=new String[0];
            String canPath = vf.getParentFile().getCanonicalPath();
            if (pathToRoot !=null) {  //ensure this file's zip-path is relative to this ancestor file
                String[] pathToFile =fu.splitPath(canPath);
                zipPath=new String[pathToFile.length-pathToRoot.length];
                System.arraycopy(pathToFile,pathToRoot.length,zipPath,0,zipPath.length);
                log.fine("converted inputpath="+canPath+" to="+ Arrays.asList(zipPath)+" base="+pathToRoot);
            }
            log.finest("add file="+vf);
            addFile(vf,zipPath,zos);
        }

//        for(File vf:l){
//            addFile(vf,"",zos);
//        }
    }
    // Here is a sample printout of ZipEntry.getName() from a well known zipfile (top-level dir is commanderbond-0.1)
//    commanderbond-0.1/src/uk/co/hcsoftware/commander/treeplugin/file/
//    commanderbond-0.1/src/uk/co/hcsoftware/commander/treeplugin/zip/
//    commanderbond-0.1/src/uk/co/hcsoftware/commander/vfs/
//
    private void addFile(File f,String[] path, TarOutputStream zos) throws IOException {
        if(ib.wasCancelled()) return;
        if(f.isDirectory()){
            String[] zipPath=zph.path(path,f.getName()); //the file is the dir, so add it to the path
            String zipPathString = zph.toZipDirPath(zipPath);
            log.finest("add entry="+zipPathString);
//            String thisPath = path+f.getName()+"/";
            zos.putNextEntry(new TarEntry(zipPathString));
            zos.closeEntry();
            File[] children = f.listFiles();
            if(children.length>0){
                for(File c:children){
                    addFile(c,zipPath,zos);
                }
            }
        }
        else{
            String zipPathString = zph.toZipPath(path, f.getName());
            log.finest("add entry="+zipPathString);
            
//            String thisPath = path+f.getName();
            final TarEntry te = new TarEntry(f);
            TarHeader th = te.getHeader();
            th.name=new StringBuffer(zipPathString);
            zos.putNextEntry(te);
            log.finer("writing entry header="+te.getHeader());
            writeFile(f, zos);
        }
        addedFiles++;
        ib.updateBlockMessage(getMessage());
    }
    private void writeFile(File f, TarOutputStream zos) throws IOException {
        InputStream is = new BufferedInputStream(new FileInputStream(f));
        try {
            int i;
            byte[] buf=new byte[1024];
            while ((i = is.read(buf)) > 0) {
                zos.write(buf, 0, i);
            }
            is.close();
            zos.closeEntry();
        } finally {
            is.close();
        }
    }
}
