package uk.co.hcsoftware.commander.gui.file;

import ns.gui.SwingWorker3;
import uk.co.hcsoftware.commander.io.ImageProvider;
import uk.co.hcsoftware.commander.io.FileEventManager;
import uk.co.hcsoftware.commander.io.FileUtils;
import uk.co.hcsoftware.commander.gui.InputBlocker;
import uk.co.hcsoftware.commander.gui.CommanduleContext;
import uk.co.hcsoftware.commander.gui.CreateArkPanel2;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.*;
import java.io.*;
import java.util.Collections;
import java.util.List;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;


/**
 * Title: CreateZipAction
 * Copyright:    Copyright (c) 13-Feb-2006
 *
 * @author spurlinn        
 */
public class CreateZipAction extends AbstractAction {
    private static final Logger log = Logger.getLogger(CreateZipAction.class.getName());
    private final static String title="Create zip file";
    private CommanduleContext cc;
    private FileSelector fs;
    private InputBlocker ib;
    private Component directoryPanel;
    private int addedFiles=0;
    private File zipFile;
    private File structureRoot;

    public CreateZipAction(Component parent, CommanduleContext cc, FileSelector fs, InputBlocker pu) {
        super(title, ImageProvider.getInstance().getIcon("icons/16/ark.png"));
        this.cc = cc;
        this.fs = fs;
        this.ib = pu;
        this.directoryPanel = parent;
    }

    public void actionPerformed(ActionEvent e) {
        File vf = cc.getDirectory();
        final File parentDir = vf==null?new File(System.getProperty("user.home")):vf;
        final List<File> l = cc.getSelectedFiles();//directoryPanel.getSelectedVfiles(true, true, false);
        boolean addFiles=false;
        if (l.size()>0) {
            //prompt for add
            Object[] possibleValues = { "Add "+l.size()+" files","Create empty zip","Cancel" };
            int selectedValue = JOptionPane.showOptionDialog(directoryPanel,
                    "Do you want to add the selected files?",title,
                    JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                    possibleValues, possibleValues[0]);
            if(0==selectedValue){ //create the zip and add these files
                addFiles=true;
            }
            else if(1==selectedValue){ //create the zip but add nothing
                addFiles=false;
            }
            else return;//abort
        }

        CreateArkPanel2.CreateArkDialogBuilder builder = CreateArkPanel2.getBuilder();
        File dest = builder
                .withTitle(title)
                .withMessage("Create zip file:")
                .withPathsuffix(".zip")
                .withInputFiles(l)
                .showDialogForFile(directoryPanel,parentDir);
        if(builder.isPreserveStructure()){
            log.fine("preservestruc commonanc="+builder.getRootAncestorFile());
            structureRoot=builder.getRootAncestorFile();
        }
//        File dest = new CreateFileDialogBuilder()
//                .withTitle(title)
//                .withMessage("Create zip file:")
//                .withPathsuffix(".zip")
//                .showDialogForFile(directoryPanel,parentDir);

        if(dest!=null /*&& dest.trim().length()>0*/){
            zipFile = dest;
            final boolean exists = zipFile.exists();
            if(exists){
                if(!zipFile.isFile()){
                    JOptionPane.showMessageDialog(directoryPanel,"That name is already in use by a directory!",title,JOptionPane.ERROR_MESSAGE);
                    return;
                }
                else if(JOptionPane.YES_OPTION!=JOptionPane.showConfirmDialog(directoryPanel,
                        "Replace existing file?\n"+zipFile.getName(),title,JOptionPane.YES_NO_CANCEL_OPTION
                        ,JOptionPane.QUESTION_MESSAGE)){
                    return;
                }
            }
            ib.blockInputWithCancel(getMessage());
            final boolean addFiles1 = addFiles;
            new SwingWorker3(){
                public Object construct() {
                    try {
                        final ZipOutputStream zos= new ZipOutputStream(new FileOutputStream(zipFile));
                        try {
                            if(addFiles1){
                                   writeToZip(l,zos);
                            }
                            else{
                                zos.putNextEntry(new ZipEntry("placeholder"));
                                zos.closeEntry();
                            }
                        } finally {
                            try {
                                zos.close(); //this throws an exception if zip is empty
                            } catch (IOException e1) {
                                log.log(Level.WARNING,"error closing zip",e1);
                            }
                        }
                        return null;
                    } catch (Exception e1) {
                        log.log(Level.WARNING, "exception", e1);
                        return e1;
                    }
                }
                public void finished() {
                    ib.unblockInput();
                    if(get() instanceof Exception){
                        if(zipFile.exists()) fireEvents(exists);
                        Exception e1=(Exception) get();
                        JOptionPane.showMessageDialog(directoryPanel,"Couldn't create zip file\n" +
                                "["+e1.getMessage()+"]",title,JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                    fireEvents(exists);
                    fs.selectFiles(Collections.singletonList(zipFile));
                }
            }.start();
        }
        else log.fine("ignoring "+title+" as null name entered");
    }

    private void fireEvents(final boolean exists) {
        if (!exists) {
            FileEventManager.getInstance().fireAdded(zipFile);
        } else {
            FileEventManager.getInstance().fireChanged(zipFile);
        }
    }

    public void writeToZip(List<File> l, ZipOutputStream zos) throws IOException {
        // if preserve struc, path should be path relative to structureRoot (ending with /)
        log.finest("write tozip files="+l);
        String[] pathToRoot =null;
        FileUtils fu = FileUtils.getInstance();
        if(structureRoot!=null){
            pathToRoot =fu.splitPath(structureRoot.getCanonicalPath());
        }
        for(File vf:l){
            if(ib.wasCancelled()){
                log.finest("returning as ib cancelled");
                return;
            }
            String[] zipPath=new String[0];
            String canPath = vf.getParentFile().getCanonicalPath();
            if (pathToRoot !=null) {  //ensure this file's zip-path is relative to this ancestor file
                String[] pathToFile =fu.splitPath(canPath);
                zipPath=new String[pathToFile.length-pathToRoot.length];
                System.arraycopy(pathToFile,pathToRoot.length,zipPath,0,zipPath.length);
                log.fine("converted inputpath="+canPath+" to="+ Arrays.asList(zipPath)+" base="+pathToRoot);
            }
            log.finest("add file="+vf);
            addFile(vf,zipPath,zos);
        }
    }
    private ZipPathHelper zph = new ZipPathHelper();
    // Here is a sample printout of ZipEntry.getName() from a well known zipfile (top-level dir is commanderbond-0.1)
//    commanderbond-0.1/src/uk/co/hcsoftware/commander/treeplugin/file/
//    commanderbond-0.1/src/uk/co/hcsoftware/commander/treeplugin/zip/
//    commanderbond-0.1/src/uk/co/hcsoftware/commander/vfs/
//
    /**
     *
     * @param f
     * @param zipPath path to this element in the zip
     * @param zos
     * @throws IOException
     */
    private void addFile(File f,String[] zipPath, ZipOutputStream zos) throws IOException {
        if(ib.wasCancelled()) return;
        log.finest("add File="+f+" path="+Arrays.asList(zipPath));
        if(f.isDirectory()){
            zipPath=zph.path(zipPath,f.getName()); //the file is the dir, so add it to the path
            String zipPathString = zph.toZipDirPath(zipPath);
            log.finest("add entry="+zipPathString);
            zos.putNextEntry(new ZipEntry(zipPathString));
            zos.closeEntry();
            File[] children = f.listFiles();
            if(children.length>0){
                for(File c:children){
                    addFile(c,zipPath,zos);
                }
            }
        }
        else{
            String zipPathString = zph.toZipPath(zipPath, f.getName());
            log.finest("add entry="+zipPathString);
            zos.putNextEntry(new ZipEntry(zipPathString));
            InputStream is = new BufferedInputStream(new FileInputStream(f));
            try {
                int i;
                byte[] buf=new byte[1024];
                while ((i = is.read(buf)) > 0) {
                    zos.write(buf, 0, i);
                }
                zos.closeEntry();
            } finally {
                is.close();
            }
        }
        addedFiles++;
        ib.updateBlockMessage(getMessage());
    }


     private String getMessage() {
        final String s = "Creating zip file "+zipFile;
        if (addedFiles>0) {
            return s+"\n (added "+addedFiles+" entries)";
        } else {
            return s;
        }
    }



}
