package uk.co.hcsoftware.commander.gui.file;

import com.ice.tar.TarArchive;
import ns.MacApp;
import ns.TaskTimer;
import ns.gui.SwingWorker3;
import org.jdesktop.swingx.decorator.Filter;
import spin.Spin;
import uk.co.hcsoftware.commander.*;
import uk.co.hcsoftware.commander.gui.*;
import uk.co.hcsoftware.commander.gui.ctable.*;
import uk.co.hcsoftware.commander.gui.usercommands.CommandManager;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.gui.util.ActionDispatcher;
import uk.co.hcsoftware.commander.gui.util.GuiUtilities;
import uk.co.hcsoftware.commander.gui.util.WeakListener;
import uk.co.hcsoftware.commander.gui.zip.CannotOpenZipException;
import uk.co.hcsoftware.commander.gui.zip.ZipPanel;
import uk.co.hcsoftware.commander.io.*;
import uk.co.hcsoftware.commander.io.treestream.Node;
import uk.co.hcsoftware.commander.io.treestream.TreeStream;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.List;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.nio.channels.FileChannel;

/**
 *   DirectoryPanel is one of the panels showing some file-nodes (e.g. the leftpanel or the rightpanel)
 */
public class DirectoryPanel extends CommanderViewPanel
        implements DndHelper.DndHandler, FileSelector,ProgressUpdater
        ,WindowListener

 {
    private static final Logger log = Logger.getLogger(DirectoryPanel.class.getName());
    protected final FileEventManager fem = FileEventManager.getInstance();

    protected JTextField locationField=new JTextField();
    private final JScrollPane jsp;
    private final FileTableModel tableModel;// = new FileTableModel(null);

    private CommanderSelectionUtilities<Vfile> csu;
    private final PasteToNewFileAction pasteToNewFileAction = new PasteToNewFileAction(this);
    private final JLabel dirLabel;
    private CommanduleContext commanduleContext=new CommanduleContext() {
        public File getDirectory() {
            return DirectoryPanel.this.getCurrentDirectory();
        }

        public List<File> getSelectedFiles() {
            return DirectoryPanel.this.getSelectedFiles(true,true,false);
        }

    };


    public DirectoryPanel(CommanderFrame mf, final CommanderFocus cf) {
        super(new BorderLayout(),cf, mf, new CommanderTable(new FileTableModel()));
        tableModel=(FileTableModel) table.getModel();

        locationField.setAutoscrolls(true);

        setupTable();

        new DndHelper.DndHelperBuilder(RECEIVE_FLAVORS,rowHeaderTable,this)
                .otherDropTargets(this,table)
                .shouldStartDragCheck(this)
                .create()
                ;
        jsp = new JScrollPane(table);
        jsp.setRowHeaderView(rowHeaderTable);
        final JPanel northPan = new JPanel(new BorderLayout(2,2));
        JScrollPane locScrollpane = new JScrollPane(locationField);
        locScrollpane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        northPan.add(locScrollpane);
        dirLabel = new JLabel(ImageProvider.getInstance().getIcon("icons/16/folder.png"));
        northPan.add(dirLabel,BorderLayout.WEST);
        dirLabel.setVerticalAlignment(SwingConstants.TOP);
        dirLabel.setBorder(BorderFactory.createEmptyBorder(2,1,0,0));
        dirLabel.setPreferredSize(new Dimension(20,20)); //icon is 16px^2 so make button about that size
        dirLabel.setFocusable(false);
        dirLabel.setOpaque(true);
        new DndHelper.DndHelperBuilder(new DataFlavor[0], dirLabel,new DndHelper.DndHandler() {
            public void drop(DropTargetDropEvent evt, Transferable transferable) {}
            public Transferable getTransferable(DragGestureEvent dge) {
                File dir = getCurrentDirectory();
                if(dir==null) return null;
                return new FileListTransferable(Collections.singletonList(dir),dirLabel);
            }
        })
                .shouldStartDragCheck(new DndHelper.StartDrag() {
                    public boolean shouldStartDrag(DragGestureEvent dge) {
                        return getDirectory()!=null;
                    }
                })
                .create()
                ;

        final JButton jb = GuiUtilities.createIconButton(refreshAction);
        final JButton opButton = GuiUtilities.createIconButton(openParentAction);
        Box buttonBox = new Box(BoxLayout.LINE_AXIS);
        buttonBox.add(jb);
        buttonBox.add(opButton);

        northPan.add(buttonBox,BorderLayout.EAST);
        add(northPan,BorderLayout.NORTH);

        add(jsp);



        setupActions();
        setupPopup();
        locationField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                File f = AppProperties.getFile(locationField.getText());
                openFile(f);
            }
        });

        //forward up/down keystrokes to the commandertree
        ActionMap am = locationField.getActionMap();
        InputMap im = locationField.getInputMap(JComponent.WHEN_FOCUSED);
        im.put(KeyStroke.getKeyStroke(KeyEvent.VK_UP,0), "up");
        im.put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN,0), "down");
        am.put("up",table.getForwardAction(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,KeyStroke.getKeyStroke(KeyEvent.VK_UP,0)));
        am.put("down",table.getForwardAction(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,KeyStroke.getKeyStroke(KeyEvent.VK_DOWN,0)));

        init();


        // register a weaklistener with cf to get frame activation events and call getModel().checkForChanges()
        log.fine("adding windowactivation listener");
        WeakListener wl = new WeakListener(this, WindowListener.class){
            public Object onNullObserver() {
                log.fine("null observer!");
                commanderFrame.removeWindowListener((WindowListener) getProxy());
                return null;
            }
        };
        commanderFrame.addWindowListener((WindowListener) wl.getProxy());
    }

    public String getRowName(int viewRow) {
        int modelRow=table.convertRowIndexToModel(viewRow);
        Vfile file = getModel().getElement(modelRow);
        if(file!=null) return file.getName();
        return null;
    }

    public boolean isDirectory(int viewRow) {
        int modelRow=table.convertRowIndexToModel(viewRow);
        Vfile file = getModel().getElement(modelRow);
        if(file!=null) return file.isDirectory();
        return false;
    }


    private void setupTable() {
        final TableRenderable tr = new TableRenderable() {
            FileRecognizer fr = FileRecognizer.getInstance();
            public void setupRenderLabel(JLabel jl, int viewRow, int viewCol) {
                int modelRow=table.convertRowIndexToModel(viewRow);
                Vfile file = getModel().getElement(modelRow);
                if (file!=null) {
                    jl.setFont(file.isDirectory()?AppProperties.boldFont:AppProperties.plainFont);
                }
            }
        };
        final ColumnSpec[] columns={
            ColumnSpec.createColumnSpec(FileTableModel.MARK,"mark_column_w",5)
                .setRenderer(new MarkRenderer(tr))
                .comparator(new BooleanComparator(),SortDirection.descending)

            ,ColumnSpec.createColumnSpec(FileTableModel.NAME,"name_column_w",200)
                .setRenderer(new StringRenderer(tr) )  //name column
                .comparator(new NameComparator(),SortDirection.ascending)

             ,ColumnSpec.createColumnSpec(FileTableModel.SIZE,"size_column_w",50)
                .setRenderer(new SizeRenderer(tr){
                 public Object getAttributeValue(Object attributeValue, int row, int column) {
                     if(((Long)attributeValue).longValue()==-1){
                          return "<DIR>";
                     }
                     return super.getAttributeValue(attributeValue, row, column);
                 }
             } )
                .comparator(new LongComparator(),SortDirection.descending)

            ,ColumnSpec.createColumnSpec(FileTableModel.DATE,"date_column_w",120)
                .setRenderer(new DateRenderer(tr))
             .comparator(new LongComparator(),SortDirection.descending)

            ,ColumnSpec.createColumnSpec(FileTableModel.EXT,"ext_column_w",50)
                .setRenderer(new StringRenderer(tr))
             .comparator(new NameComparator(),SortDirection.ascending)

        };

        table.addPostFilter(new CommanderTable.FilterFactory() {
            public Filter getFilter() {
                return createDirectoryFilter(FileTableModel.SIZE);
            }
        });
        table.configureColumns(Arrays.asList(columns));
        table.setCommanderFocusOwner(cf,this);
        csu = new CommanderSelectionUtilities<Vfile>(table,cf,this);
        table.setSort(FileTableModel.NAME);  //by default sort by name column
    }
    public void setColors(Color backColor, Color foreColor, Color selColor) {
       log.fine("setting colors.. back="+backColor+" fore="+foreColor+" sel="+selColor);
       super.setColors(backColor, foreColor, selColor);
       setBackground(backColor);
       jsp.getViewport().setBackground(backColor);
       jsp.getRowHeader().setBackground(backColor);
       locationField.setBackground(backColor);
       locationField.setForeground(foreColor);
   }

    private void setupActions() {
        table.registerAction(findAction,refreshAction,browseNewViewAction,copyPathsAction
                ,copyNamesAction,copyContentsAction,markAction,openParentAction,openChildAction,deleteAction
                ,escapeAction,enterAction,locationFieldAction,jumpDownAction,jumpUpAction
                ,pasteToNewFileAction
        );
        //fire deleteAction on backspace
        table.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE,0),deleteAction.getValue(Action.ACTION_COMMAND_KEY));

    }
    private ActionBuilder ab = ActionBuilder.getInstance();
    Action replaceSpacesAction = ab.init(new AbstractAction() {
        /**
         *
         * @param sources
         * @return count of how many of these files' counterpart desitination files already exist
         */
        private int findExistings(List<File> sources) {
            int i=0;
            for(File f:sources) if(getTargetFile(f).exists()) i++;
            return i;
        }
        private File getTargetFile(File sourceFile){
            return new File(sourceFile.getParentFile(), sourceFile.getName().replaceAll(" ","_"));
        }

        public void actionPerformed(ActionEvent e) {
            List<File> selected = getSelectedFiles(true,true,false);
            List<File> sources = new ArrayList<File>();
            for(File f:selected){
                if(f.getName().indexOf(' ')!=-1){
                    sources.add(f);
                }
            }

            if(!sources.isEmpty()){
                Object[] possibleValues = { "Rename", "Cancel" };
                int existings = findExistings(sources);
                int selectedValue = JOptionPane.showOptionDialog(DirectoryPanel.this,
                        "Are you sure you want to rename "
                        +(sources.size()==1?sources.get(0).getName():sources.size()+" files")
                        +"?"
                        +(existings==0?"":"\n "+existings+" file(s) will be overwritten")
                        ,"Replace spaces in name(s)",
                        JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                        possibleValues, possibleValues[0]);
                if(0==selectedValue){
                    List<File> renamedFrom=new ArrayList<File>(), renamedTo=new ArrayList<File>();
                    for (File sourceFile:sources) {
                        File destinationFile = getTargetFile(sourceFile);
                        log.fine("sourcefile="+sourceFile+" destinationFile="+destinationFile);
                        if(sourceFile.renameTo(destinationFile)) {
                            renamedFrom.add(sourceFile);
                        } else{
                            JOptionPane.showMessageDialog(DirectoryPanel.this
                                    ,"Couldn't rename "+sourceFile
                                    ,"Rename"
                                    ,JOptionPane.ERROR_MESSAGE);
                        }
                    }
                    if(!renamedFrom.isEmpty() || !renamedTo.isEmpty()){
                        FileEventManager.getInstance().fireEvent(renamedFrom, renamedTo, null);
                        selectFiles(renamedTo);
                    }
                }
            }
        }
        })
            .icon(ImageProvider.getInstance().getIcon("icons/16/filesaveas.png"))
            .name("Replace spaces in name(s)")
            .tooltip("Rename the file(s) replacing spaces with underscores")
    .get();


    Action locationFieldAction = ab.init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                locationField.selectAll();
                locationField.requestFocus();
            }
        }) .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_L,ctrlMask))
           .command("lbar")
    .get();
    Action jumpDownAction = ab.init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            //if there is a selected dir, jump at least to that dir, and
            //ignore the selection as it is confusing - there is no way to know if the selected child has children or not..
            final File currDir = getDirectory().getFile();
            final File startDir=getFirstSelectedDirectory();
            File f = startDir==null?currDir:startDir;
            File[] childdirs;
            while((childdirs=f.listFiles()).length==1){
                File dir=childdirs[0];
                if(!dir.isDirectory()){
                    break;
                }
                f=dir;
            }
            if(!f.equals(currDir)){
                openFile(f);
            }
            else log.fine("ignoring jumpdown f="+f);
        }
    }) .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,ctrlMask))
            .icon(ImageProvider.getInstance().getIcon("icons/16/2rightarrow.png"))
            .name("Jump over empty dirs")
            .command("jumpdown")
            .tooltip("jump to first descendant directory that has children")
            .get();
    Action jumpUpAction = ab.init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            //when jumping left, ignore the selection as it is confusing jumping up from selected child - always start form CWD
           final File currDir = getDirectory().getFile();
            File f=currDir;
            File parent;
            while((parent=f.getParentFile())!=null){
                f=parent;
                if(f.list().length>1){
                    break;
                }
            }

            if(!f.equals(currDir)){
                openFile(f);
            }
            else log.fine("ignoring jumpup f="+f);
        }
    }) .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,ctrlMask))
            .icon(ImageProvider.getInstance().getIcon("icons/16/2leftarrow.png"))
            .command("selectancestor")
            .name("Jump over empty dirs")
            .tooltip("jump to first ancestor directory that has children")
            .get();
    DeleteAction deleteAction= new DeleteAction(this,this,this);
    static final int ctrlMask = AppProperties.getInstance().getCtrlMask();
    Action browseNewViewAction = ab.init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                browseInNewView();
            }
        }) .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,ctrlMask))
           .command("browsenewview")
    .get();
    Action escapeAction = ab.init(new AbstractAction("") {
        public void actionPerformed(ActionEvent e) {
            table.clearSelection();
            getModel().clearMarked();
        }
    })
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0))
            .command("clear")
            .get();

    Action refreshAction = ab.init(new AbstractAction("") {
        public void actionPerformed(ActionEvent e) {
            refresh();
        }
    })
            .icon(ImageProvider.getInstance().getIcon("icons/16/reload.png"))
            .tooltip("Reload current directory from filesystem")
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_R,AppProperties.getInstance().getCtrlMask()))
            .command("refresh")
            .get();

    Action openParentAction = ab.init(new AbstractAction("") {
        public void actionPerformed(ActionEvent e) {
            Vfile vf = getDirectory();
            if(vf!=null){
                Vfile parent = vf.getParentFile();
                if(parent !=null){
                    openFile(parent,vf);
                }
                else log.fine("ignoring openparent as parent==null");
            }
            else log.fine("ignoring openparent as dir==null");
        }
    })
            .icon(ImageProvider.getInstance().getIcon("icons/16/1uparrow.png"))
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,0))
            .command("openparent")
            .tooltip("Open parent directory")
            .get();
    Action enterAction = ab.init(new AbstractAction("") {
        public void actionPerformed(ActionEvent e) {
            onEnter();
        }
    })
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0))
            .command("enter")
            .get();

    Action openChildAction = ab.init(new AbstractAction("") {
        public void actionPerformed(ActionEvent e) {
            List<Vfile> files =getSelectedVfiles(true,false,false);
             if(!files.isEmpty()){
                Vfile child = files.get(0);
                if(child !=null){
                    openFile(child,null);
                }
                else log.fine("ignoring openparent as child==null");
            }
            else log.fine("ignoring openchild 0 files selected");
        }
    })
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,0))
            .command("openchild")
            .tooltip("Open child directory")
            .get();
    private FindAction findAction = new FindAction(this);
    private RenameAction renameAction = new RenameAction(this,this,this);
    private Action markAction =ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            final int[] selectedViewRows = csu.getSelectedRows(true,false);
            final int[] selectedModelRows=new int[selectedViewRows.length];
            for(int i=0;i<selectedViewRows.length;i++){
                selectedModelRows[i]=table.convertRowIndexToModel(selectedViewRows[i]);
            }
            getModel().toggleMarked(selectedModelRows);

        }
    }).keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0))
           .command("mark").get();
    private Action copyPathsAction = new CopyPathsAction(this, this);
    private Action copyNamesAction = new CopyNamesAction(this, this);
    private Action copyContentsAction=new CopyContentsAction(this,this);


    private void browseInNewView() {
        List<Vfile> f = getSelectedVfiles(true,false,true);
        if(f.size()==1){
            final DirectoryPanel zp = new DirectoryPanel(commanderFrame,cf);
            commanderFrame.addTabule(this,zp,JSplitPane.VERTICAL_SPLIT);
            zp.openFile(f.get(0),null);
            zp.getFocus();
        }
        else log.fine("ignoring browseInNewView as "+f.size()+" selected files");
    }

    private void refresh() {
        final Vfile directory = getDirectory();
        if (directory!=null) {
            openFile(directory,null);
        }
    }



    public CommanderFocus getCommanderFocus() {
        return cf;
    }

    public CommanderFrame getCommanderFrame() {
        return commanderFrame;
    }



    //ACTIONS
    public interface TarStractor{
        public List<File> extractTar(File tar, File to) throws IOException;

        List<File> extractZip(File f, File destinationDir) throws IOException;
    }
    private void onSingleFileClicked(File f){
        if(f.isDirectory()){
            //if we are on a mac then .app means the directory should be treated like an app
            if(MacApp.isMac() && f.getName().endsWith(".app")){
                Object[] possibleValues = { "Launch","Browse", "Cancel" };
                int selectedValue = JOptionPane.showOptionDialog(this,
                        "What do want to do with the Application?","Application",
                        JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                        possibleValues, possibleValues[0]);
                if(0==selectedValue) fl.tryLaunch(f);
                else if(1==selectedValue) openFile(Vfile.getVfile(f),null);
                return;
            }
            //open directories as a url
            openFile(Vfile.getVfile(f),null);
            log.fine("adding to history");
            return;
        }
        else{
            final FileRecognizer fileRecognizer = FileRecognizer.getInstance();
            if (fileRecognizer.isZipFile(f)) {
                log.fine("onEnter with zipfile="+f);
                Object[] possibleValues = { "Browse in CB", "Extract", "Launch","Cancel" };
                int selectedValue = JOptionPane.showOptionDialog(this,
                        "What do want to do with the zip file?","Zip file found..",
                        JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                        possibleValues, possibleValues[0]);
                if(0==selectedValue){
                    browseZip(f);
                    return;
                }
                else if(1==selectedValue){
                    Vfile cwd = getDirectory();
                    File dir = cwd==null?new File(System.getProperty("user.home")):cwd.getFile();
                    final String title = "Extract zip contents";
                    File destinationDir = new CreateFileDialogBuilder()
                            .withTitle(title)
                            .withMessage("Extract to:")
                            .withOkText("Extract")
                            .showDialogForFile(this,dir);
                    log.fine("destinationdir="+destinationDir);
                    if(destinationDir !=null){
                        if(!destinationDir.exists()){
                            destinationDir.mkdirs();
                        }
                        if (destinationDir.isDirectory()) {
                            try {
                                blockInputWithCancel("Extracting:\n "+f);
                                try {
                                    TarStractor ts = (TarStractor) Spin.off(new TarStractor(){

                                        public List<File> extractTar(File tar, File to) {return null;}
                                        public List<File> extractZip(File f, File destinationDir) throws IOException {
                                            return extractZipfile(f,destinationDir);
                                        }
                                    });
                                    List<File> toplevels=ts.extractZip(f, destinationDir);
                                    log.finer("extracted toplevels="+toplevels);
//                                    extractZipfile(f, destinationDir);
                                    fem.fireAdded(toplevels.toArray(new File[toplevels.size()]));
                                    Vfile cdir = getDirectory();
                                    File currDir =cdir==null?null:cdir.getFile();
                                    if(destinationDir.equals(currDir)){
                                        log.finer("selecting files="+toplevels);
                                        selectFiles(toplevels);
                                    }
                                } finally {
                                    unblockInput();
                                }
                            } catch (Exception e) {
                                log.log(Level.WARNING,"couldn't extract zip",e);
                                JOptionPane.showMessageDialog(DirectoryPanel.this
                                        ,"Couldn't extract "+f
                                        ,title
                                        ,JOptionPane.ERROR_MESSAGE);
                            }
                        } else {
                            JOptionPane.showMessageDialog(DirectoryPanel.this
                                    ,"Destination must be a directory!\n ("+destinationDir +" is a file)"
                                    ,title
                                    ,JOptionPane.ERROR_MESSAGE);
                        }
                    }
                    return;
                }
                else if(2==selectedValue){
                    fl.tryLaunch(f);
                    return;
                }
                else return;
            }
            else if(fileRecognizer.isTarFile(f, true)){
                onTar(f);
                return;
            }
        }    //fall thru
        fl.tryLaunch(f);
    }

     private void browseZip(File f) {
         final ZipPanel zp = new ZipPanel(commanderFrame,cf);
         commanderFrame.addTabule(this,zp, JSplitPane.VERTICAL_SPLIT);
         try {
             zp.openFile(f,null);
             zp.getFocus();
         } catch (CannotOpenZipException e) {
             log.log(Level.WARNING, "exception", e);
             JOptionPane.showMessageDialog(this,"Cannot open zip file\n  "+e.getMessage(),"Browse zip file",JOptionPane.ERROR_MESSAGE);
         }
     }

     /**
      *
      * @param zipFile
      * @param destinationDir
      * @return list of toplevel files/dirs extracted
      * @throws IOException
      */
     private List<File> extractZipfile(File zipFile, File destinationDir) throws IOException {
         ZipFile zf = new ZipFile(zipFile,ZipFile.OPEN_READ);
         try {
             Enumeration<? extends ZipEntry> entries = zf.entries();
             int written=0;
             Set<File> toplevels=new HashSet<File>();
             while(entries.hasMoreElements()){
                 ZipEntry ze = entries.nextElement();
                 String entryName = ze.getName();
                 File newFile = new File(destinationDir,entryName);
                 if(ze.isDirectory()){
                     boolean b=newFile.mkdirs();
                     log.finer("mkdir="+newFile.getAbsolutePath()+" success="+b);
                 }
                 else{
                     InputStream is = zf.getInputStream(ze);
                     try {
                         newFile.getParentFile().mkdirs();
                         OutputStream os = new FileOutputStream(newFile);
                         try {
                             byte[] buf = new byte[1024];
                             int i,bytes=0;
                             while((i=is.read(buf))!=-1){
                                 os.write(buf,0,i);
                                 bytes+=i;
                             }
                             log.finer("wrote "+bytes+" bytes to file="+newFile.getAbsolutePath());
                         } finally {
                             os.close();
                         }
                     } finally {
                         is.close();
                     }
                     written++;
                 }
                 //add the ancestor of te newfile that is a child of destdir to toplevel files
                 File f =newFile,parent;
                 while(!destinationDir.equals(parent=f.getParentFile())){
                     f =parent;
                 }
                 toplevels.add(f);
                 updateBlockMessage("wrote "+written+" entries");
             }
             return new ArrayList<File>(toplevels);
         } finally {
             zf.close();
         }
     }

     public void onEnter() {
         java.util.List<File> l =getSelectedFiles(true,true,true);// ct.getSelectedNodes(true,true);
         if(l.isEmpty()){
             if (log.isLoggable(Level.FINEST)) {
                 log.log(Level.FINEST,"enter on zero files -where did this come from?",new Throwable());
             }
             return;
         }
         log.fine("onEnter, "+l.size()+" files selected");
         File f=l.get(0);
         if(l.size()==1){
             onSingleFileClicked(f);
         }
         else fl.tryLaunch(l);
     }

    private void onTar(final File f) {
        Object[] possibleValues = { "Extract","List", "Launch","Cancel" };
        int selectedValue = JOptionPane.showOptionDialog(this,
                "What do want to do with the tar file?","TAr file found..",
                JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                possibleValues, possibleValues[0]);
        if(0==selectedValue){
            try {
                blockInputWithCancel("Extracting "+f);
                try {
                    TarStractor ts = (TarStractor) Spin.off(new TarStractor(){
                        public List<File> extractTar(File tar, File to) throws IOException {
                            return extractTarChive(f,getCurrentDirectory());
                        }

                        public List<File> extractZip(File f, File destinationDir) {
                            return Collections.emptyList();
                        }
                    });
                    List<File> topLevelFiles=ts.extractTar(f,getCurrentDirectory());
                    //don't do anything different if cancelled, since if we don't undo the operation..
                    //if(wasCancelled()) return;
                    fem.fireAdded(topLevelFiles.toArray(new File[topLevelFiles.size()]));
                    if (!topLevelFiles.isEmpty()) {
                        selectFiles(topLevelFiles);
                    }
                } finally {
                    unblockInput();
                }
            } catch (Exception e) {
                log.log(Level.WARNING, "exception", e);
                JOptionPane.showMessageDialog(this,"Couldn't extract TAr file\n  "+e.getMessage(),(String)possibleValues[0],JOptionPane.ERROR_MESSAGE);
            }
        }
        else if(1==selectedValue){
            listTarAction.listTar(f);
        }
        else if(2==selectedValue){
            fl.tryLaunch(f);
        }
    }
    private void selectFile(File f){
        selectFiles(f);
    }
    private void selectFile(Vfile f){
        selectFiles(f);
    }
    private void selectFiles(File... files){
        selectFiles(Arrays.asList(files));
    }
     private void selectFiles(Vfile... files){
        setSelectedFiles(Arrays.asList(files));
    }
    private void setSelectedFilesList(java.util.List<File> d) {
        setSelectedFiles(Vfile.getVfiles(d));
    }
    private void setSelectedFiles(java.util.List<Vfile> d) {
        log.finer("select file="+d);
        if(d==null) return;
        selectFiles(Vfile.getFiles(d));
    }
    public void selectFiles(List<File> files) {
        log.finest("select files="+files);
        List<Vfile> l = Vfile.getVfiles(files);
        List<Integer> ll = new ArrayList<Integer>(l.size());
        FileTableModel fsm = getModel();
        if(fsm==null) return;
        for(Vfile f:l){
            final int row = fsm.indexOf(f);
            if(row>-1){
                log.finest("mrow="+row);
                ll.add(table.convertRowIndexToView(row));
            }
        }
        log.finest("table.select vrows="+ll);
        if(!ll.isEmpty()) table.selectRows(ll,ll.get(ll.size()-1));
    }

    private FileTableModel getModel() {
        return tableModel;
    }


    /**
     * extract the tar archive denoted by tarFile, (could be zipped) and return the root dir of its extraction (or the first dir extracted if no root)
     * @param tarFile
     * @return list of all the toplevel files
     */
    private List<File> extractTarChive(File tarFile, File toDir) throws IOException {
        log.fine("extracting tarfile from "+tarFile+" to "+toDir);
        final FileUtils fileUtils = FileUtils.getInstance();
        InputStream in = fileUtils.getTarInputStream(tarFile);
        try {

            TaskTimer tt= new TaskTimer();
            TarArchive ta = new TarArchive(in);
            ta.setKeepOldFiles(false);
            List<File> allFiles = ta.extractContents(toDir,this, TarArchive.ListMode.topLevelOnly);
            ta.closeArchive();
            log.fine("extracted "+allFiles.size()+" files took: "+tt.stopString()+" "+(tarFile.length()/1000)+"k");
            if(log.isLoggable(Level.FINER)){
                log.finer("extracted: "+allFiles);
            }
            return  allFiles;
        } finally {
            in.close();
        }
    }
    final ListTarAction listTarAction = new ListTarAction(this,this,this);
    private void setupPopup() {
        final JPopupMenu popup = new JPopupMenu();
        popup.add(ActionDispatcher.getInstance().createCommanderMenu(this));
        //jumpDownAction
        popup.addSeparator();
        popup.add(jumpDownAction);
        popup.add(jumpUpAction);
        popup.addSeparator();
        popup.add(findAction)   ;

        final Action touchAction = ActionBuilder.getInstance().init(new AbstractAction("Touch"){
            public void actionPerformed(ActionEvent e) {
                List<File> sel = getSelectedFiles(true,true,false);
                if(!sel.isEmpty()){
                    for (File f:sel) {
                        f.setLastModified(System.currentTimeMillis());
                    }
                    fem.fireEvent(null,null,sel);
                }
                else JOptionPane.showMessageDialog(DirectoryPanel.this,"Please select a file");
            }
        }).tooltip("Set the file(s)' last modified times to the current system time").get();
        popup.add(touchAction);
        popup.add(replaceSpacesAction);
        popup.add(new CreateEmptyFileAction(this));
        popup.add(pasteToNewFileAction);
        popup.add(new CreateZipAction(this, commanduleContext,this,this));
        final Action openZip = ActionBuilder.getInstance().init(new AbstractAction("Browse zip file"){
            public void actionPerformed(ActionEvent e) {
                List<File> sel = getSelectedFiles(true,true,false);
                if(!sel.isEmpty()){
                    File f1 = sel.get(0);
                    browseZip(f1);
                }
                else JOptionPane.showMessageDialog(DirectoryPanel.this,"Please select a file");
            }
        })      .icon(ImageProvider.getInstance().getIcon("icons/16/ark.png"))
                .tooltip("Browse the first selected file (treat it like a .zip file)").get();
        popup.add(openZip);
        final CreateTarAction mkTar = new CreateTarAction(this,commanduleContext,this,this);
        popup.add(mkTar);
        popup.add(listTarAction);

        popup.addSeparator();

        popup.add(copyNamesAction);
        popup.add(copyPathsAction);
        popup.add(copyContentsAction);
        popup.addSeparator();

        final CreateDigestAction md5 = new CreateDigestAction(this,"MD5");
        popup.add(md5);
        final CreateDigestAction sha = new CreateDigestAction(this,"SHA");
        popup.add(sha);


        final MouseAdapter ml = new MouseAdapter(){
            private void mouse(MouseEvent e) {
                if(e.isPopupTrigger()){
                    boolean isSelection =table.isSelectedFiles(true,true);
//                    harvest.setEnabled(isSelection);
                    listTarAction.setEnabled(isSelection);
                    mkTar.setEnabled(isSelection);
                    md5.setEnabled(isSelection);
                    sha.setEnabled(isSelection);
                    copyNamesAction.setEnabled(isSelection);
                    copyPathsAction.setEnabled(isSelection);
                    pasteToNewFileAction.setEnabledState();
                    popup.show(table,e.getX(),e.getY());
                }
            }
            public void mousePressed(MouseEvent e) { mouse(e); }
            public void mouseReleased(MouseEvent e) { mouse(e); }

            public void mouseClicked(MouseEvent e) {
                if(e.getClickCount()==2){
                    List<File> l = getSelectedFiles(true,false,false);
                    if(!l.isEmpty()){
                        onSingleFileClicked(l.get(0));
                    }
                }
            }
        };
        table.addMouseListener(ml);
        rowHeaderTable.addMouseListener(ml);
    }

    private static final DataFlavor[] RECEIVE_FLAVORS={TreeStream.DATAFLAVOR, DataFlavor.javaFileListFlavor, DataFlavor.getTextPlainUnicodeFlavor()};
    public FileListTransferable getTransferable(DragGestureEvent dge){
        //1. pass drag event
        //2. get row corresponding to mouse-x
        //3. if ro is selected/marked, drag all sel/marked files
        //4. else just drag that file
        if (dge==null) {
            final List<File> sel = getSelectedFiles(true,true,false);//getSelectedVfiles(true,true,false);
            if(sel.isEmpty()){
                log.fine("ignore drag as sel.length==0");
                return null;
            }
            return new FileListTransferable(sel,this);
        } else {
            Point p = dge.getDragOrigin();
            int ro=table.rowAtPoint(p);
            if(ro>=0){
                final int modelRow = table.convertRowIndexToModel(ro);
                if(table.isRowSelected(ro) || getModel().isMarked(modelRow)){
                    return getTransferable(null);  //return all selected/marked rows (which will incldue this one)
                }
                else{ //drag just this row
                    File f = getModel().getElement(modelRow).getFile();
                    List<File> l = Collections.singletonList(f);
                    return new FileListTransferable(l,this);
                }
            }
            else{
                log.fine("returning null transferable as x coord is not a row");
                return null;
            }
        }
    }
    public void drop(DropTargetDropEvent evt, Transferable transferable) {
        windowToFront();
        if (JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(this,"Save dragged data here?","Drop data",JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE)) {
            try {
                save(evt.getTransferable(), true);
            } catch (NotSupportedException e) {
                log.log(Level.WARNING, "exception", e);
                JOptionPane.showMessageDialog(DirectoryPanel.this,"This window does not allow saving!","Save",JOptionPane.ERROR_MESSAGE);
            } catch (CannotSaveException e) {
                log.log(Level.WARNING, "exception", e);
                JOptionPane.showMessageDialog(DirectoryPanel.this,"Couldn't save:\n "+e.getMessage(),"Save",JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public File getCurrentDirectory(){
        Vfile vf = getDirectory();
        if(vf==null) return null;
        return vf.getFile();
    }


    public CopyOperation.CopyMode promptForCopyMode(int overwrites, Vfile destDir,String title){
        final CopyOperation.CopyMode[] options = CopyOperation.CopyMode.values();
        int i=JOptionPane.showOptionDialog(this,//overwrites+
                //"vfile(s) already exist in the destination directory:" +
                //"\n" +destDir.getName()+"\n"
                "What do you want to do when a file already exists?"
                ,title
                ,JOptionPane.OK_CANCEL_OPTION
                ,JOptionPane.QUESTION_MESSAGE
                ,null,options,CopyOperation.CopyMode.defaultCopyMode());
        if(i>-1){
            return options[i];
        }
        else return null;
    }

    private void performMove(final MoveOperation mo, final CopyOperation.CopyMode cm) {
        blockInput("Move "+mo.getSchedule().size()+" file(s)", 100);
        mo.setProgressUpdater(this);
        //use swingworker as multiple invocations of Spin.off can cause deadlocks
        new SwingWorker3(){
            public Object construct() {
                mo.move(cm);//mo.move(copySchedule, copyMode);
                return null;  //To change body of implemented methods use File | Settings | File Templates.
            }
            public void finished() {
                unblockInput();
                fem.fireEvent(mo.getDeletedFiles(),mo.getCreatedFiles(),mo.getOverwrittenFiles());

                if(mo.wereFailures()){
                    java.util.List<CopyOperation.CopyFailure> failures = mo.getFailures();
                    String lastMess=failures.get(failures.size()-1).getReason();
                    final String message="couldn't move "+
                            (failures.size()>1
                            ?(failures.size()+" files")
                            :failures.get(0).getCopyDescriptor().getFrom().getName()+"\n" +
                            "["+lastMess+"]");
                    JOptionPane.showMessageDialog(DirectoryPanel.this,message,
                            "Move",
                            JOptionPane.WARNING_MESSAGE
                    );
                    getFocus();
                    java.util.List<File> l= new ArrayList<File>(failures.size());
                    for(CopyOperation.CopyFailure f:failures){
                        l.add(f.getCopyDescriptor().getFrom());
                    }
                    setSelectedFilesList(l);
                }
                else getFocus();
            }
        }.start();
    }


    private void setFile(final Vfile dir,final Vfile selectFile) {

        blockInput("Loading directory: "+dir.getName(), 100);
        getModel().setDirAsync(dir, new Runnable() {
            public void run() {
                unblockInput();
                if(dir!=null){
                    final String path = dir.getAbsolutePath();
                    locationField.setText(path);
                }
                else{
                    locationField.setText("");
                }
                if(selectFile!=null) selectFile(selectFile);
                else if(table.getRowCount()>0){
                    table.selectRow(0);
                }
            }
        }
        ,new ExceptionHandler(){

            public void handleException(Exception e) {

                if (!(e instanceof FileTableModel.CannotAccessDirectoryException)) {
                    JOptionPane.showMessageDialog(DirectoryPanel.this,"Cannot open file:\n" +
                            "  "+e,"Open Directory",JOptionPane.ERROR_MESSAGE);
                } else {
                    FileTableModel.CannotAccessDirectoryException ee=(FileTableModel.CannotAccessDirectoryException) e;
                     JOptionPane.showMessageDialog(DirectoryPanel.this,"Cannot access directory:\n" +
                            "  "+ee.getDir(),"Open Directory",JOptionPane.ERROR_MESSAGE);
                }
                unblockInput();
            }
        });
    }


    public Vfile getDirectory() {
        return getModel().getDirectory();
    }


    /**
     *
     * @return first selected file that is a directory, or null if none
     */
    public File getFirstSelectedDirectory(){
       List<File> fs=getSelectedFiles(true,false,false);
        if(!fs.isEmpty()){
            for(File f:fs){
                if(f.isDirectory()) return f;
            }
        }
        return null;
    }
    public List<File> getSelectedFiles(boolean includeSelected, boolean includeMarkedFiles, boolean includeParents){
        return Vfile.getFiles(getSelectedVfiles(includeSelected,includeMarkedFiles,includeParents));
    }
    List<Vfile> getSelectedVfiles(boolean includeSelected, boolean includeMarkedFiles, boolean includeParents){
        return csu.getSelectedElements(includeSelected,includeMarkedFiles);
    }
    public int getSelectedRowCount(boolean includeSelected, boolean includeMarkedFiles){
        return csu.getSelectedRowCount(includeSelected,includeMarkedFiles);
    }


    private String getString(List<Vfile> l) {
        return l.size()!=1?l.size()+" files":l.get(0).getName();
    }
    private String getName(List<File> l) {
        return l.size()!=1?l.size()+" files":l.get(0).getName();
    }

    public void copy() {
//        final java.util.List<Vfile> l=getSelectedVfiles(true, true, false);
//        if(l.size()>0){
//            final Vfile targetDir=getTargetDir();
//            if(targetDir !=null){
//                String sugTarg=targetDir.getAbsolutePath();
//                String s = l.size()!=1?l.size()+" files":l.get(0).getName();//getString(l);
//                String dest=(String)JOptionPane.showInputDialog(DirectoryPanel.this,
//                        "Copy "+s+" to:"
//                        ,"Copy",JOptionPane.QUESTION_MESSAGE,null
//                        ,null,sugTarg);
//                if(dest!=null){
//                    final Vfile d=new Vfile(dest);
//                    updateStatusWithSaveResult(copyOrMove(l, d, "Copy"));
//                }
//            }
//        } else{
            try {
                FileListTransferable t = getTransferable(null);
                if(t==null){
                    log.fine("ignoring copy as no transferable files");
                    return;
                }
                int fileCount = t.getFileCount();
                String s = fileCount !=1?fileCount+" files":"file";
                // prompt (and get overwrite question)
                Object[] possibleValues = { "Copy (skip)","Copy (overwrite)", "Cancel" };
                int selectedValue = JOptionPane.showOptionDialog(DirectoryPanel.this,
                        "Copy "+s+" to next panel?"
                        +"\n  (skip means any existing files will be left alone, overwrite means they will be replaced)"
                        ,"Copy"
                        ,JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                        possibleValues, possibleValues[0]);
                log.finer("selval="+selectedValue);
                if(selectedValue<0 || selectedValue>1){//user cancelled or something
                    return;
                }
                assert selectedValue==1 || selectedValue ==0;
                final boolean force=selectedValue==1;
                SaveTarget st = commanderFrame.getSaveTarget();
                if (st!=null) {
                    st.save(t, force);
                }
            } catch (NotSupportedException e) {
                JOptionPane.showMessageDialog(DirectoryPanel.this,"The opposite pane doesn't understand my data!","Copy",JOptionPane.ERROR_MESSAGE);
            } catch (CannotSaveException e) {
                log.log(Level.WARNING, "exception", e);
                JOptionPane.showMessageDialog(DirectoryPanel.this,"Couldn't save those data\n"+e.getMessage(),"Copy",JOptionPane.ERROR_MESSAGE);
            }
//        }
    }

     private void updateStatusWithSaveResult(SaveTreeStreamResult stsr) {
//         JOptionPane.showMessageDialog(this,"Failed to save "+stsr.getFailures().size()+" files","Save",JOptionPane.WARNING_MESSAGE);
         String statusText = stsr.getResultStatusText();
//    todo     if(stsr.wereFailures()){
//             Action detailAction= ActionBuilder.getInstance().init(new AbstractAction(){
//                 public void actionPerformed(ActionEvent e) {
////                     dialog.setVisible(true);
//                     log.warning("not impled");
//                 }
//             })
////                     .icon(ImageProvider.getInstance().getIcon(
////                             wasError?"icons/16/ledred.png"
////                     :"icons/16/ledgreen.png"
////                     ))
//                     .tooltip(
//                             "Show details of errors"
//                     )
//                     .name("Details..")
//                     .get();
//            commanderFrame.setFrameStatusText(statusText,detailAction);
//         }
//         else
    commanderFrame.setFrameStatusText(statusText);
         statusUpdater.setDisableFocusStatusUpdate(true);
     }

     /**
      *  copy or move these files to this dir; fires events and prompts user
      * @param files
      * @param destinationDir
      * @param operationTitle
      * @return result if some degree of success, null if nothing was doing
      */
     private SaveTreeStreamResult copyOrMove(List<Vfile> files, Vfile destinationDir, final String operationTitle) {
         if(!destinationDir.exists()){
             JOptionPane.showMessageDialog(DirectoryPanel.this
                     ,"Destination directory "+destinationDir +" doesn't exist!"
                     ,operationTitle
                     ,JOptionPane.ERROR_MESSAGE);
         }
         else if(!destinationDir.isDirectory()){
             JOptionPane.showMessageDialog(DirectoryPanel.this
                     ,"Destination "+destinationDir +" isn't a directory!\n" +
                     "please type a directory name"
                     ,operationTitle
                     ,JOptionPane.ERROR_MESSAGE);
         }
         else{
//             try {
             if(files.isEmpty()){
                 log.warning("savelist empty returning empty result and aborting");
                 return new SaveTreeStreamResult();

             }
             File[] filesArr = Vfile.getFilesArray(files);//new File[files.size()];

                 List<Node> roots = FileNode.getFileNodes(filesArr);
                 return saveTreeStream(destinationDir,new TreeStream(roots,this), true);
         }
         return null;
     }


     public void delete() {
         deleteAction.actionPerformed(null);
     }
     public void duplicate() {
         java.util.List<Vfile> l=getSelectedVfiles(true, true, false);
         if(l.size()>0){
             if(l.size()==1){
                 final Vfile sourceVfile = l.get(0);
                 final File sourceFile =sourceVfile.getFile();
                 String s = sourceVfile.getName();
                 String dest=(String)JOptionPane.showInputDialog(DirectoryPanel.this,
                         "Duplicate "+s+" to:"
                         ,"Duplicate",JOptionPane.QUESTION_MESSAGE,null
                         ,null,s+".1");
                 if(dest!=null){
                     final File destinationFile = new File(getCurrentDirectory(),dest);
                     if(!destinationFile.exists()){
                         try {
                             List<Node> roots =new ArrayList<Node>(1);
                             roots.add(new FileNode(sourceFile){
                                 public String getName() {
                                     return destinationFile.getName();
                                 }
                             });
                             updateStatusWithSaveResult(saveTreeStream(new TreeStream(roots,this), true));
                         } catch (CannotSaveException e) {
                             log.log(Level.WARNING,"exception",e);
                             JOptionPane.showMessageDialog(DirectoryPanel.this
                                 ,"Error saving:\n " + e.getMessage()
                                 ,"Duplicate"
                                 ,JOptionPane.ERROR_MESSAGE);
                         } catch (IOException e) {
                             log.log(Level.WARNING,"exception",e);
                             JOptionPane.showMessageDialog(DirectoryPanel.this
                                 ,"IO Error saving:\n " + e.getMessage()
                                 ,"Duplicate"
                                 ,JOptionPane.ERROR_MESSAGE);
                         }
                     }
                     else{
                         JOptionPane.showMessageDialog(DirectoryPanel.this
                                 ,"couldn't duplicate:\n" +
                                 destinationFile.getName()+" already exists!"
                                 ,"Duplicate"
                                 ,JOptionPane.ERROR_MESSAGE);
                     }
                 }
             } else{
                 JOptionPane.showMessageDialog(DirectoryPanel.this
                         ,"you can only duplicate 1 file at a time\n" +
                         "please select a single file"
                         ,"Duplicate"
                         ,JOptionPane.ERROR_MESSAGE);
             }
//             getFocus();
         }
     }

    public void shell() {
        log.fine("runscript");
        CommandManager.getInstance().launchShellCommand(commanderFrame,commanduleContext);
    }

    public void usercommand() {
        log.fine("usercommand");
        CommandManager.getInstance().showSelector(commanderFrame,commanduleContext);
    }

    FileLauncher fl = new FileLauncher();
    public void launch() {
        fl.launchWithPrompt(this, getSelectedFiles(true, true, true));
        getFocus();
    }

    public void mkdir() {
        final Vfile directory = getDirectory();
        if(directory==null) return;
        String dest=(String)JOptionPane.showInputDialog(DirectoryPanel.this,
                "Create Directory (parent is "+directory.getName()+"):"
                ,"Create Directory:",JOptionPane.QUESTION_MESSAGE,null
                ,null,"");
        if(dest!=null && dest.trim().length()>0){
            Vfile d=new Vfile(directory,dest);
            if(!d.exists() ){
                if (d.mkdirs()) {
                    while(true){
                        fem.fireEvent(new FilesChangedEvent(null,Collections.singletonList(d.getFile()),null));
                        Vfile parent=d.getParentFile();
                        if(parent.equals(directory)) break;
                        else d=parent;
                    }
                    //select the dir we just created
                    selectFile(d);
                } else {
                    JOptionPane.showMessageDialog(DirectoryPanel.this
                            ,"couldn't create directory"
                            ,"Create Directory",JOptionPane.ERROR_MESSAGE);
                }
            } else{
                JOptionPane.showMessageDialog(DirectoryPanel.this
                        ,d+" already exists"
                        ,"Create Directory"
                        ,JOptionPane.WARNING_MESSAGE);
                //select the existing dir
                selectFile(d);
            }
            getFocus();
        }
    }
    private Vfile getTargetDir(){
        Commandule c = commanderFrame.getNextCommandule();
        if(c instanceof DirectoryPanel){
            DirectoryPanel dp = (DirectoryPanel) c;
            return dp.getDirectory();
        }
        return null;
    }
//     public void move() {
//         try {
//             FileListTransferable t = getTransferable(null);
//             if(t==null){
//                 log.fine("ignoring move as no transferable files");
//                 return;
//             }
//             int fileCount = t.getFileCount();
//             String s = fileCount !=1?fileCount+" files":"file";
//             // prompt (and get overwrite question)
//             Object[] possibleValues = { "Copy (skip)","Copy (overwrite)", "Cancel" };
//             int selectedValue = JOptionPane.showOptionDialog(DirectoryPanel.this,
//                     "Copy "+s+" to next panel?"
//                     +"\n  (skip means any existing files will be left alone, overwrite means they will be replaced)"
//                     ,"Copy"
//                     ,JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
//                     possibleValues, possibleValues[0]);
//             log.finer("selval="+selectedValue);
//             if(selectedValue<0 || selectedValue>1){//user cancelled or something
//                 return;
//             }
//             assert selectedValue==1 || selectedValue ==0;
//             final boolean force=selectedValue==1;
//             SaveTarget st = commanderFrame.getSaveTarget();
//             if (st!=null) {
//                 st.save(t, force);
//             }
//         } catch (NotSupportedException e) {
//             JOptionPane.showMessageDialog(DirectoryPanel.this,"The opposite pane doesn't understand my data!","Copy",JOptionPane.ERROR_MESSAGE);
//         } catch (CannotSaveException e) {
//             log.log(Level.WARNING, "exception", e);
//             JOptionPane.showMessageDialog(DirectoryPanel.this,"Couldn't save those data\n"+e.getMessage(),"Copy",JOptionPane.ERROR_MESSAGE);
//         }
//     }

  /*test*/
  public void moveOld() {
      java.util.List<Vfile> l=getSelectedVfiles(true, true, false);
      if(!l.isEmpty()){
          Commandule c = commanderFrame.getNextCommandule();
          if(c instanceof DirectoryPanel){
              DirectoryPanel dp = (DirectoryPanel) c;
              Vfile destDir = dp.getDirectory();
              String s = getString(l);
              String dest=(String)JOptionPane.showInputDialog(DirectoryPanel.this,
                      "move "+s+" to:"
                      ,"Move",JOptionPane.QUESTION_MESSAGE,null
                      ,null,destDir.getAbsolutePath());
              if(dest!=null){
//   todo: rewrite move a la copy, is trickier than copy as it may be that some files can't be overwritten or can't be deleted
                  Vfile d=new Vfile(dest);
                  if(d.isDirectory()){
                      if(d.exists()){
                          final MoveOperation co = new MoveOperation(d.getFile(),Vfile.getFiles(l),DirectoryPanel.this);
                          final CopyOperation.CopyMode cm = co.getCopyModeWithPrompt("Move");
                          if(cm==null){
                              log.fine("abandoning move as cm==null");
                              return;
                          }
                          performMove(co, cm);
                      }
                      else{
                          //target dir deosn't exist
                          JOptionPane.showMessageDialog(DirectoryPanel.this
                                  ,"destination directory "+d+" doesn't exist!"
                                  ,"Move"
                                  ,JOptionPane.ERROR_MESSAGE);
                      }
                  } else{//input is a vfile -that's OK if only 1 sel, otherwise not
                      if(l.size()==1){
                          //check the vfile doesn't exist
                          if(!d.exists()){
                              final Vfile vfile = l.get(0);
                              if (vfile.renameTo(d)) {
                                  fem.fireEvent(new FilesChangedEvent(Collections.singletonList(vfile.getFile()),Collections.singletonList(d.getFile()),null));
                                  getFocus();
                                  if (d.getParentFile().equals(getDirectory())) {
                                      selectFile(d);
                                  }
                                  return;
                              }
                              else{
                                  JOptionPane.showMessageDialog(DirectoryPanel.this
                                          ,"Couldn't move "+vfile
                                          ,"Move"
                                          ,JOptionPane.ERROR_MESSAGE);
                              }
                          } else  JOptionPane.showMessageDialog(DirectoryPanel.this
                                  ,"destination file "+d+" already exists!"
                                  ,"Move"
                                  ,JOptionPane.ERROR_MESSAGE);
                      } else  JOptionPane.showMessageDialog(DirectoryPanel.this
                              ,"cannot rename more than 1 file to a new file name\n" +
                              "either type a new directory name, or make sure only 1 file is selected"
                              ,"Move"
                              ,JOptionPane.ERROR_MESSAGE);
                  }

              }
              getFocus();
          }
      }
  }

     public void move() {
         List<File> l=getSelectedFiles(true, true, false);
         if(!l.isEmpty()){
             Commandule c = commanderFrame.getNextCommandule();
             if(c instanceof DirectoryPanel){
                 DirectoryPanel dp = (DirectoryPanel) c;
                 File destDir = dp.getDirectory().getFile();
                 String s = getName(l);
                 String dest=(String)JOptionPane.showInputDialog(DirectoryPanel.this,
                         "move "+s+" to:"
                         ,"Move",JOptionPane.QUESTION_MESSAGE,null
                         ,null,destDir.getAbsolutePath());
                 if(dest!=null){
                     File d=new File(dest);
                     //the destination must be a directory
                     if(d.exists() && !d.isDirectory()){
                         JOptionPane.showMessageDialog(DirectoryPanel.this
                                     ,"Please select a destination directory"
                                     ,"Move"
                                     ,JOptionPane.WARNING_MESSAGE);
                         return;
                     }
                     final List<File> created=new ArrayList<File>(), changed=new ArrayList<File>(), failed=new ArrayList<File>();
                     for(File f:l){
                         File newFile = new File(d,f.getName());
                         boolean exists = newFile.exists();
                        if(FileUtils.getInstance().moveRecursive(f,newFile)){//moveToDirRecursive(f,d)){
                            if(exists) changed.add(f);
                            else created.add(f);
                        }
                        else failed.add(f);
                     }
                     fem.fireEvent(null,created,changed);
                     if(!failed.isEmpty()){
                         StringBuilder sb = new StringBuilder();

                         for(File f:failed) sb.append("  "+f.getName()+"\n");

                         JOptionPane.showMessageDialog(DirectoryPanel.this
                                 ,"Error moving "+failed.size()+" of the selected files:\n"
                                 + sb.toString()
                                 ,"Move"
                                 ,JOptionPane.ERROR_MESSAGE);
                         setSelectedFilesList(l);
                     }
                 }
                 getFocus();
             }
         }
     }

    public void rename() {
        renameAction.actionPerformed(null);
    }


    public void openFile(File f){
        openFile(new Vfile(f),null);
    }
    private void openFile(Vfile f, Vfile select) {
        if(f==null){
            log.warning("openfile f=null, ignoring");
            return;
        }
        if(f.exists() && f.isDirectory()){
            setFile(f,select);
        }
        else if(!f.exists()){
            log.log(Level.FINE,"can't open f",new Throwable());
            Object[] options = {"open first extant ancestor","cancel"};
            int i=JOptionPane.showOptionDialog(DirectoryPanel.this,
                    "can't open "+f.getAbsolutePath()+" as it doesn't exist"
                    ,"Open",
                    JOptionPane.OK_CANCEL_OPTION
                    ,JOptionPane.WARNING_MESSAGE
                    ,null,options,options[0]);


            if(i==0){
                log.fine("opening first extant ancestor");
                while(true){
                    f=f.getParentFile();
                    if(f==null) return;
                    else if(f.exists() && f.isDirectory()){
                        openFile(f, null);
                        return;
                    }
                }
            }
        }
        else{
            log.fine("ignoring open on file="+f);
            JOptionPane.showMessageDialog(DirectoryPanel.this,"Cannot open \n"+f.getAbsolutePath()+" as it is a file, not a directory","Open",JOptionPane.ERROR_MESSAGE);
        }
    }


    public void save(final Transferable t, boolean force) throws NotSupportedException, CannotSaveException {
        if(t.isDataFlavorSupported(TreeStream.DATAFLAVOR)){
            log.fine("TreeStream.DATAFLAVOR supported");
            try {
                TreeStream ts = (TreeStream) t.getTransferData(TreeStream.DATAFLAVOR);
                if(this==ts.getSource()){
                    log.fine("ignoring drop as I am the vendor of these data");
                    return;
                }
                SaveTreeStreamResult stsr = saveTreeStream(ts, force);
                //now select any created or updated files
                List<File> createdAndOverwritten = stsr.getCreatedAndOverwritten();
                //weed out any that are not parented here
                for (Iterator<File> iterator = createdAndOverwritten.iterator(); iterator.hasNext();) {
                    File file = iterator.next();
                    if(!file.getParentFile().equals(getDirectory().getFile())){
                        iterator.remove();
                    }
                }
                setSelectedFilesList(createdAndOverwritten);

                updateStatusWithSaveResult(stsr);
            } catch (UnsupportedFlavorException e) {
                log.log(Level.WARNING, "exception", e);
                throw new CannotSaveException("Cannot save this datatype here");
            } catch (IOException e) {
                log.log(Level.WARNING, "exception", e);
                throw new CannotSaveException(e.getMessage(),e);
            }
        }
        else if(t.isDataFlavorSupported(DataFlavor.javaFileListFlavor)){
            log.fine("javaFileListFlavor supported");
            try {
                Object data=t.getTransferData(DataFlavor.javaFileListFlavor);
                final java.util.List<File> vfiles = (java.util.List<File>)data;
                saveFiles(vfiles);
            } catch (UnsupportedFlavorException e) {
                log.log(Level.WARNING,"UnsupportedFlavorException",e);
            } catch (IOException e) {
                log.log(Level.WARNING,"IOException",e);
            }
        }
        else if(t.isDataFlavorSupported(DataFlavor.getTextPlainUnicodeFlavor())){
            final Vfile currentDirectory = getDirectory();
            if (currentDirectory!=null) {
                windowToFront();
                String input = JOptionPane.showInputDialog(DirectoryPanel.this,"Enter name for new file containing dropped text:"
                        ,"droptext.txt");
                if(input !=null){
                    try {
                        Reader r = DataFlavor.getTextPlainUnicodeFlavor().getReaderForText(t);
                        final File f = new File(currentDirectory.getFile(),input);
                        boolean newfile=true;
                        if(f.exists()){
                            Object[] possibleValues = { "Replace", "Cancel" };

                            if(0!=JOptionPane.showOptionDialog(DirectoryPanel.this,
                                    "Replace existing file?","Create new file",
                                    JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                                    possibleValues, possibleValues[0])){
                                log.fine("aborting drop as user doesn't want to replace existing file="+f);
                                return;
                            }
                            else newfile=false;
                        }
                        FileWriter fw = new FileWriter(f);
                        char[] buf = new char[1024];
                        int i;
                        while((i=r.read(buf,0,buf.length))!=-1){
                            fw.write(buf,0,i);
                        }
                        r.close();
                        fw.close();

                        if (newfile) {
                            fem.fireAdded(f);
                        } else {
                            fem.fireChanged(f);
                        }
                        selectFile(new Vfile(f));
                    } catch (UnsupportedFlavorException e) {
                        log.log(Level.WARNING, "exception", e);
                    } catch (IOException e) {
                        log.log(Level.WARNING, "exception", e);
                        JOptionPane.showMessageDialog(DirectoryPanel.this,"Couldn't create file:\n  "+e.getMessage(),"Create new file",JOptionPane.ERROR_MESSAGE);
                    }
                    getFocus();
                }
            }
        }
        else throw new NotSupportedException();
    }

    private void saveFiles(final List<File> vfiles) {
        log.finer("save "+vfiles.size()+" files");
        // weed out invalid files (including ancestors)
        final Vfile currDir = getDirectory();
        if (currDir!=null) {
            for (Iterator<File> iterator = vfiles.iterator(); iterator.hasNext();) {
                File f = iterator.next();
                if(currDir.getFile().equals(f.getParentFile())){
                    log.finer("weeding f="+f+" as parent is currdir");
                    iterator.remove();
                }
                else if(currDir.getFile().equals(f)){
                    log.finer("weeding f="+f+" as equals currdir");
                    iterator.remove();
                }
            }
            if(vfiles.size()<1){
                log.finer("ignoring drop as 0 files (after weeding)");
                getFocus();
                return;
            }
        }
        final CopyOperation co = new CopyOperation(getDirectory().getFile(),vfiles,DirectoryPanel.this);
        final CopyOperation.CopyMode cm = co.getCopyModeWithPrompt("Copy (drop)");
        if(cm==null){
            log.fine("abandoning drop-copy as cm==null");
            return;
        }
        blockInputWithCancel("Copy (drop) "+co.getSchedule().size()+" file(s)");
        co.setProgressUpdater(this);
        new SwingWorker3(){
            public Object construct() {
                try {
                    co.copy(cm,DirectoryPanel.this);
                } catch (Exception e) {
                    log.log(Level.WARNING,"exception ",e);
                }
                return null;
            }
            public void finished() {
                unblockInput();
                fem.fireEvent(new FilesChangedEvent(null,co.getCreatedFiles(),null));
                if(co.wereFailures()){
                    List<CopyOperation.CopyFailure> failures = co.getFailures();
                    String lastMess=failures.get(failures.size()-1).getReason();
                    final String message="couldn't copy "+
                            (failures.size()>1
                            ?(failures.size()+" files")
                            :failures.get(0).getCopyDescriptor().getFrom().getName()+"\n" +
                            "["+lastMess+"]");
                    JOptionPane.showMessageDialog(DirectoryPanel.this,message,
                            "copy (drop)",
                            JOptionPane.WARNING_MESSAGE
                    );
                }
                else{
                    //select copied files
                    setSelectedFilesList(vfiles);
                }
                getFocus();
            }
        }.start();
    }
    public SaveTreeStreamResult saveTreeStream(TreeStream ts, boolean force) throws CannotSaveException, IOException {
        Vfile currDir=getDirectory();
        if(currDir==null){
            throw new CannotSaveException("No current directory");
        }
        return saveTreeStream(currDir, ts, force);
    }

     /**
      *
      * @param targetDir target dir, this will be the parent of the new top-level streams
      * @param ts  tree of streams
      * @param force
      * @throws IOException
      */
     private SaveTreeStreamResult saveTreeStream(Vfile targetDir, TreeStream ts, boolean force)  {
         List<? extends Node> roots = ts.getRoots();
         blockInputWithCancel("Saving "+roots.size()+" files");
         try {
             TreeStreamSaver tss = new TreeStreamSaver() {
                 public SaveTreeStreamResult saveTs(List<? extends Node> roots, Vfile currDir, TreeStream ts, boolean force) /*throws IOException*/ {
                     return DirectoryPanel.this.saveTs(roots,currDir,ts, force);
                 }
             };
             tss = (TreeStreamSaver) Spin.off(tss);

             return tss.saveTs(roots, targetDir, ts, force);
         } finally {
             unblockInput();
         }
     }

     public interface TreeStreamSaver{
         SaveTreeStreamResult saveTs(List<? extends Node> roots, Vfile currDir, TreeStream ts, boolean force) /*throws IOException*/ ;
     }
     private SaveTreeStreamResult saveTs(List<? extends Node> roots, Vfile currDir, TreeStream ts, boolean force) /*throws IOException*/ {
         SaveTreeStreamResult result = new SaveTreeStreamResult();
         File firstSaved=null;
         try {
             for(Node n:roots){
                 File f=saveNode(currDir.getFile(),n,result, force);
                 if(firstSaved==null) firstSaved=f;
             }
         } finally {
             ts.close();
             final List<File> empty = Collections.emptyList();
             fem.fireEvent(empty,result.created,result.overwritten);
         }

         //select the first saved datum
         if(firstSaved!=null) selectFile(firstSaved);
         return result;
     }

     private File saveNode(File currDir, Node n, SaveTreeStreamResult stsr, boolean force) /*throws IOException*/ {
         if(wasCancelled()){
             return null;
         }
         File destinationFile = new File(currDir,n.getName());
         if(!force && destinationFile.exists()){
             log.fine("ignoring file="+destinationFile+" as !force and exists");
             return null;
         }
         if(n.hasInputStream()){
             try {
                 updateBlockMessage(getSaveUpdateMessage(stsr, destinationFile));
                 log.finer("save to file destinationFile="+destinationFile);
                 boolean existed=destinationFile.exists();
                 if(n instanceof FileNode){
                     log.finer("writing treestream file");
                     //if a FileNode we cheat a bit (we can save faster file->file than inputstream->file
                     FileNode fn = (FileNode)n;
                     File source = fn.getFile();
                     FileChannel srcChannel = new FileInputStream(source).getChannel();
                     try {
                         FileChannel dstChannel = new FileOutputStream(destinationFile).getChannel();
                         try {
                             dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
                         } finally {
                             dstChannel.close();
                         }
                     } finally {
                         srcChannel.close();
                     }
                 }
                 else {
                     InputStream is=n.getInputStream();
                     try {

                         OutputStream os = new BufferedOutputStream(new FileOutputStream(destinationFile));
                         try {
                             byte[] buf = new byte[1024];
                             int i;
                             while((i=is.read(buf))!=-1){
                                 os.write(buf,0,i);
                             }
                         }  finally {
                             try { os.close(); } catch (IOException e) { log.log(Level.WARNING,"exception",e); }
                         }


                     } finally {
                         if(is!=null){
                             try { is.close(); } catch (IOException e) { log.log(Level.WARNING,"exception",e); }
                         }
                     }

                 }
                 if(existed) stsr.overwritten.add(destinationFile);
                 else stsr.created.add(destinationFile);
                 return destinationFile;
             } catch (FileNotFoundException e) {
                 log.log(Level.WARNING,"exception",e);
                 stsr.addFailure(destinationFile,"Invalid file",e);
             }catch (IOException e) {
                 log.log(Level.WARNING,"exception",e);
                 stsr.addFailure(destinationFile,"IO Exception writing file",e);
             }

         } else{//treat it like a directory
//            File parent = new File(currDir,n.getName());
             if(!destinationFile.exists()){
                 if(!destinationFile.mkdirs()){
                     stsr.addFailure(destinationFile,"Couldn't create directory");
                     return null;
//                    throw new IOException("couldn't create directory="+destinationFile);
                 }
                 else{
                     log.finer("created dir="+destinationFile);
                     stsr.created.add(destinationFile);
                 }
             }
             else if(!destinationFile.isDirectory()){
                 stsr.addFailure(destinationFile,"File exists, but is not a directory");
                 return null;
//                throw new IOException("file="+destinationFile+" exists, but is not a directory");
             }
             List<Node> kids=n.getChildren();
             for(Node kid:kids){
                 saveNode(destinationFile,kid,stsr, true);
             }
             return destinationFile;
         }
         return null;
     }

     private String getSaveUpdateMessage(SaveTreeStreamResult stsr, File f) {
         String s = "Saved " + (stsr.created.size() + stsr.overwritten.size()) + " files";
         if(stsr.wereFailures()){
             s+="\n "+stsr.getFailures().size()+" failures\n ";
         }
//         s+=" (saving " + f.getName() + ")"; skip this as causes constant rejustification
         return s;
     }


     public Commandule createClone(){
         return new DirectoryPanel(commanderFrame,cf);
     }

    public static final String MAGIC="uk.co.hcsoftware.commander.treeplugin.file.DirectoryPanel";//";//DirectoryPanel.class.getName();
    public String saveState() throws CannotSaveStateException //save current state to a unicode string that can be used later to restore state
    {
        StringBuilder sb = new StringBuilder();
        sb.append(MAGIC);
            final Vfile vf=getDirectory();
            if(vf==null)  throw new CannotSaveStateException();
            sb.append(vf.getAbsolutePath());

        return sb.toString();
    }

    private static final String OLDMAGIC="DIRECTORY";
    public void restoreState(String state) throws CannotRestoreSavedStateException {
        final boolean newM = state.startsWith(MAGIC);
        final boolean oldM = state.startsWith(OLDMAGIC);
        if(newM || oldM){
            final String pathName = state.substring(newM?MAGIC.length():OLDMAGIC.length());
            File f  = new File(pathName);
            openFile(Vfile.getVfile(f),null);
        }
        else throw new CannotRestoreSavedStateException(state, "bad magic number");
    }





    public void windowActivated(WindowEvent e) {
        log.finest("windowActivated, telling model to check for changes");
        getModel().checkForChanges();
    }

    public void windowOpened(WindowEvent e)  {}
    public void windowClosing(WindowEvent e)  {}
    public void windowClosed(WindowEvent e)  {}
    public void windowIconified(WindowEvent e)  {}
    public void windowDeiconified(WindowEvent e)  {}
    public void windowDeactivated(WindowEvent e) {}


}
