package uk.co.hcsoftware.commander.gui.file;

import uk.co.hcsoftware.commander.AppProperties;
import uk.co.hcsoftware.commander.gui.SearchForm;
import uk.co.hcsoftware.commander.gui.search.FileSearchPanel;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.io.FileUtils;
import uk.co.hcsoftware.commander.io.ImageProvider;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;


/**
 * Title: HarvestDataAction
 * Copyright:    Copyright (c) 13-Feb-2006
 *
 * @author spurlinn
 */
public class FindAction extends AbstractAction {
    private static final Logger log = Logger.getLogger(FindAction.class.getName());
    private DirectoryPanel directoryPanel;

    public FindAction(DirectoryPanel directoryPanel) {
        super("Find..");
        this.directoryPanel = directoryPanel;

        ActionBuilder.getInstance().init(this)
                .keyStroke( KeyStroke.getKeyStroke(KeyEvent.VK_F,AppProperties.getInstance().getCtrlMask()))
                .command("find")
                .icon(ImageProvider.getInstance().getIcon("icons/16/find.png"));
    }

    public void actionPerformed(ActionEvent e) {
        List<Vfile> sel = directoryPanel.getSelectedVfiles(true,true,false);
        List<File> selectedFiles= Vfile.getFiles(sel);
        FileUtils.getInstance().weed(selectedFiles,true,false);
        final SearchForm sf = getSearchForm();
        final File cwd = directoryPanel.getCurrentDirectory();
        if(cwd==null){
            log.info("ignoring find action as CWD is null");
            return;
        }
        sf.setParameters(new SearchForm.SearchParameters(selectedFiles.isEmpty()?Collections.singletonList(cwd):selectedFiles));
//        sf.setSearchRoots();
        sf.setVisible(true);
    }


    private SearchForm getSearchForm() {
        return new SearchForm(directoryPanel.getCommanderFrame(),directoryPanel){
            public void onFindPressed(SearchParameters sp) {
                {
                    FileSearchPanel srp = new FileSearchPanel(directoryPanel.getCommanderFrame(),directoryPanel.getCommanderFocus());
                    srp.setParameters(sp);
                    directoryPanel.getCommanderFrame().addTabule(directoryPanel,srp,JSplitPane.VERTICAL_SPLIT);
                    srp.find();
                }
            }
        }       ;
    }
}
