package uk.co.hcsoftware.commander.gui.file;

import com.ice.tar.TarArchive;
import ns.gui.SwingWorker3;
import uk.co.hcsoftware.commander.gui.HtmlDialog;
import uk.co.hcsoftware.commander.gui.InputBlocker;
import uk.co.hcsoftware.commander.gui.ProgressDisplay;
import uk.co.hcsoftware.commander.io.FileRecognizer;
import uk.co.hcsoftware.commander.io.FileUtils;
import uk.co.hcsoftware.commander.io.ImageProvider;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.InputStream;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: HarvestDataAction
 * Copyright:    Copyright (c) 13-Feb-2006
 *
 * @author spurlinn
 */
public class ListTarAction extends AbstractAction {
    private static final Logger log = Logger.getLogger(ListTarAction.class.getName());
    private final Component host;
    private InputBlocker directoryPanel;
    private final FileSelector fs;
    private static final String title="List tar file";
    public ListTarAction(Component host, InputBlocker directoryPanel, FileSelector fs) {
        super(title,ImageProvider.getInstance().getIcon("icons/16/tar.png"));
        this.host = host;
        this.directoryPanel = directoryPanel;
        this.fs = fs;
    }


    public void actionPerformed(ActionEvent e) {
        final List<File> l = fs.getSelectedFiles(true,true,false);//directoryPanel.getSelectedFiles(true, true, false);
        if (l.size()==1) {
            final File tarFile = l.get(0);
            if(!FileRecognizer.getInstance().isTarFile(tarFile, true)){
                 JOptionPane.showMessageDialog(host,
                                "That doesn't look like a tar file to me.. (I like things that end .tar)",title
                                ,JOptionPane.ERROR_MESSAGE);
                        return;
            }
            listTar(tarFile);
        }
        else{
            JOptionPane.showMessageDialog(host,
                                "Please select a single tar file",title
                                ,JOptionPane.ERROR_MESSAGE);
                        return;
        }
    }

    public void listTar(final File tarFile) {
        directoryPanel.blockInputWithCancel("Listing\n"+tarFile+"..");
        new SwingWorker3(){
            int files=0;
            public Object construct() {
                try {
                    final StringBuffer sb = new StringBuffer();
                    sb.append("<html");
                    sb.append("<pre>");
                    InputStream is = FileUtils.getInstance().getTarInputStream(tarFile);
                    try {
                        TarArchive ta = new TarArchive(is);
                        ta.setTarProgressDisplay(new ProgressDisplay() {
                            public void progressMade(String msg) {
                                log.fine("mess="+msg);
                                sb.append(msg);
                                sb.append("\n");
                                directoryPanel.updateBlockMessage("Listing\n"+tarFile+"\n" +
                                        "found "+(++files));
                            }
                        });
                        ta.listContents(directoryPanel);
                    } finally {
                        is.close();
                    }
                    sb.append("</pre></html>");
                    return sb.toString();
                } catch (Exception e1) {
                    log.log(Level.WARNING,"e1",e1);
                    return e1;
                }
            }

            public void finished() {
                directoryPanel.unblockInput();
                if(directoryPanel.wasCancelled()) return;
                if(get()instanceof Exception){
                    JOptionPane.showMessageDialog(host,
                            "couldn't gather data:\n"+ ((Exception) get()).getMessage(),title
                            ,JOptionPane.ERROR_MESSAGE);
                    return;
                }
                Window f = SwingUtilities.getWindowAncestor(host);
                HtmlDialog td = new HtmlDialog((Frame) f, title, false);
                td.loadHtml((String) get());
                td.setSize(500,300);
                td.setLocationRelativeTo(host);
                td.setVisible(true);
            }
        }.start();
    }
}
