package uk.co.hcsoftware.commander.gui.file;

import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.io.ImageProvider;
import uk.co.hcsoftware.commander.AppProperties;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: PasteToNewFileAction
 * pastes the clipboard contents (if any) into a new file, prompts for the name
 * Copyright:    Copyright (c) 13-Feb-2006
 *
 * @author spurlinn
 */
public class PasteToNewFileAction extends AbstractAction {
    private static final Logger log = Logger.getLogger(PasteToNewFileAction.class.getName());
    private static final String title="Paste to new file";
    private DirectoryPanel directoryPanel;

    public PasteToNewFileAction(DirectoryPanel directoryPanel) {
        super(title, ImageProvider.getInstance().getIcon("icons/16/editpaste.png"));
        this.directoryPanel = directoryPanel;
        ActionBuilder.getInstance().init(this)
                .tooltip("Paste the current contents of the clipboard into a new file")
                .command("paste")
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_V,AppProperties.getInstance().getCtrlMask()))
                ;
    }
    public void actionPerformed(ActionEvent e) {
        Vfile vf = directoryPanel.getDirectory();
        if (vf!=null) {
            Flavors flavor = getPreferredAvailableFlavor();
            if(flavor==null){
                log.info("nothing pasteable on clipboard!");
                return;
            }
            Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
            try {
                Object data=  clipboard.getData(flavor.getFlavor());
                File parentDir=vf.getFile();

                String dest=(String)JOptionPane.showInputDialog(directoryPanel,
                        title
                        ,"Enter Name for file:",JOptionPane.QUESTION_MESSAGE,null
                        ,null,flavor.getSuffix());

                if(dest!=null && dest.trim().length()>0){

                    File newFile = new File(parentDir,dest);
                    if(newFile.exists()){
                        JOptionPane.showMessageDialog(directoryPanel,"That name is already in use!",title,JOptionPane.ERROR_MESSAGE);
                        return;
                    }

                    if(newFile.createNewFile()){
                        //paste in clipboard text
                        try {
                            if(flavor==Flavors.String){
                                String s = (String) data;
                                BufferedWriter bw = new BufferedWriter(new FileWriter(newFile));
                                try {
                                    bw.write(s);
                                } finally {
                                    bw.close();
                                }
                            }
//                            else if(flavor==Flavors.Image){
//                                Image s = (Image) data;
//                                log.fine("writing image to "+newFile);
//                                ImageIO.write(getBufferedImage(s),"jpg",newFile);
//                            }
                        } finally {
                            directoryPanel.fem.fireAdded(newFile);
                            directoryPanel.selectFiles(Collections.singletonList(newFile));
                        }
                    }
                    else{
                        JOptionPane.showMessageDialog(directoryPanel,"Couldn't create new file"
                                ,title,JOptionPane.ERROR_MESSAGE);
                    }

                }
                else log.fine("ignoring "+title+" as null name entered");
            } catch (IOException e1) {
                log.log(Level.WARNING, "exception", e1);
                JOptionPane.showMessageDialog(directoryPanel,"Couldn't create new file\n" +
                        "["+e1.getMessage()+"]",title,JOptionPane.ERROR_MESSAGE);
            } catch (UnsupportedFlavorException e1) {
                log.log(Level.WARNING, "exception", e1);
                JOptionPane.showMessageDialog(directoryPanel,"Couldn't get data from clipboard\n" +
                        "["+e1.getMessage()+"]",title,JOptionPane.ERROR_MESSAGE);
            }
        }
        else log.fine("ignoring "+title+" as parentdir is null");
    }

    public void setEnabledState() {
        setEnabled(getPreferredAvailableFlavor()!=null);
    }
    enum Flavors{   //did try and support pasting images into files, but too much of a pain converting them to buffered images, waiting for them to load etc..
        //also not sure if it will be reliable, so just paste text for now..
        String("",DataFlavor.stringFlavor);//,Image(".jpg",DataFlavor.imageFlavor);
        String suffix;
        DataFlavor flavor;

        Flavors(String suffix, DataFlavor flavor) {
            this.suffix = suffix;
            this.flavor = flavor;
        }

        public String getSuffix() {
            return suffix;
        }

        public DataFlavor getFlavor() {
            return flavor;
        }
    }
//check if there's anything pasteable on the clipboard (NB: if this every causes a visible pause, just always enable it; don't like popups athat take ages to appear..)
    private Flavors getPreferredAvailableFlavor() {
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        log.fine("dfs="+Arrays.asList(clipboard.getAvailableDataFlavors()));
        for(Flavors df:Flavors.values()){
            if(clipboard.isDataFlavorAvailable(df.getFlavor())){
                return df;
            }
        }
        return null;
    }

    private BufferedImage getBufferedImage(Image image) {
        if(image instanceof BufferedImage) return (BufferedImage) image;
        log.fine("creating buffered image");
        BufferedImage bufferedImageOut = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_ARGB);
        Graphics g = bufferedImageOut.getGraphics();
        MediaTracker mt = new MediaTracker(directoryPanel);
        mt.addImage(image,0);
        mt.addImage(bufferedImageOut,1);
        g.drawImage(image, 0, 0, null);
        try {
            mt.waitForID(0);
            mt.waitForID(1);
        } catch (InterruptedException e) {
            log.log(Level.WARNING, "exception", e);
        }
        return bufferedImageOut;
    }

}
