package uk.co.hcsoftware.commander.gui.file;

import ns.TaskTimer;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;

/**
 */
public final class SaveTreeStreamResult {
    TaskTimer tt = new TaskTimer();
    public String getResultStatusText() {
        int savedCount = created.size() + overwritten.size();
        StringBuilder sb = new StringBuilder();
        sb.append("took ");
        sb.append(tt.stopString());
        sb.append(" to save ");
        sb.append(savedCount);
        sb.append(" file");
        if(savedCount>1){
            sb.append("s");
        }
        if (overwritten.size()>0) {
            sb.append(" (");
            sb.append(overwritten.size());
            sb.append(" overwrites)");
        }
        if(wereFailures()){
            sb.append(" ");
            sb.append(failed.size());
            sb.append(" failures");
        }
        return sb.toString();
    }
    public String toString() {
        return "created="+created.size()+" overwritten="+overwritten.size()
                +" failed="+failed.size();
    }

    public List<File> getCreatedAndOverwritten() {
        List<File> l = new ArrayList<File>(created.size()+overwritten.size());
        l.addAll(created);
        l.addAll(overwritten);
        return l;
    }

    private static final class Failure{
        private String reason;
        private File failedFile;
        private Exception cause;
        public Failure(String reason, File failedFile, Exception cause) {
            this.reason = reason;
            this.failedFile = failedFile;
            this.cause = cause;
        }

        public Failure(String reason, File failedFile) {
            this(reason,failedFile,null);
        }

        public String getReason() {
            return reason;
        }

        public File getFailedFile() {
            return failedFile;
        }

        /**
         *
         * @return may be null
         */
        public Exception getCause() {
            return cause;
        }
    }


    public List<File> created=new ArrayList<File>(),overwritten = new ArrayList<File>();
   public List<Failure> failed=new ArrayList<Failure>();

    public void addFailure(File f, String s, IOException e) {
        failed.add(new Failure(s,f,e));
    }

    public void addFailure(File f, String s) {
        failed.add(new Failure(s,f));
    }

    public boolean wereFailures() {
        return !failed.isEmpty();
    }

    public List<Failure> getFailures() {
        return failed;
    }
}
