package uk.co.hcsoftware.commander.gui.search;


import ns.gui.SwingWorker3;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import uk.co.hcsoftware.commander.*;
import uk.co.hcsoftware.commander.gui.*;
import uk.co.hcsoftware.commander.gui.ctable.*;
import uk.co.hcsoftware.commander.gui.file.*;
import uk.co.hcsoftware.commander.gui.SearchForm;
import uk.co.hcsoftware.commander.gui.usercommands.CommandManager;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.gui.util.ActionDispatcher;
import uk.co.hcsoftware.commander.gui.zip.CannotOpenZipException;
import uk.co.hcsoftware.commander.gui.zip.ZipPanel;
import uk.co.hcsoftware.commander.io.*;

import javax.swing.*;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.nio.channels.FileChannel;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.MalformedInputException;
import java.nio.charset.UnmappableCharacterException;


/**
 * Title: FileSearchPanel
 * Copyright:    Copyright (c) 27-Feb-2006
 *
 * @author N Spurling
 */
public class FileSearchPanel extends CommanderViewPanel
        implements Commandule,DndHelper.DndHandler,FileSelector{
    public static final Logger log = Logger.getLogger(FileSearchPanel.class.getName());

    private final JLabel nameLabel = new JLabel();
    private final JScrollPane jsp ;//= new JScrollPane(table);

    private final DeleteAction da=new DeleteAction(this,this,this);

    private CommanderSelectionUtilities csu;
    private SearchForm.SearchParameters sp;
    private JLabel searchLabel;

    public FileSearchPanel(CommanderFrame commanderFrame, final CommanderFocus cf) {
        super(new BorderLayout(GAP,GAP),cf,commanderFrame,new CommanderTable());
        final FileListTableModel model = new FileListTableModel();
        table.setModel(model);
        rowHeaderTable = new RowHeaderTable(table,this); //need this here (dupe) as rht binds to tablemodel

        nameLabel.setOpaque(true);
        setupTable(cf);
        new DndHelper.DndHelperBuilder(new DataFlavor[0],rowHeaderTable,this)
                .shouldStartDragCheck(this)
                .otherDropTargets(table)
                .create()
                ;
        jsp = new JScrollPane(table);
        jsp.setRowHeaderView(rowHeaderTable);
        jsp.setOpaque(true);
        setOpaque(true);
        add(jsp);
        JPanel northPan = new JPanel(new BorderLayout());
        nameLabel.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
        northPan.add(nameLabel);
        JPanel buttonPan = new JPanel(new FlowLayout(FlowLayout.LEFT,2,2));
        northPan.add(buttonPan,BorderLayout.EAST);
        buttonPan.add(createButton(searchDialogAction));
        buttonPan.add(createButton(refreshAction));
        add(northPan,BorderLayout.NORTH);

        searchLabel = new JLabel(ImageProvider.getInstance().getIcon("icons/16/find.png"));
        northPan.add(searchLabel,BorderLayout.WEST);
        searchLabel.setVerticalAlignment(SwingConstants.TOP);
        searchLabel.setBorder(BorderFactory.createEmptyBorder(2,1,0,0));
        searchLabel.setPreferredSize(new Dimension(20,20)); //icon is 16px^2 so make button about that size
        searchLabel.setFocusable(false);
        searchLabel.setOpaque(true);

        //make searches bookmarkable (need new bookmark type -saved view?)
        new DndHelper.DndHelperBuilder(new DataFlavor[0], searchLabel,new DndHelper.DndHandler() {
            public void drop(DropTargetDropEvent evt, Transferable transferable) {}
            public Transferable getTransferable(DragGestureEvent dge) {
                return new DefaultTransferable(ViewStateTransferData.dataFlavor,new ViewStateTransferData(FileSearchPanel.this));
            }
        })
                .create()
                ;

        setupActions() ;
        setupPopup();

        //if we get the commanderfocus update the statusbar value
        cf.addListener(new CommanderFocus.CommanderFocusListener() {
            public void focusChanged(Component lostFocus, Component gainedFocus) {
                if(gainedFocus==FileSearchPanel.this){
                    FileSearchPanel.this.commanderFrame.setFrameStatusText(sp.searchPattern+ " ("+table.getRowCount()+" files)");
                }
            }
        });
        waitPanel.setCancelText("Stop");
        init();
    }

     public String getRowName(int viewRow) {
         int modelRow=table.convertRowIndexToModel(viewRow);
         File file = getModel().getElement(modelRow);
         if(file!=null) return file.getName();
         return null;
    }

    public boolean isDirectory(int viewRow) {
        int modelRow=table.convertRowIndexToModel(viewRow);
        File file = getModel().getElement(modelRow);
        if(file!=null) return file.isDirectory();
        return false;
    }
    private void setupTable(final CommanderFocus cf) {
        final TableRenderable tr = new TableRenderable() {
            FileRecognizer fr = FileRecognizer.getInstance();
            public void setupRenderLabel(JLabel jl, int viewRow, int viewCol) {
//                int modelRow=table.convertRowIndexToModel(viewRow);
//                int modelColumn =table.convertColumnIndexToModel(viewCol);
//                File file = getModel().getElement(modelRow);
//                jl.setText("");
//                if (file!=null) {
//                    jl.setIcon(fr.getIcon(file.getName(), file.isDirectory()));
//                }
            }
        };
        final ColumnSpec[] columns={
            ColumnSpec.createColumnSpec(FileListTableModel.MARK,"mark_column_w",5)
                .setRenderer(new MarkRenderer(tr))
                .comparator(new BooleanComparator(),SortDirection.descending)

            ,ColumnSpec.createColumnSpec(FileListTableModel.NAME,"name_column_w",200)
                .setRenderer(new StringRenderer(tr) )  //name column
                .comparator(new NameComparator(),SortDirection.ascending)
            ,ColumnSpec.createColumnSpec(FileListTableModel.PATH,"path_column_w",400)
                           .setRenderer(new StringRenderer(tr) )  //name column
                           .comparator(new NameComparator(),SortDirection.ascending)
             ,ColumnSpec.createColumnSpec(FileListTableModel.SIZE,"size_column_w",50)
                .setRenderer(new SizeRenderer(tr))
                .comparator(new LongComparator(),SortDirection.descending)
            ,ColumnSpec.createColumnSpec(FileListTableModel.DATE,"date_column_w",120)
                .setRenderer(new DateRenderer(tr))
             .comparator(new LongComparator(),SortDirection.descending)
            ,ColumnSpec.createColumnSpec(FileListTableModel.EXT,"ext_column_w",50)
                .setRenderer(new StringRenderer(tr))
             .comparator(new NameComparator(),SortDirection.ascending)
        };

        table.configureColumns(Arrays.asList(columns));
        table.setCommanderFocusOwner(cf,this);
        csu = new CommanderSelectionUtilities<Vfile>(table,cf,this);
        table.addTableModelListener(new TableModelListener() {
            public void tableChanged(TableModelEvent e) {
                 updateLabelText();
            }
        });
    }

     public void setParameters(SearchForm.SearchParameters sp) {
        this.sp=sp;
        updateLabelText();
    }

    private JButton createButton(Action a) {
        final JButton jb = new JButton(a);
        jb.setPreferredSize(new Dimension(20,20)); //icon is 16px^2 so make button about that size
        jb.setFocusable(false);
        jb.setText("");
        return jb;
    }

    private NumberFormatter nf = new NumberFormatter();
    private long totalBytes=0;


    public void setColors(Color backColor, Color foreColor, Color selColor) {
         super.setColors(backColor, foreColor, selColor);
        log.fine("setting colors.. back="+backColor+" fore="+foreColor);
        setBackground(backColor);
        jsp.setBackground(backColor);
        jsp.getViewport().setBackground(backColor);
        jsp.getRowHeader().setBackground(backColor);
        nameLabel.setBackground(backColor);
        nameLabel.setForeground(foreColor);
    }


    //state-persistence
    public static final String MAGIC="uk.co.hcsoftware.commander.gui.FileSearchPanel";
    public String saveState() throws CannotSaveStateException //save current state to a unicode string that can be used later to restore state
    {
        StringBuilder sb = new StringBuilder();
        sb.append(MAGIC);
        //return new FileSearchPanel(commanderFrame,cf,searchRoots,searchPattern,wildcard,caseSensitive);
        Element root = new Element("state");
        org.jdom.Document d = new org.jdom.Document(root);
        root.setAttribute("searchPattern",sp.searchPattern);
        root.setAttribute("wildcard",String.valueOf(sp.wildcardSelected));
        root.setAttribute("caseSensitive",String.valueOf(sp.caseSensitiveSelected));

        root.setAttribute("containsPattern",sp.containsPattern);
        root.setAttribute("containsCaseSensitive",String.valueOf(sp.containsCaseSensitive));
        root.setAttribute("containsWildcard",String.valueOf(sp.containsWildcardSelected));

        root.setAttribute("after",String.valueOf(sp.after));
        root.setAttribute("before",String.valueOf(sp.before));
        root.setAttribute("afterSelected",String.valueOf(sp.hasAfter()));
        root.setAttribute("beforeSelected",String.valueOf(sp.hasBefore()));

        Element searchRoots = new Element("searchroots");
        root.addContent(searchRoots);
        for(File f:sp.searchRoots){
            Element el = new Element("file");
            el.setAttribute("path",f.getAbsolutePath());
            searchRoots.addContent(el);
        }
        final Format format = Format.getCompactFormat();
        format.setEncoding("UTF8");
        XMLOutputter xmlo = new XMLOutputter(format);
        sb.append(xmlo.outputString(d));
        return sb.toString();
    }

    public void restoreState(String state) throws CannotRestoreSavedStateException {
        if(state.startsWith(MAGIC) ){
            try {
                final String xml = state.substring(MAGIC.length());
                SAXBuilder sb = new SAXBuilder(false);
                Document d = sb.build(new StringReader(xml));
                final Element root = d.getRootElement();
                final boolean wildcard=Boolean.parseBoolean(root.getAttributeValue("wildcard"));
                final boolean caseSensitive=Boolean.parseBoolean(root.getAttributeValue("caseSensitive"));
                final String searchPattern=root.getAttributeValue("searchPattern");

                final boolean afterSelected=Boolean.parseBoolean(root.getAttributeValue("afterSelected"));
                final boolean beforeSelected=Boolean.parseBoolean(root.getAttributeValue("beforeSelected"));

                final long after = afterSelected?Long.parseLong(root.getAttributeValue("after")):0;
                final long before = beforeSelected?Long.parseLong(root.getAttributeValue("before")):0;

                Element searchroots = root.getChild("searchroots");
                final List srels = searchroots.getChildren();
                final List<File> searchRoots=new ArrayList<File>(srels.size());
                for(Object o:srels){
                    Element el =(Element) o;
                    if("file".equals(el.getName())){
                        searchRoots.add(new File(el.getAttributeValue("path")));
                    }
                }


                SwingUtilities.invokeLater(new Runnable(){
                    public void run() {
                        final SearchForm.SearchParameters sp = new SearchForm.SearchParameters(searchRoots);
                        sp.searchPattern= searchPattern;
                        sp.wildcardSelected= wildcard;
                        sp.caseSensitiveSelected=caseSensitive;
                        if(afterSelected){
                            sp.after=after;
                        }
                        if (beforeSelected) {
                            sp.before=before;
                        }
                        String s;
                        if((s=root.getAttributeValue("containsPattern"))!=null){
                             sp.containsPattern=s;
                        }
                        if((s=root.getAttributeValue("containsCaseSensitive"))!=null){
                             sp.containsCaseSensitive=Boolean.parseBoolean(s);
                        }
                        if((s=root.getAttributeValue("containsWildcard"))!=null){
                             sp.containsWildcardSelected=Boolean.parseBoolean(s);
                        }
                        setParameters(sp);
                        find();
                    }
                });
            } catch (JDOMException e) {
                log.log(Level.WARNING, "exception", e);
                throw new CannotRestoreSavedStateException(state, "xml parse exception");
            } catch (IOException e) {
                log.log(Level.WARNING, "exception", e);
                throw new CannotRestoreSavedStateException(state, "xml read exception");
            }
        }
        else throw new CannotRestoreSavedStateException(state, "bad magic number");
    }

    public Commandule createClone() {
        final FileSearchPanel fsp = new FileSearchPanel(commanderFrame,cf);
        fsp.setParameters((SearchForm.SearchParameters) sp.clone());
        return fsp;
    }


    public void delete() {
        da.actionPerformed(null);
    }

    FileLauncher fl = new FileLauncher();
    public void launch() {
        fl.launchWithPrompt(this, getSelectedFiles(true, true, true));
        getFocus();
    }

    public void mkdir() {  }

    public void move() { }

    public void rename() {
        //don't implement rename, because rename causes delete+add fileevent, and don't want to have to work out if an
        //added file would be selectd by this search, in order to determine whether to insert it into this view..
//        ra.actionPerformed(null);
    }

    public void copy() {
        try {
            FileListTransferable t = getTransferable(null);
            if(t==null){
                log.fine("ignoring copy as no transferable files");
                return;
            }
            saveToNextPanel(t, "Copy",t.getFileCount());

        } catch (NotSupportedException e) {
            JOptionPane.showMessageDialog(FileSearchPanel.this,"The opposite pane doesn't understand my data!","Copy",JOptionPane.ERROR_MESSAGE);
        } catch (CannotSaveException e) {
            log.log(Level.WARNING, "exception", e);
            JOptionPane.showMessageDialog(FileSearchPanel.this,"Couldn't save those data\n"+e.getMessage(),"Copy",JOptionPane.ERROR_MESSAGE);
        }
    }

    private void saveToNextPanel(Transferable t, final String title, int fileCount) throws NotSupportedException, CannotSaveException {
        String s = fileCount !=1?fileCount+" files":"file";
        // prompt (and get overwrite question)
        Object[] possibleValues = { "Copy (skip)","Copy (overwrite)", "Cancel" };
        int selectedValue = JOptionPane.showOptionDialog(FileSearchPanel.this,
                "Copy "+s+" to next panel?"
                +"\n  (skip means any existing files will be left alone, overwrite means they will be replaced)"
                ,title
                ,JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                possibleValues, possibleValues[0]);
        log.finer("selval="+selectedValue);
        if(selectedValue<0 || selectedValue>1){//user cancelled or something
            return;
        }
        assert selectedValue==1 || selectedValue ==0;
        final boolean force=selectedValue==1;
        SaveTarget st = commanderFrame.getSaveTarget();
        if (st!=null) {
            st.save(t, force);
        }
    }

    public void duplicate() {  }



    public void save(Transferable t, boolean force) throws NotSupportedException, CannotSaveException { throw new NotSupportedException(); }


    //search impl
    private WildcardMatcher wm = new WildcardMatcher();
    public void find(){
        log.fine("search filenamePattern="+sp);

        blockInputWithCancel("search for: "+sp.searchPattern);
        new SwingWorker3(){
            private void findFiles(File dir) throws IOException {
                if(wasCancelled()){
                    log.finer("cancelled");
                    return;
                }
                updateBlockMessage("search for: "+sp.searchPattern+"\n " + found.size()+" matches\n scanned "+dirsScanned+" directories");
                File[] children = dir.listFiles();
                for(File f:children){
                    if(f.isDirectory()){
                        findFiles(f);
                    }
                    else if(matches(f)){
                        found.add(f);
                        totalBytes+=f.length();
                    }
                }
                dirsScanned++;
            }

            private long totalBytes=0;
            final List<File> found=new ArrayList<File>();
            private int dirsScanned=0;
            Pattern filenamePattern,containsPattern;
            public Object construct() {
                try {
                    if(!sp.matchesAllNames()) filenamePattern = sp.createSearchPattern();
                    if(!sp.matchesAllContents()) containsPattern=sp.createContainsPattern();
                    for (File f:sp.searchRoots) {
                        findFiles(f);
                    }
                    return null;
                } catch (Exception e1) {
                    log.log(Level.WARNING,"e1",e1);
                    return e1;
                }
            }

            public void finished() {
                unblockInput();
                if(get()instanceof Exception){
                    JOptionPane.showMessageDialog(FileSearchPanel.this,
                            "couldn't gather data:\n"+ ((Exception) get()).getMessage(),"Search"
                            ,JOptionPane.ERROR_MESSAGE);
                    return;
                }
                //tod: instead of dialogs, display these messages over the searchpanel itself
//                if(!tooBigFiles.isEmpty()){
//                    StringBuilder sb = new StringBuilder();
//                    for(File f:tooBigFiles){
//                        sb.append(f.getAbsolutePath());
//                        sb.append("\n");
//                    }
//                    JOptionPane.showMessageDialog(FileSearchPanel.this,
//                            "Some files were greater than the maximum size of "+ new NumberFormatter().formatBytes(MAX_LENGTH)
//                            +"\nand consequently the contents were not scanned.\n" +
//                            "These files may (or may not) contain your search text:\n"
//                              +sb.toString()
//                            ,"Search"
//                            ,JOptionPane.WARNING_MESSAGE);
//                }

                if(!wasCancelled() && found.isEmpty()){
                    JOptionPane.showMessageDialog(FileSearchPanel.this,"No matches found for \n  "+sp.searchPattern
                            +"\n(scanned "+dirsScanned+" directories)"
                    );
                    return;
                }

                getModel().setFiles(found);
                FileSearchPanel.this.totalBytes=totalBytes;
                updateLabelText();
                refreshAction.setEnabled(true);
            }
//            List<File> tooBigFiles=new ArrayList<File>();
//            private final long MAX_LENGTH=10*1000*1000;//10Mb
            // now that we stream files through the matcher, can scrap the maxfilesize limit (user can always cancel if it's taking too long)

            private boolean matches(File f) throws IOException {
                if(sp.hasAfter()){ //filedate must be after spdate
                    if(f.lastModified()<=sp.after){
                        return false;
                    }
                }
                if(sp.hasBefore()){ //filedate must be before spdate
                    if(f.lastModified()>=sp.before){
                        return false;
                    }
                }
                if(!sp.matchesAllNames()){
                    final boolean matchesName = filenamePattern.matcher(f.getName()).matches();//matches(f.getName());
                    if(!matchesName) return false;
                }
                if(!sp.matchesAllContents()){
                    try {
                        FileChannel fc = new RandomAccessFile(f, "r").getChannel();
                        ByteBuffer bbuf = fc.map(FileChannel.MapMode.READ_ONLY, 0, (int)fc.size());
                        fc.close();
                        CharBuffer cbuf = Charset.forName("UTF8").newDecoder().decode(bbuf);
                        Matcher m = containsPattern.matcher(cbuf);
                        final boolean found = m.find();
                        if(found && log.isLoggable(Level.FINER)){
                            log.finer("found match in file "+f+"  "+m.start()+"-"+m.end()+" grp="+m.group());
                        }
                        return found;
                    } catch (MalformedInputException e) {
                        log.fine("MalformedInputException file="+f+" ignoring file (probably not textfile in default encoding)");
                        return false;

                    } catch (UnmappableCharacterException e) {
                        log.fine("UnmappableCharacterException file="+f+" ignoring file (probably not textfile in default encoding)");
                        return false;
                    }
                }
                else return true;
            }
        }.start();
    }


    private void updateLabelText() {
        StringBuilder sb = new StringBuilder();
        sb.append("Search results: ");
        sb.append(sp.searchPattern);
        sb.append(" (found ");
        final int count = table.getRowCount();
        sb.append(count);
        sb.append(" file");
        sb.append(count!=1?"s":"");
        sb.append(", total size: "+nf.formatBytes(totalBytes));
        sb.append(")");
        nameLabel.setText(sb.toString());
    }







    public List<File> getSelectedFiles(boolean includeSelected, boolean includeMarkedFiles,boolean includeparents){
        return getSelectedFiles(includeSelected,includeMarkedFiles);
    }

    public void selectFiles(List<File> l) {
        log.finest("select files="+l);
        List<Integer> ll = new ArrayList<Integer>(l.size());
        FileListTableModel fsm = getModel();
        if(fsm==null) return;
        for(File f:l){
            final int row = fsm.indexOf(f);
            if(row>-1){
                log.finest("mrow="+row);
                ll.add(table.convertRowIndexToView(row));
            }
        }
        log.finest("table.select vrows="+ll);
        if(!ll.isEmpty()) table.selectRows(ll,ll.get(ll.size()-1));
    }

    private List<File> getSelectedFiles(boolean includeSelected, boolean includeMarkedFiles){
        int[] rows = csu.getSelectedRows(includeSelected, includeMarkedFiles);
        List<File> l = new ArrayList<File>(rows.length);
        FileListTableModel ftm = getModel();
        for(int ro:rows){
            final int modelIndex = table.convertRowIndexToModel(ro);
            l.add(ftm.getElement(modelIndex));
        }
        return l;
    }

    private FileListTableModel getModel() {
        FileListTableModel ftm = (FileListTableModel) table.getModel();
        return ftm;
    }
    private void setupPopup() {
        final JPopupMenu popup = new JPopupMenu();
        popup.add(ActionDispatcher.getInstance().createCommanderMenu(this));
        popup.addSeparator();
        popup.add(new CreateZipAction(this, commanduleContext,this,this));
        final CreateTarAction mkTar = new CreateTarAction(this,commanduleContext,this,this);
        popup.add(mkTar);
        popup.addSeparator();
        popup.add(copyNamesAction);
        popup.add(copyPathsAction);
        popup.addSeparator();
        popup.add(copyFilesWithStructure);
        popup.addSeparator();
        popup.add(new AbstractAction("Open parent directory"){
            public void actionPerformed(ActionEvent e) {
                List<File> sel = getSelectedFiles(true,true,false);
                if(!sel.isEmpty()){
                    File f = sel.get(0);
                    File parent = f.getParentFile();
                    if(parent!=null){
                        final DirectoryPanel zp = new DirectoryPanel(commanderFrame,cf);
                        commanderFrame.addTabule(FileSearchPanel.this,zp,JSplitPane.VERTICAL_SPLIT);
                        zp.openFile(parent);
                        zp.getFocus();
                    }
                    else log.warning("ignoring attempt to open parent of f="+f+" as parent is null");
                }
                else JOptionPane.showMessageDialog(FileSearchPanel.this,"Please select a file");
            }
        });


        final MouseAdapter ml = new MouseAdapter(){
            private void mouse(MouseEvent e) {
                if(e.isPopupTrigger()){
                    boolean isSelection = table.isSelectedFiles(true,true);//.isSelection();
                    copyNamesAction.setEnabled(isSelection);
                    copyPathsAction.setEnabled(isSelection);
                    copyFilesWithStructure.setEnabled(isSelection);
                    popup.show(table,e.getX(),e.getY());
                }
            }
            public void mousePressed(MouseEvent e) {
                mouse(e);
            }
            public void mouseReleased(MouseEvent e) {
                mouse(e);
            }
            public void mouseClicked(MouseEvent e) {
                if(e.getClickCount()==2){
                    List<File> l = getSelectedFiles(true,false,false);
                    if(!l.isEmpty()){
                        onEnter();
                    }
                }
            }
        };
        table.addMouseListener(ml);
        rowHeaderTable.addMouseListener(ml);
    }

    private void setupActions() {
        table.registerAction(enterAction,searchDialogAction,refreshAction,da,markAction,copyNamesAction,copyPathsAction);
        final InputMap ancestorInputMap = getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        ancestorInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE,0),da.getValue(Action.ACTION_COMMAND_KEY));
    }
    private Action copyPathsAction = new CopyPathsAction(this, this);
    private Action copyNamesAction = new CopyNamesAction(this, this);
    private Action enterAction =ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            onEnter();
        }
    }).keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0))
           .command("launch").get();

    private Action copyFilesWithStructure =ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            List<File> l = getSelectedFiles(true,true,false);
            if (!l.isEmpty()) {
                try {
                    StructuredFileListTransferable t = new StructuredFileListTransferable(l, FileSearchPanel.this);
                    saveToNextPanel(t, "Copy", l.size());
                } catch (CannotFindCommonAncestorException e1) {
                    JOptionPane.showMessageDialog(FileSearchPanel.this,
                            "There is no common ancestor directory to the files you have selected!\n (You may be using some oddball filesystem)","Copy files with structure"
                            ,JOptionPane.ERROR_MESSAGE);
                } catch (CannotSaveException e1) {
                    JOptionPane.showMessageDialog(FileSearchPanel.this,
                            "Couldn't save\n  "+e1.getMessage(),"Copy files with structure"
                            ,JOptionPane.ERROR_MESSAGE);
                } catch (NotSupportedException e1) {
                    JOptionPane.showMessageDialog(FileSearchPanel.this,
                            "Cannot save to next panel\n  (it doesn't understand this kind of data)","Copy files with structure"
                            ,JOptionPane.ERROR_MESSAGE);
                } catch (IOException e1) {
                    JOptionPane.showMessageDialog(FileSearchPanel.this,
                            "Cannot save to next panel\n" +
                                    "  (IO: "+e1.getMessage()+")","Copy files with structure"
                            ,JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    })
            .name("Copy files, with structure")
            .tooltip("Copy files to next panel, preserving directory structure")
            .get();

    static final int ctrlMask = AppProperties.getInstance().getCtrlMask();
    private final Action refreshAction = ActionBuilder.getInstance().init(new AbstractAction("",ImageProvider.getInstance().getIcon("icons/16/reload.png")) {
        public void actionPerformed(ActionEvent e) {
            setEnabled(false);
            find();
        }
    }).keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_R,ctrlMask))
           .command("refresh").get();

    private final Action searchDialogAction = ActionBuilder.getInstance().init(new AbstractAction("",ImageProvider.getInstance().getIcon("icons/16/configure.png")) {
        private SearchForm searchForm;
        public void actionPerformed(ActionEvent e) {
            SearchForm sf = getSearchForm();
            sf.setParameters(sp);
            sf.setVisible(true);
        }
        private SearchForm getSearchForm() {
            if (searchForm==null) {
                searchForm = new SearchForm(commanderFrame,FileSearchPanel.this){
                    public void onFindPressed(SearchParameters sp) {
                        FileSearchPanel.this.setParameters(sp);
                        find();
                    }
                };
            }
            return searchForm;
        }
    }).keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_F,ctrlMask))
           .command("find").get();


    private Action markAction =ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            final int[] selectedViewRows = csu.getSelectedRows(true,false);
            final int[] selectedModelRows=new int[selectedViewRows.length];
            int i=0;
            for(int ro:selectedViewRows){
                selectedModelRows[i]=table.convertRowIndexToModel(selectedViewRows[i]);
                i++;
            }
            getModel().toggleMarked(selectedModelRows);
        }
    }).keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0))
           .command("mark").get();


    public void onEnter() {
        java.util.List<File> l =getSelectedFiles(true,true,true);// ct.getSelectedNodes(true,true);
        if(l.isEmpty()) return;
        log.fine("onEnter, "+l.size()+" files selected");
        File f=l.get(0);
        if(l.size()==1 && f.isFile()){
            if (FileRecognizer.isZipFile(f)) {
                log.fine("onEnter with zipfile="+f);
                Object[] possibleValues = { "Browse in CB", "Launch","Cancel" };
                int selectedValue = JOptionPane.showOptionDialog(this,
                        "What do want to do with the zip file?","Zip file found..",
                        JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                        possibleValues, possibleValues[0]);
                if(0==selectedValue){
                    final ZipPanel zp = new ZipPanel(commanderFrame,cf);
                    commanderFrame.addTabule(this,zp,JSplitPane.VERTICAL_SPLIT);
                    try {
                        zp.openFile(f,null);
                        zp.getFocus();
                    } catch (CannotOpenZipException e) {
                        log.log(Level.WARNING, "exception", e);
                        JOptionPane.showMessageDialog(this,"Cannot open zip file\n  "+e.getMessage(),(String)possibleValues[0],JOptionPane.ERROR_MESSAGE);
                    }
                    return;
                }
                else if(1==selectedValue){
                    fl.tryLaunch(f);
                    return;
                }
                else return;
            }
        }    //fall thru
        fl.tryLaunch(l);
    }


    //drag n drop
    public FileListTransferable getTransferable(DragGestureEvent dge){
        if (dge==null) {
            final List<File> sel = getSelectedFiles(true,true)  ;
            if(sel.isEmpty()){
                log.fine("ignore drag as sel.length==0");
                return null;
            }
            return new FileListTransferable(sel,this);
        } else {
            Point p = dge.getDragOrigin();
            int ro=table.rowAtPoint(p);
            if(ro>=0){
                final int modelRow = table.convertRowIndexToModel(ro);
                if(table.isRowSelected(ro) || getModel().isMarked(modelRow)){
                    return getTransferable(null);  //return all selected/marked rows (which will incldue this one)
                }
                else{ //drag just this row
                    File f = getModel().getElement(modelRow);
                    List<File> l = Collections.singletonList(f);
                    return new FileListTransferable(l,this);
                }
            }
            else{
                log.fine("returning null transferable as x coord is not a row");
                return null;
            }
        }

    }

    public void drop(DropTargetDropEvent evt, Transferable transferable) {}

   public void shell() {
        log.fine("runscript");
        CommandManager.getInstance().launchShellCommand(commanderFrame,commanduleContext);
    }

    public void usercommand() {
//        can't really implement this because the idea of a 'working dir' doesn't make sense -
// all the files have different parent dirs
//        log.fine("usercommand");
        final List<File> seleFiles = getSelectedFiles(true,true,false);
        CommandManager.getInstance().showSelector(commanderFrame,new CommanduleContext() {
            public File getDirectory() {
                return seleFiles.isEmpty()?null:seleFiles.get(0).getParentFile();
            }

            public List<File> getSelectedFiles() {
                return seleFiles;
            }

        });
    }
    private CommanduleContext commanduleContext=new CommanduleContext() {
        public File getDirectory() {
            return null;
        }

        public List<File> getSelectedFiles() {
            return FileSearchPanel.this.getSelectedFiles(true,true,false);
        }

    };

}
