/*
 * CommandEditorPanel.java
 *
 * Created on 12 June 2006, 17:11
 */

package uk.co.hcsoftware.commander.gui.usercommands;

import uk.co.hcsoftware.commander.gui.CommanduleContext;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.gui.util.GuiUtilities;
import uk.co.hcsoftware.commander.io.ImageProvider;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.table.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 *
 * @author  spurlinn
 */
public class CommandEditorPanel extends javax.swing.JPanel {
    private static final Logger log = Logger.getLogger(CommandEditorPanel.class.getName());
    private UserCommand userCommand;
    private final ArgsTableModel model = new ArgsTableModel();

    /** Creates new form CommandEditorPanel */
    public CommandEditorPanel() {
        initComponents();
        argsTable.setRowHeight(commandTextfield.getPreferredSize().height);//commandTextfield.getHeight() fieldheight includes scrollbar..
        setup();
    }
    /**
     * //get the cell renderer at row zero , col modelindex and ask for it's preferred with
     */
    private void autoSizeTablecolumns(){
        log.finer("autosize tablecolumns");
        Enumeration<TableColumn> en = argsTable.getColumnModel().getColumns();
        while(en.hasMoreElements()){
            TableColumn tc=en.nextElement();

            final int col = tc.getModelIndex();
            final int row = 0;
            TableCellRenderer tcr = argsTable.getCellRenderer(row,col);
            Component jc=tcr.getTableCellRendererComponent(argsTable,model.getValueAt(row,col),false,false,row,col);
            int w= Math.max(40,jc.getPreferredSize().width+6);
            log.finest("setting col="+col+" prefw="+w);
            tc.setPreferredWidth(w);
        }
    }


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        delimiterGroup = new javax.swing.ButtonGroup();
        helpText = new javax.swing.JTextArea();
        saveButton = new javax.swing.JButton();
        runButton = new javax.swing.JButton();
        nameLabel = new javax.swing.JLabel();
        nameField = new javax.swing.JTextField();
        showOutput = new javax.swing.JCheckBox();
        surroundWithQuotes = new javax.swing.JCheckBox();
        escapeSpaces = new javax.swing.JCheckBox();
        setFilesVariable = new javax.swing.JCheckBox();
        spaceDelimited = new javax.swing.JRadioButton();
        newlineDelimited = new javax.swing.JRadioButton();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        commandTextfield = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        argsTable = new javax.swing.JTable();
        addArgButton = new javax.swing.JButton();
        delArgButton = new javax.swing.JButton();
        copyButton = new javax.swing.JButton();
        pasteButton = new javax.swing.JButton();
        refreshDir = new javax.swing.JCheckBox();

        setBorder(javax.swing.BorderFactory.createEmptyBorder(2, 2, 2, 2));
        helpText.setBackground(new java.awt.Color(255, 255, 204));
        helpText.setColumns(20);
        helpText.setEditable(false);
        helpText.setFont(new java.awt.Font("Dialog", 0, 10));
        helpText.setLineWrap(true);
        helpText.setRows(5);
        helpText.setText("FILES variable holds the selected file-name(s)\nThe filenames are relative to the current working directory (the view's directory)\n\nExample (windows):\n       cmd /c c:\\Progra~1\\Java\\jdk1.5.0_06\\bin\\java.exe -jar \"c:\\t\\jEdit_4.3pre2\\jedit.jar\" %FILES%\n(invokes jedit with the selected file(s))\n\nCurrently spaces in command invocations are not supported..\n\n\t");
        helpText.setWrapStyleWord(true);
        helpText.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(), javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1)));
        helpText.setDisabledTextColor(new java.awt.Color(0, 0, 0));
        helpText.setFocusable(false);
        helpText.setRequestFocusEnabled(false);

        saveButton.setText("Save");

        runButton.setText("Test");

        nameLabel.setLabelFor(nameField);
        nameLabel.setText("Name:");

        showOutput.setSelected(true);
        showOutput.setText("Show output window");
        showOutput.setToolTipText("Pop up a new dialog to display the output of the command (std.err, std.out)");
        showOutput.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        showOutput.setMargin(new java.awt.Insets(0, 0, 0, 0));

        surroundWithQuotes.setText("surround names with quotes");
        surroundWithQuotes.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        surroundWithQuotes.setMargin(new java.awt.Insets(0, 0, 0, 0));

        escapeSpaces.setText("escape spaces");
        escapeSpaces.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        escapeSpaces.setMargin(new java.awt.Insets(0, 0, 0, 0));

        setFilesVariable.setSelected(true);
        setFilesVariable.setText("Set FILES variable");
        setFilesVariable.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        setFilesVariable.setMargin(new java.awt.Insets(0, 0, 0, 0));

        delimiterGroup.add(spaceDelimited);
        spaceDelimited.setSelected(true);
        spaceDelimited.setText("space-delimited");
        spaceDelimited.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        spaceDelimited.setMargin(new java.awt.Insets(0, 0, 0, 0));

        delimiterGroup.add(newlineDelimited);
        newlineDelimited.setText("newline-delimited");
        newlineDelimited.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        newlineDelimited.setMargin(new java.awt.Insets(0, 0, 0, 0));

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Command invocation:"));
        jScrollPane1.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        commandTextfield.setToolTipText("command e.g. bash");
        jScrollPane1.setViewportView(commandTextfield);

        jScrollPane2.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        argsTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        argsTable.setToolTipText("command arguments");
        argsTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
        argsTable.setCellSelectionEnabled(true);
        argsTable.setShowHorizontalLines(false);
        argsTable.setShowVerticalLines(false);
        jScrollPane2.setViewportView(argsTable);

        addArgButton.setText("add arg");

        delArgButton.setText("delete arg");

        copyButton.setText("Copy");
        copyButton.setToolTipText("copy command text to clipboard");

        pasteButton.setText("Paste");

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 105, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(copyButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 42, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(pasteButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 45, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 291, Short.MAX_VALUE)
                    .add(jPanel1Layout.createSequentialGroup()
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(addArgButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 50, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(delArgButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 50, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 67, Short.MAX_VALUE)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(delArgButton)
                    .add(addArgButton)
                    .add(copyButton)
                    .add(pasteButton))
                .addContainerGap())
        );

        refreshDir.setText("Reload directory after");
        refreshDir.setToolTipText("Reload the files in the current directory after running this command (useful if the command modifies files)");
        refreshDir.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        refreshDir.setMargin(new java.awt.Insets(0, 0, 0, 0));

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .add(refreshDir)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 191, Short.MAX_VALUE)
                .add(runButton)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(saveButton))
            .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .add(layout.createSequentialGroup()
                .add(nameLabel)
                .add(26, 26, 26)
                .add(nameField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 173, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
            .add(layout.createSequentialGroup()
                .add(showOutput)
                .addContainerGap(317, Short.MAX_VALUE))
            .add(helpText)
            .add(layout.createSequentialGroup()
                .add(setFilesVariable)
                .addContainerGap())
            .add(layout.createSequentialGroup()
                .add(32, 32, 32)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(newlineDelimited)
                    .add(spaceDelimited)
                    .add(layout.createSequentialGroup()
                        .add(17, 17, 17)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(escapeSpaces)
                            .add(surroundWithQuotes))))
                .addContainerGap(232, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(nameField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(nameLabel))
                .add(16, 16, 16)
                .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(setFilesVariable)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(spaceDelimited)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(surroundWithQuotes)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(escapeSpaces)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(newlineDelimited)
                .add(22, 22, 22)
                .add(showOutput)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(saveButton)
                        .add(runButton))
                    .add(refreshDir))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(helpText, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 152, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addArgButton;
    private javax.swing.JTable argsTable;
    private javax.swing.JTextField commandTextfield;
    private javax.swing.JButton copyButton;
    private javax.swing.JButton delArgButton;
    private javax.swing.ButtonGroup delimiterGroup;
    private javax.swing.JCheckBox escapeSpaces;
    private javax.swing.JTextArea helpText;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextField nameField;
    private javax.swing.JLabel nameLabel;
    private javax.swing.JRadioButton newlineDelimited;
    private javax.swing.JButton pasteButton;
    private javax.swing.JCheckBox refreshDir;
    private javax.swing.JButton runButton;
    private javax.swing.JButton saveButton;
    private javax.swing.JCheckBox setFilesVariable;
    private javax.swing.JCheckBox showOutput;
    private javax.swing.JRadioButton spaceDelimited;
    private javax.swing.JCheckBox surroundWithQuotes;
    // End of variables declaration//GEN-END:variables



    public void setUserCommand(UserCommand userCommand) {
        this.userCommand=userCommand;
        refreshState();
    }


    private void setup() {
        saveAction.setEnabled(false);
        //connect enabledstate of save button to whether or not the namefield value is legal
        nameField.getDocument().addDocumentListener(new DocumentListener() {
            private void changed(){
                final String nameValue = nameField.getText();
                saveAction.setEnabled(nameValue!=null && nameValue.trim().length()>0);
            }
            public void insertUpdate(DocumentEvent e) { changed(); }
            public void removeUpdate(DocumentEvent e)  { changed(); }
            public void changedUpdate(DocumentEvent e) { changed(); }
        });
        saveButton.setAction(saveAction);
        runButton.setAction(testAction);
        ActionListener al = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                refreshEnabledState();
            }
        };
        setFilesVariable.addActionListener(al);
        spaceDelimited.addActionListener(al);
        newlineDelimited.addActionListener(al);

        Action addAction = ActionBuilder.getInstance().init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                saveTableEdit(false);
                int sel = argsTable.getSelectedColumn();
                final int newCol;
                if(sel>-1 && sel<model.getColumnCount()-1){//if last col is selected add arg at end
                    model.insertColumnBefore(sel);
                    newCol=sel;
                }
                else{
                    model.addColumn(null);
                    newCol = model.getColumnCount()-1;
                }
                final Rectangle cellRect = argsTable.getCellRect(0,newCol,true);
                argsTable.scrollRectToVisible(cellRect);
                argsTable.editCellAt(0,newCol);
                argsTable.requestFocus();            }
        }).icon(ImageProvider.getInstance().getIcon("icons/16/edit_add.png"))
          .tooltip("Add argument")
                .get();
        addArgButton.setAction(addAction);

        Action delAction = ActionBuilder.getInstance().init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                saveTableEdit(false);
                if(model.getColumnCount()==0) return;
                int sel = argsTable.getSelectedColumn();
                if(sel==-1){
                    sel=model.getColumnCount()-1;
                }
                model.removeColumns(sel,sel+1);
            }
        }).icon(ImageProvider.getInstance().getIcon("icons/16/editdelete.png"))
                .tooltip("Delete argument")
                .get();
        delArgButton.setAction(delAction);


        Action copyAction = ActionBuilder.getInstance().init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                 UserCommand uc=createClonedCommand();
                    //copy the selected names to the clipboard
                    String s = uc.getFullCommandString();
                    StringSelection ss = new StringSelection(s);
                    try {
                        Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
                        cb.setContents(
                                ss
                                , new ClipboardOwner() { public void lostOwnership(Clipboard clipboard, Transferable contents) { }}
                        );
                    } catch (Exception e1) {
                        log.log(Level.FINE, "couldn't copy "+ss+" to clipboard", e1);
                        JOptionPane.showMessageDialog(CommandEditorPanel.this
                                ,"Couldn't put command on clipboard:\n  "+e1,"copy command",JOptionPane.ERROR_MESSAGE);
                    }
            }
        }).icon(ImageProvider.getInstance().getIcon("icons/16/editcopy.png"))
          .tooltip("Copy command text to clipboard")
                .get();
        copyButton.setAction(copyAction);
        Action pasteAction = ActionBuilder.getInstance().init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
                if(cb.isDataFlavorAvailable(DataFlavor.stringFlavor)){
                    try {
                        String s = (String) cb.getData(DataFlavor.stringFlavor);
                        //UserCommand uc = UserCommand.readFromString(s);
                        String[] cmd = s.split(" ");
                        if (cmd.length>0) {
                            commandTextfield.setText(cmd[0]);
                            model.setData(Arrays.asList(cmd).subList(1,cmd.length));
                            autoSizeTablecolumns();
                        }
                    } catch (UnsupportedFlavorException e1) {
                        log.log(Level.WARNING, "exception", e1);
                        JOptionPane.showMessageDialog(CommandEditorPanel.this
                            ,"Clipboard doesn't contain text","paste command",JOptionPane.ERROR_MESSAGE);
                    } catch (IOException e1) {
                        log.log(Level.WARNING, "exception", e1);
                        JOptionPane.showMessageDialog(CommandEditorPanel.this
                            ,"Couldn't paste data from clipboard","paste command",JOptionPane.ERROR_MESSAGE);
                    } catch(Exception e1){
                        log.log(Level.WARNING, "exception", e1);
                        JOptionPane.showMessageDialog(CommandEditorPanel.this
                            ,"Error pasting data from clipboard","paste command",JOptionPane.ERROR_MESSAGE);
                    }
                }
                else{
                    JOptionPane.showMessageDialog(CommandEditorPanel.this
                            ,"Clipboard doesn't contain text","paste command",JOptionPane.ERROR_MESSAGE);
                }

            }
        }).icon(ImageProvider.getInstance().getIcon("icons/16/editpaste.png"))
          .tooltip("Paste command text from clipboard")
                .get();
        pasteButton.setAction(pasteAction);

        argsTable.setModel(model);
        argsTable.getTableHeader().setReorderingAllowed(false);

        final DefaultCellEditor tableCellEditor = new DefaultCellEditor(new JTextField()){
            public int getClickCountToStart() {
                log.finest("getClickCountToStart");
                return 1;
            }

            public boolean isCellEditable(EventObject evt) {
                log.finest("celled");
                if (evt instanceof MouseEvent) {
                    return ((MouseEvent)evt).getClickCount() >= 1;
                }
                return true;
            }
        };
        argsTable.setCellEditor(tableCellEditor);
        argsTable.setDefaultEditor(Object.class,tableCellEditor);
    }

    private void saveTableEdit(boolean forceCancel) {
        if (argsTable.isEditing()) {
            TableCellEditor cellEditor  = argsTable.getCellEditor();
            log.finer("saving table edit");
            if (!forceCancel) {
                if(!cellEditor.stopCellEditing()){
                    cellEditor.cancelCellEditing();
                }
            } else {
                cellEditor.cancelCellEditing();
            }
        }
    }

    private static final int COLUMN_COUNT = 3;

    public void setNameFieldEnabled(boolean enabled) {
        nameField.setEnabled(enabled);
        saveButton.setPreferredSize(new Dimension(0,0));
    }
    public void setSaveAction(Action a){
        this.saveAction= a;
        saveButton.setAction(a);
    }

    public void setHelpText(String t){
         helpText.setText(t);
    }

    class ArgsTableModel extends DefaultTableModel{
        public ArgsTableModel() {
            super(1, COLUMN_COUNT);
        }
        public String getColumnName(int column) {
            return "arg"+(column+1);
        }

        public void removeColumns(int start, int end) {
            for (Iterator iterator = dataVector.iterator(); iterator.hasNext();) {
                Vector ro   = (Vector) iterator.next();
                for(int i=end-1;i>=start;i--){
                    ro.remove(i);
                }
            }
            for(int i=end-1;i>=start;i--){
                columnIdentifiers.remove(i);
            }
            fireTableStructureChanged();
        }

        public void clearData() {
           for (Iterator iterator = dataVector.iterator(); iterator.hasNext();) {
                Vector ro   = (Vector) iterator.next();
                for (ListIterator iterator1 = ro.listIterator(); iterator1.hasNext();) {
                    iterator1.next();
                    iterator1.set(null);
                }
            }
            fireTableDataChanged();
        }

        public void setData(List<String> args) {
            Vector ro1=new Vector(args);
            dataVector=new Vector(1);
            dataVector.add(ro1);
            columnIdentifiers=new Vector(args);
            fireTableStructureChanged();
        }

        public List<String> getData() {
            List<String> ret = new ArrayList<String>();
            for (Iterator iterator = ((Vector) dataVector.get(0)).iterator(); iterator.hasNext();) {
                String o = (String) iterator.next();
                ret.add(o);
            }
            return ret;
        }

        public void insertColumnBefore(int sel) {
            for (Iterator iterator = dataVector.iterator(); iterator.hasNext();) {
                Vector ro   = (Vector) iterator.next();
                ro.insertElementAt(null,sel);
            }
            columnIdentifiers.insertElementAt(null,sel);
            fireTableStructureChanged();
        }
    };

    Action testAction = ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            saveTableEdit(false);
            Frame parent = GuiUtilities.getFrameAncestor(CommandEditorPanel.this);
            UserCommand uc = createClonedCommand();
            final List<File> selectedFiles;
            if (uc.isUseFilesVariable()) {
                String files=JOptionPane.showInputDialog(CommandEditorPanel.this,"Enter a file name for the FILES variable:");
                if(files==null) return;
                selectedFiles=Collections.singletonList(new File(files));
            }
            else selectedFiles=Collections.emptyList();

            CommandManager.getInstance().execute(parent, uc,new CommanduleContext() {
                public File getDirectory() {
                    return null;
                }
                public List<File> getSelectedFiles() {
                    return selectedFiles;
                }
            }, true);//output must be modal as this might be modal
        }
    }).name("Test").get();
    public boolean canSave(){
        return saveAction.isEnabled();
    }
    Action saveAction = ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            save();
        }
    }).name("Save").get();

    public boolean save() {
        saveTableEdit(false);
        try {
            UserCommand uc = createClonedCommand();
            CommandManager.getInstance().saveCommand(uc);
            setUserCommand(uc);
            return true;
        } catch (CommandManager.ReadWriteCommandException e1) {
            JOptionPane.showMessageDialog(this,"Couldn't save command:\n  "+e1.getMessage()
                            ,"Save command",JOptionPane.ERROR_MESSAGE);
        }
        return false;
    }
    public UserCommand getGuiStateCommand(){
        saveTableEdit(false);
        return createClonedCommand();
    }

    public void getEditingFocus(boolean selectText) {
        if (selectText) {
            nameField.setCaretPosition(0);
            nameField.moveCaretPosition(nameField.getText().length());
        }
        nameField.requestFocus();
    }

    public UserCommand getUserCommand() {
        return userCommand;
    }
    private void refreshState() {
        //if command is null create a default one and read the state from that - this ensures that
        //the default state matches the state of a New command
        setState(userCommand==null?new UserCommand(""):userCommand);
        refreshEnabledState();
    }
     private boolean testUnsaved(boolean guiAndCachedValuesEqual, String description) {
        if(!guiAndCachedValuesEqual){
            log.fine("areUnsavedChanges: "+description);
            return true;
        }
        return false;
    }

    private boolean equal(List<String> data, List<String> ucArgs) {
        if(data.size()!=ucArgs.size()){
            log.finer("datasize/argssize different");
            return false;
        }
        int i=0;
        for(String x:data){
            String y = ucArgs.get(i++);
            if(!x.equals(y)){
                log.finer("x="+x+" != y="+y);
                return false;
            }
        }
        return true;
    }

    //METHODS THAT NEED UPDATING IF CHANGE UserCommand FIELDS:
    public boolean areUnsavedChanges() {
        if(userCommand==null) return false;
        // compare usercommand to fieldstate:- if different there are unsaved changes
        saveTableEdit(false);
        List<String> ucArgs=userCommand.getArgs();
        List<String> data = getModelData();
        if(testUnsaved(equal(data,ucArgs),"table args")) return true;
        if(testUnsaved(commandTextfield.getText().equals(userCommand.getCommand()),"commandtext")) return true;
        if(testUnsaved(nameField.getText().equals(userCommand.getName()),"name")) return true;
        if(testUnsaved(setFilesVariable.isSelected()==userCommand.isUseFilesVariable(),"setFilesVar")) return true;
        if(testUnsaved(spaceDelimited.isSelected()==userCommand.isSpaceDelimited(),"space delimited")) return true;
        if(testUnsaved(newlineDelimited.isSelected()!=userCommand.isSpaceDelimited(),"newline delim")) return true;
        if(testUnsaved(escapeSpaces.isSelected()==userCommand.isEscapeSpaces(),"esc spaces")) return true;
        if(testUnsaved(showOutput.isSelected()==userCommand.isShowOutputWindow(),"show output")) return true;
        if(testUnsaved(surroundWithQuotes.isSelected()==userCommand.isSurroundWithQuotes(),"surround with quote")) return true;
        if(testUnsaved(refreshDir.isSelected()==userCommand.isRefreshDirectoryAfter(),"refresh dir")) return true;

        return false;
    }
    private List<String> getModelData(){
        List<String> l = model.getData();
        //purge any nulls
        List<String> ret = new ArrayList<String>(l.size());
        for(String s:l){
            if(s!=null) ret.add(s);
        }
        return ret;
    }


    private void setState(UserCommand userCommand) {
        saveTableEdit(false);
        commandTextfield.setText(userCommand.getCommand());
        model.setData(userCommand.getArgs());
        autoSizeTablecolumns();
        nameField.setText(userCommand.getName());
        setFilesVariable.setSelected(userCommand.isUseFilesVariable());
        spaceDelimited.setSelected(userCommand.isSpaceDelimited());
        escapeSpaces.setSelected(userCommand.isEscapeSpaces());
        showOutput.setSelected(userCommand.isShowOutputWindow());
        surroundWithQuotes.setSelected(userCommand.isSurroundWithQuotes());
        refreshDir.setSelected(userCommand.isRefreshDirectoryAfter());
    }

    private UserCommand createClonedCommand() {
        UserCommand uc = userCommand==null?new UserCommand(null) : userCommand.getCopy(true, null);
        uc.setCommand(commandTextfield.getText());
        uc.setArgs(getModelData());

        uc.setName(nameField.getText());
        uc.setUseFilesVariable(setFilesVariable.isSelected());
        uc.setSpaceDelimited(spaceDelimited.isSelected());
        uc.setEscapeSpaces(escapeSpaces.isSelected());
        uc.setShowOutputWindow(showOutput.isSelected());
        uc.setSurroundWithQuotes(surroundWithQuotes.isSelected());
        uc.setRefreshDirectoryAfter(refreshDir.isSelected());
        return uc;
    }
    private void refreshEnabledState() {
        final boolean setFilesSelected = setFilesVariable.isSelected();
        spaceDelimited.setEnabled(setFilesSelected);
        newlineDelimited.setEnabled(setFilesSelected);
        final boolean escapingEnabled=setFilesSelected && spaceDelimited.isSelected();
        surroundWithQuotes.setEnabled(escapingEnabled);
        escapeSpaces.setEnabled(escapingEnabled);
    }










    public static void main(String[] args) {
        final JFrame editorDialog = new JFrame("Configure User Commands");
        final CommandEditorPanel editorPan = new CommandEditorPanel();
        editorDialog.setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);

        JPanel mainPan = new JPanel(new BorderLayout());
        mainPan.setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
        editorDialog.setContentPane(mainPan);


        mainPan.add(editorPan);
        JPanel selectorPan = new JPanel(new BorderLayout(3,3));
        mainPan.add(selectorPan,BorderLayout.WEST);

        editorDialog.pack();
        editorDialog.setVisible(true);
        editorPan.getEditingFocus(true);
    }


}
