package uk.co.hcsoftware.commander.gui.usercommands;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import uk.co.hcsoftware.commander.AppProperties;
import uk.co.hcsoftware.commander.Main;
import uk.co.hcsoftware.commander.io.FileEventManager;
import uk.co.hcsoftware.commander.gui.AutocompleteJumper;
import uk.co.hcsoftware.commander.gui.AutocompleteJumperJlist;
import uk.co.hcsoftware.commander.gui.CommanderFrame;
import uk.co.hcsoftware.commander.gui.CommanduleContext;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.gui.util.WeakListener;

import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.Preferences;


/**
 * Title: CommandManager
 * Copyright:    Copyright (c) 12-Jun-2006
 *
 * @author N Spurling
 *
 *   example:
 *
 * keywords:
 *
 * $selected      : space-separated list of selected/marked files
 * $dir           : absolute path to current directory
 * $firstselected : absolute path of first selected file
 *
 *
 *   display output | error | nothing  ;
 *   execute{
 *
 *   }endexecute  ;
 *   browse  ?  ;
 *   refresh 0-n  ;
 */
public class CommandManager {
    private static final Logger log = Logger.getLogger(CommandManager.class.getName());
    private static CommandManager commandManager=new CommandManager();
    private SelectorDialog selectorDialog;
    private static final File commandsFile = new File(AppProperties.getInstance().getPrefsDirectory(),"commands.xml");

    public static CommandManager getInstance() {
        return commandManager;
    }

    public void showSelector(Frame parent, CommanduleContext sc){
        if (selectorDialog==null || selectorDialog.getParent()!=parent) {
            //if the dialog's parent is a different frame then create a new one
            //otherwise it will be modally owned by a different window, causing teh
            //current window to lose the focus - which is weird
            selectorDialog = new SelectorDialog(parent);
        }
        selectorDialog.getSelectorPanel().loadScripts();
        selectorDialog.setContext(sc);
        //select the first one
        selectorDialog.getSelectorPanel().selectFirst();
        selectorDialog.setLocationRelativeTo(parent);
        selectorDialog.setVisible(true);
    }
   private static final int changesSaved=0,cancelled=1,saveAndClose=2,discardAndClose=3;
    private int okToLoadNewView(CommandEditorPanel editorPan,
                    String actionDescription,
                    String actionTitle
                   , String saveAndCloseOption
                   ,String discardAndCloseOption
                                    ){
        if (editorPan.areUnsavedChanges()) {
            // prompt if unsaved chagnes
            final String[] options;
            final String cancelOption = "Cancel";
            if(editorPan.canSave()) options = new String[]{saveAndCloseOption,discardAndCloseOption,cancelOption};
            else options = new String[]{discardAndCloseOption,cancelOption};
            int i=JOptionPane.showOptionDialog(editorPan,
                    actionDescription
                    ,actionTitle,
                    JOptionPane.OK_CANCEL_OPTION
                    ,JOptionPane.WARNING_MESSAGE
                    ,null,options,options[0]);
            if(i==-1) return cancelled;
            String val = options[i];
            log.finer("option chosen="+val);
            if(cancelOption.equals(val)){
                return cancelled;
            }
            else if(saveAndCloseOption.equals(val)){
               return saveAndClose;
            }
            else{
                return discardAndClose;
            }
        }
        else{
            return changesSaved;
        }
    }
    
    public void showEditor(Frame parent) {

        final JDialog editorDialog = new JDialog(parent,"Configure User Commands",false);
        final CommandEditorPanel editorPan = new CommandEditorPanel();
        editorDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        editorDialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                log.fine("window closing");
                int i = okToLoadNewView(editorPan
                   ,"There are unsaved changes, closing will lose them"
                   ,"Close window"
                   ,"Save and close"
                   ,"Don't save, but close"
                );
                switch(i){
                    case cancelled:
                        return;
                    case saveAndClose:
                        if(editorPan.save()){
                            editorDialog.dispose();
                        }
                        return;
                    default:
                        editorDialog.dispose();
                        return;
                }

//                if(editorPan.areUnsavedChanges()){
//                    // prompt if unsaved chagnes
//                    log.fine("unsaved changes!");
//                    final String[] options;
//                    final String cancelOption = "Cancel";
//                    final String saveAndCloseOption = "Save and close";
//                    if(editorPan.canSave()) options = new String[]{saveAndCloseOption,"Don't save and close",cancelOption};
//                    else options = new String[]{"Close anyway",cancelOption};
//                    int i=JOptionPane.showOptionDialog(editorPan,
//                            "There are unsaved changes, closing will lose them"
//                            ,"Close window",
//                            JOptionPane.OK_CANCEL_OPTION
//                            ,JOptionPane.WARNING_MESSAGE
//                            ,null,options,options[0]);
//                    String val = options[i];
//                    if(cancelOption.equals(val)){
//                        return;
//                    }
//                    else if(saveAndCloseOption.equals(val)){
//                        if(editorPan.save()){
//                            editorDialog.dispose();
//                        }
//                    }
//                    else{
//                        editorDialog.dispose();
//                    }
//                }
//                else editorDialog.dispose();
            }
        });

        JPanel mainPan = new JPanel(new BorderLayout());
        mainPan.setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
        editorDialog.setContentPane(mainPan);


        mainPan.add(editorPan);
        JPanel selectorPan = new JPanel(new BorderLayout(3,3));
        mainPan.add(selectorPan,BorderLayout.WEST);
        final SelectorPanel commandSelectorPanel = new SelectorPanel();
        selectorPan.add(commandSelectorPanel);

        selectorPan.setBorder(BorderFactory.createLineBorder(Color.black,1));
        commandSelectorPanel.loadScripts();
        Action newAction = ActionBuilder.getInstance().init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                int i=okToLoadNewView(editorPan
                ,"There are unsaved changes, switching view will lose them"
                        ,"Switch view"
                        ,"Save changes"
                        ,"Discard changes"
                );
                switch(i){
                    case saveAndClose:
                        if(editorPan.save()){
                            doNew();
                        }
                        break;
                    case cancelled:
                        //reset selected item
                        commandSelectorPanel.setSelectedUserCommand(editorPan.getUserCommand());
                        editorPan.getEditingFocus(false);
                        return;
                    default: //no unsaved or continue anyway
                        doNew();
                }
            }
            private void doNew(){
                commandSelectorPanel.unselectAll();
                editorPan.setUserCommand(new UserCommand("<untitled>"));
                editorPan.getEditingFocus(true);
            }
        }).name("New").get();
        final Action delAction = ActionBuilder.getInstance().init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                UserCommand cmd=commandSelectorPanel.getSelectedCommand();
                if(cmd!=null &&
                        JOptionPane.YES_OPTION==
                        JOptionPane.showConfirmDialog(editorPan,"Are you sure you want to delete "+cmd.getName()+"?")){
                    try {
                        deleteUserCommand(cmd);
                    } catch (ReadWriteCommandException e1) {
                        JOptionPane.showMessageDialog(editorPan,"Couldn't delete command:\n  "+e1.getMessage()
                                ,"Delete command",JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        }).name("Delete").get();
        final Action copyAction = ActionBuilder.getInstance().init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                UserCommand cmd=commandSelectorPanel.getSelectedCommand();
                if(cmd!=null){
                    int i=okToLoadNewView(editorPan
                            ,"There are unsaved changes, switching view will lose them"
                            ,"Switch view"
                            ,"Save changes"
                            ,"Discard changes"
                    );
                    switch(i){
                        case saveAndClose:
                            if(editorPan.save()){
                                doCopy(cmd);
                            }
                            break;
                        case cancelled:
                            //reset selected item
                            commandSelectorPanel.setSelectedUserCommand(editorPan.getUserCommand());
                            editorPan.getEditingFocus(false);
                            return;
                        default: //no unsaved or continue anyway
                            doCopy(cmd);
                    }

                }

            }
            private void doCopy(UserCommand cmd){
                commandSelectorPanel.unselectAll();
                UserCommand uc = cmd.getCopy(false, cmd.getName()+" (copy)");
                editorPan.setUserCommand(uc);
                editorPan.getEditingFocus(true);
            }
        }).name("Copy").get();

        // add a listener to switch between lists
        commandSelectorPanel.addListListener(new Runnable(){
            public void run() {
                final UserCommand selectedCommand = commandSelectorPanel.getSelectedCommand();
                final UserCommand currentlyEditedCommand = editorPan.getUserCommand();
                if(!selectedCommand.equals(currentlyEditedCommand)){
                    log.finer("selectedCommand="+selectedCommand);
                    int i=okToLoadNewView(editorPan
                            ,"There are unsaved changes, switching view will lose them"
                            ,"Switch view"
                            ,"Save changes"
                            ,"Discard changes"
                    );
                    switch(i){
                        case saveAndClose:
                            if(editorPan.save()){
                                switchView(selectedCommand);
                            }
                            break;
                        case cancelled:
                            //reset selected item
                            commandSelectorPanel.setSelectedUserCommand(currentlyEditedCommand);
                            editorPan.getEditingFocus(false);
                            return;
                        default: //no unsaved or continue anyway
                            switchView(selectedCommand);
                    }
                }
            }
            private void switchView(UserCommand selectedCommand){
                  // switch selected command
                log.fine("switching to uc="+selectedCommand);
                editorPan.setUserCommand(selectedCommand);
                commandSelectorPanel.setSelectedUserCommand(selectedCommand);
            }
        });

        JToolBar jtb = new JToolBar(JToolBar.HORIZONTAL);
        jtb.setFloatable(false);
        selectorPan.add(jtb,BorderLayout.NORTH);
        jtb.add(newAction);
        jtb.add(delAction);
        jtb.add(copyAction);

        editorDialog.pack();
        editorDialog.setLocationRelativeTo(parent);
        editorDialog.setVisible(true);
        editorPan.getEditingFocus(true);
    }


    private static final Preferences p = Preferences.userNodeForPackage(Main.class);

    public void launchShellCommand(CommanderFrame commanderFrame, CommanduleContext commanduleContext) {
        execute(commanderFrame,getShellCommand(),commanduleContext, false);
    }
    private UserCommand shellCommand;
    public UserCommand getShellCommand() {
        if (shellCommand==null) {
            String s = p.get("shellspawn.usercommand",null);
            if(s!=null){
                try {
                    SAXBuilder sb = new SAXBuilder();
                    Document d = sb.build(s);
                    Element el = d.getRootElement();
                    return UserCommand.readFromElement(el);
                } catch (JDOMException e) {
                    log.log(Level.WARNING, "exception", e);
                } catch (IOException e) {
                    log.log(Level.WARNING, "exception", e);
                }
            }
            //use default command
            UserCommand uc = new UserCommand("shell");
            if ( System.getProperty("os.name").startsWith("Windows")){
                uc.setCommand("cmd.exe");
                uc.setArgs(list("/C","start"));
            } else if (System.getProperty("os.name").startsWith("Mac")){
                uc.setCommand("bash");
                uc.setArgs(list("open","-a","Terminal","."));
            } else {
                uc.setCommand("xterm");
            }
            uc.setUseFilesVariable(true);
            uc.setSurroundWithQuotes(true);
            uc.setEscapeSpaces(false);
            uc.setSpaceDelimited(true);
            uc.setShowOutputWindow(false);
            uc.setRefreshDirectoryAfter(false);
            shellCommand= uc;
        }
        return shellCommand;
    }
    private List<String> list(String... values){
        return Arrays.asList(values);
    }


    public void saveShellCommand(UserCommand userCommand) {
        Document d = new Document(userCommand.writeToElement());
        XMLOutputter xmlo = new XMLOutputter(Format.getCompactFormat());
        String s = xmlo.outputString(d);
        p.put("shellspawn.usercommand",s);
        shellCommand=userCommand;
    }

    public static class ReadWriteCommandException extends Exception{
        public ReadWriteCommandException(String message, Throwable cause) {
            super(message, cause);
        }
    }
    private void deleteUserCommand(UserCommand cmd) throws ReadWriteCommandException {
        try {
            List<UserCommand> cmds = readCommands();
            cmds.remove(cmd);
            writeCommands(cmds);
            fireCommandsChanged(cmds, cmd);
        } catch (IOException e) {
            throw new ReadWriteCommandException("IO Exception",e);
        } catch (JDOMException e) {
            throw new ReadWriteCommandException("XML Exception",e);
        }
    }
    public void saveCommand(UserCommand uc) throws ReadWriteCommandException {
        try {
            List<UserCommand> cmds = new ArrayList<UserCommand>(readCommands());
            int i = cmds.indexOf(uc);
            if(i>-1) cmds.set(i,uc);
            else cmds.add(uc);
            writeCommands(cmds);
            fireCommandsChanged(cmds, uc);
        }catch (IOException e) {
            throw new ReadWriteCommandException("IO Exception",e);
        } catch (JDOMException e) {
            throw new ReadWriteCommandException("XML Exception",e);
        }
    }
    private void fireCommandsChanged(List<UserCommand> cmds, UserCommand selectCommand) {
        try {
            for(CommandsListener cl:new ArrayList<CommandsListener>(listeners)){
                cl.commandsChanged(cmds, selectCommand);
            }
        } finally {
        }
    }
    /**
     * adds a weak listener for commandsChanged events
     * @param cl
     */
    public void addCommandsListener(CommandsListener cl){
        //add a weak listener
        WeakListener wl = new WeakListener(cl, CommandsListener.class){
            public Object onNullObserver() {
                for (Iterator<CommandsListener> iterator = listeners.iterator(); iterator.hasNext();) {
                    CommandsListener commandsListener = iterator.next();
                    if(getProxy()==commandsListener) iterator.remove();  //thing .equals does something weird for proxys (forwards to backing object?)
                }
                return null;
            }
        };
        listeners.add((CommandsListener) wl.getProxy());
    }
    private List<CommandsListener> listeners=new ArrayList<CommandsListener>();
    public static interface CommandsListener{
        public void commandsChanged(List<UserCommand> currentCommands, UserCommand selectCommand);
    }

    public class SelectorPanel extends JPanel implements CommandsListener{
        private boolean noScriptsMode=false;
        JList list = new JList();
        public UserCommand getSelectedCommand(){
            return (UserCommand) (noScriptsMode?null:list.getSelectedValue());
        }
        private List<ListSelectionListener> listListeners = new ArrayList<ListSelectionListener>();
        public SelectorPanel() {
            setLayout(new BorderLayout());

//            list.setPrototypeCellValue(new File("test script"));
            list.setCellRenderer(new DefaultListCellRenderer(){
                public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                    UserCommand f = (UserCommand)value;
                    value = f.getName();//getId(f);
                    return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
                }
            });
            new AutocompleteJumperJlist(list, new AutocompleteJumper.RowIdentifier() {
                public String getRowId(int viewRow) {
                    UserCommand f = (UserCommand) list.getModel().getElementAt(viewRow);
                    return f.getName();
                    //return getId(f);
                }
            });
            addCommandsListener(this);
            setSize(200,500);
        }

        public void commandsChanged(List<UserCommand> currentCommands, UserCommand selectCommand) {
            loadScripts(currentCommands);
            setSelectedUserCommand(selectCommand);
        }

        public void loadScripts() {
            List<UserCommand> userCommands=Collections.emptyList();
            try {
                userCommands = readCommands();
            } catch (IOException e) {
                log.log(Level.WARNING, "exception", e);
            } catch (JDOMException e) {
                log.log(Level.WARNING, "exception", e);
            }
            loadScripts(userCommands);
        }

        private void loadScripts(final List<UserCommand> userCommands) {
            Collections.sort(userCommands,new Comparator<UserCommand>() { //sort by name (then ID so ordering is always the same even when 2 entries have same name)
                public int compare(UserCommand userCommand, UserCommand userCommand1) {
                    int i = userCommand.getName().compareTo(userCommand1.getName());
                    if(i==0) i = userCommand.getId().compareTo(userCommand1.getId());
                    return i;
                }
            });
            Runnable runnable = new Runnable() {
                public void run() {
                    if(!userCommands.isEmpty()){
                        list.setModel(new AbstractListModel() {
                            public int getSize() { return userCommands.size(); }
                            public Object getElementAt(int i) { return userCommands.get(i); }
                        });
                        removeAll();
                        add(new JScrollPane(list));
                    }
                    else{
                        setNoScriptsMode();
                    }
                    getParent().validate();
                }
            };
            if(SwingUtilities.isEventDispatchThread()) runnable.run();
            else SwingUtilities.invokeLater(runnable);
        }



        private void setNoScriptsMode() {
            noScriptsMode=true;
//            unselectAll();//clear the list
            removeAll();
            JPanel jp = new JPanel(new BorderLayout());
            jp.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
            final JLabel jl = new JLabel("No user commands");
            jl.setEnabled(false);
            jl.setHorizontalAlignment(JLabel.CENTER);
            jl.setVerticalAlignment(JLabel.CENTER);
            jp.add(jl);
            add(jp);
        }

        public void selectFirst() {
            if(list.getModel().getSize()>0) list.setSelectedIndex(0);
        }

        public boolean isNoScriptsMode() {
            return noScriptsMode;
        }

        public void unselectAll() {
            log.finest("unselecting all");
            list.clearSelection();
        }

        /**
         * runnable is invoked (in EDT) when <em>user</em> changes selection
         *
         * @param r
         */
        public void addListListener(final Runnable r) {
            list.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    log.finer("user clicked firing list selection may have changed");
                     r.run();
                }
            });
            list.addKeyListener(new KeyAdapter() {
                public void keyReleased(KeyEvent e) {
                    final int kc = e.getKeyCode();
                    if(kc==KeyEvent.VK_DOWN || kc==KeyEvent.VK_UP ){
                        log.finer("user moved key up/down firing list selection may have changed");
                        r.run();
                    }
                }
            });
        }

        /**
         *
         * @param userCommand may be null, in which case clear selection
         */
        public void setSelectedUserCommand(UserCommand userCommand) {
            if(userCommand==null) unselectAll();
            else{
                list.setSelectedValue(userCommand,false);
            }
        }

        public boolean isSelection() {
            return list.getSelectedIndex()!=-1;
        }
    }
    public class SelectorDialog extends JDialog{
        SelectorPanel selectorPanel=new SelectorPanel();
        private CommanduleContext sc;

        private Action closeAction = new AbstractAction("Cancel") {
            public void actionPerformed(ActionEvent e) {
                  dispose();
            }
        };
        private Action okAction = new AbstractAction("OK") {
            public void actionPerformed(ActionEvent e) {
                launchSelectedCommand();
            }
        };

        private void launchSelectedCommand() {
            if (!selectorPanel.isNoScriptsMode()) {
                UserCommand cmd = selectorPanel.getSelectedCommand();
                if(cmd!=null){
                    execute((JFrame) this.getParent(), cmd,sc, false);
                    dispose();
                }
                else{
                    JOptionPane.showMessageDialog(this,"Please select a command to execute");
                }
            } else {
                dispose();
            }
        }

        public SelectorDialog(Frame owner) throws HeadlessException {
            super(owner, "Select script to run",true);
            setContext(sc);
            getContentPane().setLayout(new BorderLayout(5,5));
            getContentPane().add(selectorPanel);

            final InputMap inputMap = selectorPanel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
            final ActionMap actionMap = selectorPanel.getActionMap();
            inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),"esc");
            actionMap.put("esc",closeAction);
            inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),"ok");
            actionMap.put("ok",okAction);
            JPanel bPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT,5,5));
            setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
            final JButton ok = new JButton(okAction);
            bPanel.add(ok);
            getRootPane().setDefaultButton(ok);
            bPanel.add(new JButton(closeAction));
            getContentPane().add(bPanel, BorderLayout.SOUTH);

            selectorPanel.list.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if(e.getClickCount()==2){
                        launchSelectedCommand();
                    }
                }
            });
            setSize(200,500);
        }

        public SelectorPanel getSelectorPanel() {
            return selectorPanel;
        }

        private void setContext(CommanduleContext sc) {
            this.sc = sc;
        }
    }
    private void writeCommands(List<UserCommand> commands) throws IOException {
        if(!commandsFile.exists()){
            commandsFile.getParentFile().mkdirs();
        }
        Document d = new Document();
        final Element root = new Element("usercommands");
        d.addContent(root);
        for(UserCommand uc:commands){
            root.addContent(uc.writeToElement());
        }
        XMLOutputter xmlo = new XMLOutputter(Format.getPrettyFormat());
        OutputStream os = new BufferedOutputStream(new FileOutputStream(commandsFile));
        try {
            xmlo.output(d,os);
        } finally {
            try {
                os.close();
            } catch (IOException e) {
                log.log(Level.WARNING, "exception", e);
            }
        }
    }
    private List<UserCommand> readCommands() throws IOException, JDOMException {
        if(!commandsFile.exists()) return Collections.emptyList();
        SAXBuilder sb = new SAXBuilder(false);
        InputStream is = new BufferedInputStream(new FileInputStream(commandsFile));
        Document d = sb.build(is);
        Element root = d.getRootElement();
        List commandElements = root.getChildren("usercommand");
        List<UserCommand> commands = new ArrayList<UserCommand>(commandElements.size());
        for(Object element:commandElements){
            UserCommand uc = UserCommand.readFromElement((Element)element);
            commands.add(uc);
        }
        return commands;
    }


    public static class OutputDialog extends JDialog{
        private CommandOutputPanel commandOutputPanel;

        public OutputDialog(Frame owner, ProcessBuilder pb) throws HeadlessException {
            super(owner,"Run process");

            //setup keyboard actions
            final InputMap inputMap = ((JPanel) getContentPane()).getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
            final ActionMap actionMap = ((JPanel) getContentPane()).getActionMap();
            inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),"esc");
            actionMap.put("esc",canAction);

            this.commandOutputPanel=new CommandOutputPanel(pb);
            getContentPane().add(commandOutputPanel);
        }
        Action canAction =new AbstractAction("Cancel") {
            public void actionPerformed(ActionEvent e) {
                 dispose();
            }
        };
        public void start() throws IOException {
            commandOutputPanel.start();

        }

        public void addOnFinishAction(Runnable runnable) {
            commandOutputPanel.addOnFinishAction(runnable);
        }
    }

    /**
     * considered having a 'statusAction' to control these windows - i.e. each command has a hidden outputwindow
        //and if >0 outputwindows a button appears on the status bar to toggle their visibility.
        //The idea is that you wouldn't be bothered with the outputwindow normally but could view it if you wanted, and make that decision at runtime
        //benefits:
        //when process hangs can bring up output window and kill it
        //if output stays around after proc dies, can view output of 'failed' processes

        //downsides:
        //if output stays around, ties up lots of memory that user may not be aware of
        //if output disappears, is _only_ useful when proc hangs - once proc finishes there is no way to view the window
     * @param parent
     * @param userCommand
     * @param sc
     * @param outputDialogIsModal
     */
    public void execute(Frame parent, UserCommand userCommand, CommanduleContext sc, boolean outputDialogIsModal) {
//

        log.fine("executing userCommand="+userCommand);

        try {
            ProcessBuilder pb = userCommand.createProcessBuilder(sc);
            final File directory = sc.getDirectory();
            Runnable onFinishAction=null;
            if(userCommand.isRefreshDirectoryAfter() && directory!=null){
                onFinishAction = new Runnable(){
                    public void run() {
                        FileEventManager.getInstance().fireDirectoryChanged(directory);
                    }
                };
            }
            if(userCommand.isShowOutputWindow()){
                outputDialogIsModal=true;//doesn't retain focus if not modal
                OutputDialog od = new OutputDialog(parent,pb);
                od.setModal(outputDialogIsModal);
                od.pack();
                od.setLocationRelativeTo(parent);
                if(onFinishAction!=null) od.addOnFinishAction(onFinishAction);
                if(outputDialogIsModal) od.start(); //if modal have to start before showing, as show blocks
                od.setVisible(true);
                if(!outputDialogIsModal) od.start(); //if !modal, is more civilised to wait until visible before starting
            }
            else{
                pb.redirectErrorStream(true);
                Process p = pb.start();
                monitorStream(p.getInputStream(), Level.INFO, onFinishAction);
            }
        } catch (IOException e) {
            JOptionPane.showMessageDialog(parent,"Couln't start process:\n  "+e.getMessage()
                    ,"Start process",JOptionPane.ERROR_MESSAGE);
        } catch(IllegalStateException e){
            log.log(Level.WARNING,"IllegalStateException starting process",e);
            JOptionPane.showMessageDialog(parent,"Couln't start process:\n  "+e.getMessage()
                    ,"Start process",JOptionPane.ERROR_MESSAGE);
        } catch(IllegalArgumentException e){
            log.log(Level.WARNING,"illegal arg starting process",e);
            JOptionPane.showMessageDialog(parent,"Couln't start process (illegal arg):\n  " +
                    "(Sometimes this is because args contain quote characters and surround with quotes is checked)"
                    ,"Start process",JOptionPane.ERROR_MESSAGE);
        }  catch(Exception e){
            log.log(Level.WARNING,"start process",e);
            JOptionPane.showMessageDialog(parent,"Couln't start process :\n  " +
                    e
                    ,"Start process",JOptionPane.ERROR_MESSAGE);
        }
    }

    private void monitorStream(final InputStream is, final Level logLevel, final Runnable onFinish){
        log.fine("starting trhead to monitor output");
        Thread t = new Thread(){
            public void run() {
                LineNumberReader lnr = new LineNumberReader(new InputStreamReader(is));
                try {
                    String line;
                    while((line=lnr.readLine())!=null) {
                        final String line1 = line;
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                if(log.isLoggable(logLevel)){
                                    log.log(logLevel,line1);
                                }
                            }
                        });
                    }
                }
                catch (IOException e) {
                    log.log(Level.WARNING, "exception", e);
                }
                finally{
                    try {
                        lnr.close();
                    } catch (IOException e) {
                        log.log(Level.WARNING, "exception", e);
                    }
                    log.fine("exiting process monitor thread");
                    if(onFinish!=null){
                        log.fine("running onfinish action");
                        onFinish.run();
                    }
                }
            }
        }            ;
        t.start();
    }



}
