/*
 * CommandOutputPanel.java
 *
 * Created on 12 July 2006, 15:12
 */

package uk.co.hcsoftware.commander.gui.usercommands;

import uk.co.hcsoftware.commander.gui.util.ActionBuilder;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.List;
import java.util.ArrayList;


/**
 *  
 * @author  spurlinn
 */
public class CommandOutputPanel extends javax.swing.JPanel {
    private static final Logger log = Logger.getLogger(CommandOutputPanel.class.getName());

    final MutableAttributeSet errorAttributes,normalAttributes;
    private Process p;
    private ProcessBuilder pb;
    private int exitCode;
    private boolean processStarted,processFinished,normalFinish;
    private Action clearAction = ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            try {
                final Document d = outputTextpane.getDocument();
                d.remove(0,d.getLength());
            } catch (BadLocationException e1) {
                log.log(Level.WARNING, "exception", e1);
            }
        }
    })
            .name("Clear")
            .tooltip("Clear the output text")
            .get();


    /** Creates new form CommandOutputPanel */
    private CommandOutputPanel() {
        initComponents();
//        outputTextpane.setContentType("text/html");
        errorAttributes = new SimpleAttributeSet();
        StyleConstants.setForeground(errorAttributes, Color.red);
        normalAttributes = new SimpleAttributeSet();
        StyleConstants.setForeground(normalAttributes, Color.blue);

        outputTextpane.getDocument().addDocumentListener(new DocumentListener() {
                public void insertUpdate(DocumentEvent e) { enableClearAction(); }
                public void removeUpdate(DocumentEvent e)  { enableClearAction(); }
                public void changedUpdate(DocumentEvent e) { enableClearAction(); }
                private void enableClearAction() {
                     updateStatus();
                }
            });

        clearButton.setAction(clearAction);
    }

    public CommandOutputPanel(ProcessBuilder pb) {
        this();
        this.pb=pb;
        List<String> cmd = pb.command();
//        StringBuilder sb = new StringBuilder();
//        for(String s:cmd){
//            sb.append(s);
//            sb.append("\t");
//        }
//        if(!cmd.isEmpty()) sb.setLength(sb.length()-1);
        cmdTextArea.setText(cmd.toString());
        monitorProcess(pb);
    }


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jScrollPane1 = new javax.swing.JScrollPane();
        outputTextpane = new javax.swing.JTextPane();
        clearButton = new javax.swing.JButton();
        startButton = new javax.swing.JButton();
        statusLabel = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        cmdTextArea = new javax.swing.JTextArea();

        outputTextpane.setEditable(false);
        jScrollPane1.setViewportView(outputTextpane);

        clearButton.setText("Clear");

        startButton.setText("Start");

        statusLabel.setText("Ready..");

        cmdTextArea.setColumns(20);
        cmdTextArea.setEditable(false);
        cmdTextArea.setLineWrap(true);
        cmdTextArea.setRows(5);
        cmdTextArea.setWrapStyleWord(true);
        jScrollPane2.setViewportView(cmdTextArea);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE)
                        .addContainerGap())
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(statusLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(clearButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(startButton)
                        .addContainerGap())
                    .add(layout.createSequentialGroup()
                        .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 380, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 47, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(12, 12, 12)
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 195, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(startButton)
                    .add(clearButton)
                    .add(statusLabel))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton clearButton;
    private javax.swing.JTextArea cmdTextArea;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextPane outputTextpane;
    private javax.swing.JButton startButton;
    private javax.swing.JLabel statusLabel;
    // End of variables declaration//GEN-END:variables


    private void appendLine(String text, AttributeSet as){
        try {
            final Document document = outputTextpane.getDocument();
            document.insertString(document.getLength(), text+"\n", as);
        } catch (BadLocationException e) {
            log.log(Level.WARNING, "exception", e);
        }
    }

    Action startAction = ActionBuilder.getInstance().init(new AbstractAction("Start") {
        public void actionPerformed(ActionEvent e) {
            try {
                start();
            } catch (IOException e1) {
                JOptionPane.showMessageDialog(CommandOutputPanel.this,"Couln't start process:\n  "+e1.getMessage(),"Start process",JOptionPane.ERROR_MESSAGE);
            }
        }
    }).get();
    Action killAction = ActionBuilder.getInstance().init(new AbstractAction("Kill") {
        public void actionPerformed(ActionEvent e) {
             p.destroy();
        }
    }).get();

    public void monitorProcess(ProcessBuilder pb){
        this.pb=pb;
        updateStatus();
    }
    public void monitorProcess(Process p){
        monitorStream(p.getErrorStream(),errorAttributes);
        monitorStream(p.getInputStream(),normalAttributes);
        this.p=p;
        processStarted=true;
        processFinished=false;
        normalFinish=false;
        updateStatus();
        log.fine("starting thread to monitor process finish");
        final Thread t=new Thread(){ //on the process finishing, update the status text
            public void run() {
                try {
                    exitCode = CommandOutputPanel.this.p.waitFor();
                    normalFinish=true;
                } catch (InterruptedException e) {
                    log.log(Level.WARNING, "exception", e);
                }
                finally{
                    processFinished=true;
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            updateStatus();
                        }
                    });
                }
            }
        };
        t.setName("ProcessMonitor"+processMonitorNumber+" thread="+(threadNumber++));
        t.start();
    }
    private void updateStatus(){
        clearAction.setEnabled(outputTextpane.getDocument().getLength()>0);
        if(!processStarted){
            statusLabel.setText("Not started");
            startAction.setEnabled(true);
            startButton.setAction(startAction);
        }
        else if(!processFinished){ //process is still running
            statusLabel.setText("Running..");
            startButton.setAction(killAction);
        }
        else{//finished
            final String text = normalFinish?"Finished, exit code: "+exitCode:"Finished abnormally";
            killAction.setEnabled(false);
            startAction.setEnabled(false);
            statusLabel.setText(text);
            //run onfinish actions
            log.fine("running onfinish actions");
            for(Runnable r:onFinishActions) r.run();
        }
    }

    /**
     * reads from inputstream and writes, one line at a time, to the document -using teh text-attributes supplied
     * (creates new thread)
     * @param is
     * @param outputAttribues
     */
    private void monitorStream(final InputStream is, final AttributeSet outputAttribues){
        log.fine("starting trhead to monitor output");
        final Thread t = new Thread(){
            public void run() {
                LineNumberReader lnr = new LineNumberReader(new InputStreamReader(is));
                try {
                    String line;
                    while((line=lnr.readLine())!=null) {
                        final String line1 = line;
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                appendLine(line1, outputAttribues);
                            }
                        });
                    }
                }
                catch (IOException e) {
                    log.log(Level.WARNING, "exception", e);
                }
                finally{
                    try {
                        lnr.close();
                    } catch (IOException e) {
                        log.log(Level.WARNING, "exception", e);
                    }
                }
            }
        }            ;
        t.setName("ProcessMonitor"+processMonitorNumber+" thread="+(threadNumber++));
        t.start();
    }

    public void start() throws IOException {
        Process p = pb.start();
        monitorProcess(p);
    }

    public void addOnFinishAction(Runnable runnable) {
        onFinishActions.add(runnable);
    }
    List<Runnable> onFinishActions=new ArrayList<Runnable>();
    private static int processMonitorCount=1;
    private int processMonitorNumber=processMonitorCount++, threadNumber=1;
}
