package uk.co.hcsoftware.commander.gui.util;

import javax.swing.*;
import java.awt.*;
import java.util.logging.Logger;


/**
 *
 *
 * Title: GuiUtilities
 * Copyright:    Copyright (c) 20-Jan-2006
 *
 * @author spurlinn
 */
public class GuiUtilities {
    public static final Logger log = Logger.getLogger(GuiUtilities.class.getName());

    /**
     * remove keybindings from jcomponents
     * see http://java.sun.com/j2se/1.4.2/docs/api/javax/swing/doc-files/Key-Index.html#JTable
     * for teh standard keybindings
     * @param table
     * @param tabKs
     */
    public static void removeBindings(JComponent table, final KeyStroke tabKs) {

        table.unregisterKeyboardAction(tabKs);

        // setting keystroke to null, still has table doing its default thing.. why? NS: looking at ui code there seems
        //to be a UI inputmap that has replaced the table's inputmap (on metal)..
        final InputMap uiInputMap = SwingUtilities.getUIInputMap(table,JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        if (uiInputMap!=null) {
            uiInputMap.remove(tabKs);
        }
    }

    /**
     * returns the action so you can initialise an action in the declarataion e.g. Action a=createAction(new Action(..),...
     * @param copyAction
     * @param keystroke
     * @param name
     * @return
     */
       public static Action setupAction(Action copyAction, KeyStroke keystroke, String name){
        return setupAction(copyAction,keystroke,name,name);
    }

    /**
     * returns the action so you can initialise an action in the declarataion e.g. Action a=createAction(new Action(..),...
     * @param copyAction
     * @param keystroke
     * @param name
     * @return
     */
       public static Action setupAction(Action copyAction, KeyStroke keystroke, String name, String command){
        if (keystroke!=null) {
            copyAction.putValue(Action.ACCELERATOR_KEY,keystroke);
        }
        copyAction.putValue(Action.ACTION_COMMAND_KEY,command);
        copyAction.putValue(Action.NAME,name);
        return copyAction;
    }


    public static JButton createIconButton(Action iconAction){
        final JButton jb = new JButton(iconAction);
        jb.setPreferredSize(new Dimension(20,20)); //icon is 16px^2 so make button about that size
        jb.setFocusable(false);
        return jb;
    }

    public static Frame getFrameAncestor(JComponent shellConfigPanel) {
        Component f=shellConfigPanel;
        while(!((f=f.getParent()) instanceof Frame)){
            if(f==null) return null;
        }
        return (Frame) f;
    }
}
