package uk.co.hcsoftware.commander.gui.zip;

import spin.Spin;
import uk.co.hcsoftware.commander.*;
import uk.co.hcsoftware.commander.gui.*;
import uk.co.hcsoftware.commander.gui.ctable.*;
import uk.co.hcsoftware.commander.gui.usercommands.CommandManager;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.gui.util.ActionDispatcher;
import uk.co.hcsoftware.commander.gui.util.GuiUtilities;
import uk.co.hcsoftware.commander.io.*;
import uk.co.hcsoftware.commander.io.treestream.Node;
import uk.co.hcsoftware.commander.io.treestream.TreeStream;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;



public class ZipPanel extends CommanderViewPanel
        implements  DndHelper.DndHandler, ProgressUpdater, DndHelper.StartDrag {
    private static final Logger log = Logger.getLogger(ZipPanel.class.getName());

    private JLabel nameLabel=new JLabel();
    protected final FileEventManager fem = FileEventManager.getInstance();

    protected JTextField locationField=new JTextField();

    private File file;
    private ZipNode root;
    private ZipTableModel tableModel;

    private CommanderSelectionUtilities<ZipNode> csu;
    private final JScrollPane jsp;

    public ZipPanel(CommanderFrame mf, final CommanderFocus cf) {
        super(new BorderLayout(),cf, mf, new CommanderTable(new ZipTableModel()));
        tableModel=(ZipTableModel) table.getModel();

        setupTable();

        new DndHelper.DndHelperBuilder(RECEIVE_FLAVORS,rowHeaderTable,this)
                .otherDropTargets(this,table)
                .shouldStartDragCheck(this)
                .create()
                ;
        jsp = new JScrollPane(table);
        jsp.setRowHeaderView(rowHeaderTable);

//      DndHelper.createDndHelper(RECEIVE_FLAVORS,table,this,this);
        locationField.setAutoscrolls(true);

        final JScrollPane locSroll = new JScrollPane(locationField);
        locSroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        JPanel northPan = new JPanel(new BorderLayout(2,2));
        northPan.add(locSroll);
        northPan.add(nameLabel,BorderLayout.WEST);
        nameLabel.setVerticalAlignment(JLabel.TOP);
        nameLabel.setFocusable(false);
        nameLabel.setIcon( ImageProvider.getInstance().getIcon("icons/16/ark.png") );

        add(northPan,BorderLayout.NORTH);

        final JButton jb = GuiUtilities.createIconButton(saveAction);//new JButton(saveAction);
        saveAction.setEnabled(false);
        northPan.add(jb,BorderLayout.EAST);

        add(this.jsp);

        setupActions();
        setupPopup();
        locationField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                openPath(locationField.getText());
            }
        });
        init();
    }
    private void zipChanged(){
        saveAction.setEnabled(true);
    }

    private final ZipTableModel getModel(){
        return tableModel;
    }
    public String getRowName(int viewRow) {
        int modelRow=table.convertRowIndexToModel(viewRow);
        ZipNode zipNode = getModel().getElement(modelRow);
        if(zipNode!=null) return zipNode.getName();
        return null;
    }

    public boolean isDirectory(int viewRow) {
        int modelRow=table.convertRowIndexToModel(viewRow);
        ZipNode zipNode = getModel().getElement(modelRow);
        if(zipNode!=null) return zipNode.isDirectory();
        return false;
    }
    private void setupTable() {
        final TableRenderable tr = new TableRenderable() {
            FileRecognizer fr = FileRecognizer.getInstance();
            public void setupRenderLabel(JLabel jl, int viewRow, int viewCol) {
                int modelRow=table.convertRowIndexToModel(viewRow);
                ZipNode zipNode = getModel().getElement(modelRow);
                int modelColumn =table.convertColumnIndexToModel(viewCol);

                if (zipNode!=null) {
                    if(zipNode.isDirectory()){
                        if (zipNode.isSyntheticNode()) {
                            jl.setFont(AppProperties.italicBoldFont);
                        } else {
                            jl.setFont(AppProperties.boldFont);
                        }
                    }
                    else if(zipNode.isSyntheticNode()){
                        jl.setFont(AppProperties.italicPlainFont);
                    }
                    else{
                        jl.setFont(AppProperties.plainFont);
                    }
                }
            }
        };
        final ColumnSpec[] columns={
            ColumnSpec.createColumnSpec(ZipTableModel.MARK,"mark_column_w",5)
                .setRenderer(new MarkRenderer(tr))
                .comparator(new BooleanComparator(),SortDirection.descending)

            ,ColumnSpec.createColumnSpec(ZipTableModel.NAME,"name_column_w",200)
                .setRenderer(new StringRenderer(tr) )  //name column
                .comparator(new NameComparator(),SortDirection.ascending)

             ,ColumnSpec.createColumnSpec(ZipTableModel.SIZE,"size_column_w",50)
                .setRenderer(new SizeRenderer(tr){
                 public Object getAttributeValue(Object attributeValue, int row, int column) {
                     final long l = ((Long)attributeValue).longValue();
                     if(l==-1){
                          return "<DIR>";
                     }
                     else if(l<0){  //zips use -1 when they don't know the size, so we use -2 to signal this (because we are using -1 to denote a dir)
                         return "";
                     }
                     return super.getAttributeValue(attributeValue, row, column);
                 }
             } )
                .comparator(new LongComparator(),SortDirection.descending)

            ,ColumnSpec.createColumnSpec(ZipTableModel.DATE,"date_column_w",120)
                .setRenderer(new DateRenderer(tr))
             .comparator(new LongComparator(),SortDirection.descending)

            ,ColumnSpec.createColumnSpec(ZipTableModel.COMPRESSED,"compress_column_w",50)
                .setRenderer(new StringRenderer(tr){
                    public Component getTableCellRendererComponent(JTable table, Object attributeValue, boolean isSelected, boolean hasFocus, int row, int column) {
                        String av ="";
                        if(attributeValue!=null){
                            Integer l = (Integer) attributeValue;
                            if(l.longValue()<0) av="";
                            else av=l.toString()+"%";
                        }
                        return super.getTableCellRendererComponent(table, av, isSelected, hasFocus, row, column);
                    }
                })
             .comparator(new NameComparator(),SortDirection.ascending)

        };

        table.addPostFilter(new CommanderTable.FilterFactory() {
            public org.jdesktop.swingx.decorator.Filter getFilter() {
                return createDirectoryFilter(ZipTableModel.SIZE);
            }
        });
        table.configureColumns(Arrays.asList(columns));
        table.setCommanderFocusOwner(cf,this);
        csu=new CommanderSelectionUtilities<ZipNode>(table,cf,this);
        table.setSort(ZipTableModel.NAME);  //by default sort by name column
    }


    public SaveTarget getSaveTarget() {
        return commanderFrame.getSaveTarget();
    }

    /**
     *
     * @param file
     * @param zipPath can be null
     * @throws CannotOpenZipException
     */
    public void openFile(File file, String zipPath) throws CannotOpenZipException {
        log.fine("open file="+file+" path="+zipPath);
        try {
            blockInput("Opening archive: "+file.getName(), 100);
            ZipFile zf = new ZipFile(file);
            log.fine("zf entrycount="+zf.size());
            final ZipNode root = new ZipNode(null,new String[0], true);
            //if it's a dir the name is the dirpath and isDirectory returns true
            //if it's a file the name is the filepath and isDirectory returns false
            for(Enumeration<? extends ZipEntry> en=zf.entries();en.hasMoreElements();){
                final ZipEntry ze = en.nextElement();
                final String path = ze.getName();
                ZipNode zn = root.getOrCreateDescendant(path);
                zn.setZipEntry(ze);
                if (log.isLoggable(Level.FINER)) {
                    log.finer("path="+path+" zn="+zn);
                }
            }
            zf.close();
            this.root=root;
            ZipNode showNode=root;
            if (zipPath!=null) {
                ZipNode openNode = root.getDescendant(zipPath);
                if(openNode!=null) showNode=openNode;
                else log.info("not opening zipPath="+zipPath+" as no corresponding node found");
            }
            this.file=file;
            nameLabel.setText(file.getName());
            saveAction.setEnabled(false);
            openNode(showNode);
        } catch (IOException e) {
            throw new CannotOpenZipException(file,zipPath, "error opening zip file",e);
        } finally{
            unblockInput();
        }
    }

    private void openPath(String zipPath){
        log.fine("openpath="+zipPath);
        if(zipPath.startsWith("/")){
            //since we are resolving relative to root, remove the leading /
            zipPath=zipPath.substring(1);
        }
        ZipNode openNode = root.getDescendant(zipPath);
        if(openNode==null){
            log.warning("couldn't open zipPath="+zipPath+" as it wasn't found!");
            return;
        }
        openNode(openNode);
    }

    private void openNode(ZipNode openNode) {
        if(openNode!=null && openNode.isDirectory()){
            getModel().setZipNode(openNode);
            locationField.setText(openNode.getPathRelativeToRoot());
            if(table.getRowCount()>0){
                table.selectRow(0);
            }
        }
    }

    public void save(Transferable t, final boolean force) throws NotSupportedException, CannotSaveException {
        log.fine("saving transferable="+t);
        if(t.isDataFlavorSupported(TreeStream.DATAFLAVOR)){
            log.fine("TreeStream.DATAFLAVOR supported");
            try {
                final TreeStream ts = (TreeStream) t.getTransferData(TreeStream.DATAFLAVOR);
                if(this==ts.getSource()){
                    log.fine("ignoring drop as I am the vendor of these data");
                    return;
                }
                final Saver tss = new Saver() {
                    public void writeZipFile() throws IOException {    }
                    public int saveTreeStream(final TreeStream ts) throws IOException {
                        final ZipNode current = getModel().getDirectory();
                        if (current!=null) {
                            try {
                                //recurse down roots, whenever we hit a leaf, create a zipnode
                                // get zip file closed error when copy from zip -> zip then try to save 2nd zip
                                //this is because stream is no longer available after call TreeStream.close()
                                //so if we are going to use deferred write we either:
                                //1. need to buffer the stream content in memory
                                //2. need to save a handle to the stream that we can attempt to resuscitate later
                                //prefer 2. as with 1. we run the risk of using up all the memory for large files..
                                //or 3. buffer stream content to disk before adding synthetic nodes..
                                String zipPath = current.getZipPath();

                                for(Node n:ts.getRoots()){
                                    process(n,zipPath);
                                }

                                if(!wasCancelled()){
                                    log.fine("refreshing view");
                                    zipChanged();
                                }
                                else{
                                    created=0;
                                    log.fine("aborted: not writing zipfile");
                                    requery();
                                }
                            } finally {
                                try {
                                    ts.close();
                                } catch (Exception e) {
                                    log.log(Level.WARNING, "exception", e);
                                }
                            }
                        }
                        else{
                            log.warning("current node was null!");
                        }
                        return created;
                    }
                    private int created=0;
                    private void process(Node n, String path) throws IOException {
                        if(wasCancelled()){
                            return;
                        }
                        if(path.length()>0 && !path.endsWith("/")){
                            path+="/";
                        }
                        path+=n.getName();
                        if(n.hasInputStream()){
                            //check if the path already exists
                            if(!force && root.getDescendant(path)!=null){
                                log.fine("ignoring node at path="+path+" as exists and !force");
                            }
                            else{
                                log.finer("saving node: create descendant @ path="+path);
                                ZipNode zn = root.getOrCreateDescendant(path);
                                final InputStream inputStream = n.getInputStream();
                                try {
                                    zn.setSyntheticNode(inputStream,false);
                                } finally {
                                    if(inputStream!=null) inputStream.close();
                                }
                                updateBlockMessage("Saved "+(++created)+" zip entries");
                                log.finer("new node="+zn);
                                ZipNode p = zn.getParent();
                                if (p!=null) {
                                    log.finer("parentnode has "+p.getChildren().size()+" children");
                                }
                            }
                        }
                        List<Node> children = n.getChildren();
                        for(Node c:children){
                            process(c,path);
                        }
                    }
                };

                Saver tss1 = (Saver) Spin.off(tss);
                //block the gui
                blockInputWithCancel("Saving data to zip" );
                int saved=tss1.saveTreeStream(ts);
                
                refresh();
                if(saved==0){
                    JOptionPane.showMessageDialog(this,"No files were saved to the zip file.\n" +
                            " Zip files cannot contain empty directories, so it may be that there were no files in the selection."
                    ,"Save data to zip file",JOptionPane.WARNING_MESSAGE);
                }
            } catch (UnsupportedFlavorException e) {
                log.log(Level.WARNING, "exception", e);
            } catch (IOException e) {
                log.log(Level.WARNING, "exception", e);
                throw new CannotSaveException(e.getMessage(),e);
            } finally{
                //either way we refresh, 'cos we've mutated the mem-copy
                unblockInput();
            }
        }
        else throw new NotSupportedException();
    }


    public static interface Saver{
        /**
         *
         * @param ts
         * @return count of saved files
         * @throws IOException
         */
        int saveTreeStream(final TreeStream ts) throws IOException;
        void writeZipFile() throws IOException;
    }


    public void delete() {
        final List<ZipNode> nodes = getSelectedNodes(true, true, false);
        if(!nodes.isEmpty() && JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(ZipPanel.this,"Delete "+nodes.size()+" selected files?")){
            delete(nodes);
        }
    }

    private void delete(final List<ZipNode> nodes) {
        for(ZipNode n:nodes){
            n.removeFromParent();
        }
        zipChanged();
        refresh();
    }

    public void launch() {
        int[] sel = csu.getSelectedRows(true,true);// smh.getSelectedOrMarked(true, true,true);
        //weed out directories and nodes without zipentries
        final ArrayList<ZipEntry> l = getZipEntries(sel, false, false);
        if(l.size()>0){
            launchFiles(l);
        }
        getFocus();
    }
    FileLauncher fl = new FileLauncher();
    private void launchFiles(List<ZipEntry> l) {
        String s = l.size()==1?l.get(0).getName():l.size()+" files";
        String[] options = new String[]{"launch","cancel"};
        if ( 0== JOptionPane.showOptionDialog(ZipPanel.this
                ,"launch "+s+"?","Launch"
                ,JOptionPane.YES_NO_CANCEL_OPTION
                ,JOptionPane.QUESTION_MESSAGE,null
                ,options,options[0])) {
            try {
                List<File> files = writeTempFiles(l);
                fl.launchFiles(files);
            } catch (IOException e) {
                log.log(Level.WARNING,"ioe",e);
                JOptionPane.showMessageDialog(ZipPanel.this,"Error executing external process\n" +
                        "["+e.getMessage()+"]","Launch",JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public void mkdir() {
        String dest=(String)JOptionPane.showInputDialog(ZipPanel.this,
                "Create Directory:"
                ,"Create Directory",JOptionPane.QUESTION_MESSAGE,null
                ,null,"");
        if(dest!=null && dest.trim().length()>0){
            ZipNode curr = getModel().getDirectory();//(ZipNode) ct.getNode();
            if (curr!=null) {
                if(curr.getChild(dest)!=null){
                    JOptionPane.showMessageDialog(ZipPanel.this
                            ,"Couldn't create directory\n  a directory called "+dest+" already exists"
                            ,"Create Directory",JOptionPane.ERROR_MESSAGE);
                    return;
                }
                ZipNode zn = curr.getOrCreateChild(dest);//.getOrCreateDescendant(path);
                zn.setDirectory(true);
//                try {
//                    refresh();
//                    //select dir?
//                } catch (IOException e) {
//                    log.log(Level.WARNING, "exception", e);
//                    JOptionPane.showMessageDialog(ZipPanel.this
//                            ,"Couldn't create directory\n  "+e.getMessage()
//                            ,"Create Directory",JOptionPane.ERROR_MESSAGE);
//                } finally{
//                    requery();
//                }
                zipChanged();
                                    refresh();
            }
            getFocus();
        }
    }

    public void move() {
        List<ZipNode> nodes = getSelectedNodes(true,true,false);
        if(nodes.isEmpty()){
            return;
        }
        SaveTarget tp = commanderFrame.getSaveTarget();
        if(tp==null) return;
        if(JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(ZipPanel.this,"Move selected files to other pane?")){
            if (copy(nodes,tp)) {
                delete(nodes);
            }
        }
    }

    private List<ZipNode> getSelectedNodes(boolean includeSelected, boolean includeMarked, boolean includeParents) {
        return csu.getSelectedElements(includeSelected,includeMarked);
//        int[] sel = smh.getSelectedOrMarked(includeSelected, includeMarked, includeParents);
//        final ArrayList<ZipNode> nodes = new ArrayList<ZipNode>(sel.length);
//        for (int i = 0; i < sel.length; i++) {
//            final ZipNode zipNode = (ZipNode) ct.getNode(sel[i]);
//            nodes.add(zipNode);
//        }
//        return nodes;
    }

    public void rename() {
        List<ZipNode> l = getSelectedNodes(true,true,false);
        if(l.size()==1){
            ZipNode zipNode=l.get(0);
            String dest=(String)JOptionPane.showInputDialog(ZipPanel.this,
                    "Rename "+zipNode.getName()+" to:"
                    ,"Rename",JOptionPane.QUESTION_MESSAGE,null
                    ,null,zipNode.getName());
            if(dest!=null){
                try {
                    ZipFile zf = new ZipFile(file);
                    zipNode.cloneWithNewName(dest,zf);
                    zf.close();
                    zipNode.removeFromParent();
                } catch (IOException e) {
                    log.log(Level.FINE, "exception", e);
                    JOptionPane.showMessageDialog(ZipPanel.this
                            ,"Couldn't rename zip entry\n  "+e.getMessage()
                            ,"Rename"
                            ,JOptionPane.ERROR_MESSAGE);
                } catch (CannotCreateEntryException e) {
                    JOptionPane.showMessageDialog(ZipPanel.this
                            ,"Couldn't rename zip entry\n  "+e.getMessage()
                            ,"Rename"
                            ,JOptionPane.ERROR_MESSAGE);
                } finally{
                    refresh();
                     zipChanged();
                }
            }
        }
        else  JOptionPane.showMessageDialog(ZipPanel.this
                ,"Cannot rename: must select 1 file"
                ,"Rename"
                ,JOptionPane.ERROR_MESSAGE);

    }

    public void copy() {
        List<ZipNode> nodes = getSelectedNodes(true,true,false);
        if(nodes.isEmpty()){
            return;
        }
        SaveTarget tp = commanderFrame.getSaveTarget();
        if(tp==null) return;
        if(JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(ZipPanel.this,"Copy selected files to other pane?")){
            copy(nodes,tp);
        }
    }
    private boolean copy(List<ZipNode> nodes, SaveTarget tp){
        try {
            Transferable t = getTransferable(nodes);
            tp.save(t, true);
            return true;
        } catch (NotSupportedException e) {
            JOptionPane.showMessageDialog(ZipPanel.this,"The opposite pane doesn't understand my data!","Copy",JOptionPane.ERROR_MESSAGE);
        } catch (CannotSaveException e) {
            JOptionPane.showMessageDialog(ZipPanel.this,"Couldn't save those data\n"+e.getMessage(),"Copy",JOptionPane.ERROR_MESSAGE);
        }
        return false;
    }


    public void duplicate() {
        List<ZipNode> l = getSelectedNodes(true,true,false);
        if(l.size()==1){
            ZipNode sourceFile=l.get(0);
            String s = sourceFile.getName();
            String dest=(String)JOptionPane.showInputDialog(ZipPanel.this,
                    "Duplicate "+s+" to:"
                    ,"Duplicate",JOptionPane.QUESTION_MESSAGE,null
                    ,null,s+".1");
            if(dest!=null){
                try {
                    ZipFile zf = new ZipFile(file);
                    sourceFile.cloneWithNewName(dest,zf);
                    zf.close();
                  //  writeZipFile();
                } catch (IOException e) {
                    log.log(Level.FINE, "exception", e);
                    JOptionPane.showMessageDialog(ZipPanel.this
                            ,"Couldn't rename zip entry\n  "+e.getMessage()
                            ,"Rename"
                            ,JOptionPane.ERROR_MESSAGE);
                } catch (CannotCreateEntryException e) {
                    JOptionPane.showMessageDialog(ZipPanel.this
                            ,"Couldn't rename zip entry\n  "+e.getMessage()
                            ,"Rename"
                            ,JOptionPane.ERROR_MESSAGE);
                } finally{
                    zipChanged();
                    refresh();
                }
            }
        }
        else  JOptionPane.showMessageDialog(ZipPanel.this
                ,"Cannot rename: must select 1 file"
                ,"Rename"
                ,JOptionPane.ERROR_MESSAGE);

    }

    public void shell() {
        log.fine("runscript");
        CommandManager.getInstance().launchShellCommand(commanderFrame,commanduleContext);
    }

    public void usercommand() {
//        can't really implement this because the idea of a 'working dir' doesn't make sense - all the files have different parent dirs

//        log.fine("usercommand");
//        CommandManager.getInstance().showSelector(commanderFrame,commanduleContext);
    }
    private CommanduleContext commanduleContext=new CommanduleContext() {
        public File getDirectory() {
            return file==null?null:file.getParentFile();
        }
        public List<File> getSelectedFiles() {
//            int[] sel = csu.getSelectedRows(true,true);// smh.getSelectedOrMarked(true, true,true);
//            //weed out directories and nodes without zipentries
//            final ArrayList<ZipEntry> l = getZipEntries(sel, false, false);
//            if(!l.isEmpty()){
//                try {
//                    return writeTempFiles(l);
//                } catch (IOException e) {
//                    log.log(Level.WARNING, "couldn't write tempfiles for usercommand launch", e);
//                }
//            }
            return Collections.emptyList();
        }
    };

    /**
     * reloads the current zip (and path) from the filesystem
     */
    public void requery() {
        try {
            restoreState(saveState());

        } catch (CannotRestoreSavedStateException e) {
            log.log(Level.WARNING, "exception", e);
        } catch (CannotSaveStateException e) {
            log.log(Level.WARNING, "exception", e);
        }
    }
    /**
     * reload the current node from memory (causes requery of zipmodel)
     */
    public void refresh() {
        openNode(getModel().getDirectory());
    }


    private static final DataFlavor[] SEND_FLAVORS={TreeStream.DATAFLAVOR, DataFlavor.javaFileListFlavor};
    private static final DataFlavor[] RECEIVE_FLAVORS={TreeStream.DATAFLAVOR /*,DataFlavor.javaFileListFlavor*/};

    /**
     *
     * @return a transferable object holding references (of various sorts) to all the selected files
     */
    private Transferable getTransferable(final List<ZipNode> nodes){
        //what we want here, is to recursively write all descendants of the selection, but only put the selected files/dirs in the transferable
        //weed out nodes without zipentries
        final ArrayList<ZipEntry> zes = getZipEntries(nodes, true, true);
        final ArrayList<ZipEntry> roots = getZipEntries(nodes, true, false);
//        final List<ZipNode> znRoots = nodes;//getSelectedNodes(true, true, true);
        if(zes.isEmpty()){
            log.fine("ignore drag as sel.length==0");
            return null;
        }
        Transferable t = new Transferable() {
            public DataFlavor[] getTransferDataFlavors() {
                return SEND_FLAVORS;
            }
            public boolean isDataFlavorSupported(DataFlavor flavor) {
                for(DataFlavor f:SEND_FLAVORS) if(f.match(flavor)) return true;
                return false;
            }
            List<File> dragFilesList ;
            public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
                final boolean dataFlavorSupported = isDataFlavorSupported(flavor);
                log.finer("getTransferData dataFlavorSupported="+dataFlavorSupported);
                if(TreeStream.DATAFLAVOR.match(flavor)){
                    List<Node> nodeRoots= new ArrayList<Node>(roots.size());
                    final ZipFile zf = new ZipFile(file);
                    for(ZipNode ze:nodes){
                        nodeRoots.add(new ZipFileStreamNode(zf,ze));
                    }
                    return new TreeStream(nodeRoots,ZipPanel.this){
                        public void close() {
                            super.close();
                            log.fine("closing treestream");
                            try {
                                zf.close();
                            } catch (IOException e) {
                                log.log(Level.WARNING, "exception", e);
                            }
                        }
                    };
                }
                else if (DataFlavor.javaFileListFlavor.match(flavor)) {
                    try {
                        if (dragFilesList==null) {
                            dragFilesList = new ArrayList<File>(zes.size());
                            dragFilesList.addAll(writeTempFiles(zes));
//                            writeTempFiles(zes);
//                            dragFilesList.addAll(getTempFiles(roots));
                        }
                        return dragFilesList;
                    } catch (IOException e) {
                        log.log(Level.WARNING, "exception", e);
//showing an error message here seems to cause gui probs (message can't come to front but is blocking other input)
//                    JOptionPane.showMessageDialog(ct,"Couldn't extract file(s) from archive\n" +
//                            e.getMessage(),"Drag files",JOptionPane.ERROR_MESSAGE);
                        return Collections.emptyList();
                    }
                }
                else throw new UnsupportedFlavorException(flavor);
            }
        };
        return t;
    }

    public void drop(DropTargetDropEvent evt, Transferable transferable) {
        windowToFront();
        if (JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(ZipPanel.this,"Copy dragged data into zip file?")) {
            try {
                save(evt.getTransferable(), true);
            } catch (NotSupportedException e) {
                log.log(Level.WARNING, "exception", e);
                JOptionPane.showMessageDialog(ZipPanel.this,"This window does not allow saving!","Save",JOptionPane.ERROR_MESSAGE);
            } catch (CannotSaveException e) {
                log.log(Level.WARNING, "exception", e);
                JOptionPane.showMessageDialog(ZipPanel.this,"Couldn't save data to zipfile\n" +
                        "["+e.getMessage()+"]","Edit Zip File",JOptionPane.ERROR_MESSAGE);
            }
        }
        return;
    }

    public Transferable getTransferable(DragGestureEvent dge) {
//        List<ZipNode> nodes = getSelectedNodes(true,true,false);
//        if(nodes.size()>0){
//            return getTransferable(nodes);
//        }
//        return null;
        //1. pass drag event
        //2. get row corresponding to mouse-x
        //3. if ro is selected/marked, drag all sel/marked files
        //4. else just drag that file
        if (dge==null) {
            final List<ZipNode> sel = getSelectedNodes(true,true,false);//getSelectedVfiles(true,true,false);
            if(sel.isEmpty()){
                log.fine("ignore drag as sel.length==0");
                return null;
            }
            return getTransferable(sel);
        } else {
            Point p = dge.getDragOrigin();
            int viewRow=table.rowAtPoint(p);
            if(viewRow>=0){
                final int modelRow = table.convertRowIndexToModel(viewRow);
                log.finer("drag modelRow="+modelRow+" vr="+viewRow);
                if(table.isRowSelected(viewRow) || getModel().isMarked(modelRow)){
                    log.finer("drag selected rows");
                    return getTransferable((DragGestureEvent)null);  //return all selected/marked rows (which will incldue this one)
                }
                else{ //drag just this row
                    ZipNode f = getModel().getElement(modelRow);
                    List<ZipNode> l = Collections.singletonList(f);
                    log.finer("drag this row="+f);
                    return getTransferable(l);
                }
            }
            else{
                log.fine("returning null transferable as x coord is not a row");
                return null;
            }
        }

    }

    /**
     *
     * @param sel list of view indices
     * @param includeDirectories
     * @param recurse
     * @return
     */
    private ArrayList<ZipEntry> getZipEntries(int[] sel, boolean includeDirectories, boolean recurse) {
        final ArrayList<ZipEntry> zes = new ArrayList<ZipEntry>(sel.length);
        for (int i = 0; i < sel.length; i++) {
            int modelRow=table.convertRowIndexToModel(sel[i]);
            final ZipNode zipNode = getModel().getElement(modelRow);
            addZipEntries(zipNode, includeDirectories, zes, recurse);
        }
        return zes;
    }
    private ArrayList<ZipEntry> getZipEntries(List<ZipNode> nodes, boolean includeDirectories, boolean recurse) {
        final ArrayList<ZipEntry> zes = new ArrayList<ZipEntry>(nodes.size());
        for (ZipNode zipNode:nodes) {
            addZipEntries(zipNode, includeDirectories, zes, recurse);
        }
        return zes;
    }
    private void addZipEntries(ZipNode zipNode, boolean includeDirectories, final ArrayList<ZipEntry> zes, boolean recurse) {
        if (zipNode.hasEntry()) {
            if(!includeDirectories && zipNode.isDirectory()) return;
            zes.add(zipNode.getZipEntry());
        }
        if(recurse){
            for(ZipNode child:zipNode.getChildren()){
                addZipEntries(child,includeDirectories,zes,recurse);
            }
        }
    }
    
    public Transferable getCopyResult(){
        //copy the selected names to the clipboard
        List<ZipNode> l = getSelectedNodes(true, false, false);
        if (l.size()>0) {
            StringBuffer sb = new StringBuffer();
            for(ZipNode vf:l){
                if(sb.length()>0) sb.append("\n");
                sb.append(vf.getName());
            }
            return new StringSelection(sb.toString());
        }
        return null;
    }
    private void setupPopup() {
        final JPopupMenu popup = new JPopupMenu();
        popup.add(ActionDispatcher.getInstance().createCommanderMenu(this));

        final MouseAdapter ml = new MouseAdapter(){
            private void mouse(MouseEvent e) {
                if(e.isPopupTrigger()){
                    popup.show(table,e.getX(),e.getY());
                }
            }
            public void mousePressed(MouseEvent e) { mouse(e); }
            public void mouseReleased(MouseEvent e) { mouse(e); }
             public void mouseClicked(MouseEvent e) {
                if(e.getClickCount()==2){
                    List<ZipNode> l = getSelectedNodes(true,false,false);
                    if(!l.isEmpty()){
                        onSingleFileClicked(l.get(0));
                    }
                }
            }
        };
        table.addMouseListener(ml);
        rowHeaderTable.addMouseListener(ml);
    }

    private void setupActions() {
        table.registerAction(saveAction,deleteAction,escapeAction,refreshAction
        ,openParentAction,openChildAction,markAction,copyNamesAction,copyPathsAction
        ,locationFieldAction,enterAction);

//fire deleteAction on backspace
        table.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE,0),deleteAction.getValue(Action.ACTION_COMMAND_KEY));
        table.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_F,0),null);
        
//        new AutocompleteJumper(table,new AutocompleteJumper.RowIdentifier() {
//            public String getRowId(int viewRow) {
//                int ro = table.convertRowIndexToModel(viewRow);
//                return getModel().getElement(ro).getName();
//            }
//        });


    }
    private ActionBuilder ab = ActionBuilder.getInstance();
    Action enterAction = ab.init(new AbstractAction("") {
        public void actionPerformed(ActionEvent e) {
            onEnter();
        }
    })
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0))
            .command("enter")
            .get();

    Action locationFieldAction = ab.init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                locationField.selectAll();
                locationField.requestFocus();
            }
        }) .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_L,ctrlMask))
           .command("lbar")
    .get();
    private final Action saveAction = ab.init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            try {
                writeZipFile();
                requery();
            } catch (IOException e1) {
                log.log(Level.WARNING, "exception", e1);
                JOptionPane.showMessageDialog(ZipPanel.this,"Couldn't save zip file:\n  "+e1.getMessage(),"Save Zip File",JOptionPane.ERROR_MESSAGE);
            }
        }
    })      .icon(ImageProvider.getInstance().getIcon("icons/16/filesaveas.png"))
            .command("save").keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_S,AppProperties.getInstance().getCtrlMask()))
            .get();

    private final Action deleteAction = ab.init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            delete();
        }})
            .command("delete").keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0))
            .get();

        static final int ctrlMask = AppProperties.getInstance().getCtrlMask();
        Action escapeAction = ab.init(new AbstractAction("") {
            public void actionPerformed(ActionEvent e) {
                table.clearSelection();
                getModel().clearMarked();
            }
        })
                .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0))
                .command("clear")
                .get();

    Action refreshAction = ab.init(new AbstractAction("") {
        public void actionPerformed(ActionEvent e) {
            refresh();
        }
    })
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_R,AppProperties.getInstance().getCtrlMask()))
            .command("refresh")
            .get();

    Action openParentAction = ab.init(new AbstractAction("") {
        public void actionPerformed(ActionEvent e) {
            ZipNode vf = getModel().getDirectory();
            if(vf!=null){
                ZipNode parent = vf.getParent();
                if(parent !=null){
                    openNode(parent);
                }
                else log.fine("ignoring openparent as parent==null");
            }
            else log.fine("ignoring openparent as dir==null");
        }
    })
            .icon(ImageProvider.getInstance().getIcon("icons/16/1uparrow.png"))
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,0))
            .command("openparent")
            .tooltip("Open parent directory")
            .get();

    Action openChildAction = ab.init(new AbstractAction("") {
        public void actionPerformed(ActionEvent e) {
            List<ZipNode> files =getSelectedNodes(true,false,false);//Vfiles(true,false,false);
            if(!files.isEmpty()){
                ZipNode child = files.get(0);
                if(child !=null && child.isDirectory()){
                    openNode(child);
                }
                else log.fine("ignoring openparent as child==null");
            }
            else log.fine("ignoring openchild 0 files selected");
        }
    })
            .keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,0))
            .command("openchild")
            .tooltip("Open child directory")
            .get();

    private Action markAction =ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            final int[] selectedViewRows = csu.getSelectedRows(true,false);
            final int[] selectedModelRows=new int[selectedViewRows.length];
            int i=0;
            for(int ro:selectedViewRows){
                selectedModelRows[i]=table.convertRowIndexToModel(selectedViewRows[i]);
                i++;
            }
            getModel().toggleMarked(selectedModelRows);

        }
    }).keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0))
            .command("mark").get();

    private Action copyNamesAction =ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            Transferable copyResult = getCopyResult(false);
            if (copyResult!=null) {
                putOnClipboard(copyResult);
            }
        }

    }).keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_C,ctrlMask))
            .command("copynames").get();

    private Action copyPathsAction =ActionBuilder.getInstance().init(new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            Transferable copyResult = getCopyResult(false);
            if (copyResult!=null) {
                putOnClipboard(copyResult);
            }
        }

    }).keyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_C,ctrlMask | KeyEvent.SHIFT_DOWN_MASK))
            .command("copypaths").get();

    private void putOnClipboard(Transferable copyResult) {
        try {
            Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
            cb.setContents(
                    copyResult
                    , new ClipboardOwner() { public void lostOwnership(Clipboard clipboard, Transferable contents) { }}
            );
        } catch (Exception e1) {
            log.log(Level.WARNING, "couldn't copy "+copyResult+" to clipboard", e1);
        }
    }

    private Transferable getCopyResult(boolean copyFullPath){
        //copy the selected names to the clipboard
        List<ZipNode> l = csu.getSelectedElements(true,true);
        if (l.size()>0) {
            StringBuffer sb = new StringBuffer();
            for(ZipNode vf:l){
                if(sb.length()>0) sb.append("\n");
                sb.append(copyFullPath?vf.getPathRelativeToRoot():vf.getName());
            }
            return new StringSelection(sb.toString());
        }
        return null;
    }


    private void writeZipFile() throws IOException {

        final Saver s = new Saver() {
            private int saved=0;
            public int saveTreeStream(TreeStream ts){return 0;}
            public void writeZipFile() throws IOException {
                ZipFile zf = new ZipFile(file);
                File tmp = File.createTempFile("tmp",".zip");
                log.fine("overwrite zipfile="+file+" using tmpfile="+tmp);
                ZipOutputStream zos =new ZipOutputStream(new BufferedOutputStream(new FileOutputStream(tmp)));
                try {

                    final List<ZipNode> topLevels = root.getDescendants();
                    for(ZipNode zn:topLevels){
                        if(wasCancelled()) break;
                        zn.write(zf,zos);
                        zos.closeEntry();
                        updateBlockMessage("Saved "+(++saved)+" (out of "+topLevels.size()+")\n top-level zip entries");
                    }
                } finally {
                    if(zos!=null) zos.close();
                    zf.close();
                }
                if (!wasCancelled()) {
                    if (file.delete()) {
                        log.fine("renamed "+tmp+" to="+file);
                        if(!tmp.renameTo(file)){
                            log.warning("tmp="+tmp+" renameto="+file+" failed!");
                            final List<File> emptyList = Collections.emptyList();
                            fem.fireEvent(Collections.singletonList(file),Collections.singletonList(tmp),emptyList);
                            throw new IOException("couldn't rename file="+tmp+" to "+file);
                        }
                        else fem.fireChanged(file);//notify listeners that the file has changed
                    }
                    else{
                        fem.fireAdded(tmp);
                        throw new IOException("couldn't remove file="+file+" in order to replace it with modified version (="+tmp+")");
                    }
                }
                else{
                    log.fine("cancelled, firing tmpfile added");
                    fem.fireAdded(tmp);
                }
            }
        };
        try {
            blockInputWithCancel("Saving zip file");
            ((Saver) Spin.off(s)).writeZipFile();
        } finally {
            unblockInput();
        }

    }

    private List<File> writeTempFiles(List<ZipEntry> zes) throws IOException {
        //open the zipfile for each read - this may be inefficient, but saves issues arising from holding the file open while
        //we  do other things
        ZipFile zf = new ZipFile(file);
        List<File> outs = new ArrayList<File>(zes.size());
        for (ZipEntry ze:zes) {
            if(ze.isDirectory()) continue; //skip it
            InputStream is = zf.getInputStream(ze);
            File tempRoot= FileUtils.tempDir();
            if(!tempRoot.exists()){
                if(!tempRoot.mkdirs()) throw new IOException("couldn't create temporary directory: "+tempRoot);
            }
            File out = TempFileManager.createTempFile("tmp-zipentry",".tmp");
            OutputStream os = new BufferedOutputStream(new FileOutputStream(out));
            try {
                byte[] buf = new byte[1024];
                int i;
                while((i=is.read(buf))!=-1){
                    os.write(buf,0,i);
                }
                is.close();
            } finally {
                os.close();
            }
            outs.add(out);
        }
        zf.close();
        return outs;
    }

    private void onSingleFileClicked(ZipNode f){
        if(f.isDirectory()){
            openNode(f);
            return;
        }
        else{// if it's a jar/zip file then extract it to ${tmp} then browse it
            try {
                ZipEntry ze = f.getZipEntry();
                if(ze==null){
                    log.warning("ze for node="+f+" was null! ignoring");
                    return;
                }
                if(FileRecognizer.isZipFile(f.getName())){
                    log.fine("got ourselves a nested zip");
                    Object[] possibleValues = { "Browse in CB", "Launch","Cancel" };
                    int selectedValue = JOptionPane.showOptionDialog(this,
                            "What do want to do with the (nested) zip file?","Zip file found..",
                            JOptionPane.CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE, null,
                            possibleValues, possibleValues[0]);
                    if(0==selectedValue){

                        List<File> files = writeTempFiles(Collections.singletonList(ze));
                        final ZipPanel zp = new ZipPanel(commanderFrame,cf);
                        commanderFrame.addTabule(this,zp,JSplitPane.VERTICAL_SPLIT);
                        try {
                            zp.openFile(files.get(0),null);
                            zp.getFocus();
                        } catch (CannotOpenZipException e1) {
                            log.log(Level.WARNING, "couldn't open "+files.get(0), e1);
                            JOptionPane.showMessageDialog(this,"Cannot open zip file",(String)possibleValues[0],JOptionPane.ERROR_MESSAGE);
                        }
                    }
                    else if(1==selectedValue){
                        launchFiles(Collections.singletonList(ze)) ;
                    }
                }
                else{
                    launchFiles(Collections.singletonList(ze));
                }
            } catch(Exception e1){
                log.log(Level.WARNING,"coudln't write tempfile archive",e1);
                JOptionPane.showMessageDialog(ZipPanel.this,"Couldn't extract file from archive\n" +
                        e1.getMessage(),"Open archive",JOptionPane.ERROR_MESSAGE);
            }
            getFocus();
        }
    }


    public void onEnter() {
        List<ZipNode> l = getSelectedNodes(true,false,false);
        if(!l.isEmpty()){
            onSingleFileClicked(l.get(0));
        }

//        int[] sel = csu.getSelectedRows(true,true);//smh.getSelectedOrMarked(true, true,false);
//        //weed out directories and nodes without zipentries
//        final ArrayList<ZipEntry> l = getZipEntries(sel, false, false);
//        ZipEntry ze ;
//        if(l.size()==1 && !(ze=l.get(0)).isDirectory()){  //ignore directories as enter causes them to open
//            onSingleFileClicked();
//        }
    }

    public Commandule createClone(){
        return new ZipPanel(commanderFrame,cf);
    }

    public static final String MAGIC="uk.co.hcsoftware.commander.treeplugin.zip.ZipPanel";//ZipPanel.class.getName();
    public String saveState() throws CannotSaveStateException //save current state to a unicode string that can be used later to restore state
    {
        StringBuilder sb = new StringBuilder();
        sb.append(MAGIC);
        ZipNode zn = getModel().getDirectory();
        if (zn!=null) {
            List<String> strings = new ArrayList<String>(2);
            strings.add(file.getAbsolutePath());
            strings.add(zn.getPathRelativeToRoot());
            sb.append(sc.writeLine(strings.toArray(new String[strings.size()])));
        } else {
            throw new CannotSaveStateException();
        }
        return sb.toString();
    }
    private StringCoder sc = new StringCoder('!');
    private static final String OLDMAGIC="ZIPFILE";
    public void restoreState(String state) throws CannotRestoreSavedStateException {
        final boolean newM = state.startsWith(MAGIC);

        final boolean oldM = state.startsWith(OLDMAGIC);
        if(newM || oldM){
            final String line = state.substring(newM?MAGIC.length():OLDMAGIC.length());
            String[] strings = sc.readLine(line);
            File f = new File(strings[0]);
            try {
                openFile(f,strings.length<2?null:strings[1]);
            } catch (CannotOpenZipException e) {
                throw new CannotRestoreSavedStateException(state, "couldn't open zip",e);
            }
        }
        else throw new CannotRestoreSavedStateException(state, "bad magic number: expected="+MAGIC+" or="+OLDMAGIC+" state="+state);
    }


    public void setColors(Color backColor, Color foreColor, Color selColor) {
         super.setColors(backColor, foreColor, selColor);
       log.fine("setting colors.. back="+backColor+" fore="+foreColor);
       setBackground(backColor);
       jsp.setBackground(backColor);
       jsp.getViewport().setBackground(backColor);
       jsp.getRowHeader().setBackground(backColor);
       locationField.setBackground(backColor);
       locationField.setForeground(foreColor);
   }




}
