package uk.co.hcsoftware.commander.gui.zip;

import uk.co.hcsoftware.commander.gui.ctable.MarkableTableModel;

import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;


/**
 * Title: FileTableModel
 * Copyright:    Copyright (c) 14-Mar-2006
 *
 * @author N Spurling
 */
public class ZipTableModel extends MarkableTableModel<ZipNode> {
    private static final Logger log = Logger.getLogger(ZipTableModel.class.getName());

    public static final int MARK=0;
    public static final int NAME=1;
    public static final int SIZE=2;
    public static final int COMPRESSED=3; //attribute (column) ids
    public static final int DATE=4;

    public void setZipNode(final ZipNode zipNode) {
        final List<ZipNode> kids;
        if(zipNode!=null){
            kids=zipNode.getChildren();
        }
        else kids=Collections.emptyList();
        setDir(zipNode,kids);
    }

    public int getColumnCount() {return  5;}

    public String getColumnName(int index) {
        switch(index){
            case MARK: return "";
            case NAME: return "Name";
            case SIZE: return "Size";
            case DATE: return "Date";
            default: return "Zipped to";
        }
    }
    public Object getValueAt(int rowIndex, int attribute) {
        Node n = children.get(rowIndex);
        ZipNode zipNode=n.getElement();
        switch(attribute){
            case MARK: return n.isMarked();
            case NAME: return zipNode.getName();
            case SIZE:    //zips use -1 when they don't know the size, so we use -2 to signal this (because we are using -1 to denote a dir)
                if(zipNode.isDirectory()) return -1l;
                else{
                    long s=zipNode.getSize();
                    if(s==-1l) return -2l;
                    return s;
                }
            case DATE: return zipNode.getTime();
            default: return zipNode.getCompressedRatio();
        }
    }
}
