package uk.co.hcsoftware.commander.io;

import uk.co.hcsoftware.commander.gui.file.FilesChangedEvent;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;

public class FileEventManager {
    private static final Logger log = Logger.getLogger(FileEventManager.class.getName());
    private static FileEventManager ourInstance = new FileEventManager();

    public static FileEventManager getInstance() {
        return ourInstance;
    }

    private FileEventManager() {
    }

    public void fireDirectoryChanged(File directory) {
        log.finer("fireDirChanged action dir="+directory);
        for(FileEventListener fel: new ArrayList<FileEventListener>(l)){
            fel.directoryChanged(directory);
        }
    }

    //public static final class File
    public interface FileEventListener{
        void filesChanged(FilesChangedEvent fe);

        void directoryChanged(File directory);
    }

    private List<FileEventListener> l = Collections.synchronizedList(new ArrayList<FileEventListener>());
    public void addListener(FileEventListener fel){
        l.add(fel);
    }
    public void removeListener(FileEventListener fel){
        l.remove(fel);
    }
    public void fireEvent(final FilesChangedEvent fe){
        log.finer("fireEvent fe="+fe);
        for(FileEventListener fel: new ArrayList<FileEventListener>(l)){
            fel.filesChanged(fe);
        }
    }
    public void fireChanged(File... changed){
        List<File> empty = Collections.emptyList();
        fireEvent(empty,empty,Arrays.asList(changed));
    }
    public void fireAdded(File... added){
        List<File> empty = Collections.emptyList();
        fireEvent(empty,Arrays.asList(added),empty);
    }
    public void fireDeleted(File... deleted){
        List<File> empty = Collections.emptyList();
        fireEvent(Arrays.asList(deleted),empty,empty);
    }
    public void fireEvent(List<File> deleted, List<File> added, List<File> changed){
        if(nullOrEmpty(deleted) && nullOrEmpty(added) && nullOrEmpty(changed)){
            log.finer("ignoring fireEvent as 0 files");
            return;
        }
        fireEvent(new FilesChangedEvent(deleted,added,changed));
    }
    private boolean nullOrEmpty(List l){
        return l==null || l.isEmpty();
    }
}
