package uk.co.hcsoftware.commander.io;

import uk.co.hcsoftware.commander.io.treestream.Node;
import uk.co.hcsoftware.commander.io.treestream.TreeStream;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.util.List;
import java.util.logging.Logger;


/**
 * Title: FileListTransferable
 * Copyright:    Copyright (c) 01-Mar-2006
 *
 * @author N Spurling
 */
public class FileListTransferable implements Transferable{
    public static final Logger log = Logger.getLogger(FileListTransferable.class.getName());

    public static final DataFlavor[] FLAVORS={TreeStream.DATAFLAVOR, DataFlavor.javaFileListFlavor};

    private final List<File> sel;
    private final Object source;

    public FileListTransferable(List<File> files, Object source) {
        this.sel = files;
        this.source = source;
    }
    private TreeStream ts;
    public DataFlavor[] getTransferDataFlavors() {
        return FLAVORS;
    }
    public boolean isDataFlavorSupported(DataFlavor flavor) {
        for(DataFlavor f:FLAVORS) if(f.match(flavor)) return true;
        return false;
    }
    java.util.List<File> dragFilesList ;
    public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException {
        if (DataFlavor.javaFileListFlavor.match(flavor)) {
            return sel;
        }
        else if(TreeStream.DATAFLAVOR.match(flavor)){
            List<Node> roots;
            if (ts==null) {
                List<File> fs = sel;
                roots = FileNode.getFileNodes(fs.toArray(new File[fs.size()]));
                ts=new TreeStream(roots,source);
            }
            return ts;
        }
        else throw new UnsupportedFlavorException(flavor);
    }
    public int getFileCount(){
        return sel.size();
    }
}
