package uk.co.hcsoftware.commander.io;

import uk.co.hcsoftware.commander.io.treestream.Node;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


/**
 * wraps a file in a TreeStream.Node interface
 */
public class FileNode implements Node {
    private File f;

    public FileNode(File f) {
        this.f = f;
    }

    public String getName() {
        return f.getName();
    }

    public List<Node> getChildren() {
        if(!f.isDirectory()) return Collections.emptyList();
        File[] children = f.listFiles();
        List<Node> l = getFileNodes(children);
        return l;
    }

    public static List<Node> getFileNodes(File[] children) {
        List<Node> l = new ArrayList<Node>(children.length);
        for(File f :children) l.add(new FileNode(f));
        return l;
    }

    /**
     * if false then is a directory
     * @return
     */
    public boolean hasInputStream() {
        return f.isFile();
    }

    public InputStream getInputStream() throws IOException {
        if(!hasInputStream()){
            throw new IOException("cannot get data from file="+f);
        }
        return new FileInputStream(f);
    }

    public String toString() {
        return "FileNode hasIs="+hasInputStream()+" name="+getName();
    }

    public File getFile() {
        return f;
    }

}
