/**
 * Title: ${NAME}
 * Copyright:    Copyright (c) 10-Feb-2006
 * @author spurlinn
 */
package uk.co.hcsoftware.commander.io;

import ns.MacApp;

import javax.swing.*;
import java.io.File;


public class FileRecognizer {
    private static FileRecognizer ourInstance = new FileRecognizer();
    private static final String[] zipSuffixes = {".zip",".jar",".ear",".war",".sxw",".odt"
            ,".ods",".odp",".odf",".odg",".sar",".wmz",".wal"};

    public static FileRecognizer getInstance() {
        return ourInstance;
    }

    private FileRecognizer() {
    }
    private static final String[] tarSuffixes = {".tar"};
    private static final String[] gzTarSuffixes = {".tgz",".tar.gz"};
    private static final String[] bz2TarSuffixes = {".tar.bz2"};
    private static final String[] sevenZipTarSuffixes = {".tar.lzma"};

    enum MimeType{
        image("image.png"
                ,".png",".jpg",".jpeg",".gif",".xcf")
        ,shellscript("shellscript.png"
                ,".sh",".ksh",".bash",".bat")
        ,macapp("binary.png"
                ,".app")
        ,binary("binary.png"
                ,".exe",".class",".so")
        ,html("html.png"
                ,".html",".htm",".jsp",".asp",".aspx")
        ,jar("java_jar.png"
                ,".jar")
        ,zip( "zip.png"
                ,".zip"        )
        ,source("source.png"
                ,".java",".c",".h",".cpp")
        ,tar("tar.png"
                ,".tgz",".tar","tar.gz","tar.bz2")
        ,wordpro("wordprocessing.png"
                ,".doc",".sxw",".tex")
        ,spreadsheet("wordprocessing.png"
                ,".xls")
        ,pdf("pdf.png"
                ,".pdf")
        ,text("txt.png"
                ,".txt",".properties",".xml","")
        ;
        private String imageName;
        private String[] suffixes;

        MimeType(String imageName, String... suffix) {
            this.imageName = imageName;
            this.suffixes = suffix;
        }
        public boolean matches(String filename){
            for(String s:suffixes){
                if(filename.endsWith(s)) return true;
            }
            return false;
        }

        public String getImageName() {
            return imageName;
        }
    }

    ImageProvider ip = ImageProvider.getInstance();
    public Icon getIcon(String filename, boolean isDirectory){
        if(isDirectory){
            if(MacApp.isMac() && MimeType.macapp.matches(filename)){
                 return ip.getIcon("icons/16/mimetypes/"+MimeType.macapp.getImageName());
            }
            return ip.getIcon("icons/16/folder.png");
        }
        filename=filename.toLowerCase();
        for(MimeType mt:MimeType.values()){
            if(mt.matches(filename)){
                return ip.getIcon("icons/16/mimetypes/"+mt.getImageName());
            }
        }
        return ip.getIcon("icons/16/mimetypes/unknown.png");
    }
    public Icon getDirectoryIcon(){
        return ip.getIcon("icons/16/folder.png");
    }
    public boolean isTarFile(File f, boolean includeZipped) {
        boolean b= hasSuffix(f,tarSuffixes);
        if(!b && includeZipped){
            b=isGzippedTarFile(f) || isBzip2edTarFile(f) || isLzmaTarFile(f);
        }
        return b;
    }

    public boolean isGzippedTarFile(File f) {
        return hasSuffix(f,gzTarSuffixes);
    }
    public boolean isBzip2edTarFile(File f) {
        return hasSuffix(f,bz2TarSuffixes);
    }
    public boolean isLzmaTarFile(File f) {
        return hasSuffix(f,sevenZipTarSuffixes);
    }

    private boolean hasSuffix(File f, String[] suffixes){
        String fileName=f.getName().toLowerCase();
        for(String suffix:suffixes){
            if(fileName.endsWith(suffix)) return true;
        }
        return false;
    }

    public static boolean isZipFile(File f) {
        return isZipFile(f.getName());
    }
    public static boolean isZipFile(String fileName) {
        fileName=fileName.toLowerCase();
        for(String suffix:zipSuffixes){
            if(fileName.endsWith(suffix)) return true;
        }
        return false;
    }

}
