package uk.co.hcsoftware.commander.io;

import com.ice.tar.TarOutputStream;
import org.apache.tools.bzip2.CBZip2InputStream;
import org.apache.tools.bzip2.CBZip2OutputStream;

import java.io.*;
import java.nio.channels.FileChannel;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;


/**
 * Title: FileUtils
 * Copyright:    Copyright (c) 01-Dec-2005
 *
 * @author spurlinn
 */
public class FileUtils {
    public static final Logger log = Logger.getLogger(FileUtils.class.getName());
    private static FileUtils instance;

    public static FileUtils getInstance(){
        if(instance==null) instance = new FileUtils();
        return instance;
    }
    /**
     * recuresively deletes all files/dirs under f, and f itself
     * @param f
     * @param pd can be null, if null operation continues despite failures
     */
    public void deleteRecursive(File f, DeleteProgress pd) {
        if(f.isDirectory()){
            log.finer("delete dir="+f+", recursing to children");
            File[] kids = f.listFiles();
            for(File k:kids){
                log.finer("delete child="+k);
                deleteRecursive(k,pd);
            }
        }
        if(f.delete()){
            if(pd!=null) pd.deleted(f);
        }
        else{
            if(pd==null) log.fine("aborting as failure to delete f="+f);//throw new DeleteFailedException(deleted,f);
            else{
                pd.failedToDelete(f);
            }
        }
    }
    public boolean moveToDirRecursive(File f, File newFile){
        return moveRecursive(f,new File(newFile,f.getName()));
    }
//    private class CopiedDir {
//        public File oldDir,newDir;
//        public boolean overwrote;
//
//        public CopiedDir(File oldDir, File newDir, boolean overwrote) {
//            this.oldDir = oldDir;
//            this.newDir = newDir;
//            this.overwrote = overwrote;
//        }
//
//        public String toString() {
//            return "old="+oldDir+" new="+newDir+" overwrote="+overwrote;
//        }
    //    }
    public boolean moveRecursive(File f, File newFile){
        return f.isDirectory()
                ? moveDirectoryRecursive(f,newFile)
                : moveFile(f,newFile)
                ;
    }

    public boolean moveDirectoryRecursive(File f, File newFile){
        if (f.renameTo(newFile)){
            return true;
        }
        if(!newFile.exists() && !newFile.mkdirs()){
            log.finer("couldn't create dir="+newFile);
            return false;
        }
        for(File k:f.listFiles()){
            log.finer("move child="+k);
            final File newChild = new File(newFile, k.getName());
            if(k.isDirectory()){
                moveDirectoryRecursive(k,newChild);
            }
            else{
                moveFile(k,newChild);
            }
        }
        return f.delete();
    }

    public boolean moveFile(File f, File newFile){
        if (f.renameTo(newFile)){
            return true;
        }
        else{//try copy-overwrite then delete
            try {
                copyTo(f,newFile);
                return f.delete();
            } catch (IOException e) {
                log.log(Level.FINER,"exception",e);
                return false;
            }
        }
    }



//    public void moveRecursiveOld(File f, File newFile, final MoveProgress mp){
//        boolean newFileExists = newFile.exists();
//        if (f.renameTo(newFile)){
//            mp.moved(f,newFile,newFileExists);
//        }
//        else{
////            if(newFileExists){ //try deleting it first
////                log.finer("attempting to delete target file="+newFile);
////                FileUtils.getInstance().deleteRecursive(newFile, null);
////                if(f.isFile()){
////                    copyRecursive();
////                }
////                if(f.isDirectory()){// try manual copy/delete
//            log.finer("moving manually..");
//            final LinkedList<CopiedDir> dirsCopied = new LinkedList<CopiedDir>();
//            copyRecursive(f,newFile,new CopyProgress() {
//                public void copied(File f, File newFile, boolean overwrote) {
//                    // delete f, notify mp
//                    if(f.isFile()){
//                        if(f.delete()) mp.moved(f,newFile,overwrote);
//                        else mp.failedToMove(f,newFile);
//                    }
//                    else if(f.isDirectory()){
//                        dirsCopied.add(new CopiedDir(f,newFile,overwrote));
//                    }
//                    //don't delete directories here as we can't guarantee the order in which
//                    //they are copied (e.g. might delete a dir before we've copied its descendants
//                }
//                public void failedToCopy(File f, File newFile) {
//                    mp.failedToMove(f,newFile);
//                }
//                public void failedToCopy(File f, File newFile, IOException e) {
//                    mp.failedToMove(f,newFile,e);
//                }
//            });
//            log.finer("copied successfully count="+dirsCopied.size());
//            //now delete the files we copied successfully
//            for (ListIterator<CopiedDir> iterator = dirsCopied.listIterator(dirsCopied.size()); iterator.hasPrevious();) {
//                CopiedDir file = iterator.previous();
//                log.finer("following move details="+file);
//                if(file.oldDir.delete()){
//                    mp.moved(file.oldDir,file.newDir,file.overwrote);
//                }
//                else{
//                    mp.failedToMove(file.oldDir, file.newDir);
//                }
//            }
////                }
////            }
//        }
    //    }
    public void copyToDirRecursive(File f, File destinationDir, CopyProgress cp){
        copyRecursive(f, new File(destinationDir,f.getName()),cp);
    }
    public void copyRecursive(File f, File newFile, CopyProgress cp){
//        File newFile = new File(newFile,f.getName());
        final boolean newFileExists = newFile.exists();
        if(f.isDirectory()){
            if(!newFileExists){
                if (newFile.mkdirs()) cp.copied(f,newFile,false);
                else{
                    cp.failedToCopy(f,newFile);
                    return;
                }
            }
            log.finer("copy dir="+f+", recursing to children");
            for(File k:f.listFiles()){
                log.finer("copy child="+k);
                copyToDirRecursive(k, newFile, cp);
            }
            cp.copied(f,newFile,newFileExists);
        }
        else{
            try {
                copyTo(f,newFile);
                cp.copied(f,newFile,newFileExists);
            } catch (IOException e) {
                cp.failedToCopy(f,newFile,e);
            }
        }
    }

    public void copyTo(final File sourceFile, final File destFile) throws IOException {
        log.fine("copyfile src="+toString()+" dest="+destFile);
        FileChannel srcChannel = new FileInputStream(sourceFile).getChannel();
        FileChannel dstChannel = new FileOutputStream(destFile).getChannel();
        dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
        srcChannel.close();
        dstChannel.close();
    }

    /**
     *
     * @param l
     * @param parent
     * @return list of files in l whose parentfile is parent
     */
    public List<File> findFilesWithThisParent(List<File> l, File parent){
        List<File> subList = new ArrayList<File>();
        for(File f:l){
            final boolean b = parent.equals(f.getParentFile());
            if (log.isLoggable(Level.FINEST)) {
                log.finest("parent="+parent+" f="+f+" ispar="+b);
            }
            if(b){
                subList.add(f);
            }
        }
        return subList;
    }

    public TarOutputStream getTarOutputStream(final File tarFile) throws IOException {
        OutputStream fos = new FileOutputStream(tarFile);

        try {
            if(FileRecognizer.getInstance().isGzippedTarFile(tarFile)){
                fos = new GZIPOutputStream(fos);
            }
            else if(FileRecognizer.getInstance().isBzip2edTarFile(tarFile)){
                //write the headers
                fos.write(new byte[]{0x42,0x5a});
                fos = new CBZip2OutputStream(fos);
            }
//            else if(FileRecognizer.getInstance().isLzmaTarFile(tarFile)){
//                final PipedInputStream pis = new PipedInputStream();
//                final PipedOutputStream pos = new PipedOutputStream(pis);
//                final OutputStream out1 = fos;
//                Thread t = new Thread(){
//                    public void run() {
//                        try {
//                            try {
//                                Encoder d = new Encoder();
//                                d.Code(pis,out1,1024,1024, new ICodeProgress() {
//                                    public void SetProgress(long inSize, long outSize) {
//                                        log.fine("prog in="+inSize+" out="+outSize);
//                                    }
//                                });
//                            } catch(Exception e){
//                                log.log(Level.WARNING, "exception", e);
//                                //pos.write(-1);//.close();
//                                pos.close();
//                            }finally{
//                                out1.close();
//                            }
//                        }catch (IOException e) {
//                            log.log(Level.WARNING, "couldn'updateProgressTimer close fileinputstream for file="+tarFile);
//                        }
//                    }
//                };
//                t.start();
//                fos= pos;
//            }

            return new TarOutputStream(fos);
        } catch (IOException e) {
            fos.close();
            throw e;
        } catch (RuntimeException e) {
            fos.close();
            throw e;
        }
    }
    public InputStream getTarInputStream(final File tarFile) throws IOException {
        InputStream in = new BufferedInputStream(new FileInputStream(tarFile));
        try {
            if(FileRecognizer.getInstance().isGzippedTarFile(tarFile)){
                in = new GZIPInputStream(in);
            }
            else if(FileRecognizer.getInstance().isBzip2edTarFile(tarFile)){
                StringBuilder sb = new StringBuilder();
                for(int i=0;i<2;i++){
                    sb.append(Integer.toHexString(in.read()));
                    sb.append(' ');
                }
                log.fine("manged bytes: "+sb.toString());// mange the first 2 bytes, these ar the the file header chars
                in = new CBZip2InputStream(in);
            }
//            else if(FileRecognizer.getInstance().isLzmaTarFile(tarFile)){
//                final PipedOutputStream pos = new PipedOutputStream();
//                final PipedInputStream pis = new PipedInputStream(pos);
//                final InputStream in1 = in;
//                Thread t = new Thread(){
//                    public void run() {
//                        try {
//                            try {
//                                Decoder d = new Decoder();
//                                ByteArrayOutputStream propStream = new ByteArrayOutputStream();
//                                Encoder encoder = new Encoder();
//                                encoder.WriteCoderProperties(propStream);
//                                byte[] propArray = propStream.toByteArray();
//                                d.SetDecoderProperties(propArray);
//                                d.Code(in1,pos,1024);
//                                pos.write(-1);//.close();
//                            } catch(Exception e){
//                                log.log(Level.WARNING, "exception", e);
//                                pos.write(-1);//.close();
//                            }finally{
//                                in1.close();
//                            }
//                        }catch (IOException e) {
//                            log.log(Level.WARNING, "couldn'updateProgressTimer close fileinputstream for file="+tarFile);
//                        }
//                    }
//                };
//                t.start();
//                return pis;
//            }
            return in;
        } catch (IOException e) {
            in.close();
            throw e;
        } catch (RuntimeException e) {
            in.close();
            throw e;
        }
    }

    /**
     *  WARNING: mutates the list
     * @param files
     */
    public void weed(List<File> files, boolean includeDirectories, boolean includeFiles) {
        for (Iterator<File> iterator = files.iterator(); iterator.hasNext();) {
            File file =  iterator.next();
            if(!includeDirectories && file.isDirectory()) iterator.remove();
            if(!includeFiles && file.isFile()) iterator.remove();

        }
    }

    //maintain 1 tempdir per user, so we don't run into permission issues
    private static final File tempRoot= new File(System.getProperty("java.io.tmpdir"),"commanderbond_tempdata_"+System.getProperty("user.name"));

    public static File tempDir(){
        return tempRoot;
    }
    public static File defaultTrashDir(){
        return new File(tempRoot,"Trash");
    }

    public String[] splitPath(String filePath){
        //replace any fileseps with '/' character
        String separator = File.separator;
        if("\\".equals(separator)){
            separator="\\\\";
        }
        return filePath.split(separator);
    }
    public File findCommonAncestor(List<File> inputFiles) throws IOException {
        return new CommonAncestorFinder().findCommonAncestor(inputFiles);
    }
    /**
     * finds the common ancestor (if any) of the input file list
     */
    private static class CommonAncestorFinder{
        class Node{
            private String dir;
            private Node parent;
            List<Node> children=new ArrayList<Node>();

            public Node() {
                this(null);
            }

            public Node(String pathElement) {
                this.dir = pathElement;
            }

            public String toString() {
                return dir;
            }

            private boolean hasOneChild(){
                return children.size()==1;
            }

            public File getFile() {
                //traverse back up to root+1
                LinkedList<Node> nodePath =new LinkedList<Node>();
                Node n=this;
                nodePath.add(this);
                while(n.parent!=null){
                    nodePath.addFirst(n.parent);
                    n=n.parent;
                }
                nodePath.removeFirst();//lose the root
                //join together with separators
                StringBuilder sb = new StringBuilder();
                for(Node nn:nodePath){
                    sb.append(nn.dir);
                    sb.append(File.separator);
                }
                if(!nodePath.isEmpty()){
                    sb.setLength(sb.length()-1);
                }
                return new File(sb.toString());
            }

            public void addChild(Node child) {
                children.add(child);
                child.parent=this;
            }

            public Node getChild(String dir) {
                for(Node n:children){
                    if(dir.equals(n.dir)){
                        return n;
                    }
                }
                return null;
            }
        }
        private Node root;
        private void addPath(File f) throws IOException {
            String separator = File.separator;
            if("\\".equals(separator)){
                separator="\\\\";
            }
            String[] path = f.getCanonicalPath().split(separator);
            Node n=root;
            for(String dir:path){
                Node child=n.getChild(dir);
                if(child==null){
                    child = new Node(dir);
                    n.addChild(child);
                }
                n=child;
            }
        }
        private File findCommonAncestor(List<File> inputFiles) throws IOException {
            if(inputFiles.size()==1){
                return inputFiles.get(0).getParentFile();
            }
            root=new Node();
            for(File f:inputFiles){
                addPath(f);
            }
            //traverse the tree
            Node n = root;
            while(n.hasOneChild()){
                n=n.children.get(0);
            }
            log.fine("found deepest node with one child="+n);

            File file = n == root ? null : n.getFile();
            log.finest("found commonanc="+file+" of input="+inputFiles);
            return file;
        }

    }
}
