package uk.co.hcsoftware.commander.io;

import junit.framework.TestCase;

import java.io.File;
import java.io.IOException;
import java.util.Collections;

/**
 * $Log: TestFileUtils.java,v $
 * Revision 1.3  2006/12/10 11:49:08  ns
 * update to 3-nov-2006 version
 *
 */
public class TestFileUtils extends TestCase {
 FileUtils fu = FileUtils.getInstance();
    public void testFindCommonAnc() throws Exception {
        File f = File.createTempFile("abc", "def");
        File commonAncestor = FileUtils.getInstance().findCommonAncestor(Collections.singletonList(f));
        assertEquals(f.getParentFile(),commonAncestor);
    }

    public void testCopy() throws Exception {
        //first create some files
        File tmp = new File(System.getProperty("java.io.tmpdir"));
        final File rfile = new File(tmp, "root");
        final File r2file = new File(tmp, "root2");
        fu.deleteRecursive(rfile, new MyDeleteProgress());
        fu.deleteRecursive(r2file, new MyDeleteProgress());
        assertTrue(!rfile.exists()&& !r2file.exists() );
        File root = mkdir(rfile);
        File root2 = mkdir(r2file);
        File a = mkfile(new File(root,"a"));
        File d1 = mkdir(new File(root,"d1"));
        File d1a = mkfile(d1,"a");
        //root/d1/a
        //root/a


        fu.copyToDirRecursive(root,root2, new Cp());
        assertions(root2);

        //try the copy-overwrite
        long time = System.currentTimeMillis();
        Thread.sleep(1);
        fu.copyToDirRecursive(root,root2, new Cp());
        assertions(root2);
        final File newRoot = new File(root2, "root");
        File newA = new File(newRoot,"a");
        //make sure we actually overwrote a file
        assertTrue("newA.lastModified()="+newA.lastModified()+" t="+time,newA.lastModified()>time);
    }

    private void assertions(File root2) {
        final File newRoot = new File(root2, "root");
        assertTrue(newRoot.exists());
        assertTrue(newRoot.isDirectory());
        File newA = new File(newRoot,"a");
        assertTrue(newA.exists());
        assertTrue(newA.isFile());
        File newD = new File(newRoot,"d1");
        assertTrue(newD.exists());
        assertTrue(newD.isDirectory());
        File newA2 = new File(newD,"a");
        assertTrue(newA2.exists());
        assertTrue(newA2.isFile());
    }

    private class Cp implements CopyProgress{
        public void copied(File f, File newFile, boolean overwrote) {
            System.out.println("copied f = " + f+" newFile = " + newFile);
        }

        public void failedToCopy(File f, File newFile) {
            System.out.println("f       = " + f);
            System.out.println("newFile = " + newFile);
        }

        public void failedToCopy(File f, File newFile, IOException e) {
            System.out.println("f       = " + f);
            System.out.println("newFile = " + newFile);
            System.out.println("e       = " + e);
        }
    }
    private File mkdir(File parent,String name){
        return mkdir(new File(parent,name));
    }
    private File mkdir(File dir){
        assertTrue(dir.mkdirs());
        return dir;
    }
    private File mkfile(File parent,String name) throws IOException {
        return mkfile(new File(parent,name));
    }
    private File mkfile(File f) throws IOException {
        assertTrue(f.createNewFile());
        return f;
    }

    private static class MyDeleteProgress implements DeleteProgress {
        public void deleted(File f) {
            System.out.println("deleted="+f);
        }

        public void failedToDelete(File f) {
             System.out.println("failed="+f);
        }
    }


    public void testMove() throws Exception {
        //first create some files
        File tmp = new File(System.getProperty("java.io.tmpdir"));
        final File rfile = new File(tmp, "root");
        final File r2file = new File(tmp, "root2");
        final File r3file = new File(tmp, "root3");
        fu.deleteRecursive(rfile, new MyDeleteProgress());
        fu.deleteRecursive(r2file, new MyDeleteProgress());
        fu.deleteRecursive(r3file, new MyDeleteProgress());
        assertTrue(!rfile.exists()&& !r2file.exists() && !r3file.exists());
        File root = mkdir(rfile);
        File root2 = mkdir(r2file);
        File root3 = mkdir(r3file);
        File a = mkfile(new File(root,"a"));
        File d1 = mkdir(new File(root,"d1"));
        File d1a = mkfile(d1,"a");
        //root/d1/a
        //root/a

        fu.copyToDirRecursive(root,root3,new Cp());

        fu.moveToDirRecursive(root,root2/*, new MyMoveProgress()*/);
        assertions(root2);

        //try the move-overwrite
        final File r3 = new File(root3, "root");
        fu.moveToDirRecursive(r3,root2/*, new MyMoveProgress()*/);
        assertions(root2);
        assertTrue(!root.exists());
        assertTrue(!r3.exists());
    }

    private static class MyMoveProgress implements MoveProgress {
        public void moved(File f, File newFile,boolean overwrote) {
            System.out.println("moved f = " + f+" newFile = " + newFile);
        }
        public void failedToMove(File f, File newFile) {
            System.out.println("f       = " + f);
            System.out.println("newFile = " + newFile);
        }
        public void failedToMove(File f, File newFile, IOException e) {
            System.out.println("f       = " + f);
            System.out.println("newFile = " + newFile);
            System.out.println("e       = " + e);
        }
    }
}
