package uk.co.hcsoftware.commander.io;

import ns.gui.SwingWorker3;
import uk.co.hcsoftware.commander.gui.HtmlDialog;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;
import uk.co.hcsoftware.commander.prefs.UpdatesPanel;

import javax.swing.*;
import javax.swing.Timer;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.Preferences;
import java.text.SimpleDateFormat;
import java.text.ParseException;


/**
 * Title: UpdateChecker
 * Copyright:    Copyright (c) 19-Apr-2006
 *
 * @author N Spurling
 */
public class UpdateChecker {
    private static final Logger log = Logger.getLogger(UpdateChecker.class.getName());
    private static final Preferences prefs =Preferences.userNodeForPackage(UpdatesPanel.class);

    private String[] propertyNames = {
        "os.arch","os.name","os.version"
        ,"java.vm.vendor","java.vm.version","java.vm.info"
        ,"java.awt.graphicsenv"
        ,"java.class.version"
        ,"user.country","user.language"
    };
    private final sun.misc.BASE64Encoder encoder = new sun.misc.BASE64Encoder();
    private final String UPDATE_URL = "http://www.hcsoftware.co.uk/commanderbond/updatecheck";
//    private final String UPDATE_URL = "http://localhost:8080/commanderbond/updatecheck";



    private boolean contactServer(final HttpURLConnection huc) throws IOException {
        Properties p = new Properties();
        try {
            String value = readResource("version.txt","UTF-8");
            value=value.replaceAll("\n","");
            p.setProperty("version",value);
        } catch (IOException e) {
            log.warning("couldn't get version file value: "+ e);
        }

        try {
            String value = readResource("distribution.txt","UTF-8");
            value=value.replaceAll("\n","");
            p.setProperty("distribution",value);
        } catch (IOException e) {
            log.warning("couldn't get distribution file value: "+ e);
        }

        for(String s:propertyNames){
            p.setProperty(s,System.getProperty(s));
        }
        String query = buildQueryString(p);
        final OutputStreamWriter os = new OutputStreamWriter(huc.getOutputStream());
        try {
            log.fine("sending update request:"+query);
            os.write(query);
            os.flush();

            BufferedReader rd;
            try {
                // Get the response
                rd = new BufferedReader(new InputStreamReader(huc.getInputStream(),"UTF-8"));
                try {
                    StringBuilder sbb = new StringBuilder();
                    char[] buf=new char[1024];
                    int i;
                    while ((i = rd.read(buf)) != -1) {
                        sbb.append(buf,0,i);
                    }
                    responseText=sbb.toString();
                    log.fine("read "+responseText.length()+" bytes from server");
                } finally {
                    rd.close();
                }
                String s= huc.getHeaderField("uk.co.hcsoftware.commander.upatesexist");
                final boolean updates = "TRUE".equals(s);
                log.fine("updatesexist="+updates+" headerfield="+s);
                return updates;
            } finally {
                huc.disconnect();
            }
        } finally {
            os.close();
        }
    }

    private HttpURLConnection createConnection() throws IOException {
        log.fine("connect to: "+UPDATE_URL);
        URL url = new URL(UPDATE_URL);
        final HttpURLConnection huc=(HttpURLConnection) url.openConnection();

        huc.setRequestMethod("POST");
        huc.setDoOutput(true);
        huc.setAllowUserInteraction(false);
        huc.setConnectTimeout(timeoutMillis);
        final boolean useProxy = prefs.getBoolean("useProxy",false);
        log.fine("useProxy="+useProxy);
        if (useProxy) {
            // get from prefs (and set in prefs)
            System.setProperty("http.proxyHost",prefs.get("proxyHost",""));
            System.setProperty("http.proxyPort",prefs.get("proxyPort","8080"));
            String user=prefs.get("proxyUser","");
            String pass=prefs.get("proxyPass","");
            if(user.length()>0 || pass.length()>0){
                String userId = encoder.encode((user+":"+pass).getBytes());
                //see http://www.rgagnon.com/javadetails/java-0085.html
                huc.setRequestProperty("Proxy-Authorization", "Basic " + userId);
            }
            log.fine("http.proxyHost="+System.getProperty("http.proxyHost")+
                    " http.proxyPort="+System.getProperty("http.proxyPort")+
                    " proxyUser="+System.getProperty("proxyUser")+
                    " timeout="+timeoutMillis
            ) ;
        }
        return huc;
    }

    private String buildQueryString(Properties p) throws UnsupportedEncodingException {
        StringBuilder sb = new StringBuilder();
        Enumeration en=p.propertyNames();
        while(en.hasMoreElements()){
            String name=(String) en.nextElement();
            String value=p.getProperty(name);
            if(sb.length()>0) sb.append("&");
            sb.append(URLEncoder.encode(name,"UTF-8"));
            sb.append("=");
            sb.append(URLEncoder.encode(value,"UTF-8"));
        }
        return sb.toString();
    }

    private String responseText;

    public String getResponseText() {
        return responseText;
    }



    private String readResource(String resourceName, String encoding) throws IOException {
        InputStream is = UpdateChecker.class.getClassLoader().getResourceAsStream(resourceName);
        if (is!=null) {
            ByteArrayOutputStream baos;
            try {
                baos = new ByteArrayOutputStream();
                byte[] buf = new byte[1024];
                int i;
                while((i=is.read(buf))!=-1){
                    baos.write(buf,0,i);
                }
            } finally {
                is.close();
            }
            return new String(baos.toByteArray(),encoding);
        }
        else{
            throw new IOException("resource="+resourceName+" not found");
        }
    }

    private int timeoutMillis=20*1000;
    String title = "Check for updates";
    private class GuiChecker{
        private Frame f;
        String message = "Checking for updates";
        String note = "contacting server";
        JDialog progressDialog ;

        int progress=0;
        private final JProgressBar progressbar = new JProgressBar();
        private final JOptionPane pane;

        public GuiChecker(Frame f) {
            this.f = f;
            progressbar.setMinimum(0);
            JLabel noteLabel = new JLabel(note);
            Object[] messageList=new Object[] {message, noteLabel, progressbar};
            pane = new JOptionPane(messageList,
                    JOptionPane.INFORMATION_MESSAGE,
                    JOptionPane.DEFAULT_OPTION,
                    null,
                    new Object[]{UIManager.getString("OptionPane.cancelButtonText")},
                    null);
            progressDialog = pane.createDialog(f, title);
            progressDialog.setModal(false);
        }
        private boolean aborted=false;
        public boolean isCanceled() {
            if (pane == null) return false;
            Object v = pane.getValue();
            return ((v != null) && (v.equals(UIManager.getString("OptionPane.cancelButtonText"))));
        }
        Timer updateProgressTimer= new Timer(500,new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if(isCanceled()){
                    log.fine("cancelled");
                    stopProgress();
                    return;
                }
                if(progress<0){
                    log.fine("auto-cancelling as timer has reached 0-progress, (and network call still not returned?)");
                    html = getErrorHtml("Timeout with no response from server: "+UPDATE_URL);
                    updates=false;
                    aborted=true;
                    stopProgress();
                    showResult();
                    return;
                }
                progress-=500;
                log.fine("update prog to "+progress);
                progressbar.setValue(progress);//.setProgress(progress);
            }
        });

        private void stopProgress() {
            updateProgressTimer.stop();
            progressDialog.dispose();
        }


        public void showResult() {
            if(isCanceled()) return;
            HtmlDialog htmld = getResultDialog(f, updates, html);
            htmld.setVisible(true);
        }


        private String html;
        private boolean updates;

        public void checkNow(){
            try {
                final HttpURLConnection huc = createConnection();
                int timeoutMillis=huc.getConnectTimeout();

                final int max = timeoutMillis+200;
                progress=max;//is a countdown timer, not countup
                progressbar.setMaximum(max);
                progressbar.setValue(progress);
                progressDialog.setModal(modal);
                updateProgressTimer.start();
                new SwingWorker3(){
                    public Object construct() {
                        try {
                            updates= contactServer(huc);
                            if(aborted) return null;
                            stopProgress();
                            html=getResponseText();
                        } catch (IOException e) {
                            if(aborted) return null;
                            onError(e);
                        }
                        return null;
                    }

                    public void finished() {
                        if(aborted) return;
                        stopProgress();
                        GuiChecker.this.showResult();
                    }
                }.start();
                progressDialog.setVisible(true);
            } catch (IOException e) {
                onError(e);
                showResult();
            }
        }

        private void onError(IOException e) {
            log.log(Level.FINE, "exception", e);
            final String errorHtml = getErrorHtml(e);
            html=errorHtml;
            updates=false;
        }
    }
    private boolean modal;
    private String getErrorHtml(Exception e) {
        StringWriter sw = new StringWriter();
        e.printStackTrace(new PrintWriter(sw));
        String stackTrace = sw.toString();
        final String errorHtml = getErrorHtml(stackTrace+"\nurl="+UPDATE_URL);
        return errorHtml;
    }

    private String getErrorHtml(String preErrorDetails) {
        return new String("<html><body><h2>Error contacting server:</h2>" +
                "<p>Looks like there was a problem contacting the server.. Maybe your network settings need adjusting?</p>" +
                "<p>You can paste this text into an email and send to:" +
                "<p><a href=\"mailto:support@hcsoftware.co.uk\">support@hcsoftware.co.uk</a></p>" +
                "</p>" +
                "" +
                "<pre>" +
                preErrorDetails+
                "</pre>" +
                "</body></html>");
    }

    private HtmlDialog getResultDialog(Frame f, boolean updates, String html) {
        final HtmlDialog htmlDialog;
        htmlDialog = new HtmlDialog(f,title,false);
        htmlDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        htmlDialog.setLaunchUrls(true);
        htmlDialog.setModal(modal);
        HtmlDialog htmld=htmlDialog;
        String statusText=updates?"updates found":"no updates found";

        try {
            String versionText = readResource("version.txt","UTF-8");
            if (versionText!=null) {
                final String datePattern = "yyyyMMdd";
                final SimpleDateFormat sdf = new SimpleDateFormat(datePattern);
                sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
                try {
                    Date buildDate = sdf.parse(versionText.substring(0,datePattern.length()));
                    String buildDateString = new SimpleDateFormat("dd-MMM-yyyy").format(buildDate);
                    statusText+=" (This version build on "+buildDateString+")";
                }  catch(IndexOutOfBoundsException e){
                    log.log(Level.WARNING, "versionText probably too short="+versionText, e);
                }  catch (ParseException e) {
                    log.log(Level.WARNING, "bad versionText? ="+versionText, e);
                }
            }
            else{
                log.fine("versionText null");
            }
        }
        catch (IOException e) {
            log.log(Level.WARNING, "exception", e);
        }
        htmld.setStatusText(statusText);
        htmld.loadHtml(html);
        return htmld;
    }


    /**
     * perform the check, pop up GUI dialogs to inform the user of the results/status
     * @param f
     * @param modal
     */
    public void checkNow(Frame f, boolean modal){
        this.modal=modal;
        new GuiChecker(f).checkNow();
    }

    // send a POST request with machine details
    //receive 1. bool: whether or not there are updates for this machine
    //2. some html which we can display to the user (with links)
    public boolean checkForUpdates() throws IOException {
        final HttpURLConnection huc = createConnection();
        return contactServer(huc);
    }
    /**
     * Check for updates, then provide an appropriate action based on the result
     *      error connecting: 'show connection error details' button
     *      no updates: return null
     *      updates: 'show details' button
     *
     * @return
     */
    public Action getUpdatesAction(final Frame f){
        String html="";
        boolean wasError=false,wereUpates=false;
        try {
            wereUpates=checkForUpdates();
            if(!wereUpates){
                return null;
            }
            else{
                html = getResponseText();
            }
        } catch (Exception e) {
            html = getErrorHtml(e);
            wasError=true;
        }
        final HtmlDialog dialog= getResultDialog(f,wereUpates,html);

        return ActionBuilder.getInstance().init(new AbstractAction(){
            public void actionPerformed(ActionEvent e) {
                dialog.setVisible(true);
            }
        })
                .icon(ImageProvider.getInstance().getIcon(
                        wasError?"icons/16/ledred.png"
                :"icons/16/ledgreen.png"
                ))
                .tooltip(
                        wasError?"Error connecting to server for update info, click here for details"
                :"A Newer version is available! click here for details"
                )
                .name("")
                .get();

    }



    public static void main(String[] args) {
        final UpdateChecker uc = new UpdateChecker();
        uc.checkNow(null, true);
    }

}
