package uk.co.hcsoftware.commander.prefs;

import uk.co.hcsoftware.commander.gui.CommanderFrame;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.prefs.Preferences;


public class PrefsDialog extends JDialog{
    private CommanderFrame commanderFrame;

    public PrefsDialog(CommanderFrame cf) throws HeadlessException {
        super(cf,"Preferences",true);
        this.commanderFrame=cf;
        JPanel jp = new JPanel(new BorderLayout(5,5));
        getContentPane().add(jp);
        jp.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
        JTabbedPane jtp = new JTabbedPane();
        jp.add(jtp);
        final PrefsTab[] tabs={new GeneralConfigPanel(), new ColorPanel(),new UpdatesPanel(), new ShellConfigPanel()};
        for(PrefsTab pt:tabs){
            jtp.addTab(pt.getName(),pt.getComponent());
        }
        JPanel bPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        final JButton ok=new JButton("OK"),cancel=new JButton("Cancel"),apply=new JButton("Apply");
        ActionListener al = new ActionListener() {
            private void apply() {
                for(PrefsTab pt:tabs){
                    pt.apply();
                }
            }
            public void actionPerformed(ActionEvent e) {
                Object o = e.getSource();
                if(o==cancel){
                    dispose();
                }
                else if(o==apply){
                    apply();
                }
                else{
                    apply();
                    dispose();
                }
            }
        };
        for(JButton jb:new JButton[]{cancel,apply,ok}){
            jb.addActionListener(al);
            bPanel.add(jb);
        }
        jp.add(bPanel,BorderLayout.SOUTH);
        pack();
        setSize(400,600);
    }


    public static void saveIfNecessary(Preferences p, JTextField textField, String propName, String defaultValue){
        final String text = textField.getText();
        if(!text.equals(p.get(propName, defaultValue))){
            p.put(propName,text);
        }
    }

    public static void saveIfNecessary(Preferences p, JCheckBox textField, String propName, boolean defaultValue){
        final boolean text = textField.isSelected();
        if(!text==p.getBoolean(propName, defaultValue)){
            p.putBoolean(propName,text);
        }
    }

    public CommanderFrame getCommanderFrame() {
        return commanderFrame;
    }
}
