/*
 * ShellConfigPanel.java
 *
 * Created on 13 July 2006, 14:51
 */

package uk.co.hcsoftware.commander.prefs;

import uk.co.hcsoftware.commander.Main;
import uk.co.hcsoftware.commander.gui.usercommands.CommandEditorPanel;
import uk.co.hcsoftware.commander.gui.usercommands.CommandManager;
import uk.co.hcsoftware.commander.gui.usercommands.UserCommand;
import uk.co.hcsoftware.commander.gui.util.ActionBuilder;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.util.logging.Logger;
import java.util.prefs.Preferences;

/**
 *
 * @author  spurlinn
 */
public class ShellConfigPanel extends javax.swing.JPanel implements PrefsTab{
     private static final Logger log = Logger.getLogger(ShellConfigPanel.class.getName());
    
    private final CommandEditorPanel commandEditorPanel = new CommandEditorPanel();

    public ShellConfigPanel() {
        super(new BorderLayout());
        add(commandEditorPanel);
        commandEditorPanel.setNameFieldEnabled(false);
        Action resetAction = ActionBuilder.getInstance().init(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                 commandEditorPanel.setUserCommand(getSavedUserCommand());
            }
        }).name("Reset").tooltip("Reset the command to its previous value").get();
        commandEditorPanel.setSaveAction(resetAction);
        commandEditorPanel.setUserCommand(getSavedUserCommand());
        commandEditorPanel.setHelpText("This command is executed when you invoke the 'Shell' (ctrl+F2)." +
                "\nTypically it launches your favourite command-line programme." +
                "\n\nexample1 (UNIX): xterm\nexample2 (Mac): open -a Terminal\nexample3 (Windows): cmd.exe /c start" +
                "\n\nOptionally the FILES variable can be set: it holds the names of any currently selected files");
    }

    public String getName() {
        return "Shell";
    }
    public Component getComponent() {
        return this;
    }

    public void apply() {
        if(commandEditorPanel.areUnsavedChanges()){
            UserCommand uc = commandEditorPanel.getGuiStateCommand();
            saveUserCommand(uc);
            commandEditorPanel.setUserCommand(uc);
        }
    }

    private void saveUserCommand(UserCommand userCommand) {
        CommandManager.getInstance().saveShellCommand(userCommand);
    }
    /**
     * @return usercommand saved in prefs or default uc if none
     */
    private UserCommand getSavedUserCommand() {
        return CommandManager.getInstance().getShellCommand();
    }
}
